@@ -77,10 +77,10 @@ def main():
          'This usually indicates corruption in your gcloud installation or '
          'problems with your Python interpreter.\n\n'
          'Please verify that the following is the path to a working Python 2.7 '
-         'executable:\n'
+         'or 3.5+ executable:\n'
          '    {2}\n\n'
          'If it is not, please set the CLOUDSDK_PYTHON environment variable to '
-         'point to a working Python 2.7 executable.\n\n'
+         'point to a working Python 2.7 or 3.5+ executable.\n\n'
          'If you are still experiencing problems, please reinstall the Cloud '
          'SDK using the instructions here:\n'
          '    https://cloud.google.com/sdk/\n').format(
