@@ -20,11 +20,11 @@ from __future__ import division
 from __future__ import unicode_literals
 
 import copy
-import itertools
 import json
 import operator
 
 from apitools.base.py import encoding
+from apitools.base.py import exceptions as apitools_exceptions
 from apitools.base.py import list_pager
 from googlecloudsdk.api_lib.app import build as app_cloud_build
 from googlecloudsdk.api_lib.app import env
@@ -197,7 +197,7 @@ class AppengineApiClient(appengine_api_client_base.AppengineApiClientBase):
           build.identifier,
           params={'projectId': properties.VALUES.core.project.GetOrFail},
           collection='cloudbuild.projects.builds')
-      cloudbuild_logs.CloudBuildClient().Stream(build_ref)
+      cloudbuild_logs.CloudBuildClient().Stream(build_ref, out=log.status)
 
     done_poller = operations_util.AppEngineOperationPoller(
         self.client.apps_operations, operation_metadata_type)
@@ -356,20 +356,25 @@ class AppengineApiClient(appengine_api_client_base.AppengineApiClientBase):
       versions: list of version_util.Version
 
     Returns:
-      A generator of each instances_util.Instance for the given versions
+      A list of instances_util.Instance objects for the given versions
     """
-    iters = []
+    instances = []
     for version in versions:
       request = self.messages.AppengineAppsServicesVersionsInstancesListRequest(
           parent=self._FormatVersion(version.service, version.id))
-      iters.append(list_pager.YieldFromList(
-          self.client.apps_services_versions_instances,
-          request,
-          field='instances',
-          batch_size=100,  # Set batch size so tests can expect it.
-          batch_size_attribute='pageSize'))
-    return (instances_util.Instance.FromInstanceResource(i)
-            for i in itertools.chain.from_iterable(iters))
+      try:
+        for instance in list_pager.YieldFromList(
+            self.client.apps_services_versions_instances,
+            request,
+            field='instances',
+            batch_size=100,  # Set batch size so tests can expect it.
+            batch_size_attribute='pageSize'):
+          instances.append(
+              instances_util.Instance.FromInstanceResource(instance))
+      except apitools_exceptions.HttpNotFoundError:
+        # Drop versions that were presumed deleted since initial enumeration.
+        pass
+    return instances
 
   def GetAllInstances(self, service=None, version=None, version_filter=None):
     """Generator of all instances, optionally filtering by service or version.
@@ -536,11 +541,18 @@ class AppengineApiClient(appengine_api_client_base.AppengineApiClientBase):
       # Get the versions.
       request = self.messages.AppengineAppsServicesVersionsListRequest(
           parent=self._GetServiceRelativeName(service.id))
-      for version in list_pager.YieldFromList(
-          self.client.apps_services_versions, request, field='versions',
-          batch_size=100, batch_size_attribute='pageSize'):
-        versions.append(
-            version_util.Version.FromVersionResource(version, service))
+      try:
+        for version in list_pager.YieldFromList(
+            self.client.apps_services_versions,
+            request,
+            field='versions',
+            batch_size=100,
+            batch_size_attribute='pageSize'):
+          versions.append(
+              version_util.Version.FromVersionResource(version, service))
+      except apitools_exceptions.HttpNotFoundError:
+        # Drop services that were presumed deleted since initial enumeration.
+        pass
 
     return versions
 
@@ -728,3 +740,30 @@ class AppengineApiClient(appengine_api_client_base.AppengineApiClientBase):
 
     return operations_util.WaitForOperation(self.client.apps_operations,
                                             operation)
+
+  def UpdateDatabaseType(self, database_type):
+    """Updates an application's database_type.
+
+    Args:
+      database_type: New database type to switch to
+
+    Returns:
+      Long running operation.
+    """
+
+    # Create a configuration update request.
+    update_mask = 'databaseType'
+    application_update = self.messages.Application()
+    application_update.databaseType = database_type
+    update_request = self.messages.AppengineAppsPatchRequest(
+        name=self._FormatApp(),
+        application=application_update,
+        updateMask=update_mask)
+
+    operation = self.client.apps.Patch(update_request)
+
+    log.debug('Received operation: [{operation}] with mask [{mask}]'.format(
+        operation=operation.name, mask=update_mask))
+
+    return operations_util.WaitForOperation(self.client.apps_operations,
+                                            operation)
