@@ -15,21 +15,21 @@
 
 """A library to support auth commands."""
 
+
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
 import json
-import os
 
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
+from googlecloudsdk.core.credentials import flow as c_flow
+from googlecloudsdk.core.credentials import reauth as c_reauth
 from googlecloudsdk.core.credentials import store as c_store
-from googlecloudsdk.core.util import encoding
 from googlecloudsdk.core.util import files
 
 from oauth2client import client
-from oauth2client import clientsecrets
 
 
 # Client ID from project "usable-auth-library", configured for
@@ -46,6 +46,8 @@ DEFAULT_SCOPES = [
     CLOUD_PLATFORM_SCOPE
 ]
 
+CLIENT_SECRET_INSTALLED_TYPE = 'installed'
+
 
 class Error(exceptions.Error):
   """A base exception for this class."""
@@ -57,6 +59,28 @@ class InvalidClientSecretsError(Error):
   pass
 
 
+def DoInstalledAppBrowserFlowGoogleAuth(launch_browser,
+                                        scopes,
+                                        client_id_file=None):
+  """Launches a 3LO oauth2 flow to get google-auth credentials.
+
+  Args:
+    launch_browser: bool, True to launch the browser, false to ask users to copy
+      the auth url to a browser.
+    scopes: [str], The list of scopes to authorize.
+    client_id_file: str, The path to a file containing the client id and secret
+      to use for the flow.  If None, the default client id for the Cloud SDK is
+      used.
+  Returns:
+    google.auth.credentials.Credentials, The credentials obtained from the flow.
+  """
+  if client_id_file:
+    AssertClientSecretIsInstalledType(client_id_file)
+  google_auth_flow = c_flow.CreateGoogleAuthFlow(scopes, client_id_file)
+  user_creds = c_flow.RunGoogleAuthFlow(google_auth_flow, launch_browser)
+  return c_reauth.UserCredWithReauth.FromGoogleAuthUserCredentials(user_creds)
+
+
 def DoInstalledAppBrowserFlow(launch_browser, scopes, client_id_file=None,
                               client_id=None, client_secret=None):
   """Launches a browser to get credentials.
@@ -78,11 +102,7 @@ def DoInstalledAppBrowserFlow(launch_browser, scopes, client_id_file=None,
   """
   try:
     if client_id_file:
-      client_type = GetClientSecretsType(client_id_file)
-      if client_type != clientsecrets.TYPE_INSTALLED:
-        raise InvalidClientSecretsError(
-            'Only client IDs of type \'%s\' are allowed, but encountered '
-            'type \'%s\'' % (clientsecrets.TYPE_INSTALLED, client_type))
+      AssertClientSecretIsInstalledType(client_id_file)
       webflow = client.flow_from_clientsecrets(
           filename=client_id_file,
           scope=scopes)
@@ -101,15 +121,12 @@ def DoInstalledAppBrowserFlow(launch_browser, scopes, client_id_file=None,
     raise
 
 
-# TODO(b/29157057) make this information accessible through oauth2client
-# instead of duplicating internal code from clientsecrets
 def GetClientSecretsType(client_id_file):
   """Get the type of the client secrets file (web or installed)."""
   invalid_file_format_msg = (
       'Invalid file format. See '
       'https://developers.google.com/api-client-library/'
       'python/guide/aaa_client_secrets')
-
   try:
     obj = json.loads(files.ReadFileContents(client_id_file))
   except files.Error:
@@ -125,35 +142,9 @@ def GetClientSecretsType(client_id_file):
   return tuple(obj)[0]
 
 
-def ADCFilePath():
-  """Gets the ADC default file path.
-
-  Returns:
-    str, The path to the default ADC file.
-  """
-  # pylint:disable=protected-access
-  return client._get_well_known_file()
-
-
-def AdcEnvVariable():
-  """Gets the value of the ADC environment variable.
-
-  Returns:
-    str, The value of the env var or None if unset.
-  """
-  return encoding.GetEncodedValue(
-      os.environ, client.GOOGLE_APPLICATION_CREDENTIALS, None)
-
-
-def SaveCredentialsAsADC(creds):
-  """Saves the credentials to the given file.
-
-  Args:
-    creds: The credentials obtained from a web flow.
-
-  Returns:
-    str, The full path to the ADC file that was written.
-  """
-  adc_file = ADCFilePath()
-  c_store.SaveCredentialsAsADC(creds, adc_file)
-  return os.path.abspath(adc_file)
+def AssertClientSecretIsInstalledType(client_id_file):
+  client_type = GetClientSecretsType(client_id_file)
+  if client_type != CLIENT_SECRET_INSTALLED_TYPE:
+    raise InvalidClientSecretsError(
+        'Only client IDs of type \'%s\' are allowed, but encountered '
+        'type \'%s\'' % (CLIENT_SECRET_INSTALLED_TYPE, client_type))
