@@ -18,6 +18,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import copy
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import exceptions as calliope_exceptions
 
@@ -205,6 +206,36 @@ def AddTcpRelatedUpdateArgs(parser):
   _AddTcpRelatedArgsImpl(add_info_about_clearing=True, parser=parser)
 
 
+def AddGrpcRelatedCreationArgs(parser):
+  """Adds parser arguments for creation related to gRPC."""
+
+  _AddPortRelatedCreationArgs(
+      parser,
+      use_port_name=False,
+      use_serving_port=True,
+      port_type='TCP',
+      default_port=None)
+
+  parser.add_argument(
+      '--grpc-service-name',
+      help="""\
+      An optional gRPC service name string of up to 1024 characters to include
+      in the gRPC health check request. Only ASCII characters are allowed.""")
+
+
+def AddGrpcRelatedUpdateArgs(parser):
+  """Adds parser arguments for update subcommands related to gRPC."""
+
+  _AddPortRelatedUpdateArgs(parser, use_port_name=False)
+
+  parser.add_argument(
+      '--grpc-service-name',
+      help="""\
+      An optional gRPC service name string of up to 1024 characters to include
+      in the gRPC health check request. Pass in an empty string to unset.
+      Only ASCII characters are allowed.""")
+
+
 def AddUdpRelatedArgs(parser, request_and_response_required=True):
   """Adds parser arguments related to UDP."""
 
@@ -229,6 +260,7 @@ def AddUdpRelatedArgs(parser, request_and_response_required=True):
 
 
 def _AddPortRelatedCreationArgs(parser,
+                                use_port_name=True,
                                 use_serving_port=True,
                                 port_type='TCP',
                                 default_port=80):
@@ -239,13 +271,13 @@ def _AddPortRelatedCreationArgs(parser,
   ]
   if default_port:
     port_group_help.append(
-        'If none is specified, the default port of 80 is used; if'
+        'If none is specified, the default port of 80 is used.'
     )
-  else:
-    port_group_help.append('If')
-  port_group_help.append(
-      'both `--port` and `--port-name` are specified, `--port` takes '
-      'precedence.')
+
+  if use_port_name:
+    port_group_help.append(
+        'If both `--port` and `--port-name` are specified, `--port` takes '
+        'precedence.')
   port_group = parser.add_group(help=' '.join(port_group_help))
   port_group.add_argument(
       '--port',
@@ -255,39 +287,41 @@ def _AddPortRelatedCreationArgs(parser,
       The {} port number that this health check monitors.
       """.format(port_type))
 
-  port_group.add_argument(
-      '--port-name',
-      help="""\
-      The port name that this health check monitors. By default, this is
-      empty.
-      """)
+  if use_port_name:
+    port_group.add_argument(
+        '--port-name',
+        help="""\
+        The port name that this health check monitors. By default, this is
+        empty.
+        """)
 
   if use_serving_port:
-    _AddUseServingPortFlag(port_group)
+    _AddUseServingPortFlag(port_group, use_port_name)
 
 
-def _AddPortRelatedUpdateArgs(parser):
+def _AddPortRelatedUpdateArgs(parser, use_port_name=True):
   """Adds parser update subcommand arguments --port and --port-name."""
 
-  port_group = parser.add_group(help=(
-      'These flags configure the port that the health check monitors. '
-      'If both `--port` and `--port-name` are specified, `--port` takes '
-      'precedence.'))
+  port_group = parser.add_group(
+      help=('These flags configure the port that the health check monitors.%s' %
+            (' If both `--port` and `--port-name` are specified, `--port` takes'
+             ' precedence.' if use_port_name else '')))
 
   port_group.add_argument(
       '--port',
       type=int,
       help='The TCP port number that this health check monitors.')
 
-  port_group.add_argument(
-      '--port-name',
-      help="""\
-      The port name that this health check monitors. By default, this is
-      empty. Setting this to an empty string will clear any existing
-      port-name value.
-      """)
+  if use_port_name:
+    port_group.add_argument(
+        '--port-name',
+        help="""\
+        The port name that this health check monitors. By default, this is
+        empty. Setting this to an empty string will clear any existing
+        port-name value.
+        """)
 
-  _AddUseServingPortFlag(port_group)
+  _AddUseServingPortFlag(port_group, use_port_name)
 
 
 def _AddTcpRelatedArgsImpl(add_info_about_clearing, parser):
@@ -422,6 +456,20 @@ def ValidateAndAddPortSpecificationToHealthCheck(args, x_health_check):
       x_health_check.portSpecification = enum_class.USE_FIXED_PORT
 
 
+def ValidateAndAddPortSpecificationToGRPCHealthCheck(args, x_health_check):
+  """Modifies the gRPC health check as needed and adds port specification."""
+
+  enum_class = type(x_health_check).PortSpecificationValueValuesEnum
+  if args.use_serving_port:
+    if args.IsSpecified('port'):
+      _RaiseBadPortSpecificationError('--port', '--use-serving-port',
+                                      '--use-serving-port')
+    x_health_check.portSpecification = enum_class.USE_SERVING_PORT
+    x_health_check.port = None
+  else:
+    x_health_check.portSpecification = enum_class.USE_FIXED_PORT
+
+
 def HandlePortRelatedFlagsForUpdate(args, x_health_check):
   """Calculate port, port_name and port_specification for HC update."""
   port = x_health_check.port
@@ -461,7 +509,30 @@ def HandlePortRelatedFlagsForUpdate(args, x_health_check):
   return port, port_name, port_specification
 
 
-def _AddUseServingPortFlag(parser):
+def HandlePortRelatedFlagsForGRPCUpdate(args, x_health_check):
+  """Calculate port and port_specification for gRPC HC update."""
+  port = x_health_check.port
+  port_specification = x_health_check.portSpecification
+  enum_class = type(x_health_check).PortSpecificationValueValuesEnum
+
+  if args.use_serving_port:
+    if args.IsSpecified('port'):
+      _RaiseBadPortSpecificationError('--port', '--use-serving-port',
+                                      '--use-serving-port')
+    port = None
+    port_specification = enum_class.USE_SERVING_PORT
+
+  if args.IsSpecified('port'):
+    port = args.port
+    port_specification = enum_class.USE_FIXED_PORT
+  else:
+    # Inherited values from existing health check should remain.
+    pass
+
+  return port, port_specification
+
+
+def _AddUseServingPortFlag(parser, use_port_name=True):
   """Adds parser argument for using serving port option."""
   parser.add_argument(
       '--use-serving-port',
@@ -471,8 +542,8 @@ def _AddUseServingPortFlag(parser):
 
         - When health checking network endpoints in a Network Endpoint
           Group, use the port specified with each endpoint.
-        - When health checking other backends, use the port or named port of
-          the backend service.""")
+        - When health checking other backends, use the port%s of
+          the backend service.""" % (' or named port' if use_port_name else ''))
 
 
 def IsRegionalHealthCheckRef(health_check_ref):
@@ -485,3 +556,47 @@ def IsGlobalHealthCheckRef(health_check_ref):
   """Returns True if the health check reference is global."""
 
   return health_check_ref.Collection() == 'compute.healthChecks'
+
+
+def AddHealthCheckLoggingRelatedArgs(parser):
+  """Adds parser arguments for health check log config."""
+
+  # The parser automatically adds support for --no-enable-logging. Argument
+  # value is set to false internally, if --no-enable-logging is specified.
+  parser.add_argument(
+      '--enable-logging',
+      action='store_true',
+      default=None,
+      help="""Enable logging of health check probe results to Stackdriver.
+      Logging is disabled by default.
+
+      Use --no-enable-logging to disable logging.""")
+
+
+def CreateLogConfig(client, args):
+  """Returns a HealthCheckLogconfig message if args are valid."""
+
+  messages = client.messages
+  log_config = None
+  if args.enable_logging is not None:
+    log_config = messages.HealthCheckLogConfig(enable=args.enable_logging)
+  return log_config
+
+
+def ModifyLogConfig(client, args, existing_log_config):
+  """Returns a modified HealthCheckLogconfig message."""
+
+  messages = client.messages
+  log_config = None
+  if not existing_log_config:
+    # If log config has not changed then return early.
+    if args.enable_logging is None:
+      return log_config
+    log_config = messages.HealthCheckLogConfig()
+  else:
+    log_config = copy.deepcopy(existing_log_config)
+
+  if args.enable_logging is not None:
+    log_config.enable = args.enable_logging
+
+  return log_config
