@@ -31,9 +31,16 @@ from googlecloudsdk.core import properties
 EPHEMERAL_ADDRESS = object()
 
 
-def CreateNetworkInterfaceMessage(
-    resources, scope_lister, messages, network, region, subnet, address,
-    alias_ip_ranges_string=None, network_tier=None):
+def CreateNetworkInterfaceMessage(resources,
+                                  scope_lister,
+                                  messages,
+                                  network,
+                                  private_ip,
+                                  region,
+                                  subnet,
+                                  address,
+                                  alias_ip_ranges_string=None,
+                                  network_tier=None):
   """Creates and returns a new NetworkInterface message.
 
   Args:
@@ -41,6 +48,7 @@ def CreateNetworkInterfaceMessage(
     scope_lister: function, provides scopes for prompting subnet region,
     messages: GCE API messages,
     network: network,
+    private_ip: IPv4 internal IP address to assign to the instance.
     region: region for subnetwork,
     subnet: regional subnetwork,
     address: specify static address for instance template
@@ -78,6 +86,9 @@ def CreateNetworkInterfaceMessage(
         collection='compute.networks')
     network_interface.network = network_ref.SelfLink()
 
+  if private_ip is not None:
+    network_interface.networkIP = private_ip
+
   if address:
     access_config = messages.AccessConfig(
         name=constants.DEFAULT_ACCESS_CONFIG_NAME,
@@ -125,13 +136,12 @@ def CreateNetworkInterfaceMessages(resources, scope_lister, messages,
 
       network_tier = interface.get('network-tier', None)
 
-      result.append(CreateNetworkInterfaceMessage(
-          resources, scope_lister, messages, interface.get('network', None),
-          region,
-          interface.get('subnet', None),
-          address,
-          interface.get('aliases', None),
-          network_tier))
+      result.append(
+          CreateNetworkInterfaceMessage(
+              resources, scope_lister, messages, interface.get('network', None),
+              interface.get('private-network-ip', None), region,
+              interface.get('subnet', None), address,
+              interface.get('aliases', None), network_tier))
   return result
 
 
