@@ -1,4 +1,5 @@
-# Copyright 2014 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2014 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,9 +18,16 @@ These utility functions enable easy replacement of parameters into
 ReplicaPool template files.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import arg_parsers
-from googlecloudsdk.calliope import exceptions
-import yaml
+from googlecloudsdk.calliope import exceptions as calliope_exceptions
+from googlecloudsdk.core import yaml
+from googlecloudsdk.core.util import files
+
+import six
 
 
 def AddTemplateParamArgs(parser):
@@ -53,44 +61,39 @@ def ParseTemplate(template_file, params=None, params_from_file=None):
     The parsed template dict
 
   Raises:
-    BadFileException: When the template file cannot be read
-    ToolException: If any params are not provided or the YAML file is invalid
+    yaml.Error: When the template file cannot be read or parsed.
+    ToolException: If any params are not provided or the YAML file is invalid.
   """
   params = params or {}
   params_from_file = params_from_file or {}
 
   joined_params = dict(params)
-  for key, file_path in params_from_file.iteritems():
+  for key, file_path in six.iteritems(params_from_file):
     if key in joined_params:
-      raise exceptions.ToolException('Duplicate param key: ' + key)
+      raise calliope_exceptions.ToolException('Duplicate param key: ' + key)
     try:
-      with open(file_path) as opened_file:
-        joined_params[key] = opened_file.read()
-    except IOError as e:
-      raise exceptions.ToolException(
+      joined_params[key] = files.ReadFileContents(file_path)
+    except files.Error as e:
+      raise calliope_exceptions.ToolException(
           'Could not load param key "{0}" from file "{1}": {2}'.format(
               key, file_path, e.strerror))
 
-  try:
-    with open(template_file) as opened_file:
-      template = yaml.load(opened_file)
-  except IOError as e:
-    raise exceptions.BadFileException(e)
+  template = yaml.load_path(template_file)
 
   if not isinstance(template, dict) or 'template' not in template:
-    raise exceptions.ToolException(
+    raise calliope_exceptions.ToolException(
         'Invalid template format.  Root must be a mapping with single '
         '"template" value')
 
   (template, missing_params, used_params) = ReplaceTemplateParams(
       template, joined_params)
   if missing_params:
-    raise exceptions.ToolException(
+    raise calliope_exceptions.ToolException(
         'Some parameters were present in the template but not provided on '
         'the command line: ' + ', '.join(sorted(missing_params)))
   unused_params = set(joined_params.keys()) - used_params
   if unused_params:
-    raise exceptions.ToolException(
+    raise calliope_exceptions.ToolException(
         'Some parameters were specified on the command line but not referenced '
         'in the template: ' + ', '.join(sorted(unused_params)))
   return template
@@ -109,7 +112,7 @@ def ReplaceTemplateParams(node, params):
     references found in the node that were not provided and used_params were
     the params that we actually used.
   """
-  if isinstance(node, basestring):
+  if isinstance(node, six.string_types):
     if node.startswith('{{') and node.endswith('}}'):
       param = node[2:-2].strip()
       if param in params:
