@@ -175,6 +175,13 @@ class OperationData(object):
     """
     self.operation = operation
 
+  def IsGlobalOrganizationOperation(self):
+    if not hasattr(self.operation_service.client,
+                   'globalOrganizationOperations'):
+      return False
+    return (self.operation_service ==
+            self.operation_service.client.globalOrganizationOperations)
+
   def IsDone(self):
     """Returns true if the operation is done."""
     operation_type = self.operation_service.GetResponseType('Get')
@@ -320,7 +327,7 @@ class OperationData(object):
 
 def WaitForOperations(
     operations_data, http, batch_url, warnings, errors,
-    progress_tracker=None, timeout=None):
+    progress_tracker=None, timeout=None, log_result=True):
   """Blocks until the given operations are done or until a timeout is reached.
 
   Args:
@@ -333,6 +340,8 @@ def WaitForOperations(
                       finish.
     timeout: The maximum amount of time, in seconds, to wait for the
       operations to reach the DONE state.
+    log_result: Whether the Operation Waiter should print the result in past
+      tense of each request.
 
   Yields:
     The resources pointed to by the operations' targetLink fields if
@@ -401,17 +410,26 @@ def WaitForOperations(
         # is delete because there will be no resource left.
         if not _IsDeleteOp(operation.operationType):
           request = data.ResourceGetRequest()
-          resource_requests.append((resource_service, 'Get', request))
+          # Some operations do not have target and should not send get request.
+          if request:
+            resource_requests.append((resource_service, 'Get', request))
 
-        log.status.write('{0} [{1}].\n'.format(
-            _HumanFriendlyNameForOpPastTense(
-                operation.operationType).capitalize(), operation.targetLink))
+        # Only log when there is target link in the operation.
+        if operation.targetLink and log_result:
+          log.status.write('{0} [{1}].\n'.format(
+              _HumanFriendlyNameForOpPastTense(
+                  operation.operationType).capitalize(), operation.targetLink))
 
       else:
-        # The operation has not reached the DONE state, so we add a
-        # get request to poll the operation.
-        request = data.OperationGetRequest()
-        operation_requests.append((operation_service, 'Get', request))
+        # The operation has not reached the DONE state, so we add a request
+        # to poll the operation.
+        # TODO(b/129413862): Global org operation service supports wait API.
+        if data.IsGlobalOrganizationOperation():
+          request = data.OperationGetRequest()
+          operation_requests.append((operation_service, 'Get', request))
+        else:
+          request = data.OperationWaitRequest()
+          operation_requests.append((operation_service, 'Wait', request))
 
     requests = resource_requests + operation_requests
     if not requests:
