@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,6 +25,7 @@ from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.core import log
+import six
 
 ANDROID_INSTRUMENTATION_TEST = 'ANDROID INSTRUMENTATION TEST'
 ANDROID_ROBO_TEST = 'ANDROID ROBO TEST'
@@ -53,7 +54,19 @@ def AddCommonTestRunArgs(parser):
       '--async',
       action='store_true',
       default=None,
+      dest='async_',
       help='Invoke a test asynchronously without waiting for test results.')
+  parser.add_argument(
+      '--num-flaky-test-attempts',
+      metavar='int',
+      type=arg_validate.NONNEGATIVE_INT_PARSER,
+      help="""\
+      Specifies the number of times a test execution should be reattempted if
+      one or more of its test cases fail for any reason. An execution that
+      initially fails but succeeds on any reattempt is reported as FLAKY.\n
+      The maximum number of reruns allowed is 10. (Default: 0, which implies
+      no reruns.) All additional attempts are executed in parallel.
+      """)
   parser.add_argument(
       '--record-video',
       action='store_true',
@@ -99,11 +112,14 @@ def AddAndroidTestArgs(parser):
       '--app',
       category=base.COMMONLY_USED_FLAGS,
       help='The path to the application binary file. The path may be in the '
-      'local filesystem or in Google Cloud Storage using gs:// notation.')
+      'local filesystem or in Google Cloud Storage using gs:// notation. '
+      'Android App Bundles are specified as .aab, all other files are assumed '
+      'to be APKs.')
   parser.add_argument(
       '--app-package',
-      help='The Java package of the application under test (default: extracted '
-      'from the APK manifest).')
+      action=actions.DeprecationAction('--app-package', removed=True),
+      help='The Java package of the application under test. By default, the '
+      'application package name is parsed from the APK manifest.')
   parser.add_argument(
       '--auto-google-login',
       action='store_true',
@@ -130,22 +146,15 @@ def AddAndroidTestArgs(parser):
       metavar='KEY=VALUE',
       help="""\
       A comma-separated, key=value map of environment variables and their
-      desired values. This flag is repeatable. The environment variables are
-      mirrored as extra options to the `am instrument -e KEY1 VALUE1 ...`
-      command and passed to your test runner (typically AndroidJUnitRunner).
-      Examples:
+      desired values. The environment variables are mirrored as extra options to
+      the `am instrument -e KEY1 VALUE1 ...` command and passed to your test
+      runner (typically AndroidJUnitRunner). Examples:
 
-      Break test cases into four shards and run only the first shard:
-
-      ```
-      --environment-variables numShards=4,shardIndex=0
-      ```
-
-      Enable code coverage and provide a file path to store the coverage
+      Enable code coverage and provide a directory to store the coverage
       results when using Android Test Orchestrator (`--use-orchestrator`):
 
       ```
-      --environment-variables clearPackageData,coverage=true,coverageFile=/sdcard/coverage.ec
+      --environment-variables clearPackageData=true,coverage=true,coverageFilePath=/sdcard/
       ```
 
       Enable code coverage and provide a file path to store the coverage
@@ -197,9 +206,10 @@ def AddAndroidTestArgs(parser):
       'using a URL beginning with `gs://`.')
   parser.add_argument(
       '--test-package',
+      action=actions.DeprecationAction('--test-package', removed=True),
       category=ANDROID_INSTRUMENTATION_TEST,
-      help='The Java package name of the instrumentation test (default: '
-      'extracted from the APK manifest).')
+      help='The Java package name of the instrumentation test. By default, the '
+      'test package name is parsed from the APK manifest.')
   parser.add_argument(
       '--test-runner-class',
       category=ANDROID_INSTRUMENTATION_TEST,
@@ -240,40 +250,6 @@ def AddAndroidTestArgs(parser):
 
   # The following args are specific to Android Robo tests.
 
-  parser.add_argument(
-      '--max-steps',
-      action=actions.DeprecationAction(
-          '--max-steps',
-          warn=('The `--max-steps` flag is deprecated and no longer has any '
-                'effect on the Robo crawler. The `--timeout` flag may be '
-                'optionally used to limit the maximum length of a Robo test.')),
-      metavar='int',
-      category=ANDROID_ROBO_TEST,
-      type=arg_validate.NONNEGATIVE_INT_PARSER,
-      help='The maximum number of steps/actions a Robo test can execute '
-      '(default: no limit).')
-  parser.add_argument(
-      '--max-depth',
-      action=actions.DeprecationAction(
-          '--max-depth',
-          warn=('The `--max-depth` flag is deprecated and no longer has any '
-                'effect on the actions of the Robo crawler.')),
-      metavar='int',
-      category=ANDROID_ROBO_TEST,
-      type=arg_validate.POSITIVE_INT_PARSER,
-      help='The maximum depth of the traversal stack a Robo test can explore. '
-      'Needs to be at least 2 to make Robo explore the app beyond the first '
-      'activity (default: 50).')
-  parser.add_argument(
-      '--app-initial-activity',
-      action=actions.DeprecationAction(
-          '--app-initial-activity',
-          warn=('The `--app-initial-activity` flag is deprecated and no longer '
-                'has any effect on the Robo crawler. Alternatively, the '
-                '`--robo-script` flag (in beta) can be used to guide Robo to a '
-                'specific part of your app before the Robo test begins.')),
-      category=ANDROID_ROBO_TEST,
-      help='The initial activity used to start the app during a Robo test.')
   parser.add_argument(
       '--robo-directives',
       metavar='TYPE:RESOURCE_NAME=INPUT',
@@ -282,23 +258,28 @@ def AddAndroidTestArgs(parser):
       help='A comma-separated (`<type>:<key>=<value>`) map of '
       '`robo_directives` that you can use to customize the behavior of Robo '
       'test. The `type` specifies the action type of the directive, which may '
-      'take on values `click` or `text`. If no `type` is provided, `text` will '
-      'be used by default. Each key should be the Android resource name of a '
-      'target UI element and each value should be the text input for that '
-      'element. Values are only permitted for `text` type elements, so no '
-      'value should be specified for `click` type elements. For example, use'
+      'take on values `click`, `text` or `ignore`. If no `type` is provided, '
+      '`text` will be used by default. Each key should be the Android resource '
+      'name of a target UI element and each value should be the text input for '
+      'that element. Values are only permitted for `text` type elements, so no '
+      'value should be specified for `click` and `ignore` type elements.'
+      '\n\n'
+      'To provide custom login credentials for your app, use'
       '\n\n'
       '    --robo-directives text:username_resource=username,'
       'text:password_resource=password'
       '\n\n'
-      'to provide custom login credentials for your app, or'
+      'To instruct Robo to click on the sign-in button, use'
       '\n\n'
       '    --robo-directives click:sign_in_button='
       '\n\n'
-      'to instruct Robo to click on the sign in button. To learn more about '
-      'Robo test and robo_directives, see '
-      'https://firebase.google.com/docs/test-lab/command-line'
-      '#custom_login_and_text_input_with_robo_test.'
+      'To instruct Robo to ignore any UI elements with resource names which '
+      'equal or start with the user-defined value, use'
+      '\n\n'
+      '  --robo-directives ignore:ignored_ui_element_resource_name='
+      '\n\n'
+      'To learn more about Robo test and robo_directives, see '
+      'https://firebase.google.com/docs/test-lab/android/command-line#custom_login_and_text_input_with_robo_test.'
       '\n\n'
       'Caution: You should only use credentials for test accounts that are not '
       'associated with real users.')
@@ -340,8 +321,7 @@ def AddIosTestArgs(parser):
   parser.add_argument(
       '--type',
       category=base.COMMONLY_USED_FLAGS,
-      hidden=True,
-      choices=['xctest'],
+      choices=['xctest', 'game-loop'],
       help='The type of iOS test to run.')
   parser.add_argument(
       '--test',
@@ -362,6 +342,15 @@ def AddIosTestArgs(parser):
       'ignore, this can be useful for customizing or sharding test suites. The '
       'given path may be in the local filesystem or in Google Cloud Storage '
       'using a URL beginning with `gs://`.')
+  parser.add_argument(
+      '--xcode-version',
+      category=base.COMMONLY_USED_FLAGS,
+      help="""\
+      The version of Xcode that should be used to run an XCTest. Defaults to the
+      latest Xcode version supported in Firebase Test Lab. This Xcode version
+      must be supported by all iOS versions selected in the test matrix. The
+      list of Xcode versions supported by each version of iOS can be viewed by
+      running `$ {parent_command} versions list`.""")
   parser.add_argument(
       '--device',
       category=base.COMMONLY_USED_FLAGS,
@@ -393,6 +382,31 @@ def AddIosTestArgs(parser):
       'together in the Firebase console in a time-ordered test history list.')
 
 
+def AddBetaArgs(parser):
+  """Register args which are only available in the beta run commands.
+
+  Args:
+    parser: An argparse parser used to add args that follow a command.
+  """
+  parser.add_argument(
+      '--client-details',
+      type=arg_parsers.ArgDict(),
+      metavar='KEY=VALUE',
+      help="""\
+      Comma-separated, KEY=VALUE map of additional details to attach to the test
+      matrix. Arbitrary KEY=VALUE pairs may be attached to a test matrix to
+      provide additional context about the tests being run. When consuming the
+      test results, such as in Cloud Functions or a CI system, these details can
+      add additional context such as a link to the corresponding pull request.
+
+      Example:
+
+      ```
+      --client-details=buildNumber=1234,pullRequest=https://example.com/link/to/pull-request
+      ```
+      """)
+
+
 def AddGaArgs(parser):
   """Register args which are only available in the GA run command.
 
@@ -402,8 +416,8 @@ def AddGaArgs(parser):
   del parser  # Unused by AddGaArgs
 
 
-def AddBetaArgs(parser):
-  """Register args which are only available in the beta run command.
+def AddAndroidBetaArgs(parser):
+  """Register args which are only available in the Android beta run command.
 
   Args:
     parser: An argparse parser used to add args that follow a command.
@@ -436,23 +450,101 @@ def AddBetaArgs(parser):
   parser.add_argument(
       '--other-files',
       type=arg_parsers.ArgDict(min_length=1),
-      action='append',
-      metavar='FILE=DEVICE_DIR',
+      metavar='DEVICE_PATH=FILE_PATH',
       help="""\
-      A list of file=device-directory pairs that indicate paths of files to push
-      to the device before starting tests, and the device directory to push them
-      to.\n
-      Source file paths may be in the local filesystem or in Google Cloud
-      Storage (gs://...). Device directories must be absolute, whitelisted paths
-      (${EXTERNAL_STORAGE}, or ${ANDROID_DATA}/local/tmp).\n
+      A list of device-path=file-path pairs that indicate the device paths to
+      push files to the device before starting tests, and the paths of files to
+      push.\n
+      Device paths must be under absolute, whitelisted paths
+      (${EXTERNAL_STORAGE}, or ${ANDROID_DATA}/local/tmp). Source file paths may
+      be in the local filesystem or in Google Cloud Storage (gs://...).\n
       Examples:\n
       ```
-      --other-files local/file1=/sdcard/dir1/
-      --other-files gs://bucket/file2=/sdcard/dir2
+      --other-files /sdcard/dir1/file1.txt=local/file.txt,/sdcard/dir2/file2.jpg=gs://bucket/file.jpg
       ```\n
       This flag only copies files to the device. To install files, like OBB or
       APK files, see --obb-files and --additional-apks.
       """)
+  # Mutually exclusive sharding options group.
+  sharding_options = parser.add_group(mutex=True, help='Sharding options.')
+  sharding_options.add_argument(
+      '--num-uniform-shards',
+      metavar='int',
+      type=arg_validate.POSITIVE_INT_PARSER,
+      help="""\
+      Specifies the number of shards into which you want to evenly distribute
+      test cases. The shards are run in parallel on separate devices. For
+      example, if your test execution contains 20 test cases and you specify
+      four shards, each shard executes five test cases.
+
+      The number of shards should be less than the total number of test
+      cases. The number of shards specified must be >= 1 and <= 50.
+      """)
+  sharding_options.add_argument(
+      '--test-targets-for-shard',
+      metavar='TEST_TARGETS_FOR_SHARD',
+      action='append',
+      help="""\
+      Specifies a group of packages, classes, and/or test cases to run in
+      each shard (a group of test cases). The shards are run in parallel on
+      separate devices. You can repeat this flag up to 50 times to specify
+      multiple shards.
+
+      Note: If you include the flags --environment-variable or --test-targets
+      when running --test-targets-for-shard, the flags are applied to all the
+      shards you create.
+
+      Examples:
+
+      You can also specify multiple packages, classes, or test cases in the
+      same shard by separating each item with a comma. For example:
+
+      ```
+      --test-targets-for-shard
+      "package com.package1.for.shard1,com.package2.for.shard1"
+      ```
+
+      ```
+      --test-targets-for-shard
+      "class com.foo.ClassForShard2#testMethod1,com.foo.ClassForShard2#testMethod2"
+      ```
+
+      To specify both package and class in the same shard, separate
+      package and class with semi-colons:
+
+      ```
+      --test-targets-for-shard
+      "class com.foo.ClassForShard3;package com.package.for.shard3"
+      ```
+      """)
+
+
+def AddIosBetaArgs(parser):
+  """Register args which are only available in the iOS beta run command.
+
+  Args:
+    parser: An argparse parser used to add args that follow a command.
+  """
+
+  # The following args are specific to iOS game-loop tests.
+
+  parser.add_argument(
+      '--scenario-numbers',
+      metavar='int',
+      type=arg_parsers.ArgList(element_type=int, min_length=1, max_length=1024),
+      help='A list of game-loop scenario numbers which will be run as part of '
+           'the test (default: scenario 1). A maximum of 1024 scenarios may be '
+           'specified in one test matrix, but the maximum number may also be '
+           'limited by the overall test *--timeout* setting. This flag is only '
+           'valid when *--type=game-loop* is also set.'
+  )
+  parser.add_argument(
+      '--app',
+      help='The path to the application archive (.ipa file) for game-loop '
+           'testing. The path may be in the local filesystem or in Google '
+           'Cloud Storage using gs:// notation. This flag is only valid when '
+           '*--type=game-loop* is also set.'
+  )
 
 
 def AddMatrixArgs(parser):
@@ -578,7 +670,7 @@ def ApplyLowerPriorityArgs(args, lower_pri_args, issue_cli_warning=False):
   for arg in lower_pri_args:
     if getattr(args, arg, None) is None:
       log.debug('Applying default {0}: {1}'
-                .format(arg, str(lower_pri_args[arg])))
+                .format(arg, six.text_type(lower_pri_args[arg])))
       setattr(args, arg, lower_pri_args[arg])
     elif issue_cli_warning and getattr(args, arg) != lower_pri_args[arg]:
       ext_name = exceptions.ExternalArgNameFrom(arg)
@@ -593,4 +685,4 @@ def _FormatArgValue(value):
   if isinstance(value, list):
     return ' '.join(value)
   else:
-    return str(value)
+    return six.text_type(value)
