@@ -24,6 +24,7 @@ from apitools.base.py import extra_types
 
 from googlecloudsdk.api_lib.resource_manager import folders
 from googlecloudsdk.api_lib.util import apis as core_apis
+from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.command_lib.resource_manager import completers
 from googlecloudsdk.command_lib.util.args import common_args
 from googlecloudsdk.core import exceptions
@@ -121,14 +122,19 @@ def AddBucketLocationArg(parser, required, help_string):
     required: whether the arguments is required.
     help_string: the help string.
   """
+  # We validate that the location is non-empty since otherwise the
+  # error message from the API can be confusing. We leave the rest of the
+  # validation to the API.
   parser.add_argument(
-      '--location', required=required, metavar='LOCATION', help=help_string)
+      '--location', required=required, metavar='LOCATION',
+      type=arg_parsers.RegexpValidator(r'.+', 'must be non-empty'),
+      help=help_string)
 
 
-def GetProjectResource():
+def GetProjectResource(project):
   """Returns the resource for the current project."""
   return resources.REGISTRY.Parse(
-      properties.VALUES.core.project.Get(required=True),
+      project or properties.VALUES.core.project.Get(required=True),
       collection='cloudresourcemanager.projects')
 
 
@@ -187,7 +193,7 @@ def GetParentResourceFromArgs(args):
   elif args.billing_account:
     return GetBillingAccountResource(args.billing_account)
   else:
-    return GetProjectResource()
+    return GetProjectResource(args.project)
 
 
 def GetParentFromArgs(args):
