@@ -28,6 +28,10 @@ SERVICE_LABEL = 'serving.knative.dev/service'
 # Used to force a new revision, and also to tie a particular request for changes
 # to a particular created revision.
 NONCE_LABEL = 'client.knative.dev/nonce'
+# Annotation for the user-specified image.
+USER_IMAGE_ANNOTATION = k8s_object.CLIENT_GROUP + '/user-image'
+CLOUDSQL_ANNOTATION = k8s_object.RUN_GROUP + '/cloudsql-instances'
+VPC_ACCESS_ANNOTATION = 'run.googleapis.com/vpc-access-connector'
 
 
 class Revision(k8s_object.KubernetesObject):
@@ -41,6 +45,23 @@ class Revision(k8s_object.KubernetesObject):
       READY_CONDITION,
   }
 
+  FIELD_BLACKLIST = ['container']
+
+  @classmethod
+  def New(cls, client, namespace):
+    """Produces a new Revision object.
+
+    Args:
+      client: The Cloud Run API client.
+      namespace: str, The serving namespace.
+
+    Returns:
+      A new Revision object to be deployed.
+    """
+    ret = super(Revision, cls).New(client, namespace)
+    ret.spec.containers = [client.MESSAGES_MODULE.Container()]
+    return ret
+
   @property
   def env_vars(self):
     """Returns a mutable, dict-like object to manage env vars.
@@ -82,6 +103,39 @@ class Revision(k8s_object.KubernetesObject):
   def image(self, value):
     self.container.image = value
 
+  def UserImage(self, service_user_image=None):
+    """Human-readable "what's deployed".
+
+    Sometimes references a client.knative.dev/user-image annotation on the
+    revision or service to determine what the user intended to deploy. In that
+    case, we can display that, and show the user the hash prefix as a note that
+    it's at that specific hash.
+
+    Arguments:
+      service_user_image: Optional[str], the contents of the user image annot
+        on the service.
+    Returns:
+      a string representing the user deployment intent.
+    """
+    if not self.image:
+      return None
+    if '@' not in self.image:
+      return self.image
+    user_image = (
+        self.annotations.get(USER_IMAGE_ANNOTATION) or service_user_image)
+    if not user_image:
+      return self.image
+    # The image should  be in the format base@sha256:hashhashhash
+    base, h = self.image.split('@')
+    if ':' in h:
+      _, h = h.split(':')
+    if not user_image.startswith(base):
+      # The user-image is out of date.
+      return self.image
+    if len(h) > 8:
+      h = h[:8] + '...'
+    return user_image + ' at ' + h
+
   @property
   def active(self):
     cond = self.conditions
@@ -100,8 +154,13 @@ class Revision(k8s_object.KubernetesObject):
           self._messages.ResourceRequirements)
     # These fields are in the schema due to an error in interperetation of the
     # Knative spec. We're removing them, so never send any contents for them.
-    self.container.resources.limitsInMap = None
-    self.container.resources.requestsInMap = None
+    try:
+      self.container.resources.limitsInMap = None
+      self.container.resources.requestsInMap = None
+    except AttributeError:
+      # The fields only exist in the v1alpha1 spec, if we're working with a
+      # different version, this is safe to ignore
+      pass
 
   def _EnsureMeta(self):
     if self.metadata is None:
@@ -111,12 +170,13 @@ class Revision(k8s_object.KubernetesObject):
   @property
   def container(self):
     """The container in the revisionTemplate."""
-    if self.spec.container and self.spec.containers:
-      raise ValueError(
-          'Revision can have only one of `container` or `containers` set')
-    elif self.spec.container:
-      return self.spec.container
-    elif self.spec.containers:
+    if hasattr(self.spec, 'container'):
+      if self.spec.container and self.spec.containers:
+        raise ValueError(
+            'Revision can have only one of `container` or `containers` set')
+      elif self.spec.container:
+        return self.spec.container
+    if self.spec.containers:
       if self.spec.containers[0] is None or len(self.spec.containers) != 1:
         raise ValueError('List of containers must contain exactly one element')
       return self.spec.containers[0]
@@ -134,18 +194,6 @@ class Revision(k8s_object.KubernetesObject):
         value_field='value',
     )
 
-  @property
-  def deprecated_string_concurrency(self):
-    """The string-enum concurrency model in the revisionTemplate.
-
-    This is deprecated in favor of the numeric field containerConcurrency
-    """
-    return self.spec.concurrencyModel
-
-  @deprecated_string_concurrency.setter
-  def deprecated_string_concurrency(self, value):
-    self.spec.concurrencyModel = value
-
   @property
   def concurrency(self):
     """The concurrency number in the revisionTemplate.
@@ -158,6 +206,13 @@ class Revision(k8s_object.KubernetesObject):
 
   @concurrency.setter
   def concurrency(self, value):
+    # Clear the old, deperecated string field
+    try:
+      self.spec.concurrencyModel = None
+    except AttributeError:
+      # This field only exists in the v1alpha1 spec, if we're working with a
+      # different version, this is safe to ignore
+      pass
     self.spec.containerConcurrency = value
 
   @property
@@ -214,6 +269,14 @@ class Revision(k8s_object.KubernetesObject):
                                              self.container.volumeMounts,
                                              self._messages.VolumeMount)
 
+  def MountedVolumeJoin(self, subgroup=None):
+    vols = self.volumes
+    mounts = self.volume_mounts
+    if subgroup:
+      vols = getattr(vols, subgroup)
+      mounts = getattr(mounts, subgroup)
+    return {path: vols.get(vol) for path, vol in mounts.items()}
+
 
 class EnvVarsAsDictionaryWrapper(k8s_object.ListAsReadOnlyDictionaryWrapper):
   """Wraps a list of env vars in a dict-like object.
@@ -235,15 +298,22 @@ class EnvVarsAsDictionaryWrapper(k8s_object.ListAsReadOnlyDictionaryWrapper):
 
   @property
   def literals(self):
-    """Mutable dict-like object for env vars with a string literal."""
+    """Mutable dict-like object for env vars with a string literal.
+
+    Note that if neither value nor valueFrom is specified, the list entry will
+    be treated as a literal empty string.
+
+    Returns:
+      A mutable, dict-like object for managing string literal env vars.
+    """
     return k8s_object.ListAsDictionaryWrapper(
         self._env_vars,
         self._env_var_class,
-        filter_func=lambda env_var: env_var.value is not None)
+        filter_func=lambda env_var: env_var.valueFrom is None)
 
   @property
   def secrets(self):
-    """Mutable dict-like object for volumes with a secret source type."""
+    """Mutable dict-like object for vars with a secret source type."""
     def _FilterSecretEnvVars(env_var):
       return (env_var.valueFrom is not None and
               env_var.valueFrom.secretKeyRef is not None)
@@ -256,7 +326,7 @@ class EnvVarsAsDictionaryWrapper(k8s_object.ListAsReadOnlyDictionaryWrapper):
 
   @property
   def config_maps(self):
-    """Mutable dict-like object for volumes with a config map source type."""
+    """Mutable dict-like object for vars with a config map source type."""
     def _FilterConfigMapEnvVars(env_var):
       return (env_var.valueFrom is not None and
               env_var.valueFrom.configMapKeyRef is not None)
