@@ -19,6 +19,9 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import json
+
+from apitools.base.py import encoding
 from googlecloudsdk.api_lib.services import exceptions
 from googlecloudsdk.api_lib.util import apis_internal
 from googlecloudsdk.core import log
@@ -129,3 +132,34 @@ def PrintOperation(op):
         'The operation "{0}" resulted in a failure "{1}".\nDetails: "{2}".'.
         format(op.name, op.error.message, op.error.details))
   log.status.Print('Operation "{0}" finished successfully.'.format(op.name))
+
+
+def PrintOperationWithResponse(op):
+  """Print the operation with response.
+
+  Args:
+    op: The long running operation.
+
+  Raises:
+    OperationErrorException: if the operation fails.
+
+  Returns:
+    Nothing.
+  """
+  if not op.done:
+    log.status.Print('Operation "{0}" is still in progress.'.format(op.name))
+    return
+  if op.error:
+    raise exceptions.OperationErrorException(
+        'The operation "{0}" resulted in a failure "{1}".\nDetails: "{2}".'.
+        format(op.name, op.error.message, op.error.details))
+  if op.response:
+    log.status.Print('Operation [{0}] complete. Result: {1}'.format(
+        op.name,
+        json.dumps(
+            encoding.MessageToDict(op.response),
+            sort_keys=True,
+            indent=4,
+            separators=(',', ':'))))
+  else:
+    log.status.Print('Operation "{0}" finished successfully.'.format(op.name))
