@@ -29,8 +29,22 @@ from googlecloudsdk.core.credentials import http as http_creds
 _PROJECT_RESOURCE = 'projects/%s'
 _PROJECT_SERVICE_RESOURCE = 'projects/%s/services/%s'
 _V1_VERSION = 'v1'
+_V1BETA1_VERSION = 'v1beta1'
 _V1ALPHA_VERSION = 'v1alpha'
 
+# Map of services which should be protected from being disabled by
+# prompting the user for  confirmation
+_PROTECTED_SERVICES = {
+    'anthos.googleapis.com': ('Warning: Disabling this service will '
+                              'also automatically disable any running '
+                              'Anthos clusters.')
+}
+
+
+def GetProtectedServiceWarning(service_name):
+  """Return the warning message associated with a protected service."""
+  return _PROTECTED_SERVICES.get(service_name)
+
 
 def EnableApiCall(project, service):
   """Make API call to enable a specific service.
@@ -238,13 +252,13 @@ def GenerateServiceIdentity(project, service):
   Returns:
     The email and uid of the generated service identity.
   """
-  client = _GetClientInstance(version=_V1ALPHA_VERSION)
+  client = _GetClientInstance(version=_V1BETA1_VERSION)
   messages = client.MESSAGES_MODULE
 
-  request = messages.ServiceusageServicesGenerateIdentityRequest(
+  request = messages.ServiceusageServicesGenerateServiceIdentityRequest(
       parent=_PROJECT_SERVICE_RESOURCE % (project, service))
   try:
-    op = client.services.GenerateIdentity(request)
+    op = client.services.GenerateServiceIdentity(request)
     return _GetOperationResponseProperty(
         op, 'email'), _GetOperationResponseProperty(op, 'unique_id')
   except (apitools_exceptions.HttpForbiddenError,
