@@ -1,4 +1,5 @@
-# Copyright 2016 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,10 +14,13 @@
 # limitations under the License.
 """Spanner database API helper."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import list_pager
 from googlecloudsdk.api_lib.util import apis
-from googlecloudsdk.core import properties
-from googlecloudsdk.core import resources
+from googlecloudsdk.command_lib.iam import iam_util
 
 
 # The list of pre-defined IAM roles in Spanner.
@@ -27,14 +31,10 @@ KNOWN_ROLES = [
 ]
 
 
-def Create(instance, database, ddl):
+def Create(instance_ref, database, ddl):
   """Create a new database."""
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
-  instance_ref = resources.REGISTRY.Parse(
-      instance,
-      params={'projectsId': properties.VALUES.core.project.GetOrFail},
-      collection='spanner.projects.instances')
   req = msgs.SpannerProjectsInstancesDatabasesCreateRequest(
       parent=instance_ref.RelativeName(),
       createDatabaseRequest=msgs.CreateDatabaseRequest(
@@ -47,25 +47,19 @@ def SetPolicy(database_ref, policy):
   """Saves the given policy on the database, overwriting whatever exists."""
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
+  policy.version = iam_util.MAX_LIBRARY_IAM_SUPPORTED_VERSION
   req = msgs.SpannerProjectsInstancesDatabasesSetIamPolicyRequest(
       resource=database_ref.RelativeName(),
       setIamPolicyRequest=msgs.SetIamPolicyRequest(policy=policy))
   return client.projects_instances_databases.SetIamPolicy(req)
 
 
-def Delete(instance, database):
+def Delete(database_ref):
   """Delete a database."""
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
-  ref = resources.REGISTRY.Parse(
-      database,
-      params={
-          'projectsId': properties.VALUES.core.project.GetOrFail,
-          'instancesId': instance
-      },
-      collection='spanner.projects.instances.databases')
   req = msgs.SpannerProjectsInstancesDatabasesDropDatabaseRequest(
-      database=ref.RelativeName())
+      database=database_ref.RelativeName())
   return client.projects_instances_databases.DropDatabase(req)
 
 
@@ -74,52 +68,38 @@ def GetIamPolicy(database_ref):
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
   req = msgs.SpannerProjectsInstancesDatabasesGetIamPolicyRequest(
+      getIamPolicyRequest=msgs.GetIamPolicyRequest(
+          options=msgs.GetPolicyOptions(
+              requestedPolicyVersion=
+              iam_util.MAX_LIBRARY_IAM_SUPPORTED_VERSION)),
       resource=database_ref.RelativeName())
   return client.projects_instances_databases.GetIamPolicy(req)
 
 
-def Get(instance, database):
+def Get(database_ref):
   """Get a database by name."""
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
-  ref = resources.REGISTRY.Parse(
-      database,
-      params={
-          'projectsId': properties.VALUES.core.project.GetOrFail,
-          'instancesId': instance
-      },
-      collection='spanner.projects.instances.databases')
   req = msgs.SpannerProjectsInstancesDatabasesGetRequest(
-      name=ref.RelativeName())
+      name=database_ref.RelativeName())
   return client.projects_instances_databases.Get(req)
 
 
-def GetDdl(instance, database):
+def GetDdl(database_ref):
   """Get a database's DDL description."""
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
-  ref = resources.REGISTRY.Parse(
-      database,
-      params={
-          'instancesId': instance,
-          'projectsId': properties.VALUES.core.project.GetOrFail,
-      },
-      collection='spanner.projects.instances.databases')
   req = msgs.SpannerProjectsInstancesDatabasesGetDdlRequest(
-      database=ref.RelativeName())
+      database=database_ref.RelativeName())
   return client.projects_instances_databases.GetDdl(req).statements
 
 
-def List(instance):
+def List(instance_ref):
   """List databases in the instance."""
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
-  ref = resources.REGISTRY.Parse(
-      instance,
-      params={'projectsId': properties.VALUES.core.project.GetOrFail},
-      collection='spanner.projects.instances')
   req = msgs.SpannerProjectsInstancesDatabasesListRequest(
-      parent=ref.RelativeName())
+      parent=instance_ref.RelativeName())
   return list_pager.YieldFromList(
       client.projects_instances_databases,
       req,
@@ -127,19 +107,22 @@ def List(instance):
       batch_size_attribute='pageSize')
 
 
-def UpdateDdl(instance, database, ddl):
+def UpdateDdl(database_ref, ddl):
   """Update a database via DDL commands."""
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
-  instance_ref = resources.REGISTRY.Parse(
-      database,
-      params={
-          'projectsId': properties.VALUES.core.project.GetOrFail,
-          'instancesId': instance
-      },
-      collection='spanner.projects.instances.databases')
   req = msgs.SpannerProjectsInstancesDatabasesUpdateDdlRequest(
-      database=instance_ref.RelativeName(),
-      updateDatabaseDdlRequest=msgs.UpdateDatabaseDdlRequest(
-          statements=ddl))
+      database=database_ref.RelativeName(),
+      updateDatabaseDdlRequest=msgs.UpdateDatabaseDdlRequest(statements=ddl))
   return client.projects_instances_databases.UpdateDdl(req)
+
+
+def Restore(database_ref, backup_ref):
+  client = apis.GetClientInstance('spanner', 'v1')
+  msgs = apis.GetMessagesModule('spanner', 'v1')
+  req = msgs.SpannerProjectsInstancesDatabasesRestoreRequest(
+      parent=database_ref.Parent().RelativeName(),
+      restoreDatabaseRequest=msgs.RestoreDatabaseRequest(
+          backup=backup_ref.RelativeName(),
+          databaseId=database_ref.Name()))
+  return client.projects_instances_databases.Restore(req)
