@@ -89,7 +89,13 @@ def _GetApiDef(api_name, api_version):
     raise apis_util.UnknownAPIError(api_name)
 
   version_overrides = properties.VALUES.api_client_overrides.AllValues()
-  version_override = version_overrides.get(api_name_alias, None)
+
+  # First attempt to get api specific override, then full surface override.
+  version_override = version_overrides.get('{}/{}'.format(
+      api_name, api_version))
+  if not version_override:
+    version_override = version_overrides.get(api_name_alias, None)
+
   api_version = version_override or api_version
 
   api_versions = apis_map.MAP[api_name]
@@ -129,9 +135,12 @@ def _GetClientClassFromDef(api_def):
   return getattr(module_obj, client_class_name)
 
 
-def _GetClientInstance(api_name, api_version, no_http=False,
+def _GetClientInstance(api_name,
+                       api_version,
+                       no_http=False,
                        http_client=None,
-                       check_response_func=None):
+                       check_response_func=None,
+                       use_google_auth=False):
   """Returns an instance of the API client specified in the args.
 
   Args:
@@ -141,6 +150,9 @@ def _GetClientInstance(api_name, api_version, no_http=False,
     http_client: bring your own http client to use.
       Incompatible with no_http=True.
     check_response_func: error handling callback to give to apitools.
+    use_google_auth: bool, True if the calling command indicates to use
+      google-auth library for authentication. If False, authentication will
+      fallback to using the oauth2client library.
 
   Returns:
     base_api.BaseApiClient, An instance of the specified API client.
@@ -154,7 +166,8 @@ def _GetClientInstance(api_name, api_version, no_http=False,
     # Import http only when needed, as it depends on credential infrastructure
     # which is not needed in all cases.
     from googlecloudsdk.core.credentials import http as http_creds
-    http_client = http_creds.Http(response_encoding=http_creds.ENCODING)
+    http_client = http_creds.Http(
+        response_encoding=http_creds.ENCODING, use_google_auth=use_google_auth)
 
   client_class = _GetClientClass(api_name, api_version)
   client_instance = client_class(
@@ -171,6 +184,17 @@ def _GetClientInstance(api_name, api_version, no_http=False,
   return client_instance
 
 
+def _GetMtlsEndpointIfEnabled(api_name, api_version, client_class=None):
+  """Returns mtls endpoint if mtls is enabled for the API."""
+  client_class = client_class or _GetClientClass(api_name, api_version)
+  api_def = _GetApiDef(api_name, api_version)
+  if api_def.enable_mtls:
+    # Services with mTLS enabled should have the mTLS endpoint either in
+    # mtls_endpoint_override in the API map or in the generated client.
+    # We have tests to guarantee that.
+    return api_def.mtls_endpoint_override or client_class.MTLS_BASE_URL
+
+
 def _GetEffectiveApiEndpoint(api_name, api_version, client_class=None):
   """Returns effective endpoint for given api."""
   endpoint_overrides = properties.VALUES.api_endpoint_overrides.AllValues()
@@ -178,6 +202,11 @@ def _GetEffectiveApiEndpoint(api_name, api_version, client_class=None):
   if endpoint_override:
     return endpoint_override
   client_class = client_class or _GetClientClass(api_name, api_version)
+  if properties.VALUES.context_aware.use_client_certificate.GetBool():
+    mtls_endpoint = _GetMtlsEndpointIfEnabled(api_name, api_version,
+                                              client_class)
+    if mtls_endpoint:
+      return mtls_endpoint
   return client_class.BASE_URL
 
 
