@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2013 Google Inc. All Rights Reserved.
+# Copyright 2013 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,7 +22,7 @@ from __future__ import unicode_literals
 
 import abc
 import collections
-from functools import wraps
+from functools import wraps  # pylint:disable=g-importing-member
 import itertools
 import re
 import sys
@@ -36,6 +36,36 @@ from googlecloudsdk.core.resource import resource_printer
 
 import six
 
+# Category constants
+AI_AND_MACHINE_LEARNING_CATEGORY = 'AI and Machine Learning'
+API_PLATFORM_AND_ECOSYSTEMS_CATEGORY = 'API Platform and Ecosystems'
+ANTHOS_CLI_CATEGORY = 'Anthos CLI'
+COMPUTE_CATEGORY = 'Compute'
+DATA_ANALYTICS_CATEGORY = 'Data Analytics'
+DATABASES_CATEGORY = 'Databases'
+IDENTITY_AND_SECURITY_CATEGORY = 'Identity and Security'
+INTERNET_OF_THINGS_CATEGORY = 'Internet of Things'
+MANAGEMENT_TOOLS_CATEGORY = 'Management Tools'
+MOBILE_CATEGORY = 'Mobile'
+NETWORKING_CATEGORY = 'Networking'
+SDK_TOOLS_CATEGORY = 'SDK Tools'
+DISKS_CATEGORY = 'Disks'
+INFO_CATEGORY = 'Info'
+INSTANCES_CATEGORY = 'Instances'
+LOAD_BALANCING_CATEGORY = 'Load Balancing'
+TOOLS_CATEGORY = 'Tools'
+STORAGE_CATEGORY = 'Storage'
+BILLING_CATEGORY = 'Billing'
+SECURITY_CATEGORY = 'Security'
+IDENTITY_CATEGORY = 'Identity'
+BIG_DATA_CATEGORY = 'Big Data'
+CI_CD_CATEGORY = 'CI/CD'
+MONITORING_CATEGORY = 'Monitoring'
+SOLUTIONS_CATEGORY = 'Solutions'
+SERVERLESS_CATEGORY = 'Serverless'
+UNCATEGORIZED_CATEGORY = 'Other'
+IDENTITY_CATEGORY = 'Identity'
+
 
 # Common markdown.
 MARKDOWN_BOLD = '*'
@@ -85,9 +115,8 @@ class ReleaseTrack(object):
       '{0}(ALPHA){0} '.format(MARKDOWN_BOLD),
       'This command is currently in ALPHA and may change without notice. '
       'If this command fails with API permission errors despite specifying '
-      'the right project, you will have to apply for early access and have your'
-      ' projects registered on the API whitelist to use it. To do so, contact '
-      'Support at https://cloud.google.com/support/.')
+      'the right project, you may be trying to access an API with '
+      'an invitation-only early access whitelist.')
   _ALL = [GA, BETA, ALPHA]
 
   @staticmethod
@@ -287,6 +316,18 @@ class Argument(Action):
 
 COMMONLY_USED_FLAGS = 'COMMONLY USED'
 
+FLAGS_FILE_FLAG = Argument(
+    '--flags-file',
+    metavar='YAML_FILE',
+    default=None,
+    category=COMMONLY_USED_FLAGS,
+    help="""\
+        A YAML or JSON file that specifies a *--flag*:*value* dictionary.
+        Useful for specifying complex flag values with special characters
+        that work with any command interpreter. Additionally, each
+        *--flags-file* arg is replaced by its constituent flags. See
+        $ gcloud topic flags-file for more information.""")
+
 FLATTEN_FLAG = Argument(
     '--flatten',
     metavar='KEY',
@@ -318,13 +359,15 @@ LIST_COMMAND_FLAGS = 'LIST COMMAND'
 ASYNC_FLAG = Argument(
     '--async',
     action='store_true',
+    dest='async_',
     help="""\
-    Display information about the operation in progress, without waiting for
-    the operation to complete.""")
+    Return immediately, without waiting for the operation in progress to
+    complete.""")
 
 FILTER_FLAG = Argument(
     '--filter',
     metavar='EXPRESSION',
+    require_coverage_in_tests=False,
     category=LIST_COMMAND_FLAGS,
     help="""\
     Apply a Boolean filter _EXPRESSION_ to each resource item to be listed.
@@ -336,6 +379,7 @@ FILTER_FLAG = Argument(
 LIMIT_FLAG = Argument(
     '--limit',
     type=arg_parsers.BoundedInt(1, sys.maxsize, unlimited=True),
+    require_coverage_in_tests=False,
     category=LIST_COMMAND_FLAGS,
     help="""\
     Maximum number of resources to list. The default is *unlimited*.
@@ -346,6 +390,7 @@ LIMIT_FLAG = Argument(
 PAGE_SIZE_FLAG = Argument(
     '--page-size',
     type=arg_parsers.BoundedInt(1, sys.maxsize, unlimited=True),
+    require_coverage_in_tests=False,
     category=LIST_COMMAND_FLAGS,
     help="""\
     Some services group resource list output into pages. This flag specifies
@@ -359,6 +404,7 @@ SORT_BY_FLAG = Argument(
     '--sort-by',
     metavar='FIELD',
     type=arg_parsers.ArgList(),
+    require_coverage_in_tests=False,
     category=LIST_COMMAND_FLAGS,
     help="""\
     Comma-separated list of resource field key names to sort by. The
@@ -370,12 +416,26 @@ SORT_BY_FLAG = Argument(
 URI_FLAG = Argument(
     '--uri',
     action='store_true',
+    require_coverage_in_tests=False,
     category=LIST_COMMAND_FLAGS,
     help='Print a list of resource URIs instead of the default output.')
 
+# Binary Command Flags
+BINARY_BACKED_COMMAND_FLAGS = 'BINARY BACKED COMMAND'
+
+SHOW_EXEC_ERROR_FLAG = Argument(
+    '--show-exec-error',
+    hidden=True,
+    action='store_true',
+    required=False,
+    category=BINARY_BACKED_COMMAND_FLAGS,
+    help='If true and command fails, print the underlying command '
+         'that was executed and its exit status.')
+
 
 class _Common(six.with_metaclass(abc.ABCMeta, object)):
   """Base class for Command and Group."""
+  category = None
   _cli_generator = None
   _is_hidden = False
   _is_unicode_supported = False
@@ -438,7 +498,7 @@ class _Common(six.with_metaclass(abc.ABCMeta, object)):
       The attribute value from obj for tracks.
     """
     for track in ReleaseTrack._ALL:  # pylint: disable=protected-access
-      if track not in cls._valid_release_tracks:
+      if track not in cls._valid_release_tracks:  # pylint: disable=unsupported-membership-test
         continue
       names = []
       names.append(attribute + '_' + track.id)
@@ -478,9 +538,6 @@ class Group(_Common):
 
   IS_COMMAND_GROUP = True
 
-  _allow_py3 = True
-  _command_suggestions = {}
-
   def __init__(self):
     super(Group, self).__init__(is_group=True)
 
@@ -495,10 +552,6 @@ class Group(_Common):
     """
     pass
 
-  @classmethod
-  def CommandSuggestions(cls):
-    return cls._command_suggestions
-
 
 class Command(six.with_metaclass(abc.ABCMeta, _Common)):
   """Command is a base class for commands to implement.
@@ -602,6 +655,36 @@ class DescribeCommand(six.with_metaclass(abc.ABCMeta, Command)):
   """A command that prints one resource in the 'default' format."""
 
 
+class ImportCommand(six.with_metaclass(abc.ABCMeta, Command)):
+  """A command that imports one resource from yaml format."""
+
+
+class ExportCommand(six.with_metaclass(abc.ABCMeta, Command)):
+  """A command that outputs one resource to file in yaml format."""
+
+
+class BinaryBackedCommand(six.with_metaclass(abc.ABCMeta, Command)):
+  """A command that wraps a BinaryBackedOperation."""
+
+  @staticmethod
+  def _Flags(parser):
+    SHOW_EXEC_ERROR_FLAG.AddToParser(parser)
+
+  @staticmethod
+  def _DefaultOperationResponseHandler(response):
+    """Process results of BinaryOperation Execution."""
+    if response.stdout:
+      log.status.Print(response.stdout)
+
+    if response.failed:
+      log.error(response.stderr)
+      return None
+
+    if response.stderr:
+      log.status.Print(response.stderr)
+    return response.stdout
+
+
 class CacheCommand(six.with_metaclass(abc.ABCMeta, Command)):
   """A command that affects the resource URI cache."""
 
@@ -673,46 +756,6 @@ def Hidden(cmd_class):
   return cmd_class
 
 
-def DisallowPython3(group_class):
-  """Decorator to indicate a surface doesn't support being run undder Python 3.
-
-  Calliope will error out if sub-elements of this surface are attempted to be
-  loaded. Should only be applied to groups.
-
-  Args:
-    group_class: base.Group, A calliope group.
-
-  Returns:
-    A modified version of the provided class.
-  """
-  # pylint: disable=protected-access
-  group_class._allow_py3 = False
-  return group_class
-
-
-def CommandSuggestion(command, suggestion):
-  """Decorator for adding a suggestion when a command is mistyped.
-
-  This applies to base.Group classes. When a user tries to run the given
-  `command` that does not exist, `suggestion` will but suggested as a
-  "did you mean".
-
-  Args:
-    command: str, The name of the command (just the command itself not including
-      the group).
-    suggestion: str, The full command name to suggest (excluding the gcloud
-      prefix).
-
-  Returns:
-    The inner decorator.
-  """
-  def Inner(cmd_class):
-    # pylint: disable=protected-access
-    cmd_class._command_suggestions[command] = suggestion
-    return cmd_class
-  return Inner
-
-
 def UnicodeIsSupported(cmd_class):
   """Decorator for calliope commands and groups that support unicode.
 
