@@ -19,6 +19,7 @@ from __future__ import division
 from __future__ import unicode_literals
 
 import base64
+import datetime
 
 from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.api_lib.util import waiter
@@ -29,6 +30,8 @@ from googlecloudsdk.core.credentials import store
 
 DEFAULT_ENVIRONMENT_NAME = 'users/me/environments/default'
 
+MIN_CREDS_EXPIRY = datetime.timedelta(minutes=30)
+
 
 class UnsupportedPlatform(exceptions.Error):
   """Raised when attempting to run a command on an unsupported platform."""
@@ -56,6 +59,14 @@ def ParseCommonArgs(parser):
         *~/.ssh/google_compute_engine*.
       """,
       action='store_true')
+  parser.add_argument(
+      '--boosted',
+      help="""\
+      If provided, starts Cloud Shell in boost mode. If there is alread a
+      running Cloud Shell session that is not boosted, this will restart the
+      session.
+      """,
+      action='store_true')
 
 
 def PrepareEnvironment(args):
@@ -77,11 +88,20 @@ def PrepareEnvironment(args):
       messages.CloudshellUsersEnvironmentsGetRequest(
           name=DEFAULT_ENVIRONMENT_NAME))
 
+  if args.boosted and environment.size != messages.Environment.SizeValueValuesEnum.BOOSTED:
+    boosted_environment = messages.Environment(
+        size=messages.Environment.SizeValueValuesEnum.BOOSTED)
+    client.users_environments.Patch(
+        messages.CloudshellUsersEnvironmentsPatchRequest(
+            name=DEFAULT_ENVIRONMENT_NAME,
+            updateMask='size',
+            environment=boosted_environment))
+
   # If the environment doesn't have the public key, push it
   key_parts = keys.GetPublicKey().ToEntry().split(' ')
+  key_format = ValidateKeyType(key_parts[0].replace('-', '_').upper(), messages)
   key = messages.PublicKey(
-      format=messages.PublicKey.FormatValueValuesEnum(key_parts[0].replace(
-          '-', '_').upper()),
+      format=key_format,
       key=base64.b64decode(key_parts[1]),
   )
   has_key = False
@@ -102,6 +122,10 @@ def PrepareEnvironment(args):
     log.Print('Starting your Cloud Shell machine...')
 
     creds = store.LoadIfEnabled()
+    if creds is not None and creds.token_expiry - creds.token_expiry.utcnow(
+    ) < MIN_CREDS_EXPIRY:
+      store.Refresh(creds)
+
     access_token = None
     if creds is not None:
       access_token = creds.get_access_token().access_token
@@ -129,11 +153,26 @@ def PrepareEnvironment(args):
   )
 
 
+def ValidateKeyType(key_format, messages):
+  try:
+    return messages.PublicKey.FormatValueValuesEnum(key_format)
+  except TypeError:
+    raise ssh.InvalidKeyError('{} format of the key is not supported '
+                              'yet.'.format(key_format))
+
+
 def AuthorizeEnvironment():
+  """Pushes gcloud credentials to the user's environment."""
+
   client = apis.GetClientInstance('cloudshell', 'v1alpha1')
   messages = apis.GetMessagesModule('cloudshell', 'v1alpha1')
 
+  # Load creds and refresh them if they're about to expire
   creds = store.LoadIfEnabled()
+  if creds is not None and creds.token_expiry - creds.token_expiry.utcnow(
+  ) < MIN_CREDS_EXPIRY:
+    store.Refresh(creds)
+
   access_token = None
   if creds is not None:
     access_token = creds.get_access_token().access_token
