@@ -26,9 +26,12 @@ from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.composer import parsers
+from googlecloudsdk.command_lib.composer import util as command_util
 from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.core import properties
+
 import ipaddress
+import six
 
 
 AIRFLOW_VERSION_TYPE = arg_parsers.RegexpValidator(
@@ -64,6 +67,12 @@ _ENV_VAR_NAME_ERROR = (
     'Environment variable names may not start with a digit.')
 
 _INVALID_IPV4_CIDR_BLOCK_ERROR = ('Invalid format of IPV4 CIDR block.')
+_INVALID_GKE_MASTER_IPV4_CIDR_BLOCK_ERROR = (
+    'Not a valid IPV4 CIDR block value for the kubernetes master')
+_INVALID_WEB_SERVER_IPV4_CIDR_BLOCK_ERROR = (
+    'Not a valid IPV4 CIDR block value for the Airflow web server')
+_INVALID_CLOUD_SQL_IPV4_CIDR_BLOCK_ERROR = (
+    'Not a valid IPV4 CIDR block value for the Cloud SQL instance')
 
 AIRFLOW_CONFIGS_FLAG_GROUP_DESCRIPTION = (
     'Group of arguments for modifying the Airflow configuration.')
@@ -282,6 +291,56 @@ SERVICES_SECONDARY_RANGE_NAME_FLAG = base.Argument(
     Cannot be specified unless '--enable-ip-alias' is also specified.
     """)
 
+WEB_SERVER_ALLOW_IP = base.Argument(
+    '--web-server-allow-ip',
+    type=arg_parsers.ArgDict(spec={
+        'ip_range': str,
+        'description': str
+    }),
+    action='append',
+    help="""\
+    Specifies a list of IPv4 or IPv6 ranges that will be allowed to access the
+    Airflow web server. By default, all IPs are allowed to access the web
+    server.
+
+    *ip_range*::: IPv4 or IPv6 range of addresses allowed to access the Airflow
+    web server.
+
+    *description*::: An optional description of the IP range.
+    """)
+
+WEB_SERVER_DENY_ALL = base.Argument(
+    '--web-server-deny-all',
+    action='store_true',
+    help="""\
+    Denies all incoming traffic to the Airflow web server.
+    """)
+
+WEB_SERVER_ALLOW_ALL = base.Argument(
+    '--web-server-allow-all',
+    action='store_true',
+    help="""\
+    Allows all IP addresses to access the Airflow web server.
+    """)
+
+UPDATE_WEB_SERVER_ALLOW_IP = base.Argument(
+    '--update-web-server-allow-ip',
+    type=arg_parsers.ArgDict(spec={
+        'ip_range': str,
+        'description': str
+    }),
+    action='append',
+    help="""\
+    Specifies a list of IPv4 or IPv6 ranges that will be allowed to access the
+    Airflow web server. By default, all IPs are allowed to access the web
+    server.
+
+    *ip_range*::: IPv4 or IPv6 range of addresses allowed to access the Airflow
+    web server.
+
+    *description*::: An optional description of the IP range.
+    """)
+
 
 def _IsValidIpv4CidrBlock(ipv4_cidr_block):
   """Validates that IPV4 CIDR block arg has valid format.
@@ -357,13 +416,96 @@ ENABLE_PRIVATE_ENDPOINT_FLAG = base.Argument(
     specified.
     """)
 
+
+def _GetIpv4CidrMaskSize(ipv4_cidr_block):
+  """Returns the size of IPV4 CIDR block mask in bits.
+
+  Args:
+    ipv4_cidr_block: str, the IPV4 CIDR block string to check.
+
+  Returns:
+    int, the size of the block mask if ipv4_cidr_block is a valid CIDR block
+    string, otherwise None.
+  """
+  network = ipaddress.IPv4Network(ipv4_cidr_block)
+  if network is None:
+    return None
+
+  return 32 - (network.num_addresses.bit_length() - 1)
+
+
+def _IsValidMasterIpv4CidrBlockWithMaskSize(ipv4_cidr_block, min_mask_size,
+                                            max_mask_size):
+  """Validates that IPV4 CIDR block arg for the cluster master is a valid value.
+
+  Args:
+    ipv4_cidr_block: str, the IPV4 CIDR block string to validate.
+    min_mask_size: int, minimum allowed netmask size for CIDR block.
+    max_mask_size: int, maximum allowed netmask size for CIDR block.
+
+  Returns:
+    bool, True if and only if the IPV4 CIDR block is valid and has the mask
+    size between min_mask_size and max_mask_size.
+  """
+  is_valid = _IsValidIpv4CidrBlock(ipv4_cidr_block)
+  if not is_valid:
+    return False
+
+  mask_size = _GetIpv4CidrMaskSize(ipv4_cidr_block)
+  return min_mask_size <= mask_size and mask_size <= max_mask_size
+
+
+_IS_VALID_MASTER_IPV4_CIDR_BLOCK = (
+    lambda cidr: _IsValidMasterIpv4CidrBlockWithMaskSize(cidr, 23, 28))
+
+MASTER_IPV4_CIDR_BLOCK_FORMAT_VALIDATOR = arg_parsers.CustomFunctionValidator(
+    _IS_VALID_MASTER_IPV4_CIDR_BLOCK, _INVALID_GKE_MASTER_IPV4_CIDR_BLOCK_ERROR)
+
 MASTER_IPV4_CIDR_FLAG = base.Argument(
     '--master-ipv4-cidr',
     default=None,
-    type=IPV4_CIDR_BLOCK_FORMAT_VALIDATOR,
+    type=MASTER_IPV4_CIDR_BLOCK_FORMAT_VALIDATOR,
     help="""\
-    IPv4 CIDR range to use for the master network. This should have a netmask
-    of size /28.
+    IPv4 CIDR range to use for the cluste master network. This should have a
+    size of the netmask between 23 and 28.
+
+    Cannot be specified unless '--enable-private-environnment' is also
+    specified.
+    """)
+
+_IS_VALID_WEB_SERVER_IPV4_CIDR_BLOCK = (
+    lambda cidr: _IsValidMasterIpv4CidrBlockWithMaskSize(cidr, 24, 29))
+
+WEB_SERVER_IPV4_CIDR_BLOCK_FORMAT_VALIDATOR = arg_parsers.CustomFunctionValidator(
+    _IS_VALID_WEB_SERVER_IPV4_CIDR_BLOCK,
+    _INVALID_WEB_SERVER_IPV4_CIDR_BLOCK_ERROR)
+
+WEB_SERVER_IPV4_CIDR_FLAG = base.Argument(
+    '--web-server-ipv4-cidr',
+    default=None,
+    type=WEB_SERVER_IPV4_CIDR_BLOCK_FORMAT_VALIDATOR,
+    help="""\
+    IPv4 CIDR range to use for the Airflow web server network. This should have
+    a size of the netmask between 24 and 29.
+
+    Cannot be specified unless '--enable-private-environnment' is also
+    specified.
+    """)
+
+_IS_VALID_CLOUD_SQL_IPV4_CIDR_BLOCK = (
+    lambda cidr: _IsValidMasterIpv4CidrBlockWithMaskSize(cidr, 0, 24))
+
+CLOUD_SQL_IPV4_CIDR_BLOCK_FORMAT_VALIDATOR = arg_parsers.CustomFunctionValidator(
+    _IS_VALID_CLOUD_SQL_IPV4_CIDR_BLOCK,
+    _INVALID_CLOUD_SQL_IPV4_CIDR_BLOCK_ERROR)
+
+CLOUD_SQL_IPV4_CIDR_FLAG = base.Argument(
+    '--cloud-sql-ipv4-cidr',
+    default=None,
+    type=CLOUD_SQL_IPV4_CIDR_BLOCK_FORMAT_VALIDATOR,
+    help="""\
+    IPv4 CIDR range to use for the Cloud SQL network. This should have a size
+    of the netmask not greater than 24.
 
     Cannot be specified unless '--enable-private-environnment' is also
     specified.
@@ -557,25 +699,41 @@ def AddNodeCountUpdateFlagToGroup(update_type_group):
       help='The new number of nodes running the environment. Must be >= 3.')
 
 
-def AddPrivateIpAndIpAliasEnvironmentFlags(update_type_group):
-  """Adds flags related to private clusters and IP alias to parser.
+def AddIpAliasEnvironmentFlags(update_type_group):
+  """Adds flags related to IP aliasing to parser.
+
+  IP alias flags are related to similar flags found within GKE SDK:
+    /third_party/py/googlecloudsdk/command_lib/container/flags.py
+
+  Args:
+    update_type_group: argument group, the group to which flag should be added.
+  """
+  group = update_type_group.add_group(help='IP Alias (VPC-native)')
+  ENABLE_IP_ALIAS_FLAG.AddToParser(group)
+  CLUSTER_IPV4_CIDR_FLAG.AddToParser(group)
+  SERVICES_IPV4_CIDR_FLAG.AddToParser(group)
+  CLUSTER_SECONDARY_RANGE_NAME_FLAG.AddToParser(group)
+  SERVICES_SECONDARY_RANGE_NAME_FLAG.AddToParser(group)
+
+
+def AddPrivateIpEnvironmentFlags(update_type_group,
+                                 web_server_cloud_sql_flags):
+  """Adds flags related to private clusters to parser.
 
   Private cluster flags are related to similar flags found within GKE SDK:
     /third_party/py/googlecloudsdk/command_lib/container/flags.py
 
   Args:
     update_type_group: argument group, the group to which flag should be added.
+    web_server_cloud_sql_flags: boolean, indicates if API includes new flags.
   """
-  group = update_type_group.add_group(
-      help='Private Clusters and IP Alias (VPC-native)')
+  group = update_type_group.add_group(help='Private Clusters')
   ENABLE_PRIVATE_ENVIRONMENT_FLAG.AddToParser(group)
   ENABLE_PRIVATE_ENDPOINT_FLAG.AddToParser(group)
   MASTER_IPV4_CIDR_FLAG.AddToParser(group)
-  ENABLE_IP_ALIAS_FLAG.AddToParser(group)
-  CLUSTER_IPV4_CIDR_FLAG.AddToParser(group)
-  SERVICES_IPV4_CIDR_FLAG.AddToParser(group)
-  CLUSTER_SECONDARY_RANGE_NAME_FLAG.AddToParser(group)
-  SERVICES_SECONDARY_RANGE_NAME_FLAG.AddToParser(group)
+  if web_server_cloud_sql_flags:
+    WEB_SERVER_IPV4_CIDR_FLAG.AddToParser(group)
+    CLOUD_SQL_IPV4_CIDR_FLAG.AddToParser(group)
 
 
 def AddPypiUpdateFlagsToGroup(update_type_group):
@@ -676,3 +834,21 @@ def FallthroughToLocationProperty(location_refs, flag_name, failure_msg):
     return [parsers.ParseLocation(fallthrough_location)]
   else:
     raise exceptions.RequiredArgumentException(flag_name, failure_msg)
+
+
+def ValidateIpRanges(ip_ranges):
+  """Validates list of IP ranges.
+
+  Raises exception when any of the given strings is not a valid IPv4
+  or IPv6 network IP range.
+  Args:
+    ip_ranges: [string], list of IP ranges to validate
+  """
+  for ip_range in ip_ranges:
+    if six.PY2:
+      ip_range = ip_range.decode()
+    try:
+      ipaddress.ip_network(ip_range)
+    except:
+      raise command_util.InvalidUserInputError(
+          'Invalid IP range: [{}].'.format(ip_range))
