@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -97,37 +97,31 @@ def MakeCommitmentArg(plural):
       region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION)
 
 
-def AddCreateFlags(parser, enable_ssd_and_accelerator_support=False):
+def AddCreateFlags(parser):
   """Add general arguments for `commitments create` flag."""
-  parser.add_argument(
+  AddPlan(parser)
+  AddReservationArgGroup(parser)
+  AddResourcesArgGroup(parser)
+
+
+def AddPlan(parser):
+  return parser.add_argument(
       '--plan',
       required=True,
       choices=VALID_PLANS,
       help='Duration of the commitment.')
 
-  if enable_ssd_and_accelerator_support:
-    AddResourcesArgGroup(parser)
-  else:
-    AddResourcesFlag(parser)
 
-
-def AddResourcesFlag(parser):
-  """Add --resources flag in `commitments create`."""
-  resources_help = """\
-Resources to be included in the commitment, the ratio between number of vCPU cores and memory must conform to limits described at:
-https://cloud.google.com/compute/docs/instances/creating-instance-with-custom-machine-type#specifications.
-*memory*::: The size of the memory, should include units (e.g. 3072MB or 9GB). If no units are specified, GB is assumed.
-*vcpu*::: The number of the vCPU cores."""
-  parser.add_argument(
-      '--resources',
-      required=True,
-      help=resources_help,
-      type=arg_parsers.ArgDict(
-          spec={
-              'vcpu': int,
-              'memory': arg_parsers.BinarySize()
-          },
-          required_keys=['vcpu', 'memory']))
+def AddLicenceBasedFlags(parser):
+  parser.add_argument('--license', required=True,
+                      help='Applicable license URI. For example: '
+                           '`https://www.googleapis.com/compute/v1/projects/suse-sap-cloud/global/licenses/sles-sap-12`')  #  pylint:disable=line-too-long
+  parser.add_argument('--cores-per-license', required=False, type=str,
+                      help='Core range of the instance. Must be one of: `1-2`,'
+                           ' `3-4`, `5+`. Required for SAP licenses.')
+  parser.add_argument('--amount', required=True, type=int,
+                      help='Number of licenses purchased.')
+  AddPlan(parser)
 
 
 # TODO(b/129054682): fix the format in text render.
@@ -137,7 +131,7 @@ def AddResourcesArgGroup(parser):
       'Manage the commitment for particular resources.', required=True)
 
   resources_help = """\
-Resources to be included in the commitment, the ratio between number of vCPU cores and memory must conform to limits described at:
+Resources to be included in the commitment. The ratio between the number of vCPU cores and memory must conform to limits described at:
 https://cloud.google.com/compute/docs/instances/creating-instance-with-custom-machine-type#specifications.
 *memory*::: The size of the memory, should include units (e.g. 3072MB or 9GB). If no units are specified, GB is assumed.
 *vcpu*::: The number of the vCPU cores.
@@ -147,14 +141,12 @@ https://cloud.google.com/compute/docs/instances/creating-instance-with-custom-ma
   resources_group.add_argument(
       '--resources',
       help=resources_help,
-      required=True,
       type=arg_parsers.ArgDict(
           spec={
               'vcpu': int,
               'local-ssd': int,
               'memory': arg_parsers.BinarySize()
-          },
-          required_keys=['vcpu', 'memory']))
+          }))
   accelerator_help = """\
 Manage the configuration of the type and number of accelerator cards to include in the commitment.
 *count*::: The number of accelerators to include.
@@ -188,7 +180,7 @@ def AddReservationArgGroup(parser):
 
   reservations_manage_group.add_argument(
       '--reservations-from-file',
-      type=arg_parsers.BufferedFileInput(),
+      type=arg_parsers.FileContents(),
       help='Path to a YAML file of multiple reservations\' configuration.')
 
   single_reservation_group = reservations_manage_group.add_argument_group(
