@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,6 +25,7 @@ from googlecloudsdk.command_lib.resource_manager import completers as resource_m
 from googlecloudsdk.command_lib.util import completers
 from googlecloudsdk.command_lib.util import parameter_info_lib
 from googlecloudsdk.core import exceptions
+from googlecloudsdk.core.util import encoding
 
 
 class Error(exceptions.Error):
@@ -300,8 +301,8 @@ class MachineImagesCompleter(ListCommandCompleter):
   def __init__(self, **kwargs):
     super(MachineImagesCompleter, self).__init__(
         collection='compute.machineImages',
-        list_command='alpha compute machine-images list --uri',
-        api_version='alpha',
+        list_command='beta compute machine-images list --uri',
+        api_version='beta',
         **kwargs)
 
 
@@ -314,6 +315,36 @@ class SearchMachineImagesCompleter(completers.ResourceSearchCompleter):
         **kwargs)
 
 
+class InPlaceSnapshotsCompleter(completers.MultiResourceCompleter):
+
+  def __init__(self, **kwargs):
+    super(InPlaceSnapshotsCompleter, self).__init__(
+        completers=[
+            RegionInPlaceSnapshotsCompleter, ZoneInPlaceSnapshotsCompleter
+        ],
+        **kwargs)
+
+
+class ZoneInPlaceSnapshotsCompleter(ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(ZoneInPlaceSnapshotsCompleter, self).__init__(
+        collection='compute.zoneInPlaceSnapshots',
+        list_command='alpha compute in-place-snapshots list --uri',
+        api_version='alpha',
+        **kwargs)
+
+
+class RegionInPlaceSnapshotsCompleter(ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(RegionInPlaceSnapshotsCompleter, self).__init__(
+        collection='compute.regionInPlaceSnapshots',
+        list_command='alpha compute in-place-snapshots list --uri',
+        api_version='alpha',
+        **kwargs)
+
+
 class MachineTypesCompleter(ListCommandCompleter):
 
   def __init__(self, **kwargs):
@@ -348,7 +379,8 @@ class TestCompleter(ListCommandCompleter):
   """
 
   def __init__(self, **kwargs):
-    test_parameters = os.environ.get('_ARGCOMPLETE_TEST', 'parameters=bad')
+    test_parameters = encoding.GetEncodedValue(os.environ, '_ARGCOMPLETE_TEST',
+                                               'parameters=bad')
     kwargs = dict(kwargs)
     for pair in test_parameters.split(','):
       name, value = pair.split('=')
