@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2019 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -106,7 +106,7 @@ class BadArgumentException(ValueError):
 
 
 def AddZoneFlag(parser, resource_type, operation_type, flag_prefix=None,
-                explanation=ZONE_PROPERTY_EXPLANATION,
+                explanation=ZONE_PROPERTY_EXPLANATION, help_text=None,
                 hidden=False, plural=False, custom_plural=None):
   """Adds a --zone flag to the given parser.
 
@@ -118,6 +118,7 @@ def AddZoneFlag(parser, resource_type, operation_type, flag_prefix=None,
                     "update" or "delete".
     flag_prefix: str, flag will be named --{flag_prefix}-zone.
     explanation: str, detailed explanation of the flag.
+    help_text: str, help text will be overridden with this value.
     hidden: bool, If True, --zone argument help will be hidden.
     plural: bool, resource_type will be pluralized or not depending on value.
     custom_plural: str, If plural is True then this string will be used as
@@ -135,12 +136,12 @@ def AddZoneFlag(parser, resource_type, operation_type, flag_prefix=None,
       hidden=hidden,
       completer=completers.ZonesCompleter,
       action=actions.StoreProperty(properties.VALUES.compute.zone),
-      help='{0} {1}'.format(short_help, explanation))
+      help=help_text or '{0} {1}'.format(short_help, explanation))
 
 
 def AddRegionFlag(parser, resource_type, operation_type,
                   flag_prefix=None,
-                  explanation=REGION_PROPERTY_EXPLANATION,
+                  explanation=REGION_PROPERTY_EXPLANATION, help_text=None,
                   hidden=False, plural=False, custom_plural=None):
   """Adds a --region flag to the given parser.
 
@@ -152,6 +153,7 @@ def AddRegionFlag(parser, resource_type, operation_type,
                     "update" or "delete".
     flag_prefix: str, flag will be named --{flag_prefix}-region.
     explanation: str, detailed explanation of the flag.
+    help_text: str, help text will be overridden with this value.
     hidden: bool, If True, --region argument help will be hidden.
     plural: bool, resource_type will be pluralized or not depending on value.
     custom_plural: str, If plural is True then this string will be used as
@@ -169,7 +171,7 @@ def AddRegionFlag(parser, resource_type, operation_type,
       completer=completers.RegionsCompleter,
       action=actions.StoreProperty(properties.VALUES.compute.region),
       hidden=hidden,
-      help='{0} {1}'.format(short_help, explanation))
+      help=help_text or '{0} {1}'.format(short_help, explanation))
 
 
 class UnderSpecifiedResourceError(exceptions.Error):
@@ -346,21 +348,31 @@ class ResourceResolver(object):
           .format(default_scope,
                   ' or '.join([s.scope_enum.name for s in self.scopes])))
 
-  def _GetResourceScopeParam(
-      self, resource_scope, scope_value, project, api_resource_registry):
+  def _GetResourceScopeParam(self,
+                             resource_scope,
+                             scope_value,
+                             project,
+                             api_resource_registry,
+                             with_project=True):
+    """Gets the resource scope parameters."""
+
     if scope_value is not None:
       if resource_scope.scope_enum == compute_scope.ScopeEnum.GLOBAL:
         return None
       else:
         collection = compute_scope.ScopeEnum.CollectionForScope(
             resource_scope.scope_enum)
-        return api_resource_registry.Parse(
-            scope_value,
-            params={'project': project},
-            collection=collection).Name()
+        if with_project:
+          return api_resource_registry.Parse(
+              scope_value, params={
+                  'project': project
+              }, collection=collection).Name()
+        else:
+          return api_resource_registry.Parse(
+              scope_value, params={}, collection=collection).Name()
     else:
-      if resource_scope and (
-          resource_scope.scope_enum != compute_scope.ScopeEnum.GLOBAL):
+      if resource_scope and (resource_scope.scope_enum !=
+                             compute_scope.ScopeEnum.GLOBAL):
         return resource_scope.scope_enum.property_func
 
   def _GetRefsAndUnderspecifiedNames(
@@ -392,9 +404,13 @@ class ResourceResolver(object):
       refs.append(ref)
     return refs, underspecified_names
 
-  def _ResolveUnderspecifiedNames(
-      self, underspecified_names, default_scope, scope_lister, project,
-      api_resource_registry):
+  def _ResolveUnderspecifiedNames(self,
+                                  underspecified_names,
+                                  default_scope,
+                                  scope_lister,
+                                  project,
+                                  api_resource_registry,
+                                  with_project=True):
     """Attempt to resolve scope for unresolved names.
 
     If unresolved_names was generated with _GetRefsAndUnderspecifiedNames
@@ -406,6 +422,8 @@ class ResourceResolver(object):
       scope_lister: callback used to list potential scopes for the resources
       project: str, id of the project
       api_resource_registry: resources Registry
+      with_project: indicates whether or not project is associated. It should be
+        False for flexible resource APIs
 
     Raises:
       UnderSpecifiedResourceError: when resource scope can't be resolved.
@@ -426,7 +444,12 @@ class ResourceResolver(object):
       raise UnderSpecifiedResourceError(names, [s.flag for s in self.scopes])
 
     resource_scope = self.scopes[resource_scope_enum]
-    params = {'project': project,}
+    if with_project:
+      params = {
+          'project': project,
+      }
+    else:
+      params = {}
 
     if resource_scope.scope_enum != compute_scope.ScopeEnum.GLOBAL:
       params[resource_scope.scope_enum.param_name] = scope_value
@@ -444,7 +467,8 @@ class ResourceResolver(object):
                        scope_value,
                        api_resource_registry,
                        default_scope=None,
-                       scope_lister=None):
+                       scope_lister=None,
+                       with_project=True):
     """Resolve this resource against the arguments.
 
     Args:
@@ -468,6 +492,8 @@ class ResourceResolver(object):
           specify default_scope.
       scope_lister: func(scope, underspecified_names), a callback which returns
         list of items (with 'name' attribute) for given scope.
+      with_project: indicates whether or not project is associated. It should be
+        False for flexible resource APIs.
     Returns:
       Resource reference or list of references if plural.
     Raises:
@@ -483,14 +509,22 @@ class ResourceResolver(object):
     if default_scope is not None:
       default_scope = self.scopes[default_scope]
     project = properties.VALUES.core.project.GetOrFail
-    params = {
-        'project': project,
-    }
+    if with_project:
+      params = {
+          'project': project,
+      }
+    else:
+      params = {}
+
     if scope_value is None:
       resource_scope = self.scopes.GetImplicitScope(default_scope)
 
     resource_scope_param = self._GetResourceScopeParam(
-        resource_scope, scope_value, project, api_resource_registry)
+        resource_scope,
+        scope_value,
+        project,
+        api_resource_registry,
+        with_project=with_project)
     if resource_scope_param is not None:
       params[resource_scope.scope_enum.param_name] = resource_scope_param
 
@@ -504,8 +538,12 @@ class ResourceResolver(object):
     # If we still have some resources which need to be resolve see if we can
     # prompt the user and try to resolve these again.
     self._ResolveUnderspecifiedNames(
-        underspecified_names, default_scope, scope_lister, project,
-        api_resource_registry)
+        underspecified_names,
+        default_scope,
+        scope_lister,
+        project,
+        api_resource_registry,
+        with_project=with_project)
 
     # Now unpack each element.
     refs = [ref[0] for ref in refs]
@@ -562,12 +600,25 @@ class ResourceArgument(object):
     and zonal qualifiers (or any combination of) for each resource.
   """
 
-  def __init__(self, name=None, resource_name=None, completer=None,
-               plural=False, required=True, zonal_collection=None,
-               regional_collection=None, global_collection=None,
-               region_explanation=None, region_hidden=False,
-               zone_explanation=None, zone_hidden=False,
-               short_help=None, detailed_help=None, custom_plural=None):
+  def __init__(self,
+               name=None,
+               resource_name=None,
+               completer=None,
+               plural=False,
+               required=True,
+               zonal_collection=None,
+               regional_collection=None,
+               global_collection=None,
+               global_help_text=None,
+               region_explanation=None,
+               region_help_text=None,
+               region_hidden=False,
+               zone_explanation=None,
+               zone_help_text=None,
+               zone_hidden=False,
+               short_help=None,
+               detailed_help=None,
+               custom_plural=None):
 
     """Constructor.
 
@@ -584,11 +635,17 @@ class ResourceArgument(object):
       global_collection: str, if also zonal and/or regional adds global flag
                               and uses this collection to resolve as
                               global resource.
+      global_help_text: str, if provided, global flag help text will be
+                             overridden with this value.
       region_explanation: str, long help that will be given for region flag,
                                empty by default.
+      region_help_text: str, if provided, region flag help text will be
+                             overridden with this value.
       region_hidden: bool, Hide region in help if True.
       zone_explanation: str, long help that will be given for zone flag, empty
                              by default.
+      zone_help_text: str, if provided, zone flag help text will be overridden
+                           with this value.
       zone_hidden: bool, Hide region in help if True.
       short_help: str, help for the flag being added, if not provided help text
                        will be 'The name[s] of the ${resource_name}[s].'.
@@ -629,9 +686,12 @@ class ResourceArgument(object):
     if global_collection:
       self.scopes.AddScope(compute_scope.ScopeEnum.GLOBAL,
                            collection=global_collection)
+    self._global_help_text = global_help_text
     self._region_explanation = region_explanation or ''
+    self._region_help_text = region_help_text
     self._region_hidden = region_hidden
     self._zone_explanation = zone_explanation or ''
+    self._zone_help_text = zone_help_text
     self._zone_hidden = zone_hidden
     self._resource_resolver = ResourceResolver(self.scopes, resource_name)
 
@@ -661,6 +721,12 @@ class ResourceArgument(object):
               int(self.plural) + 1, self.resource_name or '',
               self.custom_plural),
           operation_type)
+      if self.name.startswith('instance'):
+        params['help'] += (' For details on valid instance names, refer '
+                           'to the criteria documented under the field '
+                           '\'name\' at: '
+                           'https://cloud.google.com/compute/docs/reference/'
+                           'rest/v1/instances')
 
     if self.name_arg.startswith('--'):
       params['required'] = self.required
@@ -687,6 +753,7 @@ class ResourceArgument(object):
           resource_type=self.resource_name,
           operation_type=operation_type,
           explanation=self._zone_explanation,
+          help_text=self._zone_help_text,
           hidden=self._zone_hidden,
           plural=self.plural,
           custom_plural=self.custom_plural)
@@ -698,6 +765,7 @@ class ResourceArgument(object):
           resource_type=self.resource_name,
           operation_type=operation_type,
           explanation=self._region_explanation,
+          help_text=self._region_help_text,
           hidden=self._region_hidden,
           plural=self.plural,
           custom_plural=self.custom_plural)
@@ -713,13 +781,15 @@ class ResourceArgument(object):
           self.scopes[compute_scope.ScopeEnum.GLOBAL].flag,
           action='store_true',
           default=None,
-          help='If set, the {0} global.'
+          help=self._global_help_text or 'If set, the {0} global.'
           .format(resource_mention))
 
-  def ResolveAsResource(self, args,
+  def ResolveAsResource(self,
+                        args,
                         api_resource_registry,
                         default_scope=None,
-                        scope_lister=None):
+                        scope_lister=None,
+                        with_project=True):
     """Resolve this resource against the arguments.
 
     Args:
@@ -731,6 +801,8 @@ class ResourceArgument(object):
           specify default_scope.
       scope_lister: func(scope, underspecified_names), a callback which returns
         list of items (with 'name' attribute) for given scope.
+      with_project: indicates whether or not project is associated. It should be
+        False for flexible resource APIs.
     Returns:
       Resource reference or list of references if plural.
     """
@@ -749,8 +821,13 @@ class ResourceArgument(object):
             'Can\'t specify {0} without specifying resource via {1}'.format(
                 flag, self.name))
     refs = self._resource_resolver.ResolveResources(
-        names, resource_scope, scope_value, api_resource_registry,
-        default_scope, scope_lister)
+        names,
+        resource_scope,
+        scope_value,
+        api_resource_registry,
+        default_scope,
+        scope_lister,
+        with_project=with_project)
     if self.plural:
       return refs
     if refs:
@@ -793,37 +870,54 @@ def AddStorageLocationFlag(parser, resource):
       """.format(resource))
 
 
+def AddGuestFlushFlag(parser, resource):
+  parser.add_argument(
+      '--guest-flush',
+      action='store_true',
+      default=False,
+      help=('Create an application-consistent {} by informing the OS '
+            'to prepare for the snapshot process. Currently only supported '
+            'on Windows instances using the Volume Shadow Copy Service '
+            '(VSS).'.format(resource)))
+
+
 def AddShieldedInstanceInitialStateKeyArg(parser):
   """Add the initial state for shielded instance arg."""
   parser.add_argument(
       '--platform-key-file',
       help="""\
-      Single Platform Key file path used when booting up
-      shielded instance from this image.
+      File path that points to an X.509 certificate or raw binary file. When
+      you create a shielded VM from this image, this certificate or raw binary
+      file is used as the platform key (PK).
         """)
   parser.add_argument(
       '--key-exchange-key-file',
       type=arg_parsers.ArgList(),
       metavar='KEK_VALUE',
       help="""\
-      List of key exchange key file paths used when booting up
-      shieled instance from this image.
+      Comma-separated list of file paths that point to X.509 certificates or raw
+      binary files. When you create a shielded VM from this image,
+      these certificates or files are used as key exchange keys (KEK).
         """)
   parser.add_argument(
       '--signature-database-file',
       type=arg_parsers.ArgList(),
       metavar='DB_VALUE',
       help="""\
-      List of valid public certificates file paths used when booting up
-      shieled instance from this image.
+      Comma-separated list of file paths that point to valid X.509 certificates
+      or raw binary files. When you create a shielded VM from this
+      image, these certificates or files are  added to the signature database
+      (db).
         """)
   parser.add_argument(
       '--forbidden-database-file',
       type=arg_parsers.ArgList(),
       metavar='DBX_VALUE',
       help="""\
-      List of revoked certificates file paths used when
-      booting up shieled instance from this image.
+      Comma-separated list of file paths that point to revoked X.509
+      certificates or raw binary files. When you create a shielded VM
+      from this image, these certificates or files are added to the forbidden
+      signature database (dbx).
         """)
 
 
@@ -856,3 +950,40 @@ def RewriteFilter(args, message=None, frontend_fields=None):
           args.filter, defaults=defaults)
   log.info('client_filter=%r server_filter=%r', client_filter, server_filter)
   return client_filter, server_filter
+
+
+def AddSourceDiskCsekKeyArg(parser):
+  spec = {
+      'disk': str,
+      'csek-key-file': str
+  }
+  parser.add_argument(
+      '--source-disk-csek-key',
+      type=arg_parsers.ArgDict(spec=spec),
+      action='append',
+      metavar='PROPERTY=VALUE',
+      help="""
+              Customer-supplied encryption key of the disk attached to the
+              source instance. Required if the source disk is protected by
+              a customer-supplied encryption key. This flag may be repeated to
+              specify multiple attached disks.
+
+              *disk*::: URL of the disk attached to the source instance.
+              This can be a full or   valid partial URL
+
+              *csek-key-file*::: path to customer-supplied encryption key.
+            """
+  )
+
+
+def AddEraseVssSignature(parser, resource):
+  parser.add_argument(
+      '--erase-windows-vss-signature',
+      action='store_true',
+      default=False,
+      help="""
+              Specifies whether the disk restored from {resource} should
+              erase Windows specific VSS signature.
+              See https://cloud.google.com/sdk/gcloud/reference/compute/disks/snapshot#--guest-flush
+           """.format(resource=resource)
+  )
