@@ -30,7 +30,8 @@ from googlecloudsdk.command_lib.util import completers
 
 FORWARDING_RULES_OVERVIEW = """\
         A forwarding rule directs traffic that matches a destination IP address
-        to a forwarding target (load balancer, VPN gateway or VM instance).
+        (and possibly a TCP or UDP port) to a forwarding target (load balancer,
+        VPN gateway or VM instance).
 
         Forwarding rules can be either global or regional, specified with the
         ``--global'' or ``--region=REGION'' flags. For more information about
@@ -52,7 +53,7 @@ FORWARDING_RULES_OVERVIEW = """\
         provide an IP address for an internal self-managed forwarding rule.
 
         Different types of load balancers work at different layers of the OSI
-        networking model (http://en.wikipedia.org/wiki/Network_layer). Layer 3
+        networking model (http://en.wikipedia.org/wiki/Network_layer). Layer 3/4
         targets include target pools, target SSL proxies, target TCP proxies,
         and backend services. Layer 7 targets include target HTTP proxies and
         target HTTPS proxies. For more information, refer to
@@ -62,7 +63,8 @@ FORWARDING_RULES_OVERVIEW = """\
 
 FORWARDING_RULES_OVERVIEW_ALPHA = """\
         A forwarding rule directs traffic that matches a destination IP address
-        to a forwarding target (load balancer, VPN gateway or VM instance).
+        (and possibly a TCP or UDP port) to a forwarding target (load balancer,
+        VPN gateway or VM instance).
 
         Forwarding rules can be either global or regional, specified with the
         ``--global'' or ``--region=REGION'' flag. For more information about
@@ -86,12 +88,17 @@ FORWARDING_RULES_OVERVIEW_ALPHA = """\
         Different types of load balancers work at different layers of the OSI
         networking model (http://en.wikipedia.org/wiki/Network_layer). Layer 3
         targets include target pools, target SSL proxies, target TCP proxies,
-        and backend services. Layer 7 targets include target HTTP proxies and
-        target HTTPS proxies. For more information, refer to
+        and backend services. Layer 7 targets include target HTTP proxies,
+        target HTTPS and target gRPC proxies. For more information, refer to
         https://cloud.google.com/load-balancing/docs/forwarding-rule-concepts.
         """
 
 
+# This is the list of valid values for the --target-bundle option, used
+# for Private Service Connect for Google APIs.
+PSC_GOOGLE_APIS_BUNDLES = ['all-apis', 'vpc-sc']
+
+
 class ForwardingRulesZonalCompleter(compute_completers.ListCommandCompleter):
 
   def __init__(self, **kwargs):
@@ -205,6 +212,20 @@ SUBNET_ARG = compute_flags.ResourceArgument(
                         ' region of the forwarding rule.'))
 
 
+def TargetGrpcProxyArg():
+  """Return a resource argument for parsing a target gRPC proxy."""
+
+  target_grpc_proxy_arg = compute_flags.ResourceArgument(
+      name='--target-grpc-proxy',
+      required=False,
+      resource_name='target gRPC proxy',
+      global_collection='compute.targetGrpcProxies',
+      short_help='Target gRPC proxy that receives the traffic.',
+      detailed_help=('Target gRPC proxy that receives the traffic.'),
+      region_explanation=None)
+  return target_grpc_proxy_arg
+
+
 def TargetHttpProxyArg(include_l7_internal_load_balancing=False):
   """Return a resource argument for parsing a target http proxy."""
 
@@ -328,7 +349,9 @@ def AddressArgHelp(include_l7_internal_load_balancing):
     - address-1
 
     The load-balancing-scheme (%s) and the forwarding rule's target determine
-    the type of IP address that you can use. For detailed information, refer to
+    the type of IP address that you can use. The address type must be external
+    for load-balancing-scheme EXTERNAL, and for the other load-balancing-schemes
+    the address must be internal. For detailed information, refer to
     https://cloud.google.com/load-balancing/docs/forwarding-rule-concepts#ip_address_specifications.
   """ % (
       lb_schemes)
@@ -352,10 +375,15 @@ def AddressArg(include_l7_internal_load_balancing):
 
 
 def AddUpdateArgs(parser,
-                  include_l7_internal_load_balancing=False):
+                  include_l7_internal_load_balancing=False,
+                  include_target_grpc_proxy=False,
+                  include_psc_google_apis=False):
   """Adds common flags for mutating forwarding rule targets."""
   target = parser.add_mutually_exclusive_group(required=True)
 
+  if include_target_grpc_proxy:
+    TargetGrpcProxyArg().AddArgument(parser, mutex_group=target)
+
   TargetHttpProxyArg(
       include_l7_internal_load_balancing=include_l7_internal_load_balancing
   ).AddArgument(
@@ -369,8 +397,20 @@ def AddUpdateArgs(parser,
   TARGET_SSL_PROXY_ARG.AddArgument(parser, mutex_group=target)
   TARGET_TCP_PROXY_ARG.AddArgument(parser, mutex_group=target)
   TARGET_VPN_GATEWAY_ARG.AddArgument(parser, mutex_group=target)
-
   BACKEND_SERVICE_ARG.AddArgument(parser, mutex_group=target)
+
+  if include_psc_google_apis:
+    target.add_argument(
+        '--target-google-apis-bundle',
+        required=False,
+        #      short_help='Target bundle of Google APIs to expose.',
+        help=(
+            'Target bundle of Google APIs that will receive forwarded traffic'
+            'via Private Service Connect. '
+            'Acceptable values are all-apis, meaning all Google APIs, or '
+            'vpc-sc, meaning just the APIs that support VPC Service Controls'),
+        action='store')
+
   NetworkArg(
       include_l7_internal_load_balancing=include_l7_internal_load_balancing
   ).AddArgument(parser)
@@ -378,22 +418,28 @@ def AddUpdateArgs(parser,
 
   AddLoadBalancingScheme(
       parser,
-      include_l7_ilb=include_l7_internal_load_balancing)
+      include_l7_ilb=include_l7_internal_load_balancing,
+      include_target_grpc_proxy=include_target_grpc_proxy)
 
 
-def AddLoadBalancingScheme(parser, include_l7_ilb=False):
+def AddLoadBalancingScheme(parser,
+                           include_l7_ilb=False,
+                           include_target_grpc_proxy=False):
   """Adds the load-balancing-scheme flag."""
+  td_proxies = ('--target-http-proxy, --target-https-proxy, --target-grpc-proxy'
+                if include_target_grpc_proxy else
+                '--target-http-proxy, --target-https-proxy')
   load_balancing_choices = {
       'EXTERNAL':
           'External load balancing or forwarding, used with one of '
           '--target-http-proxy, --target-https-proxy, --target-tcp-proxy, '
           '--target-ssl-proxy, --target-pool, --target-vpn-gateway, '
           '--target-instance.',
-      'INTERNAL': 'Internal load balancing or forwarding, used with '
-                  '--backend-service.',
+      'INTERNAL':
+          'Internal load balancing or forwarding, used with --backend-service.',
       'INTERNAL_SELF_MANAGED':
-          'Traffic director load balancing or forwarding, used with '
-          '--target-http-proxy, --target-https-proxy.'
+          """Traffic director load balancing or forwarding, used with
+          {0}.""".format(td_proxies)
   }
 
   if include_l7_ilb:
