@@ -270,11 +270,13 @@ def AddImageArgs(parser, enable_snapshots=False):
   image_group.add_argument(
       '--image-family',
       help="""\
-      The family of the image that the boot disk will be initialized
-      with. When a family is specified instead of an image, the latest
-      non-deprecated image associated with that family is used. It is best
-      practice to use `--image-family` when the latest version of an image is
-      needed.
+      The image family for the operating system that the boot disk will
+      be initialized with. Compute Engine offers multiple Linux
+      distributions, some of which are available as both regular and
+      Shielded VM images.  When a family is specified instead of an image,
+      the latest non-deprecated image associated with that family is
+      used. It is best practice to use `--image-family` when the latest
+      version of an image is needed.
 
       By default, ``{default_image_family}'' is assumed for this flag.
       """.format(default_image_family=constants.DEFAULT_IMAGE_FAMILY))
@@ -528,19 +530,23 @@ def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False,
       a specific version of an image is needed. If both image and image-family
       flags are omitted a blank disk will be created.
 
-      *image-family*::: The family of the image that the disk will be
-      initialized with. When a family is specified instead of an image,
+      *image-family*::: The image family for the operating system that the boot
+      disk will be initialized with. Compute Engine offers multiple Linux
+      distributions, some of which are available as both regular and
+      Shielded VM images.  When a family is specified instead of an image,
       the latest non-deprecated image associated with that family is
-      used. It is best practice to use image-family when the latest version
-      of an image is needed.
+      used. It is best practice to use --image-family when the latest
+      version of an image is needed.
 
-      *image-project*::: The project that the image or image family
-      belongs to. It is best practice to define image-project.
+      *image-project*::: The Google Cloud project against which all image and
+      image family references will be resolved. It is best practice to define
+      image-project. A full list of available projects can be generated by
+      running `gcloud projects list`.
           * If specifying one of our public images, image-project must be
             provided.
           * If there are several of the same image-family value in multiple
-            projects, image-project must be specified to clarify the image to
-            be used.
+            projects, image-project must be specified to clarify the image to be
+            used.
           * If not specified and either image or image-family is provided, the
             current default project is used.
 
@@ -560,7 +566,7 @@ def AddCreateDiskArgs(parser, enable_kms=False, enable_snapshots=False,
       *auto-delete*::: If ``yes'',  this persistent disk will be
       automatically deleted when the instance is deleted. However,
       if the disk is later detached from the instance, this option
-      won't apply. The default value for this is ``no''.
+      won't apply. The default value for this is ``yes''.
       """.format(disk_name_extra_help, disk_mode_extra_help,
                  disk_device_name_help)
   if enable_kms:
@@ -971,8 +977,15 @@ def AddAddressArgs(parser,
   addresses.add_argument(
       '--no-address',
       action='store_true',
-      help=('If provided, the instances will not be assigned external IP '
-            'addresses.'))
+      help="""\
+           If provided, the instances are not assigned external IP
+           addresses. To pull container images, you must configure private
+           Google access if using Container Registry or configure Cloud NAT
+           for instances to access container images directly. For more
+           information, see:
+             * https://cloud.google.com/vpc/docs/configure-private-google-access
+             * https://cloud.google.com/nat/docs/using-nat
+           """)
   if instances:
     address_help = """\
         Assigns the given external address to the instance that is created.
@@ -993,8 +1006,8 @@ def AddAddressArgs(parser,
       'no-address': None,
       'subnet': str,
   }
-  if instances:
-    multiple_network_interface_cards_spec['private-network-ip'] = str
+
+  multiple_network_interface_cards_spec['private-network-ip'] = str
 
   def ValidateNetworkTier(network_tier_input):
     network_tier = network_tier_input.upper()
@@ -1033,8 +1046,8 @@ def AddAddressArgs(parser,
         ``NETWORK_TIER'' must be one of: `PREMIUM`, `STANDARD`. The default
         value is `PREMIUM`.
         """
-    if instances:
-      network_interface_help += """
+
+    network_interface_help += """
         *private-network-ip*::: Assigns the given RFC1918 IP address to the
         interface.
         """
@@ -1123,13 +1136,31 @@ def AddMinCpuPlatformArgs(parser, track, required=False):
       """.format(track.prefix + ' ' if track.prefix else ''))
 
 
-def AddMinNodeCpuArg(parser):
+def AddMinNodeCpuArg(parser, is_update=False):
   parser.add_argument(
       '--min-node-cpu',
       help="""\
       Minimum number of virtual CPUs this instance will consume when running on
       a sole-tenant node.
       """)
+  if is_update:
+    parser.add_argument(
+        '--clear-min-node-cpu',
+        action='store_true',
+        help="""\
+        Removes the min-node-cpu field from the instance. If specified, the
+        instance min-node-cpu will be cleared. The instance will not be
+        overcommitted and utilize the full CPU count assigned.
+        """)
+
+
+def AddLocationHintArg(parser):
+  parser.add_argument(
+      '--location-hint',
+      hidden=True,
+      help="""\
+      Used by internal tools to control sub-zone location of the instance.
+      """)
 
 
 def AddPreemptibleVmArgs(parser):
@@ -1769,7 +1800,7 @@ def AddDeletionProtectionFlag(parser, use_default_value=True):
 def AddShieldedInstanceConfigArgs(parser,
                                   use_default_value=True,
                                   for_update=False):
-  """Adds flags for shielded Instance configuration.
+  """Adds flags for Shielded VM configuration.
 
   Args:
     parser: ArgumentParser, parser to which flags will be added.
@@ -1787,11 +1818,14 @@ def AddShieldedInstanceConfigArgs(parser,
 
   # --shielded-secure-boot
   secure_boot_help = """\
-      The instance will boot with secure boot enabled.
+      The instance boots with secure boot enabled. On Shielded VMs, Secure
+      Boot is not enabled by default. For information about how to modify
+      Shielded VM options, see
+      https://cloud.google.com/compute/docs/instances/modifying-shielded-vm.
       """
   if for_update:
     secure_boot_help += """\
-      Changes to this setting (via the update command) will only take effect
+      Changes to this setting with the update command only take effect
       after stopping and starting the instance.
       """
 
@@ -1804,13 +1838,16 @@ def AddShieldedInstanceConfigArgs(parser,
 
   # --shielded-vtpm
   vtpm_help = """\
-      The instance will boot with the TPM (Trusted Platform Module) enabled.
+      The instance boots with the TPM (Trusted Platform Module) enabled.
       A TPM is a hardware module that can be used for different security
-      operations such as remote attestation, encryption and sealing of keys.
+      operations such as remote attestation, encryption, and sealing of keys.
+      On Shielded VMs, vTPM is enabled by default. For information about how
+      to modify Shielded VM options, see
+      https://cloud.google.com/compute/docs/instances/modifying-shielded-vm.
       """
   if for_update:
     vtpm_help += """\
-      Changes to this setting (via the update command) will only take effect
+      Changes to this setting with the update command only take effect
       after stopping and starting the instance.
       """
 
@@ -1827,11 +1864,14 @@ def AddShieldedInstanceConfigArgs(parser,
       instance. The attestation is performed against the integrity policy
       baseline. This baseline is initially derived from the implicitly
       trusted boot image when the instance is created. This baseline can be
-      updated by using `--shielded-vm-learn-integrity-policy`.
+      updated by using `--shielded-vm-learn-integrity-policy`. On Shielded
+      VMs, integrity monitoring is enabled by default. For information about
+      how to modify Shielded VM options, see
+      https://cloud.google.com/compute/docs/instances/modifying-shielded-vm.
       """
   if for_update:
     integrity_monitoring_help += """\
-      Changes to this setting (via the update command) will only take effect
+      Changes to this setting with the update command only take effect
       after stopping and starting the instance.
       """
 
@@ -1860,6 +1900,21 @@ def AddShieldedInstanceIntegrityPolicyArgs(parser):
       help=help_text)
 
 
+def AddConfidentialComputeArgs(parser):
+  """Adds flags for confidential compute for instance."""
+  help_text = """\
+  The instance will boot with confidential compute enabled. Confidential
+  Compute is based on Secure Encrypted Virtualization (SEV), an AMD
+  virtualization feature for running confidential instances.
+  """
+  parser.add_argument(
+      '--confidential-compute',
+      dest='confidential_compute',
+      action='store_true',
+      default=None,
+      help=help_text)
+
+
 def AddHostnameArg(parser):
   """Adds flag for overriding hostname for instance."""
   parser.add_argument(
@@ -2346,3 +2401,18 @@ def AddUpdateContainerArgs(parser, container_mount_disk_enabled=False):
       parser,
       container_mount_disk_enabled=container_mount_disk_enabled)
   _AddContainerArgs(parser)
+
+
+def AddPostKeyRevocationActionTypeArgs(parser):
+  """Helper to add --post-key-revocation-action-type flag."""
+  parser.add_argument(
+      '--post-key-revocation-action-type',
+      choices=['noop', 'shutdown'],
+      metavar='POLICY',
+      required=False,
+      help="""\
+      The instance will be shut down when the KMS key of one of its disk is
+      revoked, if set to `SHUTDOWN`.
+
+      Default setting is `NOOP`.
+      """)
