@@ -18,20 +18,37 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
-
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.command_lib.compute.health_checks import flags as health_checks_flags
 
+
 HEALTH_CHECK_ARG = health_checks_flags.HealthCheckArgument(
     '', '--health-check', required=False)
 
 
+# Allow only up to 1 year initial delay
+# 31536000 == 1y in seconds
+# Modify both of these together.
+_MAX_INITIAL_DELAY_DURATION = 31536000
+_MAX_INITIAL_DELAY_DURATION_HUMAN_READABLE = '1y'
+
+
+def _InitialDelayValidator(value):
+  duration_parser = arg_parsers.Duration(parsed_unit='s')
+  parsed_value = duration_parser(value)
+  if parsed_value > _MAX_INITIAL_DELAY_DURATION:
+    raise arg_parsers.ArgumentTypeError(
+        'The value of initial delay must be between 0 and {max_value}'.format(
+            max_value=_MAX_INITIAL_DELAY_DURATION_HUMAN_READABLE))
+  return parsed_value
+
+
 def AddAutohealingArgs(autohealing_params_group):
   """Adds autohealing-related commandline arguments to parser."""
   autohealing_params_group.add_argument(
       '--initial-delay',
-      type=arg_parsers.Duration(),
+      type=_InitialDelayValidator,
       help="""\
       Specifies the length of time during which the instance is known to be
       initializing and should not be autohealed even if unhealthy.
