@@ -62,14 +62,16 @@ def SubnetworkResolver():
 
 def AddUpdateArgs(parser, include_alpha_logging,
                   include_l7_internal_load_balancing,
-                  include_private_ipv6_access):
+                  include_private_ipv6_access_alpha,
+                  include_private_ipv6_access_beta):
   """Add args to the parser for subnet update.
 
   Args:
     parser: The argparse parser.
     include_alpha_logging: Include alpha-specific logging args.
     include_l7_internal_load_balancing: Include Internal HTTP(S) LB args.
-    include_private_ipv6_access: Include Private Ipv6 Access args.
+    include_private_ipv6_access_alpha: Include alpha Private Ipv6 Access args.
+    include_private_ipv6_access_beta: Include beta Private Ipv6 Access args.
   """
   messages = apis.GetMessagesModule('compute',
                                     compute_api.COMPUTE_GA_API_VERSION)
@@ -112,8 +114,8 @@ def AddUpdateArgs(parser, include_alpha_logging,
   updated_field.add_argument(
       '--enable-flow-logs',
       action=arg_parsers.StoreTrueFalseAction,
-      help=('Enable/disable VPC flow logging for this subnet. More information '
-            'for VPC flow logs can be found at '
+      help=('Enable/disable VPC Flow Logs for this subnet. More information '
+            'for VPC Flow Logs can be found at '
             'https://cloud.google.com/vpc/docs/using-flow-logs.'))
 
   AddLoggingAggregationInterval(parser, messages)
@@ -121,7 +123,7 @@ def AddUpdateArgs(parser, include_alpha_logging,
       '--logging-flow-sampling',
       type=arg_parsers.BoundedFloat(lower_bound=0.0, upper_bound=1.0),
       help="""\
-      Can only be specified if VPC flow logging for this subnetwork is
+      Can only be specified if VPC Flow logs for this subnetwork is
       enabled. The value of the field must be in [0, 1]. Set the sampling rate
       of VPC flow logs within the subnetwork where 1.0 means all collected
       logs are reported and 0.0 means no logs are reported. Default is 0.5
@@ -129,21 +131,38 @@ def AddUpdateArgs(parser, include_alpha_logging,
       """)
   AddLoggingMetadata(parser, messages)
 
+  parser.add_argument(
+      '--logging-filter-expr',
+      help="""\
+      Can only be specified if VPC Flow Logs for this subnetwork is enabled.
+      Export filter used to define which logs should be generated.
+      """)
+  parser.add_argument(
+      '--logging-metadata-fields',
+      type=arg_parsers.ArgList(),
+      metavar='METADATA_FIELD',
+      default=None,
+      help="""\
+      Can only be specified if VPC Flow Logs for this subnetwork is enabled
+      and "metadata" is set to CUSTOM_METADATA. The comma-separated list of
+      metadata fields that should be added to reported logs.
+      """)
+
   if include_alpha_logging:
     messages = apis.GetMessagesModule('compute',
                                       compute_api.COMPUTE_ALPHA_API_VERSION)
-    AddLoggingAggregationIntervalAlpha(parser, messages)
+    AddLoggingAggregationIntervalDeprecated(parser, messages)
     parser.add_argument(
         '--flow-sampling',
         type=arg_parsers.BoundedFloat(lower_bound=0.0, upper_bound=1.0),
         help="""\
-        Can only be specified if VPC flow logging for this subnetwork is
-        enabled. The value of the field must be in [0, 1]. Set the sampling rate
-        of VPC flow logs within the subnetwork where 1.0 means all collected
+        Can only be specified if VPC Flow Logs for this subnetwork is enabled.
+        The value of the field must be in [0, 1]. Set the sampling rate of
+        VPC Flow Logs within the subnetwork where 1.0 means all collected
         logs are reported and 0.0 means no logs are reported. Default is 0.5
         which means half of all collected logs are reported.
         """)
-    AddLoggingMetadataAlpha(parser, messages)
+    AddLoggingMetadataDeprecated(parser, messages)
 
   if include_l7_internal_load_balancing:
     updated_field.add_argument(
@@ -171,14 +190,16 @@ def AddUpdateArgs(parser, include_alpha_logging,
         only applicable when the [--role=ACTIVE] flag is being used.
         """)
 
-  if include_private_ipv6_access:
+  if include_private_ipv6_access_alpha:
+    messages = apis.GetMessagesModule('compute',
+                                      compute_api.COMPUTE_ALPHA_API_VERSION)
     updated_field.add_argument(
         '--enable-private-ipv6-access',
         action=arg_parsers.StoreTrueFalseAction,
         help=('Enable/disable private IPv6 access for the subnet.'))
     update_private_ipv6_access_field = updated_field.add_argument_group()
-    GetPrivateIpv6GoogleAccessTypeFlagMapper(messages).choice_arg.AddToParser(
-        update_private_ipv6_access_field)
+    GetPrivateIpv6GoogleAccessTypeFlagMapperAlpha(
+        messages).choice_arg.AddToParser(update_private_ipv6_access_field)
     update_private_ipv6_access_field.add_argument(
         '--private-ipv6-google-access-service-accounts',
         default=None,
@@ -196,9 +217,15 @@ def AddUpdateArgs(parser, include_alpha_logging,
 
         $ {command} MY-SUBNET --private-ipv6-google-access-service-accounts ''
         """)
+  elif include_private_ipv6_access_beta:
+    messages = apis.GetMessagesModule('compute',
+                                      compute_api.COMPUTE_BETA_API_VERSION)
+    update_private_ipv6_access_field = updated_field.add_argument_group()
+    GetPrivateIpv6GoogleAccessTypeFlagMapperBeta(
+        messages).choice_arg.AddToParser(update_private_ipv6_access_field)
 
 
-def GetPrivateIpv6GoogleAccessTypeFlagMapper(messages):
+def GetPrivateIpv6GoogleAccessTypeFlagMapperAlpha(messages):
   return arg_utils.ChoiceEnumMapper(
       '--private-ipv6-google-access-type',
       messages.Subnetwork.PrivateIpv6GoogleAccessValueValuesEnum,
@@ -216,6 +243,22 @@ def GetPrivateIpv6GoogleAccessTypeFlagMapper(messages):
   )
 
 
+def GetPrivateIpv6GoogleAccessTypeFlagMapperBeta(messages):
+  return arg_utils.ChoiceEnumMapper(
+      '--private-ipv6-google-access-type',
+      messages.Subnetwork.PrivateIpv6GoogleAccessValueValuesEnum,
+      custom_mappings={
+          'DISABLE_GOOGLE_ACCESS':
+              'disable',
+          'ENABLE_BIDIRECTIONAL_ACCESS_TO_GOOGLE':
+              'enable-bidirectional-access',
+          'ENABLE_OUTBOUND_VM_ACCESS_TO_GOOGLE':
+              'enable-outbound-vm-access'
+      },
+      help_str='The private IPv6 google access type for the VMs in this subnet.'
+  )
+
+
 def GetLoggingAggregationIntervalArg(messages):
   return arg_utils.ChoiceEnumMapper(
       '--logging-aggregation-interval',
@@ -229,7 +272,7 @@ def GetLoggingAggregationIntervalArg(messages):
           'INTERVAL_15_MIN': 'interval-15-min'
       },
       help_str="""\
-        Can only be specified if VPC flow logging for this subnetwork is
+        Can only be specified if VPC Flow Logs for this subnetwork is
         enabled. Toggles the aggregation interval for collecting flow logs.
         Increasing the interval time will reduce the amount of generated flow
         logs for long lasting connections. Default is an interval of 5 seconds
@@ -241,7 +284,7 @@ def AddLoggingAggregationInterval(parser, messages):
   GetLoggingAggregationIntervalArg(messages).choice_arg.AddToParser(parser)
 
 
-def GetLoggingAggregationIntervalArgAlpha(messages):
+def GetLoggingAggregationIntervalArgDeprecated(messages):
   return arg_utils.ChoiceEnumMapper(
       '--aggregation-interval',
       messages.SubnetworkLogConfig.AggregationIntervalValueValuesEnum,
@@ -254,7 +297,7 @@ def GetLoggingAggregationIntervalArgAlpha(messages):
           'INTERVAL_15_MIN': 'interval-15-min'
       },
       help_str="""\
-        Can only be specified if VPC flow logging for this subnetwork is
+        Can only be specified if VPC Flow Logs for this subnetwork is
         enabled. Toggles the aggregation interval for collecting flow logs.
         Increasing the interval time will reduce the amount of generated flow
         logs for long lasting connections. Default is an interval of 5 seconds
@@ -262,8 +305,9 @@ def GetLoggingAggregationIntervalArgAlpha(messages):
         """)
 
 
-def AddLoggingAggregationIntervalAlpha(parser, messages):
-  GetLoggingAggregationIntervalArgAlpha(messages).choice_arg.AddToParser(parser)
+def AddLoggingAggregationIntervalDeprecated(parser, messages):
+  GetLoggingAggregationIntervalArgDeprecated(messages).choice_arg.AddToParser(
+      parser)
 
 
 def GetLoggingMetadataArg(messages):
@@ -272,12 +316,13 @@ def GetLoggingMetadataArg(messages):
       messages.SubnetworkLogConfig.MetadataValueValuesEnum,
       custom_mappings={
           'INCLUDE_ALL_METADATA': 'include-all',
-          'EXCLUDE_ALL_METADATA': 'exclude-all'
+          'EXCLUDE_ALL_METADATA': 'exclude-all',
+          'CUSTOM_METADATA': 'custom'
       },
       help_str="""\
-        Can only be specified if VPC flow logging for this subnetwork is
+        Can only be specified if VPC Flow Logs for this subnetwork is
         enabled. Configures whether metadata fields should be added to the
-        reported VPC flow logs. Default is to include all metadata.
+        reported logs. Default is to include all metadata.
         """)
 
 
@@ -285,7 +330,7 @@ def AddLoggingMetadata(parser, messages):
   GetLoggingMetadataArg(messages).choice_arg.AddToParser(parser)
 
 
-def GetLoggingMetadataArgAlpha(messages):
+def GetLoggingMetadataArgDeprecated(messages):
   return arg_utils.ChoiceEnumMapper(
       '--metadata',
       messages.SubnetworkLogConfig.MetadataValueValuesEnum,
@@ -294,11 +339,11 @@ def GetLoggingMetadataArgAlpha(messages):
           'EXCLUDE_ALL_METADATA': 'exclude-all-metadata'
       },
       help_str="""\
-        Can only be specified if VPC flow logging for this subnetwork is
+        Can only be specified if VPC Flow Logs for this subnetwork is
         enabled. Configures whether metadata fields should be added to the
-        reported VPC flow logs. Default is to include all metadata.
+        reported logs. Default is to include all metadata.
         """)
 
 
-def AddLoggingMetadataAlpha(parser, messages):
-  GetLoggingMetadataArgAlpha(messages).choice_arg.AddToParser(parser)
+def AddLoggingMetadataDeprecated(parser, messages):
+  GetLoggingMetadataArgDeprecated(messages).choice_arg.AddToParser(parser)
