@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import json
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.core import yaml
 from googlecloudsdk.core.resource import resource_printer
+import six
 
 
 def SecurityPolicyFromFile(input_file, messages, file_format):
@@ -45,7 +46,7 @@ def SecurityPolicyFromFile(input_file, messages, file_format):
       parsed_security_policy = json.load(input_file)
     except ValueError as e:
       raise exceptions.BadFileException('Error parsing JSON: {0}'.format(
-          str(e)))
+          six.text_type(e)))
 
   security_policy = messages.SecurityPolicy()
   if 'description' in parsed_security_policy:
@@ -53,6 +54,9 @@ def SecurityPolicyFromFile(input_file, messages, file_format):
   if 'fingerprint' in parsed_security_policy:
     security_policy.fingerprint = base64.urlsafe_b64decode(
         parsed_security_policy['fingerprint'].encode('ascii'))
+  if 'cloudArmorConfig' in parsed_security_policy:
+    security_policy.cloudArmorConfig = messages.SecurityPolicyCloudArmorConfig(
+        enableMl=parsed_security_policy['cloudArmorConfig']['enableMl'])
 
   rules = []
   for rule in parsed_security_policy['rules']:
@@ -66,6 +70,8 @@ def SecurityPolicyFromFile(input_file, messages, file_format):
     if 'versionedExpr' in rule['match']:
       match.versionedExpr = ConvertToEnum(rule['match']['versionedExpr'],
                                           messages)
+    if 'expr' in rule['match']:
+      match.expr = messages.Expr(expression=rule['match']['expr']['expression'])
     if 'config' in rule['match']:
       if 'srcIpRanges' in rule['match']['config']:
         match.config = messages.SecurityPolicyRuleMatcherConfig(
@@ -105,3 +111,14 @@ def WriteToFile(output_file, security_policy, file_format):
   """
   resource_printer.Print(
       security_policy, print_format=file_format, out=output_file)
+
+
+def CreateCloudArmorConfig(client, args):
+  """Returns a SecurityPolicyCloudArmorConfig message if args are valid."""
+
+  messages = client.messages
+  cloud_armor_config = None
+  if args.enable_ml is not None:
+    cloud_armor_config = messages.SecurityPolicyCloudArmorConfig(
+        enableMl=args.enable_ml)
+  return cloud_armor_config
