@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ from __future__ import unicode_literals
 from googlecloudsdk.api_lib.ml_engine import models
 from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.command_lib.util.args import labels_util
+from googlecloudsdk.core import exceptions as core_exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
@@ -43,14 +44,29 @@ def ParseCreateLabels(models_client, args):
       args, models_client.messages.GoogleCloudMlV1Model.LabelsValue)
 
 
-def Create(models_client, model, regions=None, enable_logging=None,
-           labels=None, description=None):
-  if regions is None:
-    log.warning('`--regions` flag will soon be required. Please explicitly '
-                'specify a region. Using [us-central1] by default.')
-    regions = ['us-central1']
-  return models_client.Create(model, regions, enable_logging, labels=labels,
-                              description=description)
+class RegionArgError(core_exceptions.Error):
+  """Indicates that both --region and --regions flag were passed."""
+  pass
+
+
+def _GetModelRegion(args):
+  if args.region and args.regions:
+    raise RegionArgError('Only one of --region or --regions can be specified.')
+  if args.region is not None:
+    return [args.region]
+  if args.regions is None:
+    log.warning(
+        'Please explicitly specify a region. Using [us-central1] by default.')
+    return ['us-central1']
+  return args.regions
+
+
+def Create(models_client, model, args, enable_logging=None,
+           enable_console_logging=None, labels=None, description=None):
+  regions = _GetModelRegion(args)
+  return models_client.Create(model, regions, enable_logging=enable_logging,
+                              enable_console_logging=enable_console_logging,
+                              labels=labels, description=description)
 
 
 def Delete(models_client, operations_client, model):
@@ -121,4 +137,32 @@ def RemoveIamPolicyBinding(models_client, model, member, role):
   model_ref = ParseModel(model)
   policy = models_client.GetIamPolicy(model_ref)
   iam_util.RemoveBindingFromIamPolicy(policy, member, role)
+  ret = models_client.SetIamPolicy(model_ref, policy, 'bindings,etag')
+  iam_util.LogSetIamPolicy(model_ref.Name(), 'model')
+  return ret
+
+
+def AddIamPolicyBindingWithCondition(models_client, model, member, role,
+                                     condition):
+  """Adds IAM binding with condition to ml engine model's IAM policy."""
+  model_ref = ParseModel(model)
+  policy = models_client.GetIamPolicy(model_ref)
+  iam_util.AddBindingToIamPolicyWithCondition(
+      models_client.messages.GoogleIamV1Binding,
+      models_client.messages.GoogleTypeExpr,
+      policy,
+      member,
+      role,
+      condition)
   return models_client.SetIamPolicy(model_ref, policy, 'bindings,etag')
+
+
+def RemoveIamPolicyBindingWithCondition(models_client, model, member, role,
+                                        condition):
+  model_ref = ParseModel(model)
+  policy = models_client.GetIamPolicy(model_ref)
+  iam_util.RemoveBindingFromIamPolicyWithCondition(policy, member, role,
+                                                   condition)
+  ret = models_client.SetIamPolicy(model_ref, policy, 'bindings,etag')
+  iam_util.LogSetIamPolicy(model_ref.Name(), 'model')
+  return ret
