@@ -30,9 +30,9 @@ import tempfile
 from googlecloudsdk.api_lib.run import gke
 from googlecloudsdk.api_lib.run import global_methods
 from googlecloudsdk.api_lib.util import apis
+from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.run import exceptions as serverless_exceptions
 from googlecloudsdk.command_lib.run import flags
-
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.util import files
 
@@ -41,6 +41,9 @@ import six
 from six.moves.urllib import parse as urlparse
 
 
+_EVENTS_API_VERSION = 'v1alpha1'
+
+
 @contextlib.contextmanager
 def _OverrideEndpointOverrides(override):
   """Context manager to override the Cloud Run endpoint overrides for a while.
@@ -61,11 +64,17 @@ def _OverrideEndpointOverrides(override):
 class ConnectionInfo(six.with_metaclass(abc.ABCMeta)):
   """Information useful in constructing an API client."""
 
-  def __init__(self):
+  def __init__(self, version):
+    """Initialize a connection context.
+
+    Args:
+      version: str, api version to use for making requests.
+    """
     self.endpoint = None
     self.ca_certs = None
     self.region = None
     self._cm = None
+    self._version = version
 
   @property
   def api_name(self):
@@ -73,7 +82,7 @@ class ConnectionInfo(six.with_metaclass(abc.ABCMeta)):
 
   @property
   def api_version(self):
-    return global_methods.SERVERLESS_API_VERSION
+    return self._version
 
   @property
   def active(self):
@@ -147,8 +156,8 @@ def _CheckTLSSupport():
 class _GKEConnectionContext(ConnectionInfo):
   """Context manager to connect to the GKE Cloud Run add-in."""
 
-  def __init__(self, cluster_ref):
-    super(_GKEConnectionContext, self).__init__()
+  def __init__(self, cluster_ref, version):
+    super(_GKEConnectionContext, self).__init__(version)
     self.cluster_ref = cluster_ref
 
   @contextlib.contextmanager
@@ -163,7 +172,7 @@ class _GKEConnectionContext(ConnectionInfo):
 
   @property
   def operator(self):
-    return 'Cloud Run on GKE'
+    return 'Cloud Run for Anthos'
 
   def HttpClient(self):
     # Import http only when needed, as it depends on credential infrastructure
@@ -200,14 +209,15 @@ class _GKEConnectionContext(ConnectionInfo):
 class _KubeconfigConnectionContext(ConnectionInfo):
   """Context manager to connect to a cluster defined in a Kubeconfig file."""
 
-  def __init__(self, kubeconfig, context=None):
+  def __init__(self, kubeconfig, version, context=None):
     """Initialize connection context based on kubeconfig file.
 
     Args:
       kubeconfig: googlecloudsdk.api_lib.container.kubeconfig.Kubeconfig object
+      version: str, api version to use for making requests
       context: str, current context name
     """
-    super(_KubeconfigConnectionContext, self).__init__()
+    super(_KubeconfigConnectionContext, self).__init__(version)
     self.kubeconfig = kubeconfig
     self.kubeconfig.SetCurrentContext(context or kubeconfig.current_context)
     self.client_cert_data = None
@@ -230,8 +240,8 @@ class _KubeconfigConnectionContext(ConnectionInfo):
           self.endpoint = 'https://{}/'.format(self.raw_hostname)
           with _OverrideEndpointOverrides(self.endpoint):
             yield self
-      except httplib2.SSLHandshakeError as e:
-        if 'CERTIFICATE_VERIFY_FAILED' in str(e):
+      except httplib2.HttpLib2Error as e:
+        if 'CERTIFICATE_VERIFY_FAILED' in six.text_type(e):
           raise gke.NoCaCertError(
               'Missing or invalid [certificate-authority] or '
               '[certificate-authority-data] field in kubeconfig file.')
@@ -244,19 +254,25 @@ class _KubeconfigConnectionContext(ConnectionInfo):
       raise ValueError(
           'Kubeconfig authentication requires a client certificate '
           'authentication method.')
-    # Import http only when needed, as it depends on credential infrastructure
-    # which is not needed in all cases.
-    from googlecloudsdk.core import http as http_core  # pylint: disable=g-import-not-at-top
-    http_client = http_core.Http(
-        response_encoding=http_core.ENCODING,
+    if self.client_cert_domain:
+      # Import http only when needed, as it depends on credential infrastructure
+      # which is not needed in all cases.
+      from googlecloudsdk.core import http as http_core  # pylint: disable=g-import-not-at-top
+      http_client = http_core.Http(
+          response_encoding=http_core.ENCODING,
+          ca_certs=self.ca_certs)
+      http_client.add_certificate(
+          self.client_key, self.client_cert, self.client_cert_domain)
+      return http_client
+    from googlecloudsdk.core.credentials import http as http_creds  # pylint: disable=g-import-not-at-top
+    http_client = http_creds.Http(
+        response_encoding=http_creds.ENCODING,
         ca_certs=self.ca_certs)
-    http_client.add_certificate(
-        self.client_key, self.client_cert, self.client_cert_domain)
     return http_client
 
   @property
   def operator(self):
-    return 'Kubernetes Cluster'
+    return 'Cloud Run for Anthos'
 
   @property
   def location_label(self):
@@ -353,8 +369,8 @@ def DeriveRegionalEndpoint(endpoint, region):
 class _RegionalConnectionContext(ConnectionInfo):
   """Context manager to connect a particular Cloud Run region."""
 
-  def __init__(self, region):
-    super(_RegionalConnectionContext, self).__init__()
+  def __init__(self, region, version):
+    super(_RegionalConnectionContext, self).__init__(version)
     self.region = region
 
   @property
@@ -373,8 +389,7 @@ class _RegionalConnectionContext(ConnectionInfo):
   @contextlib.contextmanager
   def Connect(self):
     global_endpoint = apis.GetEffectiveApiEndpoint(
-        global_methods.SERVERLESS_API_NAME,
-        global_methods.SERVERLESS_API_VERSION)
+        global_methods.SERVERLESS_API_NAME, self._version)
     self.endpoint = DeriveRegionalEndpoint(global_endpoint, self.region)
     with _OverrideEndpointOverrides(self.endpoint):
       yield self
@@ -384,11 +399,37 @@ class _RegionalConnectionContext(ConnectionInfo):
     return True
 
 
-def GetConnectionContext(args):
+def _GetApiVersion(product,
+                   release_track,
+                   is_cluster=False,
+                   version_override=None):
+  """Returns the api version to use depending on the current context."""
+  del release_track
+  del is_cluster
+
+  if version_override is not None:
+    return version_override
+
+  if product == flags.Product.RUN:
+    return global_methods.SERVERLESS_API_VERSION
+  elif product == flags.Product.EVENTS:
+    return _EVENTS_API_VERSION
+  else:
+    raise ValueError('Unrecognized product: ' + six.u(product))
+
+
+def GetConnectionContext(args,
+                         product=flags.Product.RUN,
+                         release_track=base.ReleaseTrack.GA,
+                         version_override=None):
   """Gets the regional, kubeconfig, or GKE connection context.
 
   Args:
     args: Namespace, the args namespace.
+    product: Which product is requesting connection context.
+    release_track: Release track of the command being run.
+    version_override: If specified, the given api version will be used no matter
+      the other parameters.
 
   Raises:
     ArgumentError if region or cluster is not specified.
@@ -396,23 +437,35 @@ def GetConnectionContext(args):
   Returns:
     A GKE or regional ConnectionInfo object.
   """
-  if flags.IsKubernetes(args):
+  if flags.GetPlatform() == flags.PLATFORM_KUBERNETES:
     kubeconfig = flags.GetKubeconfig(args)
-    return _KubeconfigConnectionContext(kubeconfig, args.context)
+    api_version = _GetApiVersion(
+        product,
+        release_track,
+        is_cluster=True,
+        version_override=version_override)
+    return _KubeconfigConnectionContext(kubeconfig, api_version, args.context)
 
-  if flags.IsGKE(args):
+  if flags.GetPlatform() == flags.PLATFORM_GKE:
     cluster_ref = args.CONCEPTS.cluster.Parse()
     if not cluster_ref:
       raise flags.ArgumentError(
           'You must specify a cluster in a given location. '
           'Either use the `--cluster` and `--cluster-location` flags '
           'or set the run/cluster and run/cluster_location properties.')
-    return _GKEConnectionContext(cluster_ref)
+    api_version = _GetApiVersion(
+        product,
+        release_track,
+        is_cluster=True,
+        version_override=version_override)
+    return _GKEConnectionContext(cluster_ref, api_version)
 
-  if flags.IsManaged(args):
+  if flags.GetPlatform() == flags.PLATFORM_MANAGED:
     region = flags.GetRegion(args, prompt=True)
     if not region:
       raise flags.ArgumentError(
           'You must specify a region. Either use the `--region` flag '
           'or set the run/region property.')
-    return _RegionalConnectionContext(region)
+    api_version = _GetApiVersion(
+        product, release_track, version_override=version_override)
+    return _RegionalConnectionContext(region, api_version)
