@@ -32,6 +32,7 @@ from googlecloudsdk.command_lib.run import flags
 from googlecloudsdk.command_lib.util.concepts import presentation_specs
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
+from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import console_io
 
 
@@ -108,7 +109,8 @@ class DefaultFallthrough(deps.Fallthrough):
         'Otherwise, defaults to project ID.')
 
   def _Call(self, parsed_args):
-    if flags.IsGKE(parsed_args) or flags.IsKubernetes(parsed_args):
+    if (flags.GetPlatform() == flags.PLATFORM_GKE or
+        flags.GetPlatform() == flags.PLATFORM_KUBERNETES):
       return 'default'
     elif not (getattr(parsed_args, 'project', None) or
               properties.VALUES.core.project.Get()):
@@ -124,7 +126,7 @@ class DefaultFallthrough(deps.Fallthrough):
 def NamespaceAttributeConfig():
   return concepts.ResourceParameterAttributeConfig(
       name='namespace',
-      help_text='Specific to Cloud Run on GKE: '
+      help_text='Specific to Cloud Run for Anthos: '
       'Kubernetes namespace for the {resource}.',
       fallthroughs=[
           deps.PropertyFallthrough(properties.VALUES.run.namespace),
@@ -188,7 +190,7 @@ class ClusterPromptFallthrough(PromptFallthrough):
     Returns:
       A cluster name string
     """
-    if flags.IsGKE(parsed_args):
+    if flags.GetPlatform() == flags.PLATFORM_GKE:
       cluster_location = (
           getattr(parsed_args, 'cluster_location', None) or
           properties.VALUES.run.cluster_location.Get())
@@ -199,7 +201,7 @@ class ClusterPromptFallthrough(PromptFallthrough):
       if not clusters:
         raise exceptions.ConfigurationError(
             'No compatible clusters found{}. '
-            'Ensure your cluster has Cloud Run on GKE enabled.'.format(
+            'Ensure your cluster has Cloud Run enabled.'.format(
                 cluster_location_msg))
 
       def _GetClusterDescription(cluster):
@@ -267,14 +269,14 @@ class ClusterLocationPromptFallthrough(PromptFallthrough):
     cluster_name = (
         getattr(parsed_args, 'cluster', None) or
         properties.VALUES.run.cluster.Get())
-    if flags.IsGKE(parsed_args) and cluster_name:
+    if flags.GetPlatform() == flags.PLATFORM_GKE and cluster_name:
       clusters = [
           c for c in global_methods.ListClusters() if c.name == cluster_name
       ]
       if not clusters:
         raise exceptions.ConfigurationError(
             'No cluster locations found for cluster [{}]. '
-            'Ensure your clusters have Cloud Run on GKE enabled.'
+            'Ensure your clusters have Cloud Run enabled.'
             .format(cluster_name))
       cluster_locations = [c.zone for c in clusters]
       idx = console_io.PromptChoice(
@@ -350,45 +352,22 @@ def GetDomainMappingResourceSpec():
 
 
 def GetNamespaceResourceSpec():
-  return concepts.ResourceSpec(
-      'run.namespaces',
-      namespacesId=NamespaceAttributeConfig(),
-      resource_name='namespace')
+  """Returns a resource spec for the namespace."""
+  # TODO(b/150322097): Remove this when the api has been split.
+  # This try/except block is needed because the v1alpha1 and v1 run apis
+  # have different collection names for the namespaces.
+  try:
+    return concepts.ResourceSpec(
+        'run.namespaces',
+        namespacesId=NamespaceAttributeConfig(),
+        resource_name='namespace')
+  except resources.InvalidCollectionException:
+    return concepts.ResourceSpec(
+        'run.api.v1.namespaces',
+        namespacesId=NamespaceAttributeConfig(),
+        resource_name='namespace')
 
 
-class RegionWildcardFallthrough(deps.Fallthrough):
-  """Fall through to returning the wildcard '-' for region."""
-
-  def __init__(self):
-    super(RegionWildcardFallthrough, self).__init__(
-        function=None,
-        hint='wild card matching all regions')
-
-  def _Call(self, parsed_args):
-    return '-'
-
-
-def RegionWithWildcardAttributeConfig():
-  return concepts.ResourceParameterAttributeConfig(
-      name='region',
-      help_text='Region for the {resource}.',
-      fallthroughs=[RegionWildcardFallthrough()])
-
-
-def GetOnePlatformLocationResourceSpec():
-  return concepts.ResourceSpec(
-      'run.projects.locations',
-      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
-      locationsId=RegionWithWildcardAttributeConfig(),
-      resource_name='location')
-
-CLOUD_RUN_LOCATION_PRESENTATION = presentation_specs.ResourcePresentationSpec(
-    '--region',
-    GetOnePlatformLocationResourceSpec(),
-    'Location to list or \'-\' for all locations.',
-    required=False,
-    prefixes=False)
-
 CLUSTER_PRESENTATION = presentation_specs.ResourcePresentationSpec(
     '--cluster',
     GetClusterResourceSpec(),
