@@ -22,25 +22,26 @@ from __future__ import unicode_literals
 import contextlib
 import datetime
 import functools
-import random
-import string
+
+from apitools.base.py import encoding
 from apitools.base.py import exceptions as api_exceptions
+from apitools.base.py import list_pager
+from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
 from googlecloudsdk.api_lib.run import condition as run_condition
 from googlecloudsdk.api_lib.run import configuration
 from googlecloudsdk.api_lib.run import domain_mapping
 from googlecloudsdk.api_lib.run import global_methods
-from googlecloudsdk.api_lib.run import k8s_object
 from googlecloudsdk.api_lib.run import metric_names
 from googlecloudsdk.api_lib.run import revision
 from googlecloudsdk.api_lib.run import route
 from googlecloudsdk.api_lib.run import service
 from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.api_lib.util import apis_internal
-from googlecloudsdk.api_lib.util import exceptions as exceptions_util
 from googlecloudsdk.api_lib.util import waiter
 from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.command_lib.run import config_changes as config_changes_mod
 from googlecloudsdk.command_lib.run import exceptions as serverless_exceptions
+from googlecloudsdk.command_lib.run import name_generator
 from googlecloudsdk.command_lib.run import resource_name_conversion
 from googlecloudsdk.command_lib.run import stages
 from googlecloudsdk.core import config
@@ -56,8 +57,6 @@ import six
 DEFAULT_ENDPOINT_VERSION = 'v1'
 
 
-_NONCE_LENGTH = 10
-
 # Wait 11 mins for each deployment. This is longer than the server timeout,
 # making it more likely to get a useful error message from the server.
 MAX_WAIT_MS = 660000
@@ -183,16 +182,11 @@ class ConditionPoller(waiter.OperationPoller):
     self._dependencies = {k: set() for k in tracker}
     if dependencies is not None:
       for k in dependencies:
-        error_msg = 'Condition {} is not in the dependency tracker.'
-        if k not in tracker:
-          raise ValueError(error_msg.format(k))
-        for c in dependencies[k]:
-          if c not in tracker:
-            raise ValueError(error_msg.format(c))
-        # Add dependencies, only if they're still not complete
+        # Add dependencies, only if they're still not complete. If a stage isn't
+        # in the tracker. consider it "already complete".
         self._dependencies[k] = {
-            c for c in dependencies[k] if not tracker.IsComplete(c)
-        }
+            c for c in dependencies[k]
+            if c in tracker and not tracker.IsComplete(c)}
     self._resource_getter = resource_getter
     self._tracker = tracker
     self._resource_fail_type = exceptions.Error
@@ -274,8 +268,7 @@ class ConditionPoller(waiter.OperationPoller):
       conditions_message: str, The message from the conditions object we're
         displaying..
     """
-
-    if self._tracker.IsComplete(condition):
+    if condition not in self._tracker or self._tracker.IsComplete(condition):
       return
 
     if self._IsBlocked(condition):
@@ -306,7 +299,7 @@ class ConditionPoller(waiter.OperationPoller):
     Returns:
       bool: True if stage was completed, False if no action taken
     """
-    if self._tracker.IsComplete(condition):
+    if condition not in self._tracker or self._tracker.IsComplete(condition):
       return False
     # A blocked condition is likely to remain True (indicating the previous
     # operation concerning it was successful) until the blocking condition(s)
@@ -329,6 +322,8 @@ class ConditionPoller(waiter.OperationPoller):
     # The set of stages that aren't marked started and don't have unsatisfied
     # dependencies are newly unblocked.
     for c in self._dependencies:
+      if c not in self._tracker:
+        continue
       if self._tracker.IsWaiting(c) and not self._IsBlocked(c):
         self._tracker.StartStage(c)
     # TODO(b/120679874): Should not have to manually call Tick()
@@ -345,7 +340,7 @@ class ConditionPoller(waiter.OperationPoller):
       DeploymentFailedError: If the 'Ready' condition failed.
     """
     # Don't fail an already failed stage.
-    if self._tracker.IsComplete(condition):
+    if condition not in self._tracker or self._tracker.IsComplete(condition):
       return
 
     self._tracker.FailStage(
@@ -414,21 +409,18 @@ class ServiceConditionPoller(ConditionPoller):
     return datetime.datetime.now() - self._start_time > self._five_seconds
 
 
-def _Nonce():
-  """Return a random string with unlikely collision to use as a nonce."""
-  return ''.join(
-      random.choice(string.ascii_lowercase) for _ in range(_NONCE_LENGTH))
-
-
 class _NewRevisionForcingChange(config_changes_mod.ConfigChanger):
-  """Forces a new revision to get created by posting a random nonce label."""
+  """Forces a new revision to get created by changing the revision name."""
 
-  def __init__(self, nonce):
-    self._nonce = nonce
+  def __init__(self, revision_suffix):
+    self._revision_name_change = config_changes_mod.RevisionNameChanges(
+        revision_suffix)
 
   def Adjust(self, resource):
-    resource.template.labels[revision.NONCE_LABEL] = self._nonce
-    return resource
+    """Adjust by revision name."""
+    if revision.NONCE_LABEL in resource.template.labels:
+      del resource.template.labels[revision.NONCE_LABEL]
+    return self._revision_name_change.Adjust(resource)
 
 
 def _IsDigest(url):
@@ -436,6 +428,24 @@ def _IsDigest(url):
   return '@sha256:' in url
 
 
+class RevisionNameBasedPoller(waiter.OperationPoller):
+  """Poll for the revision with the given name to exist."""
+
+  def __init__(self, operations, revision_ref_getter):
+    self._operations = operations
+    self._revision_ref_getter = revision_ref_getter
+
+  def IsDone(self, revision_obj):
+    return bool(revision_obj)
+
+  def Poll(self, revision_name):
+    revision_ref = self._revision_ref_getter(revision_name)
+    return self._operations.GetRevision(revision_ref)
+
+  def GetResult(self, revision_obj):
+    return revision_obj
+
+
 class NonceBasedRevisionPoller(waiter.OperationPoller):
   """To poll for exactly one revision with the given nonce to appear."""
 
@@ -468,12 +478,15 @@ class _SwitchToDigestChange(config_changes_mod.ConfigChanger):
       return resource
 
     # Mutates through to metadata: Save the by-tag user intent.
-    resource.annotations[configuration.USER_IMAGE_ANNOTATION] = (
+    resource.annotations[revision.USER_IMAGE_ANNOTATION] = (
+        self._base_revision.image)
+    resource.template.annotations[revision.USER_IMAGE_ANNOTATION] = (
         self._base_revision.image)
     resource.template.image = self._base_revision.image_digest
     return resource
 
 _CLIENT_NAME_ANNOTATION = 'run.googleapis.com/client-name'
+_CLIENT_NAME = 'gcloud'
 _CLIENT_VERSION_ANNOTATION = 'run.googleapis.com/client-version'
 
 
@@ -481,10 +494,11 @@ class _SetClientNameAndVersion(config_changes_mod.ConfigChanger):
   """Sets the client name and version annotations."""
 
   def Adjust(self, resource):
-    annotations = k8s_object.AnnotationsFromMetadata(resource.MessagesModule(),
-                                                     resource.metadata)
-    annotations[_CLIENT_NAME_ANNOTATION] = 'gcloud'
-    annotations[_CLIENT_VERSION_ANNOTATION] = config.CLOUD_SDK_VERSION
+    resource.annotations[_CLIENT_NAME_ANNOTATION] = _CLIENT_NAME
+    resource.template.annotations[_CLIENT_NAME_ANNOTATION] = _CLIENT_NAME
+    resource.annotations[_CLIENT_VERSION_ANNOTATION] = config.CLOUD_SDK_VERSION
+    resource.template.annotations[
+        _CLIENT_VERSION_ANNOTATION] = config.CLOUD_SDK_VERSION
     return resource
 
 
@@ -502,7 +516,7 @@ class ServerlessOperations(object):
       region: str, The region of the control plane if operating against
         hosted Cloud Run, else None.
       op_client: The API client for interacting with One Platform APIs. Or
-        None if interacting with Cloud Run on GKE.
+        None if interacting with Cloud Run for Anthos.
     """
     self._client = client
     self._registry = resources.REGISTRY.Clone()
@@ -755,8 +769,9 @@ class ServerlessOperations(object):
     we get the digest for the image to run.
 
     Getting this revision:
+      * If there's a name in the template metadata, use that
       * If there's a nonce in the revisonTemplate metadata, use that
-      * If that query produces >1 or produces 0 after a short timeout, use
+      * If that query produces >1 or 0 after a short timeout, use
         the latestCreatedRevision in status.
 
     Arguments:
@@ -767,23 +782,47 @@ class ServerlessOperations(object):
         level object.
 
     Returns:
-      The base revision of the configuration.
+      The base revision of the configuration or None if not found by revision
+        name nor nonce and latestCreatedRevisionName does not exist on the
+        Service object.
     """
-    # Or returns None if not available by nonce & the control plane has not
-    # implemented latestCreatedRevisionName on the Service object yet.
-    base_revision_nonce = template.labels.get(revision.NONCE_LABEL, None)
     base_revision = None
-    if base_revision_nonce:
+    # Try to find by revision name
+    base_revision_name = template.name
+    if base_revision_name:
       try:
-        namespace_ref = self._registry.Parse(
-            metadata.namespace,
-            collection='run.namespaces')
-        poller = NonceBasedRevisionPoller(self, namespace_ref)
-        base_revision = poller.GetResult(waiter.PollUntilDone(
-            poller, base_revision_nonce,
-            sleep_ms=500, max_wait_ms=2000))
-      except retry.WaitException:
+        revision_ref_getter = functools.partial(
+            self._registry.Parse,
+            params={'namespacesId': metadata.namespace},
+            collection='run.namespaces.revisions')
+        poller = RevisionNameBasedPoller(self, revision_ref_getter)
+        base_revision = poller.GetResult(
+            waiter.PollUntilDone(
+                poller, base_revision_name, sleep_ms=500, max_wait_ms=2000))
+      except retry.RetryException:
         pass
+    # Name polling didn't work. Fall back to nonce polling
+    if not base_revision:
+      base_revision_nonce = template.labels.get(revision.NONCE_LABEL, None)
+      if base_revision_nonce:
+        try:
+          # TODO(b/150322097): Remove this when the api has been split.
+          # This try/except block is needed because the v1alpha1 and v1 run apis
+          # have different collection names for the namespaces.
+          try:
+            namespace_ref = self._registry.Parse(
+                metadata.namespace,
+                collection='run.namespaces')
+          except resources.InvalidCollectionException:
+            namespace_ref = self._registry.Parse(
+                metadata.namespace,
+                collection='run.api.v1.namespaces')
+          poller = NonceBasedRevisionPoller(self, namespace_ref)
+          base_revision = poller.GetResult(waiter.PollUntilDone(
+              poller, base_revision_nonce,
+              sleep_ms=500, max_wait_ms=2000))
+        except retry.RetryException:
+          pass
     # Nonce polling didn't work, because some client didn't post one or didn't
     # change one. Fall back to the (slightly racy) `latestCreatedRevisionName`.
     if not base_revision:
@@ -821,18 +860,8 @@ class ServerlessOperations(object):
       The Service object we created or modified.
     """
     messages = self.messages_module
-    config_changes = [_SetClientNameAndVersion()] + config_changes
     try:
       if serv:
-        if not with_code:
-          # Avoid changing the running code by making the new revision by digest
-          self._EnsureImageDigest(serv, config_changes)
-
-        # Revision names must be unique across the namespace.
-        # To prevent the revision name being unchanged from the last revision,
-        # we reset the value so the default naming scheme will be used instead.
-        serv.template.name = None
-
         # PUT the changed Service
         for config_change in config_changes:
           serv = config_change.Adjust(serv)
@@ -865,15 +894,7 @@ class ServerlessOperations(object):
               serv_create_req)
         return service.Service(raw_service, messages)
     except api_exceptions.HttpBadRequestError as e:
-      error_payload = exceptions_util.HttpErrorPayload(e)
-      if error_payload.field_violations:
-        if (serverless_exceptions.BadImageError.IMAGE_ERROR_FIELD
-            in error_payload.field_violations):
-          exceptions.reraise(serverless_exceptions.BadImageError(e))
-        elif (serverless_exceptions.MalformedLabelError.LABEL_ERROR_FIELD
-              in error_payload.field_violations):
-          exceptions.reraise(serverless_exceptions.MalformedLabelError(e))
-      exceptions.reraise(e)
+      exceptions.reraise(serverless_exceptions.HttpError(e))
     except api_exceptions.HttpNotFoundError as e:
       platform = properties.VALUES.run.platform.Get()
       error_msg = 'Deployment endpoint was not found.'
@@ -912,8 +933,7 @@ class ServerlessOperations(object):
       raise serverless_exceptions.KubernetesError('Error{}:\n{}\n'.format(
           's' if len(k8s_error.causes) > 1 else '', causes))
 
-  def UpdateTraffic(
-      self, service_ref, config_changes, tracker, asyn, is_managed):
+  def UpdateTraffic(self, service_ref, config_changes, tracker, asyn):
     """Update traffic splits for service."""
     if tracker is None:
       tracker = progress_tracker.NoOpStagedProgressTracker(
@@ -925,29 +945,42 @@ class ServerlessOperations(object):
       raise serverless_exceptions.ServiceNotFoundError(
           'Service [{}] could not be found.'.format(service_ref.servicesId))
 
-    if not serv.spec.template:
-      if is_managed:
-        raise serverless_exceptions.UnsupportedOperationError(
-            'Your provider does not support updating traffic for this service.')
-      else:
-        raise serverless_exceptions.UnsupportedOperationError(
-            'You must upgrade your cluster to version 0.61 or greater '
-            'to update traffic.')
+    if serv.configuration:
+      raise serverless_exceptions.UnsupportedOperationError(
+          'This service is using an old version of Cloud Run for Anthos '
+          'that does not support traffic features. Please upgrade to 0.8 '
+          'or later.')
+
     self._UpdateOrCreateService(service_ref, config_changes, False, serv)
 
     if not asyn:
       getter = functools.partial(self.GetService, service_ref)
       self.WaitForCondition(ServiceConditionPoller(getter, tracker, serv=serv))
 
-  def ReleaseService(self, service_ref, config_changes, tracker=None,
-                     asyn=False, allow_unauthenticated=None, for_replace=False):
+  def _AddRevisionForcingChange(self, serv, config_changes):
+    """Get a new revision forcing config change for the given service."""
+    curr_generation = serv.generation if serv is not None else 0
+    revision_suffix = '{}-{}'.format(
+        str(curr_generation + 1).zfill(5), name_generator.GenerateName())
+    config_changes.insert(0, _NewRevisionForcingChange(revision_suffix))
+
+  def ReleaseService(self,
+                     service_ref,
+                     config_changes,
+                     tracker=None,
+                     asyn=False,
+                     allow_unauthenticated=None,
+                     for_replace=False,
+                     prefetch=False,
+                     build_op_ref=None,
+                     build_log_url=None):
     """Change the given service in prod using the given config_changes.
 
     Ensures a new revision is always created, even if the spec of the revision
     has not changed.
 
     Arguments:
-      service_ref: Resource, the service to release
+      service_ref: Resource, the service to release.
       config_changes: list, objects that implement Adjust().
       tracker: StagedProgressTracker, to report on the progress of releasing.
       asyn: bool, if True, return without waiting for the service to be updated.
@@ -957,18 +990,51 @@ class ServerlessOperations(object):
         unauthenticated access from a service.
       for_replace: bool, If the change is for a replacing the service from a
         YAML specification.
+      prefetch: the service, pre-fetched for ReleaseService. `False` indicates
+        the caller did not perform a prefetch; `None` indicates a nonexistant
+        service.
+      build_op_ref: The reference to the build.
+      build_log_url: The log url of the build result.
     """
     if tracker is None:
       tracker = progress_tracker.NoOpStagedProgressTracker(
           stages.ServiceStages(allow_unauthenticated is not None),
           interruptable=True, aborted_message='aborted')
+    if build_op_ref is not None:
+      tracker.StartStage(stages.BUILD_READY)
+      tracker.UpdateHeaderMessage('Building Container.')
+      tracker.UpdateStage(
+          stages.BUILD_READY, 'Logs are available at [{build_log_url}].'.format(
+              build_log_url=build_log_url))
+      client = cloudbuild_util.GetClientInstance()
+      poller = waiter.CloudOperationPoller(client.projects_builds,
+                                           client.operations)
+      operation = waiter.PollUntilDone(poller, build_op_ref)
+      response_dict = encoding.MessageToPyValue(operation.response)
+      if response_dict and response_dict['status'] != 'SUCCESS':
+        tracker.FailStage(
+            stages.BUILD_READY, None,
+            message='Container build failed and '
+            'logs are available at [{build_log_url}].'.format(
+                build_log_url=build_log_url))
+        return
+      else:
+        tracker.CompleteStage(stages.BUILD_READY)
+    if prefetch is None:
+      serv = None
+    else:
+      serv = prefetch or self.GetService(service_ref)
     if for_replace:
       with_image = True
     else:
       with_image = any(
           isinstance(c, config_changes_mod.ImageChange) for c in config_changes)
-      config_changes = [_NewRevisionForcingChange(_Nonce())] + config_changes
-    serv = self.GetService(service_ref)
+      self._AddRevisionForcingChange(serv, config_changes)
+      if serv and not with_image:
+        # Avoid changing the running code by making the new revision by digest
+        self._EnsureImageDigest(serv, config_changes)
+    config_changes = [_SetClientNameAndVersion()] + config_changes
+
     self._UpdateOrCreateService(
         service_ref, config_changes, with_image, serv)
 
@@ -1002,7 +1068,8 @@ class ServerlessOperations(object):
       for msg in run_condition.GetNonTerminalMessages(poller.GetConditions()):
         tracker.AddWarning(msg)
 
-  def ListRevisions(self, namespace_ref, service_name):
+  def ListRevisions(self, namespace_ref, service_name,
+                    limit=None, page_size=100):
     """List all revisions for the given service.
 
     Revision list gets sorted by service name and creation timestamp.
@@ -1010,11 +1077,17 @@ class ServerlessOperations(object):
     Args:
       namespace_ref: Resource, namespace to list revisions in
       service_name: str, The service for which to list revisions.
+      limit: Optional[int], max number of revisions to list.
+      page_size: Optional[int], number of revisions to fetch at a time
 
-    Returns:
-      A list of revisions for the given service.
+    Yields:
+      Revisions for the given surface
     """
     messages = self.messages_module
+    # NB: This is a hack to compensate for apitools not generating this line.
+    #     It's necessary to make the URL parameter be "continue".
+    encoding.AddCustomJsonFieldMapping(
+        messages.RunNamespacesRevisionsListRequest, 'continue_', 'continue')
     request = messages.RunNamespacesRevisionsListRequest(
         parent=namespace_ref.RelativeName(),
     )
@@ -1023,15 +1096,15 @@ class ServerlessOperations(object):
       # 'service-less' operation.
       request.labelSelector = 'serving.knative.dev/service = {}'.format(
           service_name)
-    with metrics.RecordDuration(metric_names.LIST_REVISIONS):
-      response = self._client.namespaces_revisions.List(request)
-
-    # Server does not sort the response so we'll need to sort client-side
-    revisions = [revision.Revision(item, messages) for item in response.items]
-    # Newest first
-    revisions.sort(key=lambda r: r.creation_timestamp, reverse=True)
-    revisions.sort(key=lambda r: r.service_name)
-    return revisions
+    for result in list_pager.YieldFromList(
+        service=self._client.namespaces_revisions,
+        request=request,
+        limit=limit,
+        batch_size=page_size,
+        current_token_attribute='continue_',
+        next_token_attribute=('metadata', 'continue_'),
+        batch_size_attribute='limit'):
+      yield revision.Revision(result, messages)
 
   def ListDomainMappings(self, namespace_ref):
     """List all domain mappings.
@@ -1050,12 +1123,16 @@ class ServerlessOperations(object):
     return [domain_mapping.DomainMapping(item, messages)
             for item in response.items]
 
-  def CreateDomainMapping(self, domain_mapping_ref, service_name):
+  def CreateDomainMapping(self,
+                          domain_mapping_ref,
+                          service_name,
+                          force_override=False):
     """Create a domain mapping.
 
     Args:
       domain_mapping_ref: Resource, domainmapping resource.
       service_name: str, the service to which to map domain.
+      force_override: bool, override an existing mapping of this domain.
 
     Returns:
       A domain_mapping.DomainMapping object.
@@ -1066,6 +1143,7 @@ class ServerlessOperations(object):
         self._client, domain_mapping_ref.namespacesId)
     new_mapping.name = domain_mapping_ref.domainmappingsId
     new_mapping.route_name = service_name
+    new_mapping.force_override = force_override
 
     request = messages.RunNamespacesDomainmappingsCreateRequest(
         domainMapping=new_mapping.Message(),
