@@ -165,11 +165,12 @@ def DisplayQueryAggregateStats(query_stats, out):
       'total_elapsed_time': _ConvertToStringValue(get_prop('elapsed_time')),
       'cpu_time': _ConvertToStringValue(get_prop('cpu_time')),
       'rows_returned': _ConvertToStringValue(get_prop('rows_returned')),
-      'rows_scanned': _ConvertToStringValue(get_prop('rows_scanned'))
+      'rows_scanned': _ConvertToStringValue(get_prop('rows_scanned')),
+      'optimizer_version': _ConvertToStringValue(get_prop('optimizer_version')),
   }
   resource_printer.Print(
       stats,
-      'table[box](total_elapsed_time, cpu_time, rows_returned, rows_scanned)',
+      'table[box](total_elapsed_time, cpu_time, rows_returned, rows_scanned, optimizer_version)',
       out=out)
 
 
