@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,11 +32,13 @@ from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.compute import utils as compute_utils
 from googlecloudsdk.api_lib.storage import storage_util
+from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.util import completers
 
+messages = apis.GetMessagesModule('sql', 'v1beta4')
 
 _IP_ADDRESS_PART = r'(25[0-5]|2[0-4][0-9]|1?[0-9]{1,2})'  # Match decimal 0-255
 _CIDR_PREFIX_PART = r'([0-9]|[1-2][0-9]|3[0-2])'  # Match decimal 0-32
@@ -88,9 +90,7 @@ def AddInstance(parser):
 def AddInstanceArgument(parser):
   """Add the 'instance' argument to the parser."""
   parser.add_argument(
-      'instance',
-      completer=InstanceCompleter,
-      help='Cloud SQL instance ID.')
+      'instance', completer=InstanceCompleter, help='Cloud SQL instance ID.')
 
 
 # Currently, 10230 is the max storage size one can set, and 10 is the minimum.
@@ -105,36 +105,21 @@ def AddInstanceResizeLimit(parser):
 
 def AddUsername(parser):
   parser.add_argument(
-      'username',
-      completer=UserCompleter,
-      help='Cloud SQL username.')
+      'username', completer=UserCompleter, help='Cloud SQL username.')
 
 
 def AddHost(parser):
-  """Add the 'deprecated_host' arg and '--host' flag to the parser."""
-  help_text = ('Cloud SQL user\'s host name expressed as a specific IP address'
-               ' or address range. `%` denotes an unrestricted host name. '
-               'Applicable flag for MySQL instances; ignored for PostgreSQL '
-               'instances. Note, if you connect to your instance using IP '
-               'addresses, you must add your client IP address as an Authorized'
-               ' Address, even if your host name is unrestricted. For help on '
-               'how to do so, read: '
-               'https://cloud.google.com/sql/docs/mysql/configure-ip')
-
-  host = parser.add_argument_group(mutex=True)
-  # NOTICE: is_required = False is a short-term solution -- don't use elsewhere.
-  host.add_argument(
-      'deprecated_host',
-      help=help_text,
-      action=actions.DeprecationAction(
-          'deprecated_host',
-          removed=True,
-          error='Positional argument {flag_name} has been removed. Use '
-                '--host instead.'),
-      hidden=True).is_required = False
-  host.add_argument(
+  """Add the '--host' flag to the parser."""
+  parser.add_argument(
       '--host',
-      help=help_text)
+      help=('Cloud SQL user\'s host name expressed as a specific IP address'
+            ' or address range. `%` denotes an unrestricted host name. '
+            'Applicable flag for MySQL instances; ignored for all other '
+            'engines. Note, if you connect to your instance using IP '
+            'addresses, you must add your client IP address as an Authorized'
+            ' Address, even if your host name is unrestricted. For help on '
+            'how to do so, read: '
+            'https://cloud.google.com/sql/docs/mysql/configure-ip'))
 
 
 def AddAvailabilityType(parser):
@@ -148,15 +133,20 @@ def AddAvailabilityType(parser):
                       'to another zone within your selected region.',
           'zonal': 'Provides no failover capability. This is the default.'
       },
-      help_str=('Specifies level of availability. Only applies to PostgreSQL '
-                'instances.'))
+      help_str=('Specifies level of availability.'))
   availabilty_type_flag.AddToParser(parser)
 
 
 def AddPassword(parser):
+  parser.add_argument('--password', help='Cloud SQL user\'s password.')
+
+
+def AddRootPassword(parser):
+  """Add the root password field to the parser."""
   parser.add_argument(
-      '--password',
-      help='Cloud SQL user\'s password.')
+      '--root-password',
+      required=False,
+      help='Root Cloud SQL user\'s password.')
 
 
 def AddPromptForPassword(parser):
@@ -168,6 +158,14 @@ def AddPromptForPassword(parser):
             'including the RETURN or ENTER key.'))
 
 
+def AddType(parser):
+  parser.add_argument('--type', help='Cloud SQL user\'s type. It determines '
+                      'the method to authenticate the user during login. '
+                      'See the list of user types at '
+                      'https://cloud.google.com/sql/docs/postgres/admin-api/'
+                      'v1beta4/users#type')
+
+
 # Instance create and patch flags
 
 
@@ -181,18 +179,17 @@ def AddActivationPolicy(parser):
                 'the instance should be activated and is applicable only when '
                 'the instance state is `RUNNABLE`. The default is `on-demand`. '
                 'More information on activation policies can be found here: '
-                'https://cloud.google.com/sql/faq#activation_policy')
-  ).AddToParser(parser)
+                'https://cloud.google.com/sql/faq#activation_policy'
+               )).AddToParser(parser)
 
 
-def AddAssignIp(parser, show_negated_in_help=False):
-  kwargs = _GetKwargsForBoolFlag(show_negated_in_help)
+def AddAssignIp(parser):
   parser.add_argument(
       '--assign-ip',
-      help='Assign an IPv4 external address to this instance.  This setting is '
-           'enabled by default.  To create an instance which only has a '
-           'private IP, use --no-assign-ip and specify a private network.',
-      **kwargs)
+      help='Assign an IPv4 external address to this instance. This setting is '
+      'enabled by default when creating a new instance, but can be '
+      'disabled to use private IP connectivity.',
+      action=arg_parsers.StoreTrueFalseAction)
 
 
 def AddAuthorizedGAEApps(parser, update=False):
@@ -239,6 +236,15 @@ def AddBackupStartTime(parser):
             'format - HH:MM, in the UTC timezone.'))
 
 
+def AddBackupLocation(parser, allow_empty):
+  help_text = (
+      'Choose where to store your backups. Backups are stored in the closest '
+      'multi-region location to you by default. Only customize if needed.')
+  if allow_empty:
+    help_text += ' Specify empty string to revert to default.'
+  parser.add_argument('--backup-location', required=False, help=help_text)
+
+
 def AddDatabaseFlags(parser, update=False):
   """Adds the `--database-flags` flag."""
   help_ = ('Comma-separated list of database flags to set on the '
@@ -261,14 +267,24 @@ def AddDatabaseFlags(parser, update=False):
 
 def AddDatabaseVersion(parser, restrict_choices=True):
   """Adds `--database-version` to the parser with choices restricted or not."""
-  choices = ['MYSQL_5_5', 'MYSQL_5_6', 'MYSQL_5_7', 'POSTGRES_9_6']
+  choices = [
+      'MYSQL_5_5',
+      'MYSQL_5_6',
+      'MYSQL_5_7',
+      'POSTGRES_9_6',
+      'POSTGRES_10',
+      'POSTGRES_11',
+      'SQLSERVER_2017_EXPRESS',
+      'SQLSERVER_2017_WEB',
+      'SQLSERVER_2017_STANDARD',
+      'SQLSERVER_2017_ENTERPRISE',
+  ]
   help_text = (
       'The database engine type and version. If left unspecified, the API '
       'defaults will be used.')
   if not restrict_choices:
     help_text = ' '.join([
-        help_text,
-        'See the list of database versions at '
+        help_text, 'See the list of database versions at '
         'https://cloud.google.com/sql/docs/mysql/admin-api/v1beta4/instances'
         '#databaseVersion'
     ])
@@ -296,10 +312,7 @@ def _GetKwargsForBoolFlag(show_negated_in_help):
         'action': arg_parsers.StoreTrueFalseAction,
     }
   else:
-    return {
-        'action': 'store_true',
-        'default': None
-    }
+    return {'action': 'store_true', 'default': None}
 
 
 def AddEnableBinLog(parser, show_negated_in_help=False):
@@ -307,9 +320,19 @@ def AddEnableBinLog(parser, show_negated_in_help=False):
   parser.add_argument(
       '--enable-bin-log',
       required=False,
-      help=(
-          'Specified if binary log should be enabled. If backup '
-          'configuration is disabled, binary log must be disabled as well.'),
+      help=('Specified if binary log should be enabled. If backup '
+            'configuration is disabled, binary log must be disabled as well.'),
+      **kwargs)
+
+
+def AddEnablePointInTimeRecovery(parser, show_negated_in_help=False):
+  kwargs = _GetKwargsForBoolFlag(show_negated_in_help)
+  parser.add_argument(
+      '--enable-point-in-time-recovery',
+      required=False,
+      help=('Specified if point-in-time recovery (using write-ahead log '
+            'archiving) should be enabled. If backup configuration is '
+            'disabled, point-in-time recovery must be disabled as well.'),
       **kwargs)
 
 
@@ -414,10 +437,9 @@ def AddMaintenanceReleaseChannel(parser):
                      'their compatibility with your application prior '
                      'to the production release.'
       },
-
       help_str=("Which channel's updates to apply during the maintenance "
-                "window. If not specified, Cloud SQL chooses the timing of "
-                "updates to your instance.")).AddToParser(parser)
+                'window. If not specified, Cloud SQL chooses the timing of '
+                'updates to your instance.')).AddToParser(parser)
 
 
 def AddMaintenanceWindowDay(parser):
@@ -490,11 +512,12 @@ def AddStorageSize(parser):
       '--storage-size',
       type=arg_parsers.BinarySize(
           lower_bound='10GB',
-          upper_bound='10230GB',
+          upper_bound='30720GB',
           suggested_binary_size_scales=['GB']),
       help=('Amount of storage allocated to the instance. Must be an integer '
-            'number of GB between 10GB and 10230GB inclusive. The default is '
-            '10GB.'))
+            'number of GB. The default is 10GB. Information on storage '
+            'limits can be found here: '
+            'https://cloud.google.com/sql/docs/quotas#storage_limits'))
 
 
 def AddTier(parser, is_patch=False):
@@ -506,11 +529,6 @@ def AddTier(parser, is_patch=False):
                'available here: https://cloud.google.com/sql/pricing.')
   if is_patch:
     help_text += ' WARNING: Instance will be restarted.'
-  # TODO(b/122660263): Remove when V1 instances are no longer supported.
-  # V1 deprecation notice.
-  help_text += ('\n\nIMPORTANT: First Generation instances are deprecated. '
-                'If you\'re considering any First Generation tiers, we '
-                'recommend using Second Generation instead.')
 
   parser.add_argument('--tier', '-t', required=False, help=help_text)
 
@@ -527,10 +545,7 @@ def AddZone(parser, help_text):
           warn=('Flag `{flag_name}` is deprecated and will be removed by '
                 'release 255.0.0. Use `--zone` instead.')),
       help=help_text)
-  zone_group.add_argument(
-      '--zone',
-      required=False,
-      help=help_text)
+  zone_group.add_argument('--zone', required=False, help=help_text)
 
 
 # Database specific flags
@@ -538,9 +553,7 @@ def AddZone(parser, help_text):
 
 def AddDatabaseName(parser):
   parser.add_argument(
-      'database',
-      completer=DatabaseCompleter,
-      help='Cloud SQL database name.')
+      'database', completer=DatabaseCompleter, help='Cloud SQL database name.')
 
 
 def AddCharset(parser):
@@ -556,7 +569,9 @@ def AddCollation(parser):
       '--collation',
       help='Cloud SQL database collation setting, which specifies '
       'the set of rules for comparing characters in a character set. Each'
-      ' database version may support a different set of collations.')
+      ' database version may support a different set of collations. For '
+      'PostgreSQL database versions, this may only be set to the collation of '
+      'the template database.')
 
 
 def AddOperationArgument(parser):
@@ -571,35 +586,140 @@ def AddOperationArgument(parser):
 
 def AddUriArgument(parser, help_text):
   """Add the 'uri' argument to the parser, with help text help_text."""
-  parser.add_argument(
-      'uri',
-      help=help_text)
+  parser.add_argument('uri', help=help_text)
+
+
+DEFAULT_DATABASE_IMPORT_HELP_TEXT = (
+    'Database to which the import is made. If not set, it is assumed that '
+    'the database is specified in the file to be imported. If your SQL '
+    'dump file includes a database statement, it will override the '
+    'database set in this flag.')
+
+SQLSERVER_DATABASE_IMPORT_HELP_TEXT = (
+    'A new database into which the import is made.')
 
 
 def AddDatabase(parser, help_text, required=False):
-  """Add the '--database' flag to the parser, with help text help_text."""
-  parser.add_argument(
-      '--database',
-      '-d',
-      required=required,
-      help=help_text)
+  """Add the '--database' and '-d' flags to the parser.
+
+  Args:
+    parser: The current argparse parser to add these database flags to.
+    help_text: String, specifies the help text for the database flags.
+    required: Boolean, specifies whether the database flag is required.
+  """
+  parser.add_argument('--database', '-d', required=required, help=help_text)
 
 
-def AddDatabaseList(parser, help_text):
-  """Add the '--database' list flag to the parser, with help text help_text."""
-  parser.add_argument(
-      '--database',
-      '-d',
-      type=arg_parsers.ArgList(min_length=1),
-      metavar='DATABASE',
-      required=False,
-      help=help_text)
+DEFAULT_DATABASE_LIST_EXPORT_HELP_TEXT = (
+    'Database(s) from which the export is made. Information on requirements '
+    'can be found here: https://cloud.google.com/sql/docs/mysql/admin-api/'
+    'v1beta4/instances/export#exportContext.databases')
+
+SQLSERVER_DATABASE_LIST_EXPORT_HELP_TEXT = (
+    'Database from which the export is made. Information on requirements '
+    'can be found here: https://cloud.google.com/sql/docs/sqlserver/admin-api/'
+    'v1beta4/instances/export#exportContext.databases')
+
+
+def AddDatabaseList(parser, help_text, required=False):
+  """Add the '--database' and '-d' list flags to the parser.
+
+  Args:
+    parser: The current argparse parser to add these database flags to.
+    help_text: String, specifies the help text for the database flags.
+    required: Boolean, specifies whether the database flag is required.
+  """
+  if required:
+    group = parser.add_group(mutex=False, required=True)
+    group.add_argument(
+        '--database',
+        '-d',
+        type=arg_parsers.ArgList(min_length=1),
+        metavar='DATABASE',
+        help=help_text)
+  else:
+    parser.add_argument(
+        '--database',
+        '-d',
+        type=arg_parsers.ArgList(min_length=1),
+        metavar='DATABASE',
+        required=False,
+        help=help_text)
 
 
 def AddUser(parser, help_text):
   """Add the '--user' flag to the parser, with help text help_text."""
   parser.add_argument('--user', help=help_text)
 
+
+def AddEncryptedBakFlags(parser):
+  """Add the flags for importing encrypted BAK files.
+
+  Add the --cert-path, --pvk-path, --pvk-password and
+  --prompt-for-pvk-password flags to the parser
+
+  Args:
+    parser: The current argparse parser to add these database flags to.
+  """
+  enc_group = parser.add_group(
+      mutex=False,
+      required=False,
+      help='Encryption info to support importing an encrypted .bak file')
+  enc_group.add_argument(
+      '--cert-path',
+      required=True,
+      help=('Path to the encryption certificate file in Google Cloud Storage '
+            'associated with the BAK file. The URI is in the form '
+            '`gs://bucketName/fileName`.'))
+  enc_group.add_argument(
+      '--pvk-path',
+      required=True,
+      help=('Path to the encryption private key file in Google Cloud Storage '
+            'associated with the BAK file. The URI is in the form '
+            '`gs://bucketName/fileName`.'))
+  password_group = enc_group.add_group(mutex=True, required=True)
+  password_group.add_argument(
+      '--pvk-password',
+      help='The private key password associated with the BAK file.')
+  password_group.add_argument(
+      '--prompt-for-pvk-password',
+      action='store_true',
+      help=(
+          'Prompt for the private key password associated with the BAK file '
+          'with character echo disabled. The password is all typed characters '
+          'up to but not including the RETURN or ENTER key.'))
+
+
+def AddRescheduleType(parser):
+  """Add the flag to specify reschedule type.
+
+  Args:
+    parser: The current argparse parser to add this to.
+  """
+  choices = [
+      messages.Reschedule.RescheduleTypeValueValuesEnum.IMMEDIATE.name,
+      messages.Reschedule.RescheduleTypeValueValuesEnum.NEXT_AVAILABLE_WINDOW
+      .name,
+      messages.Reschedule.RescheduleTypeValueValuesEnum.SPECIFIC_TIME.name,
+  ]
+  help_text = 'The type of reschedule operation to perform.'
+  parser.add_argument(
+      '--reschedule-type', choices=choices, required=True, help=help_text)
+
+
+def AddScheduleTime(parser):
+  """Add the flag for maintenance reschedule schedule time.
+
+  Args:
+    parser: The current argparse parser to add this to.
+  """
+  parser.add_argument(
+      '--schedule-time',
+      type=arg_parsers.Datetime.Parse,
+      help=('When specifying SPECIFIC_TIME, the date and time at which to '
+            'schedule the maintenance in ISO 8601 format.'))
+
+
 INSTANCES_USERLABELS_FORMAT = ':(settings.userLabels:alias=labels:label=LABELS)'
 
 INSTANCES_FORMAT_COLUMNS = [
@@ -608,8 +728,7 @@ INSTANCES_FORMAT_COLUMNS = [
     'ip_addresses.filter("type:PRIMARY").*extract(ip_address).flatten()'
     '.yesno(no="-"):label=PRIMARY_ADDRESS',
     'ip_addresses.filter("type:PRIVATE").*extract(ip_address).flatten()'
-    '.yesno(no="-"):label=PRIVATE_ADDRESS',
-    'state:label=STATUS'
+    '.yesno(no="-"):label=PRIVATE_ADDRESS', 'state:label=STATUS'
 ]
 
 
@@ -666,9 +785,17 @@ TIERS_FORMAT = """
   )
 """
 
-USERS_FORMAT_BETA = """
+USERS_FORMAT = """
   table(
     name.yesno(no='(anonymous)'),
     host
   )
 """
+
+USERS_FORMAT_ALPHA = """
+  table(
+    name.yesno(no='(anonymous)'),
+    host,
+    type.yesno(no='NATIVE')
+  )
+"""
