@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,10 +21,12 @@ from __future__ import unicode_literals
 
 from apitools.base.protorpclite import messages
 from googlecloudsdk.calliope import base
+from googlecloudsdk.calliope.concepts import concepts
 from googlecloudsdk.calliope.concepts import multitype
 from googlecloudsdk.command_lib.util.apis import arg_utils
-from googlecloudsdk.command_lib.util.apis import resource_arg_schema
+from googlecloudsdk.command_lib.util.apis import update
 from googlecloudsdk.command_lib.util.apis import yaml_command_schema
+from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
 from googlecloudsdk.command_lib.util.concepts import presentation_specs
 from googlecloudsdk.core import resources
@@ -37,6 +39,51 @@ class Error(Exception):
   pass
 
 
+def _GetLabelsClass(message, api_field):
+  return arg_utils.GetFieldFromMessage(message, api_field).type
+
+
+def _ParseLabelsIntoCreateMessage(message, args, api_field):
+  labels_cls = _GetLabelsClass(message, api_field)
+  labels_field = labels_util.ParseCreateArgs(args, labels_cls)
+  arg_utils.SetFieldInMessage(message, api_field, labels_field)
+
+
+def _AddLabelsToUpdateMask(static_field, update_mask_path):
+  if (update_mask_path not in static_field) or (
+      not static_field[update_mask_path]):
+    static_field[update_mask_path] = 'labels'
+    return
+
+  if 'labels' in static_field[update_mask_path].split(','):
+    return
+
+  static_field[
+      update_mask_path] = static_field[update_mask_path] + ',' + 'labels'
+
+
+def _RetrieveFieldValueFromMessage(message, api_field):
+  path = api_field.split('.')
+  for field_name in path:
+    try:
+      message = getattr(message, field_name)
+    except AttributeError:
+      raise AttributeError(
+          'The message does not have field specified in {}.'.format(api_field))
+  return message
+
+
+def _ParseLabelsIntoUpdateMessage(message, args, api_field):
+  existing_labels = _RetrieveFieldValueFromMessage(message, api_field)
+  diff = labels_util.Diff.FromUpdateArgs(args)
+  label_cls = _GetLabelsClass(message, api_field)
+  update_result = diff.Apply(label_cls, existing_labels)
+  if not update_result.needs_update:
+    return False
+  arg_utils.SetFieldInMessage(message, api_field, update_result.labels)
+  return True
+
+
 class DeclarativeArgumentGenerator(object):
   """An argument generator that operates off a declarative configuration.
 
@@ -76,6 +123,8 @@ class DeclarativeArgumentGenerator(object):
                     namespace,
                     static_fields=None,
                     resource_method_params=None,
+                    labels=None,
+                    command_type=None,
                     use_relative_name=True,
                     override_method=None,
                     parse_resource_into_request=True,
@@ -91,6 +140,8 @@ class DeclarativeArgumentGenerator(object):
       resource_method_params: {str: str}, A mapping of API method parameter name
         to resource ref attribute name when the API method uses non-standard
         names.
+      labels: The labels section of the command spec.
+      command_type: Type of the command, i.e. CREATE, UPDATE.
       use_relative_name: Use ref.RelativeName() if True otherwise ref.Name().
       override_method: APIMethod, The method other than self.method, this is
         used when the command has more than one API call.
@@ -112,6 +163,17 @@ class DeclarativeArgumentGenerator(object):
       message = arg_utils.ParseExistingMessageIntoMessage(
           message, existing_message, self.method)
 
+    # Add labels into message
+    if labels:
+      if command_type == yaml_command_schema.CommandType.CREATE:
+        _ParseLabelsIntoCreateMessage(message, namespace, labels.api_field)
+      elif command_type == yaml_command_schema.CommandType.UPDATE:
+        need_update = _ParseLabelsIntoUpdateMessage(message, namespace,
+                                                    labels.api_field)
+        if need_update:
+          update_mask_path = update.GetMaskFieldPath(override_method)
+          _AddLabelsToUpdateMask(static_fields, update_mask_path)
+
     # Insert static fields into message.
     arg_utils.ParseStaticFieldsIntoMessage(message, static_fields=static_fields)
 
@@ -160,6 +222,7 @@ class DeclarativeArgumentGenerator(object):
     return resources.REGISTRY.Parse(
         id_value,
         collection=self.method.collection.full_name,
+        api_version=self.method.collection.api_version,
         params=parent_ref.AsDict())
 
   def Limit(self, namespace):
@@ -184,8 +247,12 @@ class DeclarativeArgumentGenerator(object):
     else:
       flag_name = self.resource_arg.name or self.resource_spec.name
 
-    anchor_arg_is_flag = (
-        not self.resource_arg.is_positional or self.method.IsList())
+    # If left unspecified, decide whether the resource is positional based on
+    # the method.
+    if self.resource_arg.is_positional is None:
+      anchor_arg_is_flag = self.method.IsList()
+    else:
+      anchor_arg_is_flag = not self.resource_arg.is_positional
     anchor_arg_name = (
         '--' + flag_name if anchor_arg_is_flag
         else flag_name)
@@ -204,9 +271,11 @@ class DeclarativeArgumentGenerator(object):
     # attributes or for list commands (where everything should be a flag since
     # the parent resource collection is being used).
     anchor_arg_name = self._GetAnchorArgName()
-    no_gen = {n: '' for _, n
-              in six.iteritems(resource_arg_schema.IGNORED_FIELDS)
-              if n in self.resource_arg.attribute_names}
+    no_gen = {
+        n: ''
+        for _, n in six.iteritems(concepts.IGNORED_FIELDS)
+        if n in self.resource_arg.attribute_names
+    }
     no_gen.update({n: '' for n in self.resource_arg.removed_flags})
     command_level_fallthroughs = {}
     concept_parsers.UpdateFallthroughsMap(
@@ -507,6 +576,7 @@ class AutoArgumentGenerator(object):
     defaults.update(params)
     return resources.REGISTRY.Parse(
         r, collection=self.method.request_collection.full_name,
+        api_version=self.method.request_collection.api_version,
         params=defaults)
 
   def _GetArgName(self, field_name, field_help=None):
