@@ -26,9 +26,11 @@ from __future__ import division
 from __future__ import unicode_literals
 
 import json
+import sys
 
 from apitools.base.protorpclite import messages as apitools_messages
 from apitools.base.py import encoding
+from apitools.base.py import exceptions as apitools_exceptions
 from apitools.base.py.exceptions import HttpBadRequestError
 from googlecloudsdk.api_lib.util import waiter
 from googlecloudsdk.calliope import base
@@ -46,6 +48,7 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.resource import resource_transform
+from googlecloudsdk.core.util import files
 
 import six
 
@@ -99,17 +102,20 @@ class CommandBuilder(object):
   def __init__(self, spec, path):
     self.spec = spec
     self.path = path
-    self.method = registry.GetMethod(
-        self.spec.request.collection, self.spec.request.method,
-        self.spec.request.api_version)
+    self.ConfigureCommand()
+
+  def ConfigureCommand(self):
+    """Allows command to be reconfigured if needed."""
+    self.method = registry.GetMethod(self.spec.request.collection,
+                                     self.spec.request.method,
+                                     self.spec.request.api_version,
+                                     self.spec.request.use_google_auth)
     resource_arg = self.spec.arguments.resource
     self.arg_generator = arg_marshalling.DeclarativeArgumentGenerator(
-        self.method,
-        self.spec.arguments.params,
-        resource_arg)
+        self.method, self.spec.arguments.params, resource_arg)
     self.display_resource_type = self.spec.request.display_resource_type
-    if (not self.display_resource_type
-        and resource_arg and not resource_arg.is_parent_resource):
+    if (not self.display_resource_type and resource_arg and
+        not resource_arg.is_parent_resource):
       self.display_resource_type = resource_arg.name if resource_arg else None
 
   def Generate(self):
@@ -146,6 +152,10 @@ class CommandBuilder(object):
       command = self._GenerateRemoveIamPolicyBindingCommand()
     elif self.spec.command_type == yaml_command_schema.CommandType.UPDATE:
       command = self._GenerateUpdateCommand()
+    elif self.spec.command_type == yaml_command_schema.CommandType.IMPORT:
+      command = self._GenerateImportCommand()
+    elif self.spec.command_type == yaml_command_schema.CommandType.EXPORT:
+      command = self._GenerateExportCommand()
     elif self.spec.command_type == yaml_command_schema.CommandType.GENERIC:
       command = self._GenerateGenericCommand()
     else:
@@ -199,6 +209,7 @@ class CommandBuilder(object):
     # pylint: disable=protected-access, The linter gets confused about 'self'
     # and thinks we are accessing something protected.
     class Command(base.ListCommand):
+    # pylint: disable=missing-docstring
 
       @staticmethod
       def Args(parser):
@@ -234,6 +245,7 @@ class CommandBuilder(object):
     # pylint: disable=protected-access, The linter gets confused about 'self'
     # and thinks we are accessing something protected.
     class Command(base.DeleteCommand):
+    # pylint: disable=missing-docstring
 
       @staticmethod
       def Args(parser):
@@ -280,6 +292,7 @@ class CommandBuilder(object):
     # pylint: disable=protected-access, The linter gets confused about 'self'
     # and thinks we are accessing something protected.
     class Command(base.CreateCommand):
+    # pylint: disable=missing-docstring
 
       @staticmethod
       def Args(parser):
@@ -342,6 +355,7 @@ class CommandBuilder(object):
     # pylint: disable=protected-access, The linter gets confused about 'self'
     # and thinks we are accessing something protected.
     class Command(base.Command):
+    # pylint: disable=missing-docstring
 
       @staticmethod
       def Args(parser):
@@ -385,6 +399,11 @@ class CommandBuilder(object):
         base.URI_FLAG.RemoveFromParser(parser)
 
       def Run(self_, args):
+        if self.spec.iam and self.spec.iam.policy_version:
+          self.spec.request.static_fields[
+              self.spec.iam
+              .get_iam_policy_version_path] = self.spec.iam.policy_version
+
         _, response = self._CommonRun(args)
         return self._HandleResponse(response, args)
 
@@ -423,8 +442,10 @@ class CommandBuilder(object):
         # Use Policy message and set IAM request field name overrides for API's
         # with non-standard naming (if provided)
         if self.spec.iam:
-          policy_type_name = (self.spec.iam.message_type_overrides['policy'] or
-                              policy_type_name)
+          if 'policy' in self.spec.iam.message_type_overrides:
+            policy_type_name = (self.spec.iam
+                                .message_type_overrides['policy'] or
+                                policy_type_name)
           policy_request_path = (self.spec.iam.set_iam_policy_request_path or
                                  policy_request_path)
 
@@ -436,6 +457,10 @@ class CommandBuilder(object):
         policy, update_mask = iam_util.ParsePolicyFileWithUpdateMask(
             args.policy_file, policy_type)
 
+        # override policy version
+        if self.spec.iam and self.spec.iam.policy_version:
+          policy.version = self.spec.iam.policy_version
+
         self.spec.request.static_fields[policy_field_path] = policy
         self._SetPolicyUpdateMask(update_mask)
         try:
@@ -619,6 +644,159 @@ class CommandBuilder(object):
 
     return Command
 
+  def _GenerateImportCommand(self):
+    """Generates an export command.
+
+    An export command has a single resource argument and an API method to call
+    to get the resource. The result is exported to a local yaml file provided
+    by the `--destination` flag, or to stdout if nothing is provided.
+
+    Returns:
+      calliope.base.Command, The command that implements the spec.
+    """
+    # Lazy import to prevent drag on startup time.
+    from googlecloudsdk.command_lib.export import util as export_util  # pylint:disable=g-import-not-at-top
+
+    # pylint: disable=no-self-argument, The class closure throws off the linter
+    # a bit. We want to use the generator class, not the class being generated.
+    # pylint: disable=protected-access, The linter gets confused about 'self'
+    # and thinks we are accessing something protected.
+    class Command(base.ImportCommand):
+      """Export command enclosure."""
+
+      @staticmethod
+      def Args(parser):
+        self._CommonArgs(parser)
+        if self.spec.async_:
+          base.ASYNC_FLAG.AddToParser(parser)
+        parser.add_argument(
+            '--source',
+            help="""
+            Path to a YAML file containing the configuration export data. The
+            YAML file must not contain any output-only fields. Alternatively, you
+            may omit this flag to read from standard input. A schema describing
+            the export/import format can be found in:
+            $CLOUDSDKROOT/lib/googlecloudsdk/schemas/...
+          """)
+
+      def Run(self_, args):
+        # Determine message to parse resource into from yaml
+        message_type = self.method.GetRequestType()
+        request_field = self.method.request_field
+        resource_message_class = message_type.field_by_name(request_field).type
+
+        # Set up information for export utility.
+        data = console_io.ReadFromFileOrStdin(args.source or '-', binary=False)
+        schema_path = export_util.GetSchemaPath(self.method.collection.api_name,
+                                                self.spec.request.api_version,
+                                                resource_message_class.__name__)
+        # Import resource from yaml.
+        imported_resource = export_util.Import(
+            message_type=resource_message_class,
+            stream=data,
+            schema_path=schema_path)
+
+        # If any special configuration has been made for the import command...
+        existing_resource = None
+        if self.spec.import_:
+          abort_if_equivalent = self.spec.import_.abort_if_equivalent
+          create_if_not_exists = self.spec.import_.create_if_not_exists
+
+          # Try to get the existing resource from the service.
+          try:
+            existing_resource = self._GetExistingResource(args)
+          except apitools_exceptions.HttpError as error:
+            # Raise error if command is configured to not create a new resource
+            # or if error other than "Does Not Exist" occurs.
+            if error.status_code != 404 or not create_if_not_exists:
+              raise error
+            else:
+              # Configure command to use fallback create request configuration.
+              self.spec.request = self.spec.import_.create_request
+
+              # Configure command to use fallback create async configuration.
+              if self.spec.import_.no_create_async:
+                self.spec.async_ = None
+              elif self.spec.import_.create_async:
+                self.spec.async_ = self.spec.import_.create_async
+              # Reset command with updated configuration.
+              self.ConfigureCommand()
+
+          # Abort command early if no changes are detected.
+          if abort_if_equivalent:
+            if imported_resource == existing_resource:
+              return log.status.Print(
+                  'Request not sent for [{}]: No changes detected.'.format(
+                      imported_resource.name))
+
+        ref, response = self._CommonRun(
+            args, existing_message=imported_resource)
+
+        # Handle asynchronous behavior.
+        if self.spec.async_:
+          request_string = None
+          if ref is not None:
+            request_string = 'Request issued for: [{{{}}}]'.format(
+                yaml_command_schema.NAME_FORMAT_KEY)
+          response = self._HandleAsync(args, ref, response, request_string)
+
+        return self._HandleResponse(response, args)
+
+    return Command
+
+  def _GenerateExportCommand(self):
+    """Generates an export command.
+
+    An export command has a single resource argument and an API method to call
+    to get the resource. The result is exported to a local yaml file provided
+    by the `--destination` flag, or to stdout if nothing is provided.
+
+    Returns:
+      calliope.base.Command, The command that implements the spec.
+    """
+
+    # Lazy import to prevent drag on startup time.
+    from googlecloudsdk.command_lib.export import util as export_util  # pylint:disable=g-import-not-at-top
+
+    # pylint: disable=no-self-argument, The class closure throws off the linter
+    # a bit. We want to use the generator class, not the class being generated.
+    # pylint: disable=protected-access, The linter gets confused about 'self'
+    # and thinks we are accessing something protected.
+    class Command(base.ExportCommand):
+      """Export command enclosure."""
+
+      @staticmethod
+      def Args(parser):
+        self._CommonArgs(parser)
+        parser.add_argument(
+            '--destination',
+            help="""
+            Path to a YAML file where the configuration will be exported.
+            The exported data will not contain any output-only fields.
+            Alternatively, you may omit this flag to write to standard output. A
+            schema describing the export/import format can be found in
+            $CLOUDSDKROOT/lib/googlecloudsdk/schemas/...
+          """)
+
+      def Run(self_, args):
+        unused_ref, response = self._CommonRun(args)
+        schema_path = export_util.GetSchemaPath(self.method.collection.api_name,
+                                                self.spec.request.api_version,
+                                                type(response).__name__)
+
+        # Export parsed yaml to selected destination.
+        if args.IsSpecified('destination'):
+          with files.FileWriter(args.destination) as stream:
+            export_util.Export(
+                message=response, stream=stream, schema_path=schema_path)
+          return log.status.Print('Exported [{}] to \'{}\'.'.format(
+              response.name, args.destination))
+        else:
+          export_util.Export(
+              message=response, stream=sys.stdout, schema_path=schema_path)
+
+    return Command
+
   def _GenerateUpdateCommand(self):
     """Generates an update command.
 
@@ -692,6 +870,7 @@ class CommandBuilder(object):
       parser: The argparse parser.
     """
     args = self.arg_generator.GenerateArgs()
+    parser = self._Exclude(parser)
     for arg in args:
       arg.AddToParser(parser)
     if self.spec.arguments.additional_arguments_hook:
@@ -702,6 +881,19 @@ class CommandBuilder(object):
     if self.spec.output.flatten:
       parser.display_info.AddFlatten(self.spec.output.flatten)
 
+  def _Exclude(self, parser):
+    """Excludes specified arguments from the parser.
+
+    Args:
+      parser: The argparse parser.
+
+    Returns:
+      The argparse parser.
+    """
+    for arg in self.spec.arguments.exclude:
+      base.Argument('--{}'.format(arg), help='').RemoveFromParser(parser)
+    return parser
+
   def _CommonRun(self, args, existing_message=None):
     """Performs run actions common to all commands.
 
@@ -776,8 +968,9 @@ class CommandBuilder(object):
     # for API's with non-standard naming (if provided)
     if self.spec.iam:
       overrides = self.spec.iam.message_type_overrides
-      set_iam_policy_request = (overrides['set_iam_policy_request']
-                                or set_iam_policy_request)
+      if 'set_iam_policy_request' in overrides:
+        set_iam_policy_request = (overrides['set_iam_policy_request']
+                                  or set_iam_policy_request)
       policy_request_path = (self.spec.iam.set_iam_policy_request_path
                              or policy_request_path)
 
@@ -1019,6 +1212,8 @@ class CommandBuilder(object):
       command = base.Hidden(command)
     if self.spec.release_tracks:
       command = base.ReleaseTracks(*self.spec.release_tracks)(command)
+    if self.spec.deprecated_data:
+      command = base.Deprecate(**self.spec.deprecated_data)(command)
     if not hasattr(command, 'detailed_help'):
       key_map = {
           'description': 'DESCRIPTION',
@@ -1093,7 +1288,7 @@ class AsyncOperationPoller(waiter.OperationPoller):
     request_type = self.method.GetRequestType()
     relative_name = operation_ref.RelativeName()
     fields = {
-        f.name: getattr(
+        f.name: getattr(  # pylint:disable=g-complex-comprehension
             operation_ref,
             self.spec.async_.operation_get_method_params.get(f.name, f.name),
             relative_name)
