@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -524,6 +524,27 @@ class Keys(object):
       cmd = KeygenCommand(self.key_file, allow_passphrase=allow_passphrase)
       cmd.Run(self.env)
 
+    if self.env.suite is Suite.PUTTY:
+      # This is to fix an encoding issue with PPK's we generated that was
+      # ignored in versions of PuTTY <=0.70, but became invalid in version 0.71.
+      # Since this only affects the PPK, we don't need to generate a new key; we
+      # can just correct the encoding of the PPK if necessary. We use a sentinel
+      # file in the config dir to check if the encoding is already correct.
+      valid_ppk_sentinel = config.Paths().valid_ppk_sentinel_file
+      if not os.path.exists(valid_ppk_sentinel):
+        if key_files_validity is KeyFileStatus.PRESENT:  # Initial validity
+          cmd = KeygenCommand(
+              self.key_file, allow_passphrase=False, reencode_ppk=True)
+          cmd.Run(self.env)
+        try:
+          files.WriteFileContents(valid_ppk_sentinel, '')
+        except files.Error as e:
+          # It's possible that writing the sentinel file fails, which means
+          # we'll potentially have to re-encode the PPK again the next time an
+          # SSH/SCP command is run. But we shouldn't let this prevent the user
+          # from running their current command.
+          log.debug('Failed to create sentinel file: [{}]'.format(e))
+
 
 class KnownHosts(object):
   """Represents known hosts file, supports read, write and basic key management.
@@ -611,6 +632,42 @@ class KnownHosts(object):
     else:
       self.known_hosts.append(new_key_entry)
 
+  def AddMultiple(self, hostname, host_keys, overwrite=False):
+    """Add or update multiple entries for the given hostname.
+
+    If there is no entry for the given hostname, the keys will be added. If
+    there is an entry already, and overwrite keys is False, nothing will be
+    changed. If there is an entry and overwrite_keys is True, all  current
+    entries for the given hostname will be removed and the new keys added.
+
+    Args:
+      hostname: str, The hostname for the known_hosts entry.
+      host_keys: list, A list of host keys for the given hostname.
+      overwrite: bool, If true, will overwrite the entries corresponding to
+        hostname with the new host_key if it already exists. If false and an
+        entry already exists for hostname, will ignore the new host_key values.
+    Returns:
+      bool, True if new keys were added.
+    """
+    new_keys_added = False
+    new_key_entries = ['{0} {1}'.format(hostname, host_key)
+                       for host_key in host_keys]
+    if not new_key_entries:
+      return new_keys_added
+    existing_entries = [key for key in self.known_hosts
+                        if key.startswith(hostname)]
+    if existing_entries:
+      if overwrite:
+        self.known_hosts = [key for key in self.known_hosts
+                            if not key.startswith(hostname)]
+        self.known_hosts.extend(new_key_entries)
+        new_keys_added = True
+    else:
+      self.known_hosts.extend(new_key_entries)
+      new_keys_added = True
+
+    return new_keys_added
+
   def Write(self):
     """Writes the file to disk."""
     files.WriteFileContents(
@@ -665,7 +722,8 @@ def _MetadataHasOsloginEnable(metadata):
 
 
 def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
-                              release_track):
+                              expiration_time, release_track,
+                              username_requested=False):
   """Check instance/project metadata for oslogin and return updated username.
 
   Check to see if OS Login is enabled in metadata and if it is, return
@@ -677,7 +735,13 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
     project: project, The object representing the current project.
     requested_user: str, The default or requested username to connect as.
     public_key: str, The public key of the user connecting.
+    expiration_time: int, Microseconds after epoch when the ssh key should
+      expire. If None, an existing key will not be modified and a new key will
+      not be set to expire.  If not None, an existing key may be modified
+      with the new expiry.
     release_track: release_track, The object representing the release track.
+    username_requested: bool, True if the user has passed a specific username in
+      the args.
 
   Returns:
     tuple, A string containing the oslogin username and a boolean indicating
@@ -702,16 +766,24 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
         'OS Login is enabled on Instance/Project, but is not available '
         'in the {0} version of gcloud.'.format(release_track.id))
     return requested_user, use_oslogin
-  user_email = properties.VALUES.core.account.Get()
+  user_email = (properties.VALUES.auth.impersonate_service_account.Get()
+                or properties.VALUES.core.account.Get())
 
-  # Check to see if public key is already in profile, and import if not.
+  # Check to see if public key is already in profile and POSIX information
+  # exists associated with the project. If either are not set, import an SSH
+  # public key. Otherwise update the expiration time if needed.
   login_profile = oslogin.GetLoginProfile(user_email, project.name)
   keys = oslogin_utils.GetKeyDictionaryFromProfile(
       user_email, oslogin, profile=login_profile)
   fingerprint = oslogin_utils.FindKeyInKeyList(public_key, keys)
-  if not fingerprint:
-    import_response = oslogin.ImportSshPublicKey(user_email, public_key)
+  if not fingerprint or not login_profile.posixAccounts:
+    import_response = oslogin.ImportSshPublicKey(user_email, public_key,
+                                                 expiration_time)
     login_profile = import_response.loginProfile
+  elif expiration_time:
+    oslogin.UpdateSshPublicKey(user_email, fingerprint, keys[fingerprint],
+                               'expirationTimeUsec',
+                               expiration_time=expiration_time)
   use_oslogin = True
 
   # Get the username for the oslogin user. If the username is the same as the
@@ -725,12 +797,20 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
     elif pa.primary:
       oslogin_user = pa.username
 
-  log.warning('Using OS Login user [{0}] instead of default user [{1}]'
-              .format(oslogin_user, requested_user))
+  # If the user passed in a specific username to the command, show a message
+  # to the user, otherwise just add a message to the log.
+  if username_requested:
+    log.status.Print(
+        'Using OS Login user [{0}] instead of requested user [{1}]'
+        .format(oslogin_user, requested_user))
+  else:
+    log.info('Using OS Login user [{0}] instead of default user [{1}]'.format(
+        oslogin_user, requested_user))
   return oslogin_user, use_oslogin
 
 
-def ParseAndSubstituteSSHFlags(args, remote, ip_address):
+def ParseAndSubstituteSSHFlags(args, remote, instance_address,
+                               internal_address):
   """Obtain extra flags from the command arguments."""
   extra_flags = []
   if args.ssh_flag:
@@ -738,7 +818,8 @@ def ParseAndSubstituteSSHFlags(args, remote, ip_address):
       for flag_part in flag.split():  # We want grouping here
         dereferenced_flag = (
             flag_part.replace('%USER%', remote.user)
-            .replace('%INSTANCE%', ip_address))
+            .replace('%INSTANCE%', instance_address)
+            .replace('%INTERNAL%', internal_address))
         extra_flags.append(dereferenced_flag)
   return extra_flags
 
@@ -936,7 +1017,7 @@ def _BuildIapTunnelProxyCommandArgs(iap_tunnel_args, env):
   # correct as long as the python executable path doesn't contain a doublequote
   # or end with a backslash, which should never happen.
   gcloud_command = [_EscapeProxyCommandArg(x, env) for x in gcloud_command]
-  # track, project, zone, instance, interface, verbosity should only contain
+  # track, project, zone, instance, verbosity should only contain
   # characters that don't need escaping, so don't bother escaping them.
   if iap_tunnel_args.track:
     gcloud_command.append(iap_tunnel_args.track)
@@ -945,8 +1026,7 @@ def _BuildIapTunnelProxyCommandArgs(iap_tunnel_args, env):
       'compute', 'start-iap-tunnel', iap_tunnel_args.instance, port_token,
       '--listen-on-stdin',
       '--project=' + iap_tunnel_args.project,
-      '--zone=' + iap_tunnel_args.zone,
-      '--network-interface=' + iap_tunnel_args.interface])
+      '--zone=' + iap_tunnel_args.zone])
   for arg in iap_tunnel_args.pass_through_args:
     gcloud_command.append(_EscapeProxyCommandArg(arg, env))
 
@@ -982,12 +1062,16 @@ class KeygenCommand(object):
       - Running in an OpenSSH environment (Linux and Mac)
       - Running in interactive mode (from an actual TTY)
       - Prompts are enabled in gcloud
+    reencode_ppk: bool, If True, reencode the PPK file if it was generated with
+      a bad encoding, instead of generating a new key. This is only valid for
+      PuTTY.
   """
 
-  def __init__(self, identity_file, allow_passphrase=True):
+  def __init__(self, identity_file, allow_passphrase=True, reencode_ppk=False):
     """Construct a suite independent `ssh-keygen` command."""
     self.identity_file = identity_file
     self.allow_passphrase = allow_passphrase
+    self.reencode_ppk = reencode_ppk
 
   def Build(self, env=None):
     """Construct the actual command according to the given environment.
@@ -1012,6 +1096,8 @@ class KeygenCommand(object):
         args.extend(['-N', ''])  # Empty passphrase
       args.extend(['-t', 'rsa', '-f', self.identity_file])
     else:
+      if self.reencode_ppk:
+        args.append('--reencode-ppk')
       args.append(self.identity_file)
 
     return args
@@ -1136,9 +1222,11 @@ class SSHCommand(object):
       args.extend(self.remainder)
 
     if self.remote_command:
-      if env.suite is Suite.OPENSSH:  # Putty doesn't like double dash
+      if env.suite is Suite.OPENSSH:
         args.append('--')
-      args.extend(self.remote_command)
+        args.extend(self.remote_command)
+      else:
+        args.append(' '.join(self.remote_command))
     return args
 
   def Run(self, env=None, force_connect=False,
