@@ -29,6 +29,8 @@ from googlecloudsdk.core.util import encoding
 from googlecloudsdk.core.util import files as file_utils
 from googlecloudsdk.core.util import pkg_resources
 from googlecloudsdk.core.util import platforms
+
+from oauth2client import client
 import six
 
 
@@ -134,7 +136,7 @@ class InstallationConfig(object):
                release_channel, config_suffix):
     # JSON returns all unicode.  We know these are regular strings and using
     # unicode in environment variables on Windows doesn't work.
-    self.version = str(version)
+    self.version = version
     self.revision = revision
     self.user_agent = str(user_agent)
     self.documentation_url = str(documentation_url)
@@ -168,6 +170,7 @@ CLOUDSDK_USER_AGENT = INSTALLATION_CONFIG.user_agent
 
 # Do not add more scopes here.
 CLOUDSDK_SCOPES = (
+    'openid',
     'https://www.googleapis.com/auth/userinfo.email',
     'https://www.googleapis.com/auth/cloud-platform',
     # TODO(b/19019218): remove the following now that 'cloud-platform'
@@ -349,6 +352,19 @@ class Paths(object):
     """
     return os.path.join(self.global_config_dir, '.last_survey_prompt.yaml')
 
+  @property
+  def opt_in_prompting_cache_path(self):
+    """Gets the path to the file to cache information about opt-in prompting.
+
+    This is stored in the config directory instead of the installation state
+    because if the SDK is installed as root, it will fail to persist the cache
+    when you are running gcloud as a normal user.
+
+    Returns:
+      str, The path to the file.
+    """
+    return os.path.join(self.global_config_dir, '.last_opt_in_prompt.yaml')
+
   @property
   def installation_properties_path(self):
     """Gets the path to the installation-wide properties file.
@@ -493,3 +509,23 @@ class Paths(object):
       str, The path to the GCE cache.
     """
     return os.path.join(self.global_config_dir, 'gce')
+
+
+def ADCFilePath():
+  """Gets the ADC default file path.
+
+  Returns:
+    str, The path to the default ADC file.
+  """
+  # pylint:disable=protected-access
+  return client._get_well_known_file()
+
+
+def ADCEnvVariable():
+  """Gets the value of the ADC environment variable.
+
+  Returns:
+    str, The value of the env var or None if unset.
+  """
+  return encoding.GetEncodedValue(
+      os.environ, client.GOOGLE_APPLICATION_CREDENTIALS, None)
