@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,9 +14,6 @@
 # limitations under the License.
 
 """Utilities for configuring platform specific installation."""
-# This file contains platform specific code which is not currently handled
-# by pytype.
-# pytype: skip-file
 
 from __future__ import absolute_import
 from __future__ import division
@@ -59,11 +56,9 @@ def _UpdatePathForWindows(bin_path):
   # pylint:disable=g-import-not-at-top, we want to only attempt these imports
   # on windows.
   try:
-    # pytype: disable=import-error
     import win32con
     import win32gui
     from six.moves import winreg
-    # pytype: enable=import-error
   except ImportError:
     _TraceAction("""\
 The installer is unable to automatically update your system PATH. Please add
@@ -252,7 +247,7 @@ class _RcUpdater(object):
       _TraceAction(console_io.FormatRequiredUserAction(
           'Start a new shell for the changes to take effect.'))
 
-    if not self.completion_update:
+    if not self.completion_update and self._CompletionExists():
       _TraceAction(
           '==> Source [{rc}] in your profile to enable shell command '
           'completion for gcloud.'.format(rc=self.completion))
@@ -334,7 +329,7 @@ def _GetAndUpdateRcPath(completion_update, path_update, rc_path, host_os):
   preferred_shell = _GetPreferredShell(
       encoding.GetEncodedValue(os.environ, 'SHELL', '/bin/sh'))
   default_rc_path = os.path.join(
-      platforms.GetHomePath(), _GetShellRcFileName(preferred_shell, host_os))
+      files.GetHomeDir(), _GetShellRcFileName(preferred_shell, host_os))
   # If in quiet mode, we'll use default path.
   if not console_io.CanPrompt():
     _TraceAction('You specified that you wanted to update your rc file. The '
@@ -346,7 +341,7 @@ def _GetAndUpdateRcPath(completion_update, path_update, rc_path, host_os):
       'file to bring the Google Cloud CLIs into your environment.\n\n'
       'Enter a path to an rc file to update, or leave blank to use '
       '[{rc_path}]:  ').format(rc_path=default_rc_path))
-  return (os.path.expanduser(rc_path_update) if rc_path_update
+  return (files.ExpandHomeDir(rc_path_update) if rc_path_update
           else default_rc_path)
 
 
