@@ -24,6 +24,7 @@ import re
 import sys
 import enum
 
+from googlecloudsdk.core import argv_utils
 from googlecloudsdk.core import config
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core.configurations import named_configs
@@ -39,7 +40,7 @@ import six
 # the --configuration flag.  If they did, this could affect the value of the
 # properties defined in that configuration.  Since some libraries (like logging)
 # use properties at startup, we want to use the correct configuration for that.
-named_configs.FLAG_OVERRIDE_STACK.PushFromArgs(sys.argv)
+named_configs.FLAG_OVERRIDE_STACK.PushFromArgs(argv_utils.GetDecodedArgv())
 
 _SET_PROJECT_HELP = """\
 To set your project, run:
@@ -252,8 +253,8 @@ class _Sections(object):
     billing: Section, The section containing billing properties for the Cloud
       SDK.
     builds: Section, The section containing builds properties for the Cloud SDK.
-    build_artifacts: Section, The section containing build artifacts properties
-      for the Cloud SDK.
+    artifacts: Section, The section containing artifacts properties for the
+      Cloud SDK.
     component_manager: Section, The section containing properties for the
       component_manager.
     composer: Section, The section containing composer properties for the Cloud
@@ -268,6 +269,8 @@ class _Sections(object):
     scc: Section, The section containing scc properties for the Cloud SDK.
     dataproc: Section, The section containing dataproc properties for the Cloud
       SDK.
+    dataflow: Section, The section containing dataflow properties for the Cloud
+      SDK.
     datafusion: Section, The section containing datafusion properties for the
       Cloud SDK.
     default_section: Section, The main section of the properties file (core).
@@ -295,10 +298,14 @@ class _Sections(object):
       Cloud SDK.
     lifesciences: Section, The section containing lifesciencs properties for the
       Cloud SDK.
+    memcache: Section, The section containing memcache properties for the Cloud
+      SDK.
     metrics: Section, The section containing metrics properties for the Cloud
       SDK.
     ml_engine: Section, The section containing ml_engine properties for the
       Cloud SDK.
+    notebooks: Section, The section containing notebook properties for the
+      Cloud SDK.
     proxy: Section, The section containing proxy properties for the Cloud SDK.
     pubsub: Section, The section containing pubsub properties for the Cloud SDK.
     redis: Section, The section containing redis properties for the Cloud SDK.
@@ -311,6 +318,8 @@ class _Sections(object):
       SDK.
     survey: Section, The section containing survey properties for the Cloud SDK.
     test: Section, The section containing test properties for the Cloud SDK.
+    workflows: Section, The section containing workflows properties for the
+      Cloud SDK.
   """
 
   class _ValueFlag(object):
@@ -325,10 +334,10 @@ class _Sections(object):
     self.api_client_overrides = _SectionApiClientOverrides()
     self.api_endpoint_overrides = _SectionApiEndpointOverrides()
     self.app = _SectionApp()
+    self.artifacts = _SectionArtifacts()
     self.auth = _SectionAuth()
     self.billing = _SectionBilling()
     self.builds = _SectionBuilds()
-    self.build_artifacts = _SectionBuildArtifacts()
     self.component_manager = _SectionComponentManager()
     self.composer = _SectionComposer()
     self.compute = _SectionCompute()
@@ -337,6 +346,7 @@ class _Sections(object):
     self.core = _SectionCore()
     self.scc = _SectionScc()
     self.dataproc = _SectionDataproc()
+    self.dataflow = _SectionDataflow()
     self.datafusion = _SectionDatafusion()
     self.deployment_manager = _SectionDeploymentManager()
     self.devshell = _SectionDevshell()
@@ -350,8 +360,10 @@ class _Sections(object):
     self.healthcare = _SectionHealthcare()
     self.interactive = _SectionInteractive()
     self.lifesciences = _SectionLifeSciences()
+    self.memcache = _SectionMemcache()
     self.metrics = _SectionMetrics()
     self.ml_engine = _SectionMlEngine()
+    self.notebooks = _SectionNotebooks()
     self.proxy = _SectionProxy()
     self.pubsub = _SectionPubsub()
     self.redis = _SectionRedis()
@@ -361,6 +373,7 @@ class _Sections(object):
     self.storage = _SectionStorage()
     self.survey = _SectionSurvey()
     self.test = _SectionTest()
+    self.workflows = _SectionWorkflows()
 
     sections = [
         self.access_context_manager,
@@ -371,7 +384,7 @@ class _Sections(object):
         self.auth,
         self.billing,
         self.builds,
-        self.build_artifacts,
+        self.artifacts,
         self.component_manager,
         self.composer,
         self.compute,
@@ -380,6 +393,7 @@ class _Sections(object):
         self.core,
         self.scc,
         self.dataproc,
+        self.dataflow,
         self.datafusion,
         self.deployment_manager,
         self.devshell,
@@ -393,8 +407,11 @@ class _Sections(object):
         self.healthcare,
         self.interactive,
         self.lifesciences,
+        self.memcache,
         self.metrics,
         self.ml_engine,
+        self.notebooks,
+        self.pubsub,
         self.proxy,
         self.redis,
         self.run,
@@ -402,6 +419,7 @@ class _Sections(object):
         self.spanner,
         self.survey,
         self.test,
+        self.workflows,
     ]
     self.__sections = {section.name: section for section in sections}
     self.__invocation_value_stack = [{}]
@@ -574,7 +592,8 @@ class _Section(object):
   def __le__(self, other):
     return self.name <= other.name
 
-  def _Add(self,  # pylint: disable=missing-docstring
+  #  pylint: disable=missing-docstring
+  def _Add(self,
            name,
            help_text=None,
            internal=False,
@@ -746,7 +765,18 @@ class _SectionSecrets(_Section):
   """Contains the properties for the 'secrets' section."""
 
   def __init__(self):
-    super(_SectionSecrets, self).__init__('secrets', hidden=True)
+    super(_SectionSecrets, self).__init__('secrets')
+    self.replication_policy = self._Add(
+        'replication-policy',
+        choices=['automatic', 'user-managed'],
+        help_text='The type of replication policy to apply to secrets. Allowed '
+        'values are "automatic" and "user-managed". If user-managed then '
+        'locations must also be provided.',
+    )
+    self.locations = self._Add(
+        'locations',
+        help_text='A comma separated list of the locations to replicate '
+        'secrets to. Only applies to secrets with a user-managed policy.')
 
 
 class _SectionSpanner(_Section):
@@ -784,7 +814,15 @@ class _SectionCompute(_Section):
         completer=('googlecloudsdk.command_lib.compute.completers:'
                    'RegionsCompleter'))
     self.gce_metadata_read_timeout_sec = self._Add(
-        'gce_metadata_read_timeout_sec', default=1, hidden=True)
+        'gce_metadata_read_timeout_sec',
+        default=20,
+        help_text='Timeout of requesting data from gce metadata endpoints.',
+        hidden=True)
+    self.gce_metadata_check_timeout_sec = self._Add(
+        'gce_metadata_check_timeout_sec',
+        default=3,
+        help_text='Timeout of checking if it is on gce environment.',
+        hidden=True)
     self.use_new_list_usable_subnets_api = self._AddBool(
         'use_new_list_usable_subnets_api',
         default=False,
@@ -859,12 +897,29 @@ class _SectionGameServices(_Section):
 
   def __init__(self):
     super(_SectionGameServices, self).__init__('game_services')
+    self.deployment = self._Add(
+        'default_deployment',
+        default='-',
+        help_text=('Default deployment to use when working with Cloud Game '
+                   'Services list configs. When a --deployment flag is '
+                   'required in a list command but not provided, the command '
+                   'will fall back to this value which envokes aggregated '
+                   'list from the backend.'))
     self.location = self._Add(
         'location',
         default='global',
-        help_text='Default location to use when working with Cloud Game '
-        'Services resources. When a `--location` flag is required but not '
-        'provided, the command will fall back to this value.')
+        help_text=(
+            'Default location to use when working with Cloud Game Services '
+            'resources. When a `--location` flag is required but not provided, '
+            'the command will fall back to this value.'))
+    self.realm = self._Add(
+        'default_realm',
+        default='-',
+        help_text=(
+            'Default realm to use when working with Cloud Game Services list '
+            'clusters. When a --realm flag is required in a list command but '
+            'not provided, the command will fall back to this value which '
+            'envokes aggregated list from the backend.'))
 
 
 class _SectionAccessibility(_Section):
@@ -965,7 +1020,7 @@ class _SectionBuilds(_Section):
         default=True,
         hidden=True,
         help_text='If True, validate that the --tag value to builds '
-        'submit is in the gcr.io or *.gcr.io namespace.')
+        'submit is in the gcr.io, *.gcr.io, or *.pkg.dev namespace.')
     # TODO(b/118509363): Remove this after its default is True.
     self.use_kaniko = self._AddBool(
         'use_kaniko',
@@ -985,17 +1040,24 @@ class _SectionBuilds(_Section):
         'to gcr.io/kaniko-project/executor:latest')
 
 
-class _SectionBuildArtifacts(_Section):
-  """Contains the properties for the 'build_artifacts' section."""
+class _SectionArtifacts(_Section):
+  """Contains the properties for the 'artifacts' section."""
 
   def __init__(self):
-    super(_SectionBuildArtifacts, self).__init__('build_artifacts')
+    super(_SectionArtifacts, self).__init__('artifacts')
 
     self.repository = self._Add(
         'repository',
-        help_text='Default repository to use when working with Cloud '
-        'Build Artifacts resources. When a `repository` is required but not '
-        'provided by a flag, the command will fall back to this value, if set.')
+        help_text='Default repository to use when working with Artifact '
+        'Registry resources. When a `repository` value is required but not '
+        'provided, the command will fall back to this value, if set.')
+
+    self.location = self._Add(
+        'location',
+        help_text='Default location to use when working with Artifact Registry '
+        'resources. When a `location` value is required but not provided, the '
+        'command will fall back to this value, if set. If this value is unset, '
+        'the default location is `global` when `location` value is optional.')
 
 
 class _SectionContainer(_Section):
@@ -1320,8 +1382,17 @@ class _SectionAuth(_Section):
     self.credential_file_override = self._Add(
         'credential_file_override', hidden=True)
     self.impersonate_service_account = self._Add(
-        'impersonate_service_account', hidden=True)
-    self.pkce_code_verifier = self._Add('pkce_code_verifier', hidden=True)
+        'impersonate_service_account',
+        help_text='After setting this property, all API requests will be made '
+        'as the given service account instead of the currently selected '
+        'account. This is done without needing to create, download, and '
+        'activate a key for the account. In order to perform operations as the '
+        'service account, your currently selected account must have an IAM '
+        'role that includes the iam.serviceAccounts.getAccessToken permission '
+        'for the service account. The roles/iam.serviceAccountTokenCreator '
+        'role has this permission or you may create a custom role.')
+    self.disable_google_auth = self._AddBool(
+        'disable_google_auth', default=False, hidden=True)
 
 
 class _SectionBilling(_Section):
@@ -1432,6 +1503,19 @@ class _SectionMlEngine(_Section):
                    'interpreter found on system `PATH`.'))
 
 
+class _SectionNotebooks(_Section):
+  """Contains the properties for the 'notebooks' section."""
+
+  def __init__(self):
+    super(_SectionNotebooks, self).__init__('notebooks')
+
+    self.location = self._Add(
+        'location',
+        help_text='Default location to use when working with Notebook '
+        'resources. When a `location` value is required but not provided, the '
+        'command will fall back to this value, if set.')
+
+
 class _SectionPubsub(_Section):
   """Contains the properties for the 'pubsub' section."""
 
@@ -1440,9 +1524,20 @@ class _SectionPubsub(_Section):
     self.legacy_output = self._AddBool(
         'legacy_output',
         default=False,
+        internal=True,
+        hidden=True,
         help_text=('Use the legacy output for beta pubsub commands. The legacy '
                    'output from beta is being deprecated. This property will '
                    'eventually be removed.'))
+    self.zone = self._Add(
+        'lite_zone',
+        # TODO(b/152969311): Create a Completer for the active zones.
+        # TODO(b/149560300): Add this to
+        # https://cloud.google.com/sdk/docs/properties.
+        help_text='Default zone to use when working with zonal Pub/Sub Lite '
+        'resources. When a `--zone` flag is required but not provided, '
+        'the command will fall back to this value, if set.',
+        hidden=True)
 
 
 class _SectionComposer(_Section):
@@ -1461,6 +1556,23 @@ class _SectionComposer(_Section):
             'referenced.'))
 
 
+class _SectionDataflow(_Section):
+  """Contains the properties for the 'dataflow' section."""
+
+  def __init__(self):
+    super(_SectionDataflow, self).__init__('dataflow')
+    self.disable_public_ips = self._AddBool(
+        'disable_public_ips',
+        help_text='Specifies that Cloud Dataflow workers '
+        'must not use public IP addresses.',
+        default=False)
+    self.print_only = self._AddBool(
+        'print_only',
+        help_text='Prints the container spec to stdout. Does not save in '
+        'Google Cloud Storage.',
+        default=False)
+
+
 class _SectionDatafusion(_Section):
   """Contains the properties for the 'datafusion' section."""
 
@@ -1650,14 +1762,15 @@ class _SectionApiEndpointOverrides(_Section):
     super(_SectionApiEndpointOverrides, self).__init__(
         'api_endpoint_overrides', hidden=True)
     self.remotebuildexecution = self._Add('remotebuildexecution')
+    self.accessapproval = self._Add('accessapproval')
     self.accesscontextmanager = self._Add('accesscontextmanager')
+    self.apigateway = self._Add('apigateway')
     self.appengine = self._Add('appengine')
     self.bigtableadmin = self._Add('bigtableadmin')
-    self.bigtableclusteradmin = self._Add('bigtableclusteradmin')
     self.binaryauthorization = self._Add('binaryauthorization')
-    self.bio = self._Add('bio')
-    self.buildartifacts = self._Add('buildartifacts')
+    self.artifactregistry = self._Add('artifactregistry')
     self.categorymanager = self._Add('categorymanager')
+    self.cloudasset = self._Add('cloudasset')
     self.cloudbilling = self._Add('cloudbilling')
     self.cloudbuild = self._Add('cloudbuild')
     self.clouddebugger = self._Add('clouddebugger')
@@ -1699,18 +1812,24 @@ class _SectionApiEndpointOverrides(_Section):
     self.logging = self._Add('logging')
     self.managedidentities = self._Add('managedidentities')
     self.manager = self._Add('manager')
+    self.memcache = self._Add('memcache')
     self.ml = self._Add('ml')
     self.monitoring = self._Add('monitoring')
+    self.networkmanagement = self._Add('networkmanagement')
+    self.networkservices = self._Add('networkservices')
     self.orgpolicy = self._Add('orgpolicy')
     self.osconfig = self._Add('osconfig')
     self.oslogin = self._Add('oslogin')
     self.policytroubleshooter = self._Add('policytroubleshooter')
+    self.privateca = self._Add('privateca')
     self.pubsub = self._Add('pubsub')
+    self.pubsublite = self._Add('pubsublite')
     self.recommender = self._Add('recommender')
     self.replicapoolupdater = self._Add('replicapoolupdater')
     self.runtimeconfig = self._Add('runtimeconfig')
     self.redis = self._Add('redis')
     self.run = self._Add('run')
+    self.scc = self._Add('scc')
     self.servicemanagement = self._Add('servicemanagement')
     self.serviceregistry = self._Add('serviceregistry')
     self.serviceusage = self._Add('serviceusage')
@@ -1718,6 +1837,7 @@ class _SectionApiEndpointOverrides(_Section):
     self.source = self._Add('source')
     self.sourcerepo = self._Add('sourcerepo')
     self.secrets = self._Add('secretmanager')
+    self.servicedirectory = self._Add('servicedirectory')
     self.spanner = self._Add('spanner')
     self.speech = self._Add('speech')
     self.sql = self._Add('sql')
@@ -1727,6 +1847,8 @@ class _SectionApiEndpointOverrides(_Section):
     self.tpu = self._Add('tpu')
     self.vision = self._Add('vision')
     self.vpcaccess = self._Add('vpcaccess')
+    self.workflowexecutions = self._Add('workflowexecutions')
+    self.workflows = self._Add('workflows')
 
   def EndpointValidator(self, value):
     """Checks to see if the endpoint override string is valid."""
@@ -1755,8 +1877,13 @@ class _SectionApiClientOverrides(_Section):
     self.appengine = self._Add('appengine')
     self.cloudidentity = self._Add('cloudidentity')
     self.compute = self._Add('compute')
+    self.compute_alpha = self._Add('compute/alpha')
+    self.compute_beta = self._Add('compute/beta')
+    self.compute_v1 = self._Add('compute/v1')
     self.container = self._Add('container')
+    self.speech = self._Add('speech')
     self.sql = self._Add('sql')
+    self.run = self._Add('run')
 
 
 class _SectionEmulator(_Section):
@@ -1811,7 +1938,12 @@ class _SectionContextAware(_Section):
     self.use_client_certificate = self._AddBool(
         'use_client_certificate',
         help_text=('If True, use client certificate to authorize user '
-                   'device using context aware access.'))
+                   'device using Context-aware access. Some services may not '
+                   'support client certificate authorization. If a command '
+                   'sends requests to such services, the client certificate '
+                   'will not be validated. '
+                   'Run `gcloud topic client-certificate` for list of services '
+                   'supporting this feature.'))
     self.auto_discovery_file_path = self._Add(
         'auto_discovery_file_path',
         validator=ExistingAbsoluteFilepathValidator,
@@ -1819,6 +1951,18 @@ class _SectionContextAware(_Section):
         hidden=True)
 
 
+class _SectionMemcache(_Section):
+  """Contains the properties for the 'memcache' section."""
+
+  def __init__(self):
+    super(_SectionMemcache, self).__init__('memcache')
+    self.region = self._Add(
+        'region',
+        help_text='Default region to use when working with Cloud Memorystore '
+        'for Memcached resources. When a `region` is required but not provided '
+        'by a flag, the command will fall back to this value, if set.')
+
+
 class _SectionRedis(_Section):
   """Contains the properties for the 'redis' section."""
 
@@ -1862,6 +2006,19 @@ class _SectionSurvey(_Section):
         'experience surveys.')
 
 
+class _SectionWorkflows(_Section):
+  """Contains the properties for the 'workflows' section."""
+
+  def __init__(self):
+    super(_SectionWorkflows, self).__init__('workflows', hidden=True)
+    self.location = self._Add(
+        'location',
+        default='us-central1',
+        help_text='The default region to use when working with Cloud '
+        'Workflows resources. When a `--location` flag is required '
+        'but not provided, the command will fall back to this value, if set.')
+
+
 class _Property(object):
   """An individual property that can be gotten from the properties file.
 
@@ -2253,17 +2410,16 @@ def PersistProperty(prop, value, scope=None):
   else:
     active_config = named_configs.ConfigurationStore.ActiveConfig()
     active_config.PersistProperty(prop.section, prop.name, value)
-  # Print message if value being set is overridden by environment var
+  # Print message if value being set/unset is overridden by environment var
   # to prevent user confusion
-  if value is not None:
-    env_name = prop.EnvironmentName()
-    override = os.getenv(env_name)
-    if override:
-      warning_message = ('WARNING: Property [{0}] is overridden '
-                         'by environment setting [{1}={2}]\n')
-      # Writing to sys.stderr because of circular dependency
-      # in googlecloudsdk.core.log on properties
-      sys.stderr.write(warning_message.format(prop.name, env_name, override))
+  env_name = prop.EnvironmentName()
+  override = encoding.GetEncodedValue(os.environ, env_name)
+  if override:
+    warning_message = ('WARNING: Property [{0}] is overridden '
+                       'by environment setting [{1}={2}]\n')
+    # Writing to sys.stderr because of circular dependency
+    # in googlecloudsdk.core.log on properties
+    sys.stderr.write(warning_message.format(prop.name, env_name, override))
 
 
 def _GetProperty(prop, properties_file, required):
