@@ -163,12 +163,13 @@ def _ShouldRetryOperation(func, exc_info):
   Returns:
     True if the error can be retried or false if we should just fail.
   """
-  if not (func == os.remove or func == os.rmdir):
+  # os.unlink is the same as os.remove
+  if not (func == os.remove or func == os.rmdir or func == os.unlink):
     return False
-  if not WindowsError or exc_info[0] != WindowsError:
+  if not WindowsError:
     return False
   e = exc_info[1]
-  return e.winerror in RETRY_ERROR_CODES
+  return getattr(e, 'winerror', None) in RETRY_ERROR_CODES
 
 
 def _RetryOperation(exc_info, func, args,
@@ -217,7 +218,11 @@ def _HandleRemoveError(func, failed_path, exc_info):
 
   # Access denied on Windows. This happens when trying to delete a readonly
   # file. Change the permissions and retry the delete.
-  if exc_info[0] == WindowsError and exc_info[1].winerror == 5:
+  #
+  # In python 3.3+, WindowsError is an alias of OSError and exc_info[0] can be
+  # a subclass of OSError.
+  if (WindowsError and issubclass(exc_info[0], WindowsError) and
+      getattr(exc_info[1], 'winerror', None) == 5):
     os.chmod(failed_path, stat.S_IWUSR)
 
   # Don't remove the trailing comma in the passed arg tuple.  It indicates that
@@ -987,7 +992,7 @@ def WriteFileAtomically(file_name, contents):
 
 
 def GetTreeSizeBytes(path, predicate=None):
-  """Returns sum of sizes of not-ingnored files under given path, in bytes."""
+  """Returns sum of sizes of not-ignored files under given path, in bytes."""
   result = 0
   if predicate is None:
     predicate = lambda x: True
@@ -999,6 +1004,46 @@ def GetTreeSizeBytes(path, predicate=None):
   return result
 
 
+def GetDirectoryTreeListing(path,
+                            include_dirs=False,
+                            file_predicate=None,
+                            dir_sort_func=None,
+                            file_sort_func=None):
+  """Yields a generator that list all the files in a directory tree.
+
+  Walks directory tree from path and yeilds all files that it finds. Will expand
+  paths relative to home dir e.g. those that start with '~'.
+
+  Args:
+    path: string, base of file tree to walk.
+    include_dirs: bool, if true will yield directory names in addition to files.
+    file_predicate: function, boolean function to determine which files should
+      be included in the output. Default is all files.
+    dir_sort_func: function, function that will determine order directories are
+      processed. Default is lexical ordering.
+    file_sort_func:  function, function that will determine order directories
+      are processed. Default is lexical ordering.
+  Yields:
+    Generator: yields all files and directory paths matching supplied criteria.
+  """
+  if not file_sort_func:
+    file_sort_func = sorted
+  if file_predicate is None:
+    file_predicate = lambda x: True
+  if dir_sort_func is None:
+    dir_sort_func = lambda x: x.sort()
+
+  for root, dirs, files in os.walk(ExpandHomeDir(six.text_type(path))):
+    dir_sort_func(dirs)
+    if include_dirs:
+      for dirname in dirs:
+        yield dirname
+    for file_name in file_sort_func(files):
+      file_path = os.path.join(root, file_name)
+      if file_predicate(file_path):
+        yield file_path
+
+
 def ReadFileContents(path):
   """Reads the text contents from the given path.
 
@@ -1041,7 +1086,11 @@ def ReadBinaryFileContents(path):
     raise Error('Unable to read file [{0}]: {1}'.format(path, e))
 
 
-def WriteFileContents(path, contents, overwrite=True, private=False):
+def WriteFileContents(path,
+                      contents,
+                      overwrite=True,
+                      private=False,
+                      create_path=True):
   """Writes the given text contents to a file at the given path.
 
   Args:
@@ -1049,13 +1098,14 @@ def WriteFileContents(path, contents, overwrite=True, private=False):
     contents: str, The text string to write.
     overwrite: bool, False to error out if the file already exists.
     private: bool, True to make the file have 0o600 permissions.
+    create_path: bool, True to create intermediate directories, if needed.
 
   Raises:
     Error: If the file cannot be written.
   """
   try:
     _CheckOverwrite(path, overwrite)
-    with FileWriter(path, private=private) as f:
+    with FileWriter(path, private=private, create_path=create_path) as f:
       # This decode is here because a lot of libraries on Python 2 can return
       # both text or bytes depending on if unicode is present. If you truly
       # pass binary data to this, the decode will fail (as it should). If you
@@ -1068,7 +1118,11 @@ def WriteFileContents(path, contents, overwrite=True, private=False):
     raise Error('Unable to write file [{0}]: {1}'.format(path, e))
 
 
-def WriteBinaryFileContents(path, contents, overwrite=True, private=False):
+def WriteBinaryFileContents(path,
+                            contents,
+                            overwrite=True,
+                            private=False,
+                            create_path=True):
   """Writes the given binary contents to a file at the given path.
 
   Args:
@@ -1076,13 +1130,14 @@ def WriteBinaryFileContents(path, contents, overwrite=True, private=False):
     contents: str, The byte string to write.
     overwrite: bool, False to error out if the file already exists.
     private: bool, True to make the file have 0o600 permissions.
+    create_path: bool, True to create intermediate directories, if needed.
 
   Raises:
     Error: If the file cannot be written.
   """
   try:
     _CheckOverwrite(path, overwrite)
-    with BinaryFileWriter(path, private=private) as f:
+    with BinaryFileWriter(path, private=private, create_path=create_path) as f:
       f.write(contents)
   except EnvironmentError as e:
     # EnvironmentError is parent of IOError, OSError and WindowsError.
@@ -1117,41 +1172,57 @@ def BinaryFileReader(path):
   Returns:
     A file-like object opened for read in binary mode.
   """
-  return _FileOpener(path, 'rb', 'read')
+  return _FileOpener(encoding_util.Encode(path, encoding='utf-8'), 'rb', 'read')
 
 
-def FileWriter(path, private=False, append=False):
+def FileWriter(path, private=False, append=False, create_path=False):
   """Opens the given file for text write for use in a 'with' statement.
 
   Args:
     path: str, The file path to write to.
     private: bool, True to create or update the file permission to be 0o600.
     append: bool, True to append to an existing file.
+    create_path: bool, True to create intermediate directories, if needed.
 
   Returns:
     A file-like object opened for write in text mode.
   """
   mode = 'at' if append else 'wt'
-  return _FileOpener(path, mode, 'write', encoding='utf8', private=private)
+  return _FileOpener(
+      path,
+      mode,
+      'write',
+      encoding='utf8',
+      private=private,
+      create_path=create_path)
 
 
-def BinaryFileWriter(path, private=False):
+def BinaryFileWriter(path, private=False, create_path=False):
   """Opens the given file for binary write for use in a 'with' statement.
 
   Args:
     path: str, The file path to write to.
     private: bool, True to create or update the file permission to be 0o600.
+    create_path: bool, True to create intermediate directories, if needed.
 
   Returns:
     A file-like object opened for write in binary mode.
   """
-  return _FileOpener(path, 'wb', 'write', private=private)
+  return _FileOpener(
+      path, 'wb', 'write', private=private, create_path=create_path)
 
 
-def _FileOpener(path, mode, verb, encoding=None, private=False):
+def _FileOpener(path,
+                mode,
+                verb,
+                encoding=None,
+                private=False,
+                create_path=False):
   """Opens a file in various modes and does error handling."""
   if private:
     PrivatizeFile(path)
+  if create_path:
+    _MakePathToFile(path)
   try:
     return io.open(path, mode, encoding=encoding)
   except EnvironmentError as e:
@@ -1173,6 +1244,12 @@ def ExpandHomeDir(path):
   return encoding_util.Decode(os.path.expanduser(path))
 
 
+def _MakePathToFile(path, mode=0o777):
+  parent_dir_path, _ = os.path.split(path)
+  full_parent_dir_path = os.path.realpath(ExpandHomeDir(parent_dir_path))
+  MakeDir(full_parent_dir_path, mode)
+
+
 def PrivatizeFile(path):
   """Makes an existing file private or creates a new, empty private file.
 
@@ -1188,10 +1265,7 @@ def PrivatizeFile(path):
     if os.path.exists(path):
       os.chmod(path, 0o600)
     else:
-      parent_dir_path, _ = os.path.split(path)
-      full_parent_dir_path = os.path.realpath(ExpandHomeDir(parent_dir_path))
-      MakeDir(full_parent_dir_path, mode=0o700)
-
+      _MakePathToFile(path, mode=0o700)
       flags = os.O_RDWR | os.O_CREAT | os.O_TRUNC
       # Accommodate Windows; stolen from python2.6/tempfile.py.
       if hasattr(os, 'O_NOINHERIT'):
