@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2013 Google Inc. All Rights Reserved.
+# Copyright 2013 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -394,7 +394,8 @@ class PythonVersion(object):
   # See class docstring for descriptions of what these mean
   MIN_REQUIRED_PY2_VERSION = (2, 6)
   MIN_SUPPORTED_PY2_VERSION = (2, 7)
-  MIN_SUPPORTED_PY3_VERSION = (3, 4)
+  MIN_REQUIRED_PY3_VERSION = (3, 4)
+  MIN_SUPPORTED_PY3_VERSION = (3, 5)
   ENV_VAR_MESSAGE = """\
 
 If you have a compatible Python interpreter installed, you can use it by setting
@@ -410,29 +411,23 @@ the CLOUDSDK_PYTHON environment variable to point to it.
     else:
       self.version = None
 
-  def SupportedVersionMessage(self, allow_py3):
-    if allow_py3:
-      return 'Please use Python version {0}.{1}.x or {2}.{3} and up.'.format(
-          PythonVersion.MIN_SUPPORTED_PY2_VERSION[0],
-          PythonVersion.MIN_SUPPORTED_PY2_VERSION[1],
-          PythonVersion.MIN_SUPPORTED_PY3_VERSION[0],
-          PythonVersion.MIN_SUPPORTED_PY3_VERSION[1])
-    else:
-      return 'Please use Python version {0}.{1}.x.'.format(
-          PythonVersion.MIN_SUPPORTED_PY2_VERSION[0],
-          PythonVersion.MIN_SUPPORTED_PY2_VERSION[1])
+  def SupportedVersionMessage(self):
+    return 'Please use Python version {0}.{1}.x or {2}.{3} and up.'.format(
+        PythonVersion.MIN_SUPPORTED_PY2_VERSION[0],
+        PythonVersion.MIN_SUPPORTED_PY2_VERSION[1],
+        PythonVersion.MIN_SUPPORTED_PY3_VERSION[0],
+        PythonVersion.MIN_SUPPORTED_PY3_VERSION[1])
 
-  def IsCompatible(self, allow_py3=False, raise_exception=False):
+  def IsCompatible(self, raise_exception=False):
     """Ensure that the Python version we are using is compatible.
 
     This will print an error message if not compatible.
 
-    Compatible versions are 2.6 and 2.7 and > 3.4 if allow_py3 is True.
+    Compatible versions are 2.6 and 2.7 and > 3.4.
     We don't guarantee support for 2.6 so we want to warn about it.
+    We don't guarantee support for 3.4 so we want to warn about it.
 
     Args:
-      allow_py3: bool, True if we should allow a Python 3 interpreter to run
-        gcloud. If False, this returns an error for Python 3.
       raise_exception: bool, True to raise an exception rather than printing
         the error and exiting.
 
@@ -447,7 +442,7 @@ the CLOUDSDK_PYTHON environment variable to point to it.
       # We don't know the version, not a good sign.
       error = ('ERROR: Your current version of Python is not compatible with '
                'the Google Cloud SDK. {0}\n'
-               .format(self.SupportedVersionMessage(allow_py3)))
+               .format(self.SupportedVersionMessage()))
     else:
       if self.version[0] < 3:
         # Python 2 Mode
@@ -455,18 +450,14 @@ the CLOUDSDK_PYTHON environment variable to point to it.
           error = ('ERROR: Python {0}.{1} is not compatible with the Google '
                    'Cloud SDK. {2}\n'
                    .format(self.version[0], self.version[1],
-                           self.SupportedVersionMessage(allow_py3)))
+                           self.SupportedVersionMessage()))
       else:
         # Python 3 Mode
-        if not allow_py3:
-          error = ('ERROR: Python 3 and later is not compatible with the '
-                   'Google Cloud SDK. {0}\n'
-                   .format(self.SupportedVersionMessage(allow_py3)))
-        elif self.version < PythonVersion.MIN_SUPPORTED_PY3_VERSION:
+        if self.version < PythonVersion.MIN_REQUIRED_PY3_VERSION:
           error = ('ERROR: Python {0}.{1} is not compatible with the Google '
                    'Cloud SDK. {2}\n'
                    .format(self.version[0], self.version[1],
-                           self.SupportedVersionMessage(allow_py3)))
+                           self.SupportedVersionMessage()))
 
     if error:
       if raise_exception:
@@ -481,7 +472,16 @@ the CLOUDSDK_PYTHON environment variable to point to it.
       sys.stderr.write("""\
 WARNING:  Python 2.6.x is no longer officially supported by the Google Cloud SDK
 and may not function correctly.  {0}
-{1}""".format(self.SupportedVersionMessage(allow_py3),
+{1}""".format(self.SupportedVersionMessage(),
+              PythonVersion.ENV_VAR_MESSAGE))
+
+    # Warn that 3.4 might not work. XXX this logic needs some work
+    if (self.version >= self.MIN_REQUIRED_PY3_VERSION and
+        self.version < self.MIN_SUPPORTED_PY3_VERSION):
+      sys.stderr.write("""\
+WARNING:  Python 3.4.x is no longer officially supported by the Google Cloud SDK
+and may not function correctly.  {0}
+{1}""".format(self.SupportedVersionMessage(),
               PythonVersion.ENV_VAR_MESSAGE))
 
     return True
