@@ -1,4 +1,5 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Utilities for manipulating text."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import collections
 
 
@@ -76,5 +81,5 @@ def PrettyTimeDelta(delta):
     if seconds < seconds_per:
       break
     unit = u
-    num = seconds / seconds_per
+    num = seconds // seconds_per
   return '{0} {1}'.format(num, Pluralize(num, unit))
