@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2018 Google Inc. All Rights Reserved.
+# Copyright 2018 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ from __future__ import unicode_literals
 from googlecloudsdk.api_lib.accesscontextmanager import zones as zones_api
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.accesscontextmanager import perimeters
+from googlecloudsdk.command_lib.accesscontextmanager import policies
 from googlecloudsdk.command_lib.util.args import repeated
 
 
@@ -32,20 +33,24 @@ class UpdatePerimetersGA(base.UpdateCommand):
 
   @staticmethod
   def Args(parser):
-    UpdatePerimetersGA.ArgsVersioned(parser, version='v1')
+    UpdatePerimetersGA.ArgsVersioned(parser, version='v1', track='GA')
 
   @staticmethod
-  def ArgsVersioned(parser, version='v1'):
+  def ArgsVersioned(parser, version='v1', track='GA'):
     perimeters.AddResourceArg(parser, 'to update')
-    perimeters.AddPerimeterUpdateArgs(parser, version=version)
+    perimeters.AddPerimeterUpdateArgs(parser, version=version, track=track)
 
   def Run(self, args):
     client = zones_api.Client(version=self._API_VERSION)
     perimeter_ref = args.CONCEPTS.perimeter.Parse()
     result = repeated.CachedResult.FromFunc(client.Get, perimeter_ref)
+    policies.ValidateAccessPolicyArg(perimeter_ref, args)
 
-    return client.Patch(
-        perimeter_ref,
+    return self.Patch(
+        client=client,
+        args=args,
+        result=result,
+        perimeter_ref=perimeter_ref,
         description=args.description,
         title=args.title,
         perimeter_type=perimeters.GetTypeEnumMapper(
@@ -53,15 +58,56 @@ class UpdatePerimetersGA(base.UpdateCommand):
         resources=perimeters.ParseResources(args, result),
         restricted_services=perimeters.ParseRestrictedServices(args, result),
         levels=perimeters.ParseLevels(args, result,
-                                      perimeter_ref.accessPoliciesId))
+                                      perimeter_ref.accessPoliciesId),
+    )
+
+  def Patch(self, client, args, result, perimeter_ref, description, title,
+            perimeter_type, resources, restricted_services, levels):
+    return client.Patch(
+        perimeter_ref,
+        description=description,
+        title=title,
+        perimeter_type=perimeter_type,
+        resources=resources,
+        restricted_services=restricted_services,
+        levels=levels)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
 class UpdatePerimetersBeta(UpdatePerimetersGA):
   """Update an existing access zone."""
   _INCLUDE_UNRESTRICTED = False
-  _API_VERSION = 'v1beta'
+  _API_VERSION = 'v1'
 
   @staticmethod
   def Args(parser):
-    UpdatePerimetersGA.ArgsVersioned(parser, version='v1beta')
+    UpdatePerimetersGA.ArgsVersioned(parser, version='v1', track='BETA')
+
+  def Patch(self, client, args, perimeter_ref, result, description, title,
+            perimeter_type, resources, restricted_services, levels):
+    vpc_allowed_services = perimeters.ParseVpcRestriction(
+        args, result, self._API_VERSION)
+    enable_vpc_accessible_services = args.enable_vpc_accessible_services
+
+    return client.Patch(
+        perimeter_ref,
+        description=description,
+        title=title,
+        perimeter_type=perimeter_type,
+        resources=resources,
+        restricted_services=restricted_services,
+        levels=levels,
+        vpc_allowed_services=vpc_allowed_services,
+        enable_vpc_accessible_services=enable_vpc_accessible_services,
+    )
+
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class UpdatePerimetersAlpha(UpdatePerimetersBeta):
+  """Update an existing access zone."""
+  _INCLUDE_UNRESTRICTED = False
+  _API_VERSION = 'v1alpha'
+
+  @staticmethod
+  def Args(parser):
+    UpdatePerimetersGA.ArgsVersioned(parser, version='v1alpha', track='ALPHA')
