@@ -20,24 +20,48 @@ from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.ml_engine import operations
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.ml_engine import endpoint_util
+from googlecloudsdk.command_lib.ml_engine import flags
 from googlecloudsdk.command_lib.ml_engine import operations_util
 
 
-_LIST_FORMAT = """\
+def _AddListArgs(parser, hide_region_flag=True):
+  list_format = """\
     table(
         name.basename(),
         metadata.operationType,
         done
     )
 """
+  parser.display_info.AddFormat(list_format)
+  flags.GetRegionArg(hidden=hide_region_flag).AddToParser(parser)
 
 
+def _Run(args):
+  with endpoint_util.MlEndpointOverrides(region=args.region):
+    client = operations.OperationsClient()
+    return operations_util.List(client)
+
+
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 class List(base.ListCommand):
   """List existing AI Platform jobs."""
 
   @staticmethod
   def Args(parser):
-    parser.display_info.AddFormat(_LIST_FORMAT)
+    _AddListArgs(parser)
 
   def Run(self, args):
-    return operations_util.List(operations.OperationsClient())
+    return _Run(args)
+
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
+class ListBeta(base.ListCommand):
+  """List existing AI Platform jobs."""
+
+  @staticmethod
+  def Args(parser):
+    _AddListArgs(parser, hide_region_flag=False)
+
+  def Run(self, args):
+    return _Run(args)
