@@ -12,17 +12,20 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """The gcloud app deploy command."""
 
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import hashlib
+
 from googlecloudsdk.api_lib.app import appengine_api_client
+from googlecloudsdk.api_lib.app import runtime_builders
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.app import deploy_util
-
+from googlecloudsdk.core import log
+from googlecloudsdk.core import properties
 
 _DETAILED_HELP = {
     'brief': ('Deploy the local code and/or configuration of your app to App '
@@ -40,9 +43,12 @@ _DETAILED_HELP = {
         `appengine-web.xml` file inside the WEB-INF directory. {command}
         skips files specified in the .gcloudignore file (see `gcloud topic
         gcloudignore` for more information).
-        For Java11 Standard, you can either use the yaml file, or, if the
+        For Java11 Standard, you can either use the yaml file, or a Maven pom.xml or, if the
         application is a single self-contained jar, you can give the path to the
-        jar and a simple service configuration will be generated.
+        jar and a simple service configuration will be generated. You can deploy
+        Java11 Maven source projects by specifying the location of your
+        project's pom.xml file, and it will be built and deployed using
+        App Engine Buildpacks.
         """,
     'EXAMPLES':
         """\
@@ -58,6 +64,10 @@ _DETAILED_HELP = {
 
           $ {command} ~/my_app/my_jar.jar
 
+        To deploy an App Engine Standard Java11 Maven source project, run:
+
+          $ {command} ~/my_app/pom.xml
+
         By default, the service is deployed the current project configured via:
 
           $ gcloud config set core/project PROJECT
@@ -92,11 +102,48 @@ class DeployGA(base.SilentCommand):
     runtime_builder_strategy = deploy_util.GetRuntimeBuilderStrategy(
         base.ReleaseTrack.GA)
     api_client = appengine_api_client.GetApiClientForTrack(self.ReleaseTrack())
+    if (runtime_builder_strategy !=
+        runtime_builders.RuntimeBuilderStrategy.NEVER and
+        self._ServerSideExperimentEnabled()):
+      flex_image_build_option_default = deploy_util.FlexImageBuildOptions.ON_SERVER
+    else:
+      flex_image_build_option_default = deploy_util.FlexImageBuildOptions.ON_CLIENT
     return deploy_util.RunDeploy(
         args,
         api_client,
         runtime_builder_strategy=runtime_builder_strategy,
-        parallel_build=False)
+        parallel_build=False,
+        flex_image_build_option=deploy_util.GetFlexImageBuildOption(
+            default_strategy=flex_image_build_option_default))
+
+  def _ServerSideExperimentEnabled(self):
+    """Evaluates whether the build on server-side experiment is enabled for the project.
+
+      The experiment is enabled for a project if the sha256 hash of the
+      projectID mod 100 is smaller than the current experiment rollout percent.
+
+    Returns:
+      false if the experiment is not enabled for this project or the
+      experiment config cannot be read due to an error
+    """
+    runtimes_builder_root = properties.VALUES.app.runtime_builders_root.Get(
+        required=True)
+    try:
+      experiment_config = runtime_builders.Experiments.LoadFromURI(
+          runtimes_builder_root)
+      experiment_percent = experiment_config.GetExperimentPercentWithDefault(
+          runtime_builders.Experiments.TRIGGER_BUILD_SERVER_SIDE, 0)
+      project_hash = int(
+          hashlib.sha256(
+              properties.VALUES.core.project.Get().encode('utf-8')).hexdigest(),
+          16) % 100
+      return project_hash < experiment_percent
+    except runtime_builders.ExperimentsError as e:
+      log.debug(
+          'Experiment config file could not be read. This error is '
+          'informational, and does not cause a deployment to fail. '
+          'Reason: %s' % e, exc_info=True)
+      return False
 
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
