@@ -1,4 +1,5 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,16 +15,20 @@
 
 """The Start command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.app import appengine_api_client
 from googlecloudsdk.api_lib.app import operations_util
 from googlecloudsdk.api_lib.app import version_util
 from googlecloudsdk.calliope import base
-from googlecloudsdk.calliope import exceptions as calliope_exceptions
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.console import progress_tracker
 from googlecloudsdk.core.resource import resource_printer
+import six
 
 
 class VersionsStartError(exceptions.Error):
@@ -70,14 +75,14 @@ class Start(base.Command):
     # TODO(b/36052475): This fails with "module/version does not exist" even
     # when it exists if the scaling mode is set to auto.  It would be good
     # to improve that error message.
-    api_client = appengine_api_client.GetApiClient()
+    api_client = appengine_api_client.GetApiClientForTrack(self.ReleaseTrack())
     services = api_client.ListServices()
     versions = version_util.GetMatchingVersions(
         api_client.ListVersions(services),
         args.versions, args.service)
 
     if not versions:
-      log.warn('No matching versions found.')
+      log.warning('No matching versions found.')
       return
 
     fmt = 'list[title="Starting the following versions:"]'
@@ -86,14 +91,13 @@ class Start(base.Command):
 
     errors = {}
     # Sort versions to make behavior deterministic enough for unit testing.
-    for version in sorted(versions):
+    for version in sorted(versions, key=str):
       try:
         with progress_tracker.ProgressTracker('Starting [{0}]'.format(version)):
-          api_client.StartVersion(version.service, version.id)
-      except (calliope_exceptions.HttpException,
-              operations_util.OperationError,
-              operations_util.OperationTimeoutError) as err:
-        errors[version] = str(err)
+          operations_util.CallAndCollectOpErrors(
+              api_client.StartVersion, version.service, version.id)
+      except operations_util.MiscOperationError as err:
+        errors[version] = six.text_type(err)
     if errors:
       printable_errors = {}
       for version, error_msg in errors.items():
@@ -101,5 +105,5 @@ class Start(base.Command):
         printable_errors[short_name] = '{0}: {1}'.format(short_name, error_msg)
       raise VersionsStartError(
           'Issues starting version(s): {0}\n\n'.format(
-              ', '.join(printable_errors.keys())) +
-          '\n\n'.join(printable_errors.values()))
+              ', '.join(list(printable_errors.keys()))) +
+          '\n\n'.join(list(printable_errors.values())))
