@@ -24,6 +24,8 @@ import textwrap
 from googlecloudsdk.api_lib.auth import exceptions as auth_exceptions
 from googlecloudsdk.api_lib.auth import util as auth_util
 from googlecloudsdk.calliope import base
+from googlecloudsdk.calliope import exceptions as calliope_exceptions
+from googlecloudsdk.command_lib.auth import auth_util as command_auth_util
 from googlecloudsdk.command_lib.util import check_browser
 from googlecloudsdk.core import config
 from googlecloudsdk.core import log
@@ -33,6 +35,8 @@ from googlecloudsdk.core.credentials import devshell as c_devshell
 from googlecloudsdk.core.credentials import gce as c_gce
 from googlecloudsdk.core.credentials import store as c_store
 
+from google.auth import jwt
+
 
 class Login(base.Command):
   """Authorize gcloud to access the Cloud Platform with Google user credentials.
@@ -56,6 +60,12 @@ class Login(base.Command):
 
   For more information on authorization and credential types, see:
   [](https://cloud.google.com/sdk/docs/authorizing).
+
+  ## EXAMPLES
+
+  To obtain access credentials for your user account, run:
+
+    $ {command}
   """
 
   @staticmethod
@@ -85,8 +95,33 @@ class Login(base.Command):
     parser.add_argument(
         '--enable-gdrive-access', action='store_true',
         help='Enable Google Drive access.')
+    parser.add_argument(
+        '--update-adc',
+        action='store_true',
+        default=False,
+        help='Write the obtained credentials to the well-known location for '
+        'Application Default Credentials (ADC). Run '
+        '$ gcloud auth application-default --help to learn more about ADC. '
+        'Any credentials previously generated by '
+        '`gcloud auth application-default login` will be overwritten.')
+    parser.add_argument(
+        '--add-quota-project-to-adc',
+        action='store_true',
+        default=False,
+        hidden=True,
+        help="Read the project from gcloud's context and write to application "
+        'default credentials as the quota project. Use this flag only '
+        'when --update-adc is specified.')
     parser.add_argument(
         'account', nargs='?', help='User account used for authorization.')
+    parser.add_argument(
+        '--use-oauth2client',
+        action='store_true',
+        default=False,
+        hidden=True,
+        help='The gcloud command-line tool is using google-auth-library-python '
+             'as its new auth library during login. Use this flag to switch '
+             'back to the oauth2client.')
     parser.display_info.AddFormat('none')
 
   def Run(self, args):
@@ -138,13 +173,22 @@ class Login(base.Command):
         creds = None
       if creds:
         # Account already has valid creds, just switch to it.
-        return self.LoginAs(account, creds, args.project, args.activate,
-                            args.brief)
+        log.warning('Re-using locally stored credentials for [{}]. '
+                    'To fetch new credentials, re-run the command with the '
+                    '`--force` flag.'.format(account))
+        return LoginAs(account, creds, args.project, args.activate, args.brief,
+                       args.update_adc, args.add_quota_project_to_adc)
 
     # No valid creds, do the web flow.
     launch_browser = check_browser.ShouldLaunchBrowser(args.launch_browser)
-    creds = auth_util.DoInstalledAppBrowserFlow(launch_browser, scopes)
-    web_flow_account = creds.id_token['email']
+    if args.use_oauth2client:
+      creds = auth_util.DoInstalledAppBrowserFlow(launch_browser, scopes)
+      web_flow_account = creds.id_token['email']
+    else:
+      creds = auth_util.DoInstalledAppBrowserFlowGoogleAuth(
+          launch_browser, scopes)
+      decoded_id_token = jwt.decode(creds.id_token, verify=False)
+      web_flow_account = decoded_id_token['email']
     if account and account.lower() != web_flow_account.lower():
       raise auth_exceptions.WrongAccountError(
           'You attempted to log in as account [{account}] but the received '
@@ -156,27 +200,51 @@ class Login(base.Command):
     account = web_flow_account
     # We got new creds, and they are for the correct user.
     c_store.Store(creds, account, scopes)
-    return self.LoginAs(account, creds, args.project, args.activate,
-                        args.brief)
+    return LoginAs(account, creds, args.project, args.activate, args.brief,
+                   args.update_adc, args.add_quota_project_to_adc)
 
-  def LoginAs(self, account, creds, project, activate, brief):
-    """Logs in with valid credentials."""
-    if not activate:
-      return creds
-    properties.PersistProperty(properties.VALUES.core.account, account)
-    if project:
-      properties.PersistProperty(properties.VALUES.core.project, project)
 
-    if not brief:
-      log.warning(
-          '`gcloud auth login` no longer writes application default '
-          'credentials.\nIf you need to use ADC, see:\n'
-          '  gcloud auth application-default --help')
-
-    if not brief:
-      log.status.write(
-          '\nYou are now logged in as [{account}].\n'
-          'Your current project is [{project}].  You can change this setting '
-          'by running:\n  $ gcloud config set project PROJECT_ID\n'.format(
-              account=account, project=properties.VALUES.core.project.Get()))
+def LoginAs(account, creds, project, activate, brief, update_adc,
+            add_quota_project_to_adc):
+  """Logs in with valid credentials."""
+  _ValidateADCFlags(update_adc, add_quota_project_to_adc)
+  if update_adc:
+    _UpdateADC(creds, add_quota_project_to_adc)
+  if not activate:
     return creds
+  properties.PersistProperty(properties.VALUES.core.account, account)
+  if project:
+    properties.PersistProperty(properties.VALUES.core.project, project)
+
+  if not brief:
+    log.status.write(
+        '\nYou are now logged in as [{account}].\n'
+        'Your current project is [{project}].  You can change this setting '
+        'by running:\n  $ gcloud config set project PROJECT_ID\n'.format(
+            account=account, project=properties.VALUES.core.project.Get()))
+  return creds
+
+
+def _UpdateADC(creds, add_quota_project_to_adc):
+  """Updates the ADC json with the credentials creds."""
+  old_adc_json = command_auth_util.GetADCAsJson()
+  command_auth_util.WriteGcloudCredentialsToADC(creds, add_quota_project_to_adc)
+  new_adc_json = command_auth_util.GetADCAsJson()
+  if new_adc_json and new_adc_json != old_adc_json:
+    adc_msg = '\nApplication default credentials (ADC) were updated.'
+    quota_project = command_auth_util.GetQuotaProjectFromADC()
+    if quota_project:
+      adc_msg = adc_msg + (
+          "\n'{}' is added to ADC as the quota project.\nTo "
+          'just update the quota project in ADC, use $gcloud auth '
+          'application-default set-quota-project.'.format(quota_project))
+    log.status.Print(adc_msg)
+
+
+def _ValidateADCFlags(update_adc, add_quota_project_to_adc):
+  if not update_adc and add_quota_project_to_adc:
+    raise calliope_exceptions.InvalidArgumentException(
+        '--add-quota-project-to-adc',
+        '--add-quota-project-to-adc cannot be specified without specifying '
+        '--update-adc.'
+    )
