@@ -28,8 +28,7 @@ from googlecloudsdk.command_lib.compute import flags as compute_flags
 from googlecloudsdk.command_lib.compute.backend_services import flags
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
-                    base.ReleaseTrack.ALPHA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class RemoveBackend(base.UpdateCommand):
   """Remove a backend from a backend service.
 
@@ -41,12 +40,18 @@ class RemoveBackend(base.UpdateCommand):
   capacity scaler to zero through 'gcloud compute
   backend-services edit'.
   """
-  _BACKEND_SERVICE_ARG = flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG
+
+  support_global_neg = True
+  support_region_neg = False
 
   @classmethod
   def Args(cls, parser):
     flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG.AddArgument(parser)
-    flags.AddInstanceGroupAndNetworkEndpointGroupArgs(parser, 'remove from')
+    flags.AddInstanceGroupAndNetworkEndpointGroupArgs(
+        parser,
+        'remove from',
+        support_global_neg=cls.support_global_neg,
+        support_region_neg=cls.support_region_neg)
 
   def GetGetRequest(self, client, backend_service_ref):
     if backend_service_ref.Collection() == 'compute.regionBackendServices':
@@ -85,10 +90,12 @@ class RemoveBackend(base.UpdateCommand):
           resources,
           scope_lister=compute_flags.GetDefaultScopeLister(client))
     if args.network_endpoint_group:
-      return flags.NETWORK_ENDPOINT_GROUP_ARG.ResolveAsResource(
-          args,
-          resources,
-          scope_lister=compute_flags.GetDefaultScopeLister(client))
+      return flags.GetNetworkEndpointGroupArg(
+          support_global_neg=self.support_global_neg,
+          support_region_neg=self.support_region_neg).ResolveAsResource(
+              args,
+              resources,
+              scope_lister=compute_flags.GetDefaultScopeLister(client))
 
   def Modify(self, client, resources, backend_service_ref, args, existing):
     replacement = encoding.CopyProtoMessage(existing)
@@ -138,3 +145,19 @@ class RemoveBackend(base.UpdateCommand):
 
     return client.MakeRequests(
         [self.GetSetRequest(client, backend_service_ref, new_object)])
+
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class RemoveBackendAlpha(RemoveBackend):
+  """Remove a backend from a backend service.
+
+  *{command}* is used to remove a backend from a backend
+  service.
+
+  Before removing a backend, it is a good idea to "drain" the
+  backend first. A backend can be drained by setting its
+  capacity scaler to zero through 'gcloud compute
+  backend-services edit'.
+  """
+
+  support_region_neg = True
