@@ -34,6 +34,7 @@ from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.command_lib.compute import flags
+from googlecloudsdk.command_lib.compute import scope as compute_scope
 from googlecloudsdk.command_lib.compute.disks import create
 from googlecloudsdk.command_lib.compute.disks import flags as disks_flags
 from googlecloudsdk.command_lib.compute.kms import resource_args as kms_resource_args
@@ -44,8 +45,10 @@ from googlecloudsdk.core import log
 import six
 
 DETAILED_HELP = {
-    'brief': 'Create Google Compute Engine persistent disks',
-    'DESCRIPTION': """\
+    'brief':
+        'Create Google Compute Engine persistent disks',
+    'DESCRIPTION':
+        """\
         *{command}* creates one or more Google Compute Engine
         persistent disks. When creating virtual machine instances,
         disks can be attached to the instances through the
@@ -62,15 +65,20 @@ DETAILED_HELP = {
         image (using `gcloud compute images create`) and creating a
         new disk using `--image` in the desired project and/or
         zone.
+        """,
+    'EXAMPLES':
+        """\
+        When creating disks, be sure to include the `--zone` option. To create
+        disks 'my-disk-1' and 'my-disk-2' in zone us-east1-a:
 
-        When creating disks, be sure to include the `--zone` option:
-
-          $ {command} my-disk-1 my-disk-2 --zone us-east1-a
+          $ {command} my-disk-1 my-disk-2 --zone=us-east1-a
         """,
 }
 
 
-def _SourceArgs(parser, source_disk_enabled=False):
+def _SourceArgs(parser,
+                source_disk_enabled=False,
+                source_in_place_snapshot_enabled=False):
   """Add mutually exclusive source args."""
   source_parent_group = parser.add_group()
   source_group = source_parent_group.add_mutually_exclusive_group()
@@ -87,19 +95,25 @@ def _SourceArgs(parser, source_disk_enabled=False):
         """
     return template
 
-  source_group.add_argument(
-      '--image',
-      help=AddImageHelp)
+  source_group.add_argument('--image', help=AddImageHelp)
 
   image_utils.AddImageProjectFlag(source_parent_group)
 
   source_group.add_argument(
       '--image-family',
-      help=('The family of the image that the boot disk will be initialized '
-            'with. When a family is used instead of an image, the latest '
-            'non-deprecated image associated with that family is used.')
-  )
+      help="""\
+        The image family for the operating system that the boot disk will be
+        initialized with. Compute Engine offers multiple Linux
+        distributions, some of which are available as both regular and
+        Shielded VM images.  When a family is specified instead of an image,
+        the latest non-deprecated image associated with that family is
+        used. It is best practice to use --image-family when the latest
+        version of an image is needed.
+        """)
   disks_flags.SOURCE_SNAPSHOT_ARG.AddArgument(source_group)
+  if source_in_place_snapshot_enabled:
+    source_ips = source_group.add_group('Source in place snapshot options')
+    disks_flags.SOURCE_IN_PLACE_SNAPSHOT_ARG.AddArgument(source_ips)
   if source_disk_enabled:
     source_disk = source_group.add_group('Source disk options')
     disks_flags.SOURCE_DISK_ARG.AddArgument(source_disk)
@@ -108,7 +122,8 @@ def _SourceArgs(parser, source_disk_enabled=False):
 def _CommonArgs(parser,
                 include_physical_block_size_support=False,
                 vss_erase_enabled=False,
-                source_disk_enabled=False):
+                source_disk_enabled=False,
+                source_in_place_snapshot_enabled=False):
   """Add arguments used for parsing in all command tracks."""
   Create.disks_arg.AddArgument(parser, operation_type='create')
   parser.add_argument(
@@ -152,7 +167,7 @@ def _CommonArgs(parser,
             'be added onto the created disks to indicate the licensing and '
             'billing policies.'))
 
-  _SourceArgs(parser, source_disk_enabled)
+  _SourceArgs(parser, source_disk_enabled, source_in_place_snapshot_enabled)
 
   csek_utils.AddCsekKeyArgs(parser)
   labels_util.AddCreateLabelsFlags(parser)
@@ -202,12 +217,15 @@ class Create(base.Command):
   """Create Google Compute Engine persistent disks."""
 
   source_disk_enabled = False
+  pd_balanced_enabled = False
+  source_in_place_snapshot_enabled = False
 
-  @staticmethod
-  def Args(parser):
+  @classmethod
+  def Args(cls, parser):
+    messages = cls._GetApiHolder(no_http=True).client.messages
     Create.disks_arg = disks_flags.MakeDiskArg(plural=True)
     _CommonArgs(parser)
-    image_utils.AddGuestOsFeaturesArg(parser, base.ReleaseTrack.GA)
+    image_utils.AddGuestOsFeaturesArg(parser, messages)
     _AddReplicaZonesArg(parser)
     kms_resource_args.AddKmsKeyResourceArg(
         parser, 'disk', region_fallthrough=True)
@@ -241,21 +259,24 @@ class Create(base.Command):
   def GetDiskSizeGb(self, args, from_image):
     size_gb = utils.BytesToGb(args.size)
 
-    if (not size_gb and not args.source_snapshot and not from_image
-        and not self.GetFromSourceDisk(args)):
-      if args.type and 'pd-ssd' in args.type:
+    if (not size_gb and not args.source_snapshot and not from_image and
+        not self.GetFromSourceDisk(args)):
+      pd_disk_types = ['pd-ssd']
+      if self.pd_balanced_enabled:
+        pd_disk_types.append('pd-balanced')
+      if args.type and args.type in pd_disk_types:
         size_gb = constants.DEFAULT_SSD_DISK_SIZE_GB
       else:
         size_gb = constants.DEFAULT_STANDARD_DISK_SIZE_GB
     utils.WarnIfDiskSizeIsTooSmall(size_gb, args.type)
     return size_gb
 
-  def GetProjectToSourceImageDict(
-      self, args, disk_refs, compute_holder, from_image):
+  def GetProjectToSourceImageDict(self, args, disk_refs, compute_holder,
+                                  from_image):
     project_to_source_image = {}
 
-    image_expander = image_utils.ImageExpander(
-        compute_holder.client, compute_holder.resources)
+    image_expander = image_utils.ImageExpander(compute_holder.client,
+                                               compute_holder.resources)
 
     for disk_ref in disk_refs:
       if from_image:
@@ -290,6 +311,13 @@ class Create(base.Command):
       return snapshot_ref.SelfLink()
     return None
 
+  def GetSourceInPlaceSnapshotUri(self, args, compute_holder, default_scope):
+    in_place_snapshot_ref = disks_flags.SOURCE_IN_PLACE_SNAPSHOT_ARG.ResolveAsResource(
+        args, compute_holder.resources, default_scope=default_scope)
+    if in_place_snapshot_ref:
+      return in_place_snapshot_ref.SelfLink()
+    return None
+
   def GetSourceDiskUri(self, args, compute_holder):
     disk_ref = disks_flags.SOURCE_DISK_ARG.ResolveAsResource(
         args, compute_holder.resources)
@@ -304,21 +332,24 @@ class Create(base.Command):
       labels = client.messages.Disk.LabelsValue(additionalProperties=[
           client.messages.Disk.LabelsValue.AdditionalProperty(
               key=key, value=value)
-          for key, value in sorted(six.iteritems(args.labels))])
+          for key, value in sorted(six.iteritems(args.labels))
+      ])
     return labels
 
   def GetDiskTypeUri(self, args, disk_ref, compute_holder):
     if args.type:
       if disk_ref.Collection() == 'compute.disks':
         type_ref = compute_holder.resources.Parse(
-            args.type, collection='compute.diskTypes',
+            args.type,
+            collection='compute.diskTypes',
             params={
                 'project': disk_ref.project,
                 'zone': disk_ref.zone
             })
       elif disk_ref.Collection() == 'compute.regionDisks':
         type_ref = compute_holder.resources.Parse(
-            args.type, collection='compute.regionDiskTypes',
+            args.type,
+            collection='compute.regionDiskTypes',
             params={
                 'project': disk_ref.project,
                 'region': disk_ref.region
@@ -332,18 +363,24 @@ class Create(base.Command):
       zone_ref = compute_holder.resources.Parse(
           zone,
           collection='compute.zones',
-          params={
-              'project': disk_ref.project
-          })
+          params={'project': disk_ref.project})
       result.append(zone_ref.SelfLink())
     return result
 
+  @classmethod
+  def _GetApiHolder(cls, no_http=False):
+    return base_classes.ComputeApiHolder(cls.ReleaseTrack(), no_http)
+
   def Run(self, args):
     return self._Run(args, supports_kms_keys=True)
 
-  def _Run(self, args, supports_kms_keys=False, supports_physical_block=False,
-           support_shared_disk=False, support_vss_erase=False):
-    compute_holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
+  def _Run(self,
+           args,
+           supports_kms_keys=False,
+           supports_physical_block=False,
+           support_shared_disk=False,
+           support_vss_erase=False):
+    compute_holder = self._GetApiHolder()
     client = compute_holder.client
 
     self.show_unformated_message = not (args.IsSpecified('image') or
@@ -352,6 +389,9 @@ class Create(base.Command):
     if self.source_disk_enabled:
       self.show_unformated_message = self.show_unformated_message and not (
           args.IsSpecified('source_disk'))
+    if self.source_in_place_snapshot_enabled:
+      self.show_unformated_message = self.show_unformated_message and not (
+          args.IsSpecified('source_in_place_snapshot'))
 
     disk_refs = self.ValidateAndParseDiskRefs(args, compute_holder)
     from_image = self.GetFromImage(args)
@@ -365,8 +405,9 @@ class Create(base.Command):
     # code supporting them only in alpha and beta versions of the command.
     labels = self.GetLabels(args, client)
 
-    allow_rsa_encrypted = self.ReleaseTrack() in [base.ReleaseTrack.ALPHA,
-                                                  base.ReleaseTrack.BETA]
+    allow_rsa_encrypted = self.ReleaseTrack() in [
+        base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA
+    ]
     csek_keys = csek_utils.CsekKeyStore.FromArgs(args, allow_rsa_encrypted)
 
     for project in project_to_source_image:
@@ -390,8 +431,8 @@ class Create(base.Command):
       # TODO(b/65161039): Stop checking release path in the middle of GA code.
       kwargs = {}
       if csek_keys:
-        disk_key_or_none = csek_keys.LookupKey(
-            disk_ref, args.require_csek_key_create)
+        disk_key_or_none = csek_keys.LookupKey(disk_ref,
+                                               args.require_csek_key_create)
         disk_key_message_or_none = csek_utils.MaybeToMessage(
             disk_key_or_none, client.apitools_client)
         kwargs['diskEncryptionKey'] = disk_key_message_or_none
@@ -440,10 +481,19 @@ class Create(base.Command):
       if self.source_disk_enabled:
         source_disk_ref = self.GetSourceDiskUri(args, compute_holder)
         disk.sourceDisk = source_disk_ref
-      if (support_shared_disk and disk_ref.Collection() == 'compute.regionDisks'
-          and args.IsSpecified('multi_writer')):
-        raise exceptions.InvalidArgumentException('--multi-writer', (
-            '--multi-writer can be used only with --zone flag'))
+      if self.source_in_place_snapshot_enabled:
+        if disk_ref.Collection() == 'compute.regionDisks':
+          disk.sourceInPlaceSnapshot = self.GetSourceInPlaceSnapshotUri(
+              args, compute_holder, compute_scope.ScopeEnum.REGION)
+        else:
+          disk.sourceInPlaceSnapshot = self.GetSourceInPlaceSnapshotUri(
+              args, compute_holder, compute_scope.ScopeEnum.ZONE)
+      if (support_shared_disk and
+          disk_ref.Collection() == 'compute.regionDisks' and
+          args.IsSpecified('multi_writer')):
+        raise exceptions.InvalidArgumentException(
+            '--multi-writer',
+            ('--multi-writer can be used only with --zone flag'))
 
       if (support_shared_disk and disk_ref.Collection() == 'compute.disks' and
           args.IsSpecified('multi_writer')):
@@ -496,39 +546,49 @@ class CreateBeta(Create):
   """Create Google Compute Engine persistent disks."""
 
   source_disk_enabled = False
+  pd_balanced_enabled = False
+  source_in_place_snapshot_enabled = False
 
-  @staticmethod
-  def Args(parser):
+  @classmethod
+  def Args(cls, parser):
+    messages = cls._GetApiHolder(no_http=True).client.messages
     Create.disks_arg = disks_flags.MakeDiskArg(plural=True)
-
     _CommonArgs(
         parser,
-        include_physical_block_size_support=True)
-    image_utils.AddGuestOsFeaturesArg(parser, base.ReleaseTrack.BETA)
+        include_physical_block_size_support=True,
+        vss_erase_enabled=True)
+    image_utils.AddGuestOsFeaturesArg(parser, messages)
     _AddReplicaZonesArg(parser)
     kms_resource_args.AddKmsKeyResourceArg(
         parser, 'disk', region_fallthrough=True)
 
   def Run(self, args):
-    return self._Run(args, supports_kms_keys=True, supports_physical_block=True)
+    return self._Run(
+        args,
+        supports_kms_keys=True,
+        supports_physical_block=True,
+        support_vss_erase=True)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class CreateAlpha(Create):
+class CreateAlpha(CreateBeta):
   """Create Google Compute Engine persistent disks."""
 
   source_disk_enabled = True
+  pd_balanced_enabled = True
+  source_in_place_snapshot_enabled = True
 
-  @staticmethod
-  def Args(parser):
+  @classmethod
+  def Args(cls, parser):
+    messages = cls._GetApiHolder(no_http=True).client.messages
     Create.disks_arg = disks_flags.MakeDiskArg(plural=True)
-
     _CommonArgs(
         parser,
         include_physical_block_size_support=True,
         vss_erase_enabled=True,
-        source_disk_enabled=True)
-    image_utils.AddGuestOsFeaturesArg(parser, base.ReleaseTrack.ALPHA)
+        source_disk_enabled=True,
+        source_in_place_snapshot_enabled=True)
+    image_utils.AddGuestOsFeaturesArg(parser, messages)
     _AddReplicaZonesArg(parser)
     kms_resource_args.AddKmsKeyResourceArg(
         parser, 'disk', region_fallthrough=True)
@@ -542,8 +602,12 @@ class CreateAlpha(Create):
               'resize and snapshot operations.'))
 
   def Run(self, args):
-    return self._Run(args, supports_kms_keys=True, supports_physical_block=True,
-                     support_shared_disk=True, support_vss_erase=True)
+    return self._Run(
+        args,
+        supports_kms_keys=True,
+        supports_physical_block=True,
+        support_shared_disk=True,
+        support_vss_erase=True)
 
 
 def _ValidateAndParseDiskRefsRegionalReplica(args, compute_holder):
@@ -563,9 +627,9 @@ def _ValidateAndParseDiskRefsRegionalReplica(args, compute_holder):
         '--replica-zones',
         '--replica-zones is required for regional disk creation')
   if args.replica_zones is not None:
-    return create.ParseRegionDisksResources(
-        compute_holder.resources, args.DISK_NAME, args.replica_zones,
-        args.project, args.region)
+    return create.ParseRegionDisksResources(compute_holder.resources,
+                                            args.DISK_NAME, args.replica_zones,
+                                            args.project, args.region)
 
   disk_refs = Create.disks_arg.ResolveAsResource(
       args,
@@ -578,8 +642,8 @@ def _ValidateAndParseDiskRefsRegionalReplica(args, compute_holder):
     if disk_ref.Collection() == 'compute.regionDisks':
       raise exceptions.RequiredArgumentException(
           '--replica-zones',
-          '--replica-zones is required for regional disk creation [{}]'.
-          format(disk_ref.SelfLink()))
+          '--replica-zones is required for regional disk creation [{}]'.format(
+              disk_ref.SelfLink()))
 
   return disk_refs
 
