@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2019 Google Inc. All Rights Reserved.
+# Copyright 2019 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,16 +21,25 @@ from __future__ import unicode_literals
 from googlecloudsdk.calliope import base
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
+                    base.ReleaseTrack.GA)
 class ExternalVpnGateways(base.Group):
   """List, create, delete and update External VPN Gateways."""
 
+  category = base.NETWORKING_CATEGORY
+
   # Placeholder to indicate that a detailed_help field exists and should
   # be set outside the class definition.
   detailed_help = None
 
 
 ExternalVpnGateways.detailed_help = {
-    'brief': ('Read and manipulate Google Compute Engine External '
-              'VPN Gateways.'),
+    'DESCRIPTION': """
+        Read and manipulate external VPN gateways for Cloud VPN.
+
+        For more information about external VPN Gateways, see the
+        [external VPN Gateways documentation](https://cloud.google.com/vpn/docs/concepts/overview#external-vpn-gateway-definition).
+
+        See also: [External VPN Gateways API](https://cloud.google.com/compute/docs/reference/rest/v1/externalVpnGateways).
+    """,
 }
