@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2014 Google Inc. All Rights Reserved.
+# Copyright 2014 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,15 @@ class FirewallRules(base.Group):
   """List, create, update, and delete Google Compute Engine firewall rules."""
 
 
+FirewallRules.category = base.NETWORKING_CATEGORY
+
 FirewallRules.detailed_help = {
-    'brief': ('List, create, update, and delete Google Compute Engine firewall '
-              'rules'),
+    'DESCRIPTION': """
+        Read and manipulate VPC firewall rules.
+
+        For more information about firewall rules, see the
+        [firewall rules documentation](https://cloud.google.com/vpc/docs/firewalls).
+
+        See also: [Firewall rules API](https://cloud.google.com/compute/docs/reference/rest/v1/firewalls).
+    """,
 }
