@@ -22,6 +22,7 @@ from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import health_checks_utils
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute import completers
+from googlecloudsdk.command_lib.compute import scope as compute_scope
 from googlecloudsdk.command_lib.compute.health_checks import flags
 
 
@@ -42,7 +43,7 @@ def _DetailedHelp():
   }
 
 
-def _Args(parser, include_l7_internal_load_balancing):
+def _Args(parser, include_l7_internal_load_balancing, include_log_config):
   """Set up arguments to create an HTTPS HealthCheck."""
   parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
   flags.HealthCheckArgument(
@@ -53,12 +54,14 @@ def _Args(parser, include_l7_internal_load_balancing):
   health_checks_utils.AddHttpRelatedCreationArgs(parser)
   health_checks_utils.AddProtocolAgnosticCreationArgs(parser, 'HTTPS')
   health_checks_utils.AddHttpRelatedResponseArg(parser)
+  if include_log_config:
+    health_checks_utils.AddHealthCheckLoggingRelatedArgs(parser)
   parser.display_info.AddCacheUpdater(completers.HealthChecksCompleterAlpha
                                       if include_l7_internal_load_balancing else
                                       completers.HttpsHealthChecksCompleter)
 
 
-def _Run(args, holder, include_l7_internal_load_balancing):
+def _Run(args, holder, include_l7_internal_load_balancing, include_log_config):
   """Issues the request necessary for adding the health check."""
   client = holder.client
   messages = client.messages
@@ -66,7 +69,8 @@ def _Run(args, holder, include_l7_internal_load_balancing):
   health_check_ref = flags.HealthCheckArgument(
       'HTTPS',
       include_l7_internal_load_balancing=include_l7_internal_load_balancing
-  ).ResolveAsResource(args, holder.resources)
+  ).ResolveAsResource(
+      args, holder.resources, default_scope=compute_scope.ScopeEnum.GLOBAL)
   proxy_header = messages.HTTPSHealthCheck.ProxyHeaderValueValuesEnum(
       args.proxy_header)
 
@@ -109,6 +113,10 @@ def _Run(args, holder, include_l7_internal_load_balancing):
         project=health_check_ref.project)
     collection = client.apitools_client.healthChecks
 
+  if include_log_config:
+    request.healthCheck.logConfig = health_checks_utils.CreateLogConfig(
+        client, args)
+
   return client.MakeRequests([(collection, 'Insert', request)])
 
 
@@ -116,24 +124,28 @@ def _Run(args, holder, include_l7_internal_load_balancing):
 class Create(base.CreateCommand):
   """Create a HTTPS health check."""
 
-  _include_l7_internal_load_balancing = False
+  _include_l7_internal_load_balancing = True
+  _include_log_config = False
   detailed_help = _DetailedHelp()
 
   @classmethod
   def Args(cls, parser):
-    _Args(parser, cls._include_l7_internal_load_balancing)
+    _Args(parser, cls._include_l7_internal_load_balancing,
+          cls._include_log_config)
 
   def Run(self, args):
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
-    return _Run(args, holder, self._include_l7_internal_load_balancing)
+    return _Run(args, holder, self._include_l7_internal_load_balancing,
+                self._include_log_config)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
 class CreateBeta(Create):
 
-  _include_l7_internal_load_balancing = True
+  _include_log_config = True
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class CreateAlpha(CreateBeta):
+
   pass
