@@ -21,6 +21,7 @@ from __future__ import unicode_literals
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import health_checks_utils
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.compute import scope as compute_scope
 from googlecloudsdk.command_lib.compute.health_checks import flags
 
 
@@ -39,7 +40,7 @@ def _DetailedHelp():
   }
 
 
-def _Args(parser, include_l7_internal_load_balancing):
+def _Args(parser, include_l7_internal_load_balancing, include_log_config):
   """Set up arguments to create an SSL HealthCheck."""
   parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
   flags.HealthCheckArgument(
@@ -49,9 +50,11 @@ def _Args(parser, include_l7_internal_load_balancing):
       parser, operation_type='create')
   health_checks_utils.AddTcpRelatedCreationArgs(parser)
   health_checks_utils.AddProtocolAgnosticCreationArgs(parser, 'SSL')
+  if include_log_config:
+    health_checks_utils.AddHealthCheckLoggingRelatedArgs(parser)
 
 
-def _Run(args, holder, include_l7_internal_load_balancing):
+def _Run(args, holder, include_l7_internal_load_balancing, include_log_config):
   """Issues the request necessary for adding the health check."""
   client = holder.client
   messages = client.messages
@@ -59,7 +62,8 @@ def _Run(args, holder, include_l7_internal_load_balancing):
   health_check_ref = flags.HealthCheckArgument(
       'SSL',
       include_l7_internal_load_balancing=include_l7_internal_load_balancing
-  ).ResolveAsResource(args, holder.resources)
+  ).ResolveAsResource(
+      args, holder.resources, default_scope=compute_scope.ScopeEnum.GLOBAL)
   proxy_header = messages.SSLHealthCheck.ProxyHeaderValueValuesEnum(
       args.proxy_header)
   ssl_health_check = messages.SSLHealthCheck(
@@ -100,6 +104,10 @@ def _Run(args, holder, include_l7_internal_load_balancing):
         project=health_check_ref.project)
     collection = client.apitools_client.healthChecks
 
+  if include_log_config:
+    request.healthCheck.logConfig = health_checks_utils.CreateLogConfig(
+        client, args)
+
   return client.MakeRequests([(collection, 'Insert', request)])
 
 
@@ -107,24 +115,28 @@ def _Run(args, holder, include_l7_internal_load_balancing):
 class Create(base.CreateCommand):
   """Create a SSL health check."""
 
-  _include_l7_internal_load_balancing = False
+  _include_l7_internal_load_balancing = True
+  _include_log_config = False
   detailed_help = _DetailedHelp()
 
   @classmethod
   def Args(cls, parser):
-    _Args(parser, cls._include_l7_internal_load_balancing)
+    _Args(parser, cls._include_l7_internal_load_balancing,
+          cls._include_log_config)
 
   def Run(self, args):
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
-    return _Run(args, holder, self._include_l7_internal_load_balancing)
+    return _Run(args, holder, self._include_l7_internal_load_balancing,
+                self._include_log_config)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
 class CreateBeta(Create):
 
-  _include_l7_internal_load_balancing = True
+  _include_log_config = True
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class CreateAlpha(CreateBeta):
+
   pass
