@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,13 +24,13 @@ from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA)
-class List(base_classes.BaseLister):
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
+class List(base_classes.MultiScopeLister):
   """List health checks in GA."""
 
   @staticmethod
   def Args(parser):
-    base_classes.BaseLister.Args(parser)
+    lister.AddMultiScopeListerFlags(parser, regional=True, global_=True)
 
     parser.add_argument(
         '--protocol',
@@ -50,6 +50,7 @@ class List(base_classes.BaseLister):
     return [
         self.messages.HealthCheck.TypeValueValuesEnum.HTTP.number,
         self.messages.HealthCheck.TypeValueValuesEnum.HTTPS.number,
+        self.messages.HealthCheck.TypeValueValuesEnum.HTTP2.number,
         self.messages.HealthCheck.TypeValueValuesEnum.TCP.number,
         self.messages.HealthCheck.TypeValueValuesEnum.SSL.number
     ]
@@ -75,6 +76,12 @@ class List(base_classes.BaseLister):
                         'httpsHealthCheck.port:label=PORT',
                         'httpsHealthCheck.requestPath:label=REQUEST_PATH',
                         'httpsHealthCheck.proxyHeader:label=PROXY_HEADER'])
+      elif (protocol_value ==
+            self.messages.HealthCheck.TypeValueValuesEnum.HTTP2.number):
+        columns.extend(['http2HealthCheck.host:label=HOST',
+                        'http2HealthCheck.port:label=PORT',
+                        'http2HealthCheck.requestPath:label=REQUEST_PATH',
+                        'http2HealthCheck.proxyHeader:label=PROXY_HEADER'])
       elif (protocol_value ==
             self.messages.HealthCheck.TypeValueValuesEnum.TCP.number):
         columns.extend(['tcpHealthCheck.port:label=PORT',
@@ -111,91 +118,6 @@ class List(base_classes.BaseLister):
   def resource_type(self):
     return 'healthChecks'
 
-  def GetResources(self, args, errors):
-    """Gets a list of global healthcheck resources."""
-    health_checks = lister.GetGlobalResourcesDicts(
-        service=self.service,
-        project=self.project,
-        filter_expr=self.GetFilterExpr(args),
-        http=self.http,
-        batch_url=self.batch_url,
-        errors=errors)
-
-    # If a protocol is specified, check that it is one we support, and convert
-    # it to a number.
-    protocol_value = None
-    if args.protocol is not None:
-      protocol_value = self._ConvertProtocolArgToValue(args)
-      if protocol_value not in self._ProtocolWhitelist():
-        raise exceptions.ToolException(
-            'Invalid health check protocol ' + args.protocol + '.')
-
-    for health_check in health_checks:
-      if (protocol_value is None or
-          health_check['type'] == args.protocol.upper()):
-        yield health_check
-
-
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
-class ListBeta(List):
-  """List health checks in Beta."""
-
-  def _ProtocolWhitelist(self):
-    # Returns a list of whitelisted protocols.
-    whitelist = super(ListBeta, self)._ProtocolWhitelist()
-    whitelist.append(self.messages.HealthCheck.TypeValueValuesEnum.HTTP2.number)
-    return whitelist
-
-  def _GetValidColumns(self, args):
-    """Returns a list of valid columns."""
-    columns = super(ListBeta, self)._GetValidColumns(args)
-    if args.protocol is not None:
-      protocol_value = self._ConvertProtocolArgToValue(args)
-      if (protocol_value ==
-          self.messages.HealthCheck.TypeValueValuesEnum.HTTP2.number):
-        columns.extend(['http2HealthCheck.host:label=HOST',
-                        'http2HealthCheck.port:label=PORT',
-                        'http2HealthCheck.requestPath:label=REQUEST_PATH',
-                        'http2HealthCheck.proxyHeader:label=PROXY_HEADER'])
-    return columns
-
-  def _Format(self, args):
-    columns = self._GetValidColumns(args)
-    return 'table[]({columns})'.format(columns=','.join(columns))
-
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class ListAlpha(base_classes.MultiScopeLister, ListBeta):
-  """List health checks in Alpha."""
-
-  @staticmethod
-  def Args(parser):
-    lister.AddMultiScopeListerFlags(parser, regional=True, global_=True)
-
-    parser.add_argument(
-        '--protocol',
-        help="""\
-        If protocol is specified, only health checks for that protocol are
-        listed, and protocol-specific columns are added to the output. By
-        default, health checks for all protocols are listed.
-        """)
-
-  def _ProtocolWhitelist(self):
-    # Returns a list of whitelisted protocols.
-    whitelist = super(ListAlpha, self)._ProtocolWhitelist()
-    return whitelist
-
-  def _Format(self, args):
-    columns = super(ListAlpha, self)._GetValidColumns(args)
-    if args.protocol is not None:
-      protocol_value = self._ConvertProtocolArgToValue(args)
-      if (protocol_value ==
-          self.messages.HealthCheck.TypeValueValuesEnum.UDP.number):
-        columns.extend(['udpHealthCheck.port:label=PORT',
-                        'udpHealthCheck.request:label=REQUEST',
-                        'udpHealthCheck.response:label=RESPONSE'])
-    return 'table[]({columns})'.format(columns=','.join(columns))
-
   @property
   def global_service(self):
     """The service used to list global resources."""
@@ -217,7 +139,7 @@ class ListAlpha(base_classes.MultiScopeLister, ListBeta):
     return self.compute.healthChecks
 
   def GetResources(self, args, errors):
-    health_checks = super(ListAlpha, self).GetResources(args, errors)
+    health_checks = super(List, self).GetResources(args, errors)
 
     # If a protocol is specified, check that it is one we support, and convert
     # it to a number.
@@ -235,6 +157,34 @@ class ListAlpha(base_classes.MultiScopeLister, ListBeta):
         yield health_check
 
 
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class ListAlpha(List):
+  """List health checks in Alpha."""
+
+  def _ProtocolWhitelist(self):
+    # Returns a list of whitelisted protocols.
+    whitelist = super(ListAlpha, self)._ProtocolWhitelist()
+    whitelist.append(self.messages.HealthCheck.TypeValueValuesEnum.GRPC.number)
+    return whitelist
+
+  def _Format(self, args):
+    columns = super(ListAlpha, self)._GetValidColumns(args)
+    if args.protocol is not None:
+      protocol_value = self._ConvertProtocolArgToValue(args)
+      if (protocol_value ==
+          self.messages.HealthCheck.TypeValueValuesEnum.GRPC.number):
+        columns.extend([
+            'grpcHealthCheck.port:label=PORT',
+            'grpcHealthCheck.grpcServiceName:label=GRPC_SERVICE_NAME'])
+      elif (protocol_value ==
+            self.messages.HealthCheck.TypeValueValuesEnum.UDP.number):
+        columns.extend([
+            'udpHealthCheck.port:label=PORT',
+            'udpHealthCheck.request:label=REQUEST',
+            'udpHealthCheck.response:label=RESPONSE'
+        ])
+    return 'table[]({columns})'.format(columns=','.join(columns))
+
+
 List.detailed_help = base_classes.GetGlobalListerHelp('health checks')
-ListBeta.detailed_help = base_classes.GetGlobalListerHelp('health checks')
 ListAlpha.detailed_help = base_classes.GetGlobalListerHelp('health checks')
