@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2014 Google Inc. All Rights Reserved.
+# Copyright 2014 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,10 +31,10 @@ from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.core import resources
 import six
 
+POLL_TIMEOUT = 36000  # 10 hours is recommended by PD team b/131850402#comment20
 
-def _Args(parser, release_track, supports_force_create=False,
-          supports_storage_location=False,
-          supports_shielded_instance_initial_state=False):
+
+def _Args(parser, messages, supports_force_create=False):
   """Set Args based on Release Track."""
   # GA Args
   parser.display_info.AddFormat(flags.LIST_FORMAT)
@@ -52,7 +52,7 @@ def _Args(parser, release_track, supports_force_create=False,
   flags.AddCloningImagesArgs(parser, sources_group)
   flags.AddCreatingImageFromSnapshotArgs(parser, sources_group)
 
-  image_utils.AddGuestOsFeaturesArg(parser, release_track)
+  image_utils.AddGuestOsFeaturesArg(parser, messages)
   kms_resource_args.AddKmsKeyResourceArg(parser, 'image')
 
   # Alpha and Beta Args
@@ -60,11 +60,16 @@ def _Args(parser, release_track, supports_force_create=False,
     # Deprecated as of Aug 2017.
     flags.MakeForceCreateArg().AddToParser(parser)
 
-  if supports_storage_location:
-    compute_flags.AddStorageLocationFlag(parser, 'image')
+  parser.add_argument(
+      '--storage-location',
+      metavar='LOCATION',
+      help="""\
+    Specifies a Cloud Storage location, either regional or multi-regional,
+    where image content is to be stored. If not specified, the multi-region
+    location closest to the source is chosen automatically.
+    """)
 
-  if supports_shielded_instance_initial_state:
-    compute_flags.AddShieldedInstanceInitialStateKeyArg(parser)
+  compute_flags.AddShieldedInstanceInitialStateKeyArg(parser)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
@@ -75,16 +80,20 @@ class Create(base.CreateCommand):
 
   @classmethod
   def Args(cls, parser):
-    _Args(parser, cls.ReleaseTrack())
+    messages = cls._GetApiHolder(no_http=True).client.messages
+    _Args(parser, messages)
     parser.display_info.AddCacheUpdater(flags.ImagesCompleter)
 
+  @classmethod
+  def _GetApiHolder(cls, no_http=False):
+    return base_classes.ComputeApiHolder(cls.ReleaseTrack(), no_http)
+
   def Run(self, args):
     return self._Run(args)
 
-  def _Run(self, args, supports_storage_location=False,
-           supports_shielded_instance_initial_state=False):
+  def _Run(self, args):
     """Returns a list of requests necessary for adding images."""
-    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
+    holder = self._GetApiHolder()
     client = holder.client
     messages = client.messages
     resource_parser = holder.resources
@@ -134,7 +143,7 @@ class Create(base.CreateCommand):
           csek_keys, source_image_ref, client.apitools_client)
 
     if args.source_uri:
-      source_uri = str(resources.REGISTRY.Parse(args.source_uri))
+      source_uri = six.text_type(resources.REGISTRY.Parse(args.source_uri))
       image.rawDisk = messages.Image.RawDiskValue(source=source_uri)
     elif args.source_disk:
       source_disk_ref = flags.SOURCE_DISK_ARG.ResolveAsResource(
@@ -165,13 +174,12 @@ class Create(base.CreateCommand):
         guest_os_feature_messages.append(guest_os_feature)
       image.guestOsFeatures = guest_os_feature_messages
 
-    if supports_shielded_instance_initial_state:
-      initial_state, has_set =\
-          image_utils.CreateInitialStateConfig(args, messages)
-      if has_set:
-        image.shieldedInstanceInitialState = initial_state
+    initial_state, has_set =\
+        image_utils.CreateInitialStateConfig(args, messages)
+    if has_set:
+      image.shieldedInstanceInitialState = initial_state
 
-    if (supports_storage_location and args.IsSpecified('storage_location')):
+    if args.IsSpecified('storage_location'):
       image.storageLocations = [args.storage_location]
 
     request = messages.ComputeImagesInsertRequest(
@@ -191,7 +199,7 @@ class Create(base.CreateCommand):
       request.forceCreate = True
 
     return client.MakeRequests([(client.apitools_client.images, 'Insert',
-                                 request)])
+                                 request)], timeout=POLL_TIMEOUT)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
@@ -204,9 +212,15 @@ class CreateBeta(Create):
 
   @classmethod
   def Args(cls, parser):
-    _Args(parser, cls.ReleaseTrack(), supports_force_create=True)
+    messages = cls._GetApiHolder(no_http=True).client.messages
+    _Args(parser,
+          messages,
+          supports_force_create=True)
     parser.display_info.AddCacheUpdater(flags.ImagesCompleter)
 
+  def Run(self, args):
+    return self._Run(args)
+
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class CreateAlpha(Create):
@@ -216,16 +230,14 @@ class CreateAlpha(Create):
 
   @classmethod
   def Args(cls, parser):
+    messages = cls._GetApiHolder(no_http=True).client.messages
     _Args(parser,
-          cls.ReleaseTrack(),
-          supports_force_create=True,
-          supports_storage_location=True,
-          supports_shielded_instance_initial_state=True)
+          messages,
+          supports_force_create=True)
     parser.display_info.AddCacheUpdater(flags.ImagesCompleter)
 
   def Run(self, args):
-    return self._Run(args, supports_storage_location=True,
-                     supports_shielded_instance_initial_state=True)
+    return self._Run(args)
 
 
 Create.detailed_help = {
@@ -246,7 +258,27 @@ Create.detailed_help = {
         projects.
 
         To learn more about creating image tarballs, visit
-        [](https://cloud.google.com/compute/docs/creating-custom-image)
+        [](https://cloud.google.com/compute/docs/creating-custom-image).
+        """,
+    'EXAMPLES':
+        """\
+        To create an image 'my-image' from a disk 'my-disk' in zone 'us-east1-a', run:
+
+            $ {command} my-image --source-disk=my-disk --source-disk-zone=us-east1-a
+
+        To create an image 'my-image' from another image 'source-image'
+        with source image project 'source-image-project', run:
+
+            $ {command} my-image --source-image=source-image --source-image-project=source-image-project
+
+        To create an image 'my-image' from the latest non-deprecated image in the family 'source-image-family'
+        with source image project 'source-image-project', run:
+
+            $ {command} my-image --source-image-family=source-image-family --source-image-project=source-image-project
+
+        To create an image 'my-image' from a snapshot 'source-snapshot', run:
+
+            $ {command} my-image --source-snapshot=source-snapshot
         """,
 }
 
