@@ -1,4 +1,5 @@
-# Copyright 2014 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2014 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,16 +13,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Command for listing unmanaged instance groups."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import instance_groups_utils
+from googlecloudsdk.api_lib.compute import lister
+from googlecloudsdk.api_lib.compute import utils
+from googlecloudsdk.calliope import base
 
 
-class List(base_classes.ZonalLister):
+class List(base.ListCommand):
   """List Google Compute Engine unmanaged instance groups."""
 
   @staticmethod
   def Args(parser):
-    base_classes.ZonalLister.Args(parser)
     parser.display_info.AddFormat("""
           table(
             name,
@@ -32,33 +40,26 @@ class List(base_classes.ZonalLister):
             size:label=INSTANCES
           )
     """)
+    parser.display_info.AddUriFunc(utils.MakeGetUriFunc())
+    lister.AddZonalListerArgs(parser)
 
-  def GetResources(self, args, errors):
-    resources = super(List, self).GetResources(args, errors)
-    return (resource for resource in resources if resource.zone)
+  def Run(self, args):
+    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
+    client = holder.client
+
+    request_data = lister.ParseZonalFlags(args, holder.resources)
+
+    list_implementation = lister.ZonalLister(
+        client, client.apitools_client.instanceGroups)
+
+    results = lister.Invoke(request_data, list_implementation)
+    results = (resource for resource in results if 'zone' in resource)
 
-  def ComputeDynamicProperties(self, args, items):
-    mode = (
-        instance_groups_utils.InstanceGroupFilteringMode.ONLY_UNMANAGED_GROUPS)
     return instance_groups_utils.ComputeInstanceGroupManagerMembership(
-        compute=self.compute,
-        resources=self.resources,
-        http=self.http,
-        batch_url=self.batch_url,
-        items=items,
-        filter_mode=mode)
-
-  def Collection(self):
-    """Override the default collection from the base class."""
-    return None
-
-  @property
-  def service(self):
-    return self.compute.instanceGroups
-
-  @property
-  def resource_type(self):
-    return 'instanceGroups'
+        compute_holder=holder,
+        items=results,
+        filter_mode=instance_groups_utils.InstanceGroupFilteringMode.
+        ONLY_UNMANAGED_GROUPS)
 
 
 List.detailed_help = base_classes.GetZonalListerHelp('unmanaged '
