@@ -23,13 +23,26 @@ from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute.instances import flags
 
+DETAILED_HELP = {
+    'brief': 'Reset a virtual machine instance.',
+    'DESCRIPTION':
+        """\
+          *{command}* is used to perform a hard reset on a Google
+        Compute Engine virtual machine.
+
+        This will not perform a clean shutdown of the guest OS on the instance.
+        """,
+    'EXAMPLES':
+        """\
+        To reset an instance named ``test-instance'', run:
+
+          $ {command} test-instance
+        """
+}
+
 
 class Reset(base.SilentCommand):
-  """Reset a virtual machine instance.
-
-    *{command}* is used to perform a hard reset on a Google
-  Compute Engine virtual machine.
-  """
+  """Reset a virtual machine instance."""
 
   @staticmethod
   def Args(parser):
@@ -51,3 +64,6 @@ class Reset(base.SilentCommand):
 
       request_list.append((client.apitools_client.instances, 'Reset', request))
     return client.MakeRequests(request_list)
+
+
+Reset.detailed_help = DETAILED_HELP
