@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,7 +22,18 @@ from googlecloudsdk.calliope import base
 
 
 class Interconnects(base.Group):
-  """Read and manipulate Google Compute Engine interconnects."""
+  """Read and manipulate Compute Engine interconnects."""
 
 
 Interconnects.category = base.NETWORKING_CATEGORY
+
+Interconnects.detailed_help = {
+    'DESCRIPTION': """
+        Read and manipulate Cloud Interconnect connections.
+
+        For more information about Cloud Interconnect, see the
+        [Cloud Interconnect documentation](https://cloud.google.com/interconnect/docs/concepts/overview).
+
+        See also: [Interconnects API](https://cloud.google.com/compute/docs/reference/rest/v1/interconnects).
+    """,
+}
