@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,3 +25,15 @@ from googlecloudsdk.calliope import base
 class InterconnectAttachments(base.Group):
   """Create or manipulate partner interconnect attachments."""
   pass
+
+InterconnectAttachments.detailed_help = {
+    'DESCRIPTION': """
+        Create or manipulate Partner Interconnect attachments.
+
+        For more information about interconnect attachments for Partner
+        Interconnect, see the documentation for
+        [Partner interconnect attachments](https://cloud.google.com/interconnect/docs/how-to/dedicated/creating-vlan-attachments).
+
+        See also: [Interconnect attachments API](https://cloud.google.com/compute/docs/reference/rest/v1/interconnectAttachments).
+    """,
+}
