@@ -43,7 +43,8 @@ def _DetailedHelp():
 
 
 def _AddArgs(parser, include_alpha_logging, include_l7_internal_load_balancing,
-             include_private_ipv6_access):
+             include_private_ipv6_access_alpha,
+             include_private_ipv6_access_beta, include_aggregate_purpose):
   """Add subnetwork create arguments to parser."""
   parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
 
@@ -88,8 +89,8 @@ def _AddArgs(parser, include_alpha_logging, include_l7_internal_load_balancing,
       '--enable-flow-logs',
       action='store_true',
       default=None,
-      help=('Enable/disable VPC flow logging for this subnet. More information '
-            'for VPC flow logs can be found at '
+      help=('Enable/disable VPC Flow Logs for this subnet. More information '
+            'for VPC Flow Logs can be found at '
             'https://cloud.google.com/vpc/docs/using-flow-logs.'))
 
   flags.AddLoggingAggregationInterval(parser, messages)
@@ -97,18 +98,35 @@ def _AddArgs(parser, include_alpha_logging, include_l7_internal_load_balancing,
       '--logging-flow-sampling',
       type=arg_parsers.BoundedFloat(lower_bound=0.0, upper_bound=1.0),
       help="""\
-      Can only be specified if VPC flow logging for this subnetwork is
+      Can only be specified if VPC Flow Logs for this subnetwork is
       enabled. The value of the field must be in [0, 1]. Set the sampling rate
       of VPC flow logs within the subnetwork where 1.0 means all collected
       logs are reported and 0.0 means no logs are reported. Default is 0.5
       which means half of all collected logs are reported.
       """)
+
+  parser.add_argument(
+      '--logging-filter-expr',
+      help="""\
+        Can only be specified if VPC Flow Logs for this subnetwork is enabled.
+        Export filter used to define which logs should be generated.
+        """)
   flags.AddLoggingMetadata(parser, messages)
+  parser.add_argument(
+      '--logging-metadata-fields',
+      type=arg_parsers.ArgList(),
+      metavar='METADATA_FIELD',
+      default=None,
+      help="""\
+      Can only be specified if VPC Flow Logs for this subnetwork is enabled
+      and "metadata" is set to CUSTOM_METADATA. The comma-separated list of
+      metadata fields that should be added to reported logs.
+      """)
 
   if include_alpha_logging:
     messages = apis.GetMessagesModule('compute',
                                       compute_api.COMPUTE_ALPHA_API_VERSION)
-    flags.AddLoggingAggregationIntervalAlpha(parser, messages)
+    flags.AddLoggingAggregationIntervalDeprecated(parser, messages)
     parser.add_argument(
         '--flow-sampling',
         type=arg_parsers.BoundedFloat(lower_bound=0.0, upper_bound=1.0),
@@ -119,20 +137,32 @@ def _AddArgs(parser, include_alpha_logging, include_l7_internal_load_balancing,
         logs are reported and 0.0 means no logs are reported. Default is 0.5
         which means half of all collected logs are reported.
         """)
-    flags.AddLoggingMetadataAlpha(parser, messages)
+    flags.AddLoggingMetadataDeprecated(parser, messages)
 
+  purpose_choices = {
+      'PRIVATE':
+          'Regular user created or automatically created subnet.',
+      'INTERNAL_HTTPS_LOAD_BALANCER':
+          'Reserved for Internal HTTP(S) Load Balancing.',
+  }
+
+  if include_aggregate_purpose:
+    purpose_choices['AGGREGATE'] = (
+        'Reserved for Aggregate Ranges used for aggregating '
+        'private subnetworks.')
+
+  # Subnetwork purpose is introduced with L7ILB feature. Aggregate purpose
+  # will have to be enabled for a given release track only after L7ILB feature
+  # is enabled for that release track. Hence if include_aggregate_purpose
+  # true, this code assumes that L7ILB purpose is enabled.
   if include_l7_internal_load_balancing:
     parser.add_argument(
         '--purpose',
-        choices={
-            'PRIVATE':
-                'Regular user created or automatically created subnet.',
-            'INTERNAL_HTTPS_LOAD_BALANCER':
-                'Reserved for Internal HTTP(S) Load Balancing.'
-        },
-        type=lambda x: x.replace('-', '_').upper(),
+        choices=purpose_choices,
+        type=arg_utils.ChoiceToEnumName,
         help='The purpose of this subnetwork.')
 
+  if include_l7_internal_load_balancing:
     parser.add_argument(
         '--role',
         choices={
@@ -140,22 +170,24 @@ def _AddArgs(parser, include_alpha_logging, include_l7_internal_load_balancing,
             'BACKUP': 'The BACKUP subnet that could be promoted to ACTIVE.'
         },
         type=lambda x: x.replace('-', '_').upper(),
-        help=('The role of subnetwork. This field is only used when'
+        help=('The role of subnetwork. This field is required when '
               'purpose=INTERNAL_HTTPS_LOAD_BALANCER. The value can be set to '
               'ACTIVE or BACKUP. An ACTIVE subnetwork is one that is currently '
               'being used for Internal HTTP(S) Load Balancing. A BACKUP '
               'subnetwork is one that is ready to be promoted to ACTIVE or is '
               'currently draining.'))
 
-  if include_private_ipv6_access:
+  if include_private_ipv6_access_alpha:
+    messages = apis.GetMessagesModule('compute',
+                                      compute_api.COMPUTE_ALPHA_API_VERSION)
     parser.add_argument(
         '--enable-private-ipv6-access',
         action='store_true',
         default=None,
         help=('Enable/disable private IPv6 access for the subnet.'))
 
-    GetPrivateIpv6GoogleAccessTypeFlagMapper(messages).choice_arg.AddToParser(
-        parser)
+    GetPrivateIpv6GoogleAccessTypeFlagMapperAlpha(
+        messages).choice_arg.AddToParser(parser)
 
     parser.add_argument(
         '--private-ipv6-google-access-service-accounts',
@@ -167,11 +199,16 @@ def _AddArgs(parser, include_alpha_logging, include_l7_internal_load_balancing,
         Google Access only on the VMs primary service account matching the
         value.
         """)
+  elif include_private_ipv6_access_beta:
+    messages = apis.GetMessagesModule('compute',
+                                      compute_api.COMPUTE_BETA_API_VERSION)
+    GetPrivateIpv6GoogleAccessTypeFlagMapperBeta(
+        messages).choice_arg.AddToParser(parser)
 
   parser.display_info.AddCacheUpdater(network_flags.NetworksCompleter)
 
 
-def GetPrivateIpv6GoogleAccessTypeFlagMapper(messages):
+def GetPrivateIpv6GoogleAccessTypeFlagMapperAlpha(messages):
   return arg_utils.ChoiceEnumMapper(
       '--private-ipv6-google-access-type',
       messages.Subnetwork.PrivateIpv6GoogleAccessValueValuesEnum,
@@ -189,9 +226,27 @@ def GetPrivateIpv6GoogleAccessTypeFlagMapper(messages):
   )
 
 
+def GetPrivateIpv6GoogleAccessTypeFlagMapperBeta(messages):
+  return arg_utils.ChoiceEnumMapper(
+      '--private-ipv6-google-access-type',
+      messages.Subnetwork.PrivateIpv6GoogleAccessValueValuesEnum,
+      custom_mappings={
+          'DISABLE_GOOGLE_ACCESS':
+              'disable',
+          'ENABLE_BIDIRECTIONAL_ACCESS_TO_GOOGLE':
+              'enable-bidirectional-access',
+          'ENABLE_OUTBOUND_VM_ACCESS_TO_GOOGLE':
+              'enable-outbound-vm-access'
+      },
+      help_str='The private IPv6 google access type for the VMs in this subnet.'
+  )
+
+
 def _CreateSubnetwork(messages, subnet_ref, network_ref, args,
                       include_alpha_logging, include_l7_internal_load_balancing,
-                      include_private_ipv6_access):
+                      include_private_ipv6_access_alpha,
+                      include_private_ipv6_access_beta,
+                      include_aggregate_purpose):
   """Create the subnet resource."""
   subnetwork = messages.Subnetwork(
       name=subnet_ref.Name(),
@@ -203,7 +258,9 @@ def _CreateSubnetwork(messages, subnet_ref, network_ref, args,
   if (args.enable_flow_logs is not None or
       args.logging_aggregation_interval is not None or
       args.logging_flow_sampling is not None or
-      args.logging_metadata is not None):
+      args.logging_metadata is not None or
+      args.logging_filter_expr is not None or
+      args.logging_metadata_fields is not None):
     log_config = messages.SubnetworkLogConfig(enable=args.enable_flow_logs)
     if args.logging_aggregation_interval:
       log_config.aggregationInterval = flags.GetLoggingAggregationIntervalArg(
@@ -213,6 +270,10 @@ def _CreateSubnetwork(messages, subnet_ref, network_ref, args,
     if args.logging_metadata:
       log_config.metadata = flags.GetLoggingMetadataArg(
           messages).GetEnumForChoice(args.logging_metadata)
+    if args.logging_filter_expr is not None:
+      log_config.filterExpr = args.logging_filter_expr
+    if args.logging_metadata_fields is not None:
+      log_config.metadataFields = args.logging_metadata_fields
     subnetwork.logConfig = log_config
 
   if include_alpha_logging:
@@ -224,13 +285,17 @@ def _CreateSubnetwork(messages, subnet_ref, network_ref, args,
           messages.SubnetworkLogConfig(enable=args.enable_flow_logs))
       if args.aggregation_interval:
         log_config.aggregationInterval = (
-            flags.GetLoggingAggregationIntervalArgAlpha(
+            flags.GetLoggingAggregationIntervalArgDeprecated(
                 messages).GetEnumForChoice(args.aggregation_interval))
       if args.flow_sampling is not None:
         log_config.flowSampling = args.flow_sampling
       if args.metadata:
-        log_config.metadata = flags.GetLoggingMetadataArgAlpha(
+        log_config.metadata = flags.GetLoggingMetadataArgDeprecated(
             messages).GetEnumForChoice(args.metadata)
+      if args.logging_filter_expr is not None:
+        log_config.filterExpr = args.logging_filter_expr
+      if args.logging_metadata_fields is not None:
+        log_config.metadataFields = args.logging_metadata_fields
       subnetwork.logConfig = log_config
 
   if include_l7_internal_load_balancing:
@@ -246,22 +311,42 @@ def _CreateSubnetwork(messages, subnet_ref, network_ref, args,
     if getattr(args, 'role', None):
       subnetwork.role = messages.Subnetwork.RoleValueValuesEnum(args.role)
 
-  if include_private_ipv6_access:
+  # At present aggregate purpose is available only in alpha whereas
+  # https_load_balancer is available in Beta. Given Aggregate Purpose Enum
+  # is not available in Beta, the code duplication below is necessary.
+  if include_aggregate_purpose:
+    if args.purpose:
+      subnetwork.purpose = messages.Subnetwork.PurposeValueValuesEnum(
+          args.purpose)
+      if (subnetwork.purpose ==
+          messages.Subnetwork.PurposeValueValuesEnum.AGGREGATE):
+        # Clear unsupported fields in the subnet resource
+        subnetwork.privateIpGoogleAccess = None
+        subnetwork.enableFlowLogs = None
+        subnetwork.logConfig = None
+
+  if include_private_ipv6_access_alpha:
     if args.enable_private_ipv6_access is not None:
       subnetwork.enablePrivateV6Access = args.enable_private_ipv6_access
     if args.private_ipv6_google_access_type is not None:
       subnetwork.privateIpv6GoogleAccess = (
-          flags.GetPrivateIpv6GoogleAccessTypeFlagMapper(
+          flags.GetPrivateIpv6GoogleAccessTypeFlagMapperAlpha(
               messages).GetEnumForChoice(args.private_ipv6_google_access_type))
     if args.private_ipv6_google_access_service_accounts is not None:
       subnetwork.privateIpv6GoogleAccessServiceAccounts = (
           args.private_ipv6_google_access_service_accounts)
+  elif include_private_ipv6_access_beta:
+    if args.private_ipv6_google_access_type is not None:
+      subnetwork.privateIpv6GoogleAccess = (
+          flags.GetPrivateIpv6GoogleAccessTypeFlagMapperBeta(
+              messages).GetEnumForChoice(args.private_ipv6_google_access_type))
 
   return subnetwork
 
 
 def _Run(args, holder, include_alpha_logging,
-         include_l7_internal_load_balancing, include_private_ipv6_access):
+         include_l7_internal_load_balancing, include_private_ipv6_access_alpha,
+         include_private_ipv6_access_beta, include_aggregate_purpose):
   """Issues a list of requests necessary for adding a subnetwork."""
   client = holder.client
 
@@ -276,7 +361,9 @@ def _Run(args, holder, include_alpha_logging,
   subnetwork = _CreateSubnetwork(client.messages, subnet_ref, network_ref, args,
                                  include_alpha_logging,
                                  include_l7_internal_load_balancing,
-                                 include_private_ipv6_access)
+                                 include_private_ipv6_access_alpha,
+                                 include_private_ipv6_access_beta,
+                                 include_aggregate_purpose)
   request = client.messages.ComputeSubnetworksInsertRequest(
       subnetwork=subnetwork,
       region=subnet_ref.region,
@@ -300,8 +387,11 @@ class Create(base.CreateCommand):
   """Create a GA subnet."""
 
   _include_alpha_logging = False
-  _include_l7_internal_load_balancing = False
-  _include_private_ipv6_access = False
+  # TODO(b/144022508): Remove _include_l7_internal_load_balancing
+  _include_l7_internal_load_balancing = True
+  _include_private_ipv6_access_alpha = False
+  _include_private_ipv6_access_beta = False
+  _include_aggregate_purpose = False
 
   detailed_help = _DetailedHelp()
 
@@ -309,24 +399,29 @@ class Create(base.CreateCommand):
   def Args(cls, parser):
     _AddArgs(parser, cls._include_alpha_logging,
              cls._include_l7_internal_load_balancing,
-             cls._include_private_ipv6_access)
+             cls._include_private_ipv6_access_alpha,
+             cls._include_private_ipv6_access_beta,
+             cls._include_aggregate_purpose)
 
   def Run(self, args):
     """Issues a list of requests necessary for adding a subnetwork."""
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
     return _Run(args, holder, self._include_alpha_logging,
                 self._include_l7_internal_load_balancing,
-                self._include_private_ipv6_access)
+                self._include_private_ipv6_access_alpha,
+                self._include_private_ipv6_access_beta,
+                self._include_aggregate_purpose)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
 class CreateBeta(Create):
 
-  _include_l7_internal_load_balancing = True
+  _include_private_ipv6_access_beta = True
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class CreateAlpha(CreateBeta):
 
   _include_alpha_logging = True
-  _include_private_ipv6_access = True
+  _include_private_ipv6_access_alpha = True
+  _include_aggregate_purpose = True
