@@ -29,8 +29,10 @@ class Update(base.UpdateCommand):
   """Updates properties of an existing Google Compute Engine subnetwork."""
 
   _include_alpha_logging = False
-  _include_l7_internal_load_balancing = False
-  _include_private_ipv6_access = False
+  # TODO(b/144022508): Remove _include_l7_internal_load_balancing
+  _include_l7_internal_load_balancing = True
+  _include_private_ipv6_access_alpha = False
+  _include_private_ipv6_access_beta = False
 
   @classmethod
   def Args(cls, parser):
@@ -44,7 +46,8 @@ class Update(base.UpdateCommand):
 
     flags.AddUpdateArgs(parser, cls._include_alpha_logging,
                         cls._include_l7_internal_load_balancing,
-                        cls._include_private_ipv6_access)
+                        cls._include_private_ipv6_access_alpha,
+                        cls._include_private_ipv6_access_beta)
 
   def Run(self, args):
     """Issues requests necessary to update Subnetworks."""
@@ -55,6 +58,8 @@ class Update(base.UpdateCommand):
     aggregation_interval = args.logging_aggregation_interval
     flow_sampling = args.logging_flow_sampling
     metadata = args.logging_metadata
+    filter_expr = args.logging_filter_expr
+    metadata_fields = args.logging_metadata_fields
 
     if self._include_alpha_logging:
       if args.aggregation_interval is not None:
@@ -74,23 +79,26 @@ class Update(base.UpdateCommand):
     enable_private_ipv6_access = None
     private_ipv6_google_access_type = None
     private_ipv6_google_access_service_accounts = None
-    if self._include_private_ipv6_access:
+    if self._include_private_ipv6_access_alpha:
       enable_private_ipv6_access = args.enable_private_ipv6_access
       private_ipv6_google_access_type = args.private_ipv6_google_access_type
       private_ipv6_google_access_service_accounts = (
           args.private_ipv6_google_access_service_accounts)
+    elif self._include_private_ipv6_access_beta:
+      private_ipv6_google_access_type = args.private_ipv6_google_access_type
 
     return subnets_utils.MakeSubnetworkUpdateRequest(
         client,
         subnet_ref,
-        self._include_alpha_logging,
         enable_private_ip_google_access=args.enable_private_ip_google_access,
         add_secondary_ranges=args.add_secondary_ranges,
         remove_secondary_ranges=args.remove_secondary_ranges,
+        enable_flow_logs=args.enable_flow_logs,
         aggregation_interval=aggregation_interval,
         flow_sampling=flow_sampling,
         metadata=metadata,
-        enable_flow_logs=args.enable_flow_logs,
+        filter_expr=filter_expr,
+        metadata_fields=metadata_fields,
         set_role_active=set_role_active,
         drain_timeout_seconds=drain_timeout_seconds,
         enable_private_ipv6_access=enable_private_ipv6_access,
@@ -103,7 +111,7 @@ class Update(base.UpdateCommand):
 class UpdateBeta(Update):
   """Updates properties of an existing Google Compute Engine subnetwork."""
 
-  _include_l7_internal_load_balancing = True
+  _include_private_ipv6_access_beta = True
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@@ -111,4 +119,4 @@ class UpdateAlpha(UpdateBeta):
   """Updates properties of an existing Google Compute Engine subnetwork."""
 
   _include_alpha_logging = True
-  _include_private_ipv6_access = True
+  _include_private_ipv6_access_alpha = True
