@@ -1,4 +1,5 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +15,10 @@
 
 """Command for removing a BGP peer from a Google Compute Engine router."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import encoding
 
 from googlecloudsdk.api_lib.compute import base_classes
@@ -59,14 +64,15 @@ class RemoveBgpPeer(base.UpdateCommand):
 
   def GetSetRequest(self, client, router_ref, replacement):
     return (client.apitools_client.routers,
-            'Update',
-            client.messages.ComputeRoutersUpdateRequest(
+            'Patch',
+            client.messages.ComputeRoutersPatchRequest(
                 router=router_ref.Name(),
                 routerResource=replacement,
                 region=router_ref.region,
                 project=router_ref.project))
 
-  def Modify(self, args, existing):
+  def Modify(self, args, existing, cleared_fields):
+    """Mutate the router and record any cleared_fields for Patch request."""
     replacement = encoding.CopyProtoMessage(existing)
 
     # remove peer if exists
@@ -75,6 +81,8 @@ class RemoveBgpPeer(base.UpdateCommand):
       if p.name == args.peer_name:
         peer = p
         replacement.bgpPeers.remove(peer)
+        if not replacement.bgpPeers:
+          cleared_fields.append('bgpPeers')
         break
 
     if peer is None:
@@ -91,8 +99,12 @@ class RemoveBgpPeer(base.UpdateCommand):
 
     objects = client.MakeRequests([get_request])
 
-    # There is only one response because one request is made above
-    new_object = self.Modify(args, objects[0])
+    # Cleared list fields need to be explicitly identified for Patch API.
+    cleared_fields = []
+    new_object = self.Modify(args, objects[0], cleared_fields)
 
-    return client.MakeRequests(
-        [self.GetSetRequest(client, router_ref, new_object)])
+    with client.apitools_client.IncludeFields(cleared_fields):
+      # There is only one response because one request is made above
+      result = client.MakeRequests(
+          [self.GetSetRequest(client, router_ref, new_object)])
+    return result
