@@ -12,7 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """Command for creating routes."""
 
 from __future__ import absolute_import
@@ -62,12 +61,11 @@ def _AddGaHops(next_hop_group):
       help=('The target VPN tunnel that will receive forwarded traffic.'))
 
 
-def _Args(parser, support_next_hop_ilb=False):
+def _Args(parser):
   """Add arguments for route creation."""
 
   parser.add_argument(
-      '--description',
-      help='An optional, textual description for the route.')
+      '--description', help='An optional, textual description for the route.')
 
   parser.add_argument(
       '--network',
@@ -117,26 +115,25 @@ def _Args(parser, support_next_hop_ilb=False):
       help=('The region of the next hop vpn tunnel. ' +
             compute_flags.REGION_PROPERTY_EXPLANATION))
 
-  if support_next_hop_ilb:
-    next_hop.add_argument(
-        '--next-hop-ilb',
-        help="""\
-        The target forwarding rule that will receive forwarded traffic. This
-        can only be used when the destination_range is a public (non-RFC 1918)
-        IP CIDR range. Requires --load-balancing-scheme=INTERNAL on the
-        corresponding forwarding rule.
-        """)
-    parser.add_argument(
-        '--next-hop-ilb-region',
-        help=('The region of the next hop forwarding rule. ' +
-              compute_flags.REGION_PROPERTY_EXPLANATION))
+  next_hop.add_argument(
+      '--next-hop-ilb',
+      help="""\
+      The target forwarding rule that will receive forwarded traffic. This
+      can only be used when the destination_range is a public (non-RFC 1918)
+      IP CIDR range. Requires --load-balancing-scheme=INTERNAL on the
+      corresponding forwarding rule.
+      """)
+  parser.add_argument(
+      '--next-hop-ilb-region',
+      help=('The region of the next hop forwarding rule. ' +
+            compute_flags.REGION_PROPERTY_EXPLANATION))
 
   parser.display_info.AddCacheUpdater(completers.RoutesCompleter)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
 class Create(base.CreateCommand):
-  """Create a new route.
+  r"""Create a new route.
 
     *{command}* is used to create routes. A route is a rule that
   specifies how certain packets should be handled by the virtual
@@ -161,9 +158,17 @@ class Create(base.CreateCommand):
   Exactly one of ``--next-hop-address'', ``--next-hop-gateway'',
   ``--next-hop-vpn-tunnel'', or ``--next-hop-instance'' must be
   provided with this command.
-  """
 
-  _support_next_hop_ilb = False
+  ## EXAMPLES
+
+  To create a route with the name 'route-name' with destination range
+  '0.0.0.0/0' and with next hop gateway 'default-internet-gateway', run:
+
+    $ {command} route-name \
+      --destination-range=0.0.0.0/0 \
+      --next-hop-gateway=default-internet-gateway
+
+  """
 
   NETWORK_ARG = None
   INSTANCE_ARG = None
@@ -179,11 +184,10 @@ class Create(base.CreateCommand):
         required=False)
     cls.INSTANCE_ARG = instance_flags.InstanceArgumentForRoute(required=False)
     cls.VPN_TUNNEL_ARG = vpn_flags.VpnTunnelArgumentForRoute(required=False)
-    if cls._support_next_hop_ilb:
-      cls.ILB_ARG = ilb_flags.ForwardingRuleArgumentForRoute(required=False)
+    cls.ILB_ARG = ilb_flags.ForwardingRuleArgumentForRoute(required=False)
     cls.ROUTE_ARG = flags.RouteArgument()
     cls.ROUTE_ARG.AddArgument(parser, operation_type='create')
-    _Args(parser, support_next_hop_ilb=cls._support_next_hop_ilb)
+    _Args(parser)
 
   def Run(self, args):
     """Issue API requests for route creation, callable from multiple tracks."""
@@ -229,17 +233,15 @@ class Create(base.CreateCommand):
 
     next_hop_ilb_uri = None
 
-    if self._support_next_hop_ilb:
-      if args.next_hop_ilb:
-        next_hop_ilb_uri = self.ILB_ARG.ResolveAsResource(
-            args,
-            holder.resources,
-            scope_lister=compute_flags.GetDefaultScopeLister(
-                client)).SelfLink()
-      elif args.next_hop_ilb_region:
-        raise exceptions.ToolException(
-            '[--next-hop-ilb-region] can only be specified in '
-            'conjunction with [--next-hop-ilb].')
+    if args.next_hop_ilb:
+      next_hop_ilb_uri = self.ILB_ARG.ResolveAsResource(
+          args,
+          holder.resources,
+          scope_lister=compute_flags.GetDefaultScopeLister(client)).SelfLink()
+    elif args.next_hop_ilb_region:
+      raise exceptions.ToolException(
+          '[--next-hop-ilb-region] can only be specified in '
+          'conjunction with [--next-hop-ilb].')
 
     request = client.messages.ComputeRoutesInsertRequest(
         project=route_ref.project,
@@ -255,8 +257,7 @@ class Create(base.CreateCommand):
             priority=args.priority,
             tags=args.tags,
         ))
-    if self._support_next_hop_ilb:
-      request.route.nextHopIlb = next_hop_ilb_uri
+    request.route.nextHopIlb = next_hop_ilb_uri
 
     return client.MakeRequests([(client.apitools_client.routes, 'Insert',
                                  request)])
@@ -264,7 +265,7 @@ class Create(base.CreateCommand):
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
 class CreateAlphaBeta(Create):
-  """Create a new route.
+  r"""Create a new route.
 
     *{command}* is used to create routes. A route is a rule that
   specifies how certain packets should be handled by the virtual
@@ -289,6 +290,14 @@ class CreateAlphaBeta(Create):
   Exactly one of ``--next-hop-address'', ``--next-hop-gateway'',
   ``--next-hop-vpn-tunnel'', ``--next-hop-instance'', or ``--next-hop-ilb''
   must be provided with this command.
-  """
 
-  _support_next_hop_ilb = True
+  ## EXAMPLES
+
+  To create a route with the name 'route-name' with destination range
+  '0.0.0.0/0' and with next hop gateway 'default-internet-gateway', run:
+
+    $ {command} route-name \
+      --destination-range=0.0.0.0/0 \
+      --next-hop-gateway=default-internet-gateway
+
+  """
