@@ -21,6 +21,7 @@ from __future__ import unicode_literals
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import utils as compute_utils
 from googlecloudsdk.calliope import base
+from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 from googlecloudsdk.command_lib.compute.sole_tenancy.node_groups import flags
 from googlecloudsdk.command_lib.compute.sole_tenancy.node_groups import util
@@ -28,7 +29,16 @@ from googlecloudsdk.command_lib.compute.sole_tenancy.node_groups import util
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
 class Create(base.CreateCommand):
-  """Creates a Google Compute Engine node group."""
+  """Create a Compute Engine node group."""
+
+  detailed_help = {
+      'brief': 'Create a Compute Engine node group.',
+      'EXAMPLES': """
+         To create a node group, run:
+
+           $ {command} my-node-group --node-template=example-template --target-size=4
+       """,
+  }
 
   @staticmethod
   def Args(parser):
@@ -59,15 +69,22 @@ class Create(base.CreateCommand):
       maintenance_policy = mapper.GetEnumForChoice(args.maintenance_policy)
       node_group.maintenancePolicy = maintenance_policy
 
-    if hasattr(args, 'autoscaling_policy') and args.autoscaling_policy:
-      mapper = flags.GetAutoscalingModeEnumMapper(messages)
-      mode = mapper.GetEnumForChoice(args.autoscaling_policy['mode'])
-      autoscaling_policy = messages.NodeGroupAutoscalingPolicy(
-          mode=mode,
-          minSize=args.autoscaling_policy.get('min-size'),
-          maxSize=args.autoscaling_policy.get('max-size'))
+    if hasattr(args, 'autoscaler_mode') and args.autoscaler_mode:
+      if args.autoscaler_mode != 'off' and args.max_nodes is None:
+        raise exceptions.RequiredArgumentException('--max-nodes',
+                                                   '--autoscaler-mode is on')
+      autoscaling_policy = util.BuildAutoscaling(args, messages)
       node_group.autoscalingPolicy = autoscaling_policy
 
+    if hasattr(
+        args,
+        'maintenance_window_start_time') and args.maintenance_window_start_time:
+      node_group.maintenanceWindow = messages.NodeGroupMaintenanceWindow(
+          startTime=args.maintenance_window_start_time)
+
+    if hasattr(args, 'location_hint') and args.location_hint:
+      node_group.locationHint = args.location_hint
+
     request = messages.ComputeNodeGroupsInsertRequest(
         nodeGroup=node_group,
         initialNodeCount=args.target_size,
@@ -80,17 +97,25 @@ class Create(base.CreateCommand):
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
 class CreateBeta(Create):
-  """Creates a Google Compute Engine node group."""
-
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class CreateAlpha(CreateBeta):
-  """Creates a Google Compute Engine node group."""
+  """Create a Compute Engine node group."""
 
   @staticmethod
   def Args(parser):
     flags.MakeNodeGroupArg().AddArgument(parser)
     flags.AddCreateArgsToParser(parser)
     flags.AddMaintenancePolicyArgToParser(parser)
-    flags.AddAutoscalingPolicyArgToParser(parser)
+    flags.AddAutoscalingPolicyArgToParser(parser, required_mode=True)
 
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class CreateAlpha(CreateBeta):
+  """Create a Compute Engine node group."""
+
+  @staticmethod
+  def Args(parser):
+    flags.MakeNodeGroupArg().AddArgument(parser)
+    flags.AddCreateArgsToParser(parser)
+    flags.AddMaintenancePolicyArgToParser(parser)
+    flags.AddAutoscalingPolicyArgToParser(parser, required_mode=True)
+    flags.AddMaintenanceWindowArgToParser(parser)
+    flags.AddLocationHintArgToParser(parser)
