@@ -40,7 +40,7 @@ def _Args(parser, include_l7_internal_load_balancing):
   parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
   if include_l7_internal_load_balancing:
     lister.AddMultiScopeListerFlags(parser, regional=True, global_=True)
-    parser.display_info.AddCacheUpdater(flags.TargetHttpProxiesCompleterAlpha)
+    parser.display_info.AddCacheUpdater(flags.TargetHttpProxiesCompleter)
   else:
     lister.AddBaseListerArgs(parser)
     parser.display_info.AddCacheUpdater(flags.TargetHttpProxiesCompleter)
@@ -65,11 +65,13 @@ def _Run(args, holder, include_l7_internal_load_balancing):
   return lister.Invoke(request_data, list_implementation)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA)
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
+                    base.ReleaseTrack.GA)
 class List(base.ListCommand):
   """List target HTTP proxies."""
 
-  _include_l7_internal_load_balancing = False
+  # TODO(b/144022508): Remove _include_l7_internal_load_balancing
+  _include_l7_internal_load_balancing = True
 
   detailed_help = _DetailedHelp(_include_l7_internal_load_balancing)
 
@@ -80,16 +82,3 @@ class List(base.ListCommand):
   def Run(self, args):
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
     return _Run(args, holder, self._include_l7_internal_load_balancing)
-
-
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
-class ListBeta(List):
-
-  _include_l7_internal_load_balancing = True
-
-  detailed_help = _DetailedHelp(_include_l7_internal_load_balancing)
-
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class ListAlpha(ListBeta):
-  pass
