@@ -84,9 +84,9 @@ characters.
   parser.add_argument(
       '--timeout',
       type=int,
-      default=1800,
+      default=3600,
       hidden=True,
-      help='THIS ARGUMENT NEEDS HELP TEXT.')
+      help='Timeout (seconds) for waiting on the operation to complete.')
   flags.AddAsyncFlag(parser)
   parser.add_argument(
       '--num-nodes',
@@ -179,6 +179,9 @@ for examples.
   flags.AddAcceleratorArgs(parser)
   flags.AddDiskTypeFlag(parser)
   flags.AddMetadataFlags(parser)
+  flags.AddDatabaseEncryptionFlag(parser)
+  flags.AddShieldedInstanceFlags(parser)
+  flags.AddEnableShieldedNodesFlags(parser)
 
 
 def ValidateBasicAuthFlags(args):
@@ -206,17 +209,6 @@ def ValidateBasicAuthFlags(args):
 
 def ParseCreateOptionsBase(args):
   """Parses the flags provided with the cluster creation command."""
-  if not (args.IsSpecified('enable_basic_auth') or
-          args.IsSpecified('username')):
-    log.warning('Starting in 1.12, new clusters will have basic '
-                'authentication disabled by default. Basic authentication '
-                'can be enabled (or disabled) manually using the '
-                '`--[no-]enable-basic-auth` flag.')
-  if not args.IsSpecified('issue_client_certificate'):
-    log.warning('Starting in 1.12, new clusters will not have a client '
-                'certificate issued. You can manually enable (or disable) the '
-                'issuance of the client certificate using the '
-                '`--[no-]issue-client-certificate` flag.')
   if args.IsSpecified('addons') and api_adapter.DASHBOARD in args.addons:
     log.warning(
         'The `KubernetesDashboard` addon is deprecated, and will be removed as '
@@ -252,12 +244,14 @@ def ParseCreateOptionsBase(args):
       enable_autorepair=enable_autorepair,
       enable_autoscaling=args.enable_autoscaling,
       enable_autoupgrade=cmd_util.GetAutoUpgrade(args),
+      enable_binauthz=args.enable_binauthz,
       enable_stackdriver_kubernetes=args.enable_stackdriver_kubernetes if args.IsSpecified('enable_stackdriver_kubernetes') else None,
       enable_cloud_logging=args.enable_cloud_logging if args.IsSpecified('enable_cloud_logging') else None,
       enable_cloud_monitoring=args.enable_cloud_monitoring if args.IsSpecified('enable_cloud_monitoring') else None,
       enable_ip_alias=args.enable_ip_alias,
       enable_intra_node_visibility=args.enable_intra_node_visibility,
       enable_kubernetes_alpha=args.enable_kubernetes_alpha,
+      enable_cloud_run_alpha=args.enable_cloud_run_alpha if args.IsSpecified('enable_cloud_run_alpha') else None,
       enable_legacy_authorization=args.enable_legacy_authorization,
       enable_master_authorized_networks=args.enable_master_authorized_networks,
       enable_network_policy=args.enable_network_policy,
@@ -271,6 +265,9 @@ def ParseCreateOptionsBase(args):
       labels=args.labels,
       local_ssd_count=args.local_ssd_count,
       maintenance_window=args.maintenance_window,
+      maintenance_window_start=args.maintenance_window_start,
+      maintenance_window_end=args.maintenance_window_end,
+      maintenance_window_recurrence=args.maintenance_window_recurrence,
       master_authorized_networks=args.master_authorized_networks,
       master_ipv4_cidr=args.master_ipv4_cidr,
       max_nodes=args.max_nodes,
@@ -301,13 +298,52 @@ def ParseCreateOptionsBase(args):
       resource_usage_bigquery_dataset=args.resource_usage_bigquery_dataset,
       enable_network_egress_metering=args.enable_network_egress_metering,
       enable_resource_consumption_metering=\
-          args.enable_resource_consumption_metering)
+          args.enable_resource_consumption_metering,
+      database_encryption_key=args.database_encryption_key,
+      workload_pool=args.workload_pool,
+      workload_metadata=args.workload_metadata,
+      workload_metadata_from_node=args.workload_metadata_from_node,
+      enable_vertical_pod_autoscaling=args.enable_vertical_pod_autoscaling,
+      enable_autoprovisioning=args.enable_autoprovisioning,
+      autoprovisioning_config_file=args.autoprovisioning_config_file,
+      autoprovisioning_service_account=args.autoprovisioning_service_account,
+      autoprovisioning_scopes=args.autoprovisioning_scopes,
+      autoprovisioning_locations=args.autoprovisioning_locations,
+      autoprovisioning_max_surge_upgrade=getattr(args, 'autoprovisioning_max_surge_upgrade', None),
+      autoprovisioning_max_unavailable_upgrade=getattr(args, 'autoprovisioning_max_unavailable_upgrade', None),
+      enable_autoprovisioning_autorepair=getattr(args, 'enable_autoprovisioning_autorepair', None),
+      enable_autoprovisioning_autoupgrade=getattr(args, 'enable_autoprovisioning_autoupgrade', None),
+      autoprovisioning_min_cpu_platform=getattr(args, 'autoprovisioning_min_cpu_platform', None),
+      min_cpu=args.min_cpu,
+      max_cpu=args.max_cpu,
+      min_memory=args.min_memory,
+      max_memory=args.max_memory,
+      min_accelerator=args.min_accelerator,
+      max_accelerator=args.max_accelerator,
+      shielded_secure_boot=args.shielded_secure_boot,
+      shielded_integrity_monitoring=args.shielded_integrity_monitoring,
+      reservation_affinity=getattr(args, 'reservation_affinity', None),
+      reservation=getattr(args, 'reservation', None),
+      enable_shielded_nodes=args.enable_shielded_nodes,
+      max_surge_upgrade=args.max_surge_upgrade,
+      max_unavailable_upgrade=args.max_unavailable_upgrade)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
 class Create(base.CreateCommand):
   """Create a cluster for running containers."""
 
+  detailed_help = {
+      'DESCRIPTION':
+          '{description}',
+      'EXAMPLES':
+          """\
+          To create a cluster with the default configuration, run:
+
+            $ {command} sample-cluster
+          """,
+  }
+
   @staticmethod
   def Args(parser):
     _Args(parser)
@@ -317,31 +353,38 @@ class Create(base.CreateCommand):
     flags.AddClusterAutoscalingFlags(parser)
     flags.AddMaxPodsPerNodeFlag(parser)
     flags.AddEnableAutoRepairFlag(parser, for_create=True)
+    flags.AddEnableBinAuthzFlag(parser)
     flags.AddEnableKubernetesAlphaFlag(parser)
+    flags.AddEnableCloudRunAlphaFlag(parser)
     flags.AddEnableStackdriverKubernetesFlag(parser)
     flags.AddEnableLegacyAuthorizationFlag(parser)
     flags.AddIPAliasFlags(parser)
     flags.AddLabelsFlag(parser)
     flags.AddLocalSSDFlag(parser)
-    flags.AddMaintenanceWindowGroup(
-        parser, add_emw_flags=False, emw_hidden=True)
+    flags.AddMaintenanceWindowGroup(parser)
     flags.AddMasterAuthorizedNetworksFlags(parser)
     flags.AddMinCpuPlatformFlag(parser)
     flags.AddNetworkPolicyFlags(parser)
     flags.AddNodeTaintsFlag(parser)
     flags.AddPreemptibleFlag(parser)
     flags.AddClusterNodeIdentityFlags(parser)
-    flags.AddPrivateClusterFlags(
-        parser, with_deprecated=False, with_alpha=False)
+    flags.AddPrivateClusterFlags(parser, with_deprecated=False)
     flags.AddClusterVersionFlag(parser)
     flags.AddNodeVersionFlag(parser)
-    flags.AddEnableAutoUpgradeFlag(parser)
+    flags.AddEnableAutoUpgradeFlag(parser, default=True)
     flags.AddEnableIntraNodeVisibilityFlag(parser)
     flags.AddTpuFlags(parser, hidden=False)
+    flags.AddAutoprovisioningFlags(
+        parser, hidden=False, for_create=True, ga=True)
     flags.AddResourceUsageExportFlags(parser)
+    flags.AddVerticalPodAutoscalingFlag(parser)
+    flags.AddWorkloadIdentityFlags(parser)
+    flags.AddWorkloadMetadataFlag(parser)
+    flags.AddReservationAffinityFlags(parser)
+    flags.AddSurgeUpgradeFlag(parser)
+    flags.AddMaxUnavailableUpgradeFlag(parser)
 
   def ParseCreateOptions(self, args):
-    flags.WarnGAForFutureAutoUpgradeChange()
     return ParseCreateOptionsBase(args)
 
   def Run(self, args):
@@ -380,14 +423,10 @@ class Create(base.CreateCommand):
           'https://cloud.google.com'
           '/kubernetes-engine/docs/how-to/private-clusters')
 
-    if not (options.metadata and
-            'disable-legacy-endpoints' in options.metadata):
-      log.warning('Starting in 1.12, default node pools in new clusters '
-                  'will have their legacy Compute Engine instance metadata '
-                  'endpoints disabled by default. To create a cluster with '
-                  'legacy instance metadata endpoints disabled in the default '
-                  'node pool, run `clusters create` with the flag '
-                  '`--metadata disable-legacy-endpoints=true`.')
+    if not options.enable_shielded_nodes:
+      log.warning(
+          'Starting with version 1.18, clusters will have shielded GKE nodes by default.'
+      )
 
     if options.enable_ip_alias:
       log.warning(
@@ -459,74 +498,69 @@ class CreateBeta(Create):
     _AddAdditionalZonesFlag(group, deprecated=True)
     flags.AddNodeLocationsFlag(group)
     flags.AddBetaAddonsFlags(parser)
+    flags.AddBootDiskKmsKeyFlag(parser)
     flags.AddClusterAutoscalingFlags(parser)
     flags.AddMaxPodsPerNodeFlag(parser)
     flags.AddEnableAutoRepairFlag(parser, for_create=True)
     flags.AddEnableBinAuthzFlag(parser)
     flags.AddEnableKubernetesAlphaFlag(parser)
+    flags.AddEnableLoggingMonitoringSystemOnlyFlag(parser)
+    flags.AddEnableCloudRunAlphaFlag(parser)
     flags.AddEnableLegacyAuthorizationFlag(parser)
     flags.AddIPAliasFlags(parser)
     flags.AddIstioConfigFlag(parser)
     flags.AddLabelsFlag(parser)
     flags.AddLocalSSDFlag(parser)
-    flags.AddMaintenanceWindowGroup(parser, emw_hidden=True, add_emw_flags=True)
+    flags.AddMaintenanceWindowGroup(parser)
     flags.AddMasterAuthorizedNetworksFlags(parser)
     flags.AddMinCpuPlatformFlag(parser)
-    flags.AddWorkloadMetadataFromNodeFlag(parser)
     flags.AddNetworkPolicyFlags(parser)
     flags.AddNodeTaintsFlag(parser)
     flags.AddPreemptibleFlag(parser)
     flags.AddPodSecurityPolicyFlag(parser)
     flags.AddAllowRouteOverlapFlag(parser)
     flags.AddClusterNodeIdentityFlags(parser)
-    flags.AddPrivateClusterFlags(parser, with_deprecated=True, with_alpha=False)
+    flags.AddPrivateClusterFlags(parser, with_deprecated=True)
     flags.AddEnableStackdriverKubernetesFlag(parser)
-    flags.AddTpuFlags(parser, hidden=False)
+    flags.AddTpuFlags(parser, hidden=False, enable_tpu_service_networking=True)
     flags.AddAutoprovisioningFlags(parser, hidden=False, for_create=True)
+    flags.AddAutoscalingProfilesFlag(parser)
     flags.AddVerticalPodAutoscalingFlag(parser)
     flags.AddResourceUsageExportFlags(parser)
     flags.AddAuthenticatorSecurityGroupFlags(parser)
     flags.AddEnableIntraNodeVisibilityFlag(parser)
-    flags.AddWorkloadIdentityFlags(parser)
-    flags.AddEnableShieldedNodesFlags(parser)
+    flags.AddWorkloadIdentityFlags(parser, use_workload_pool=False)
+    flags.AddWorkloadMetadataFlag(parser, use_mode=False)
     flags.AddEnableAutoUpgradeFlag(parser, default=True)
-    flags.AddDatabaseEncryptionFlag(parser)
-    flags.AddShieldedInstanceFlags(parser)
+    flags.AddSurgeUpgradeFlag(parser, default=1)
+    flags.AddMaxUnavailableUpgradeFlag(parser, is_create=True)
+    flags.AddReservationAffinityFlags(parser)
+    flags.AddMasterGlobalAccessFlag(parser)
+    flags.AddEnableGvnicFlag(parser)
     _AddReleaseChannelGroup(parser)
 
   def ParseCreateOptions(self, args):
     ops = ParseCreateOptionsBase(args)
     flags.WarnForNodeVersionAutoUpgrade(args)
-    ops.enable_autoprovisioning = args.enable_autoprovisioning
-    ops.autoprovisioning_config_file = args.autoprovisioning_config_file
-    ops.autoprovisioning_service_account = args.autoprovisioning_service_account
-    ops.autoprovisioning_scopes = args.autoprovisioning_scopes
-    ops.autoprovisioning_locations = args.autoprovisioning_locations
-    ops.min_cpu = args.min_cpu
-    ops.max_cpu = args.max_cpu
-    ops.min_memory = args.min_memory
-    ops.max_memory = args.max_memory
-    ops.min_accelerator = args.min_accelerator
-    ops.max_accelerator = args.max_accelerator
+    flags.ValidateSurgeUpgradeSettings(args)
+    ops.boot_disk_kms_key = args.boot_disk_kms_key
     ops.min_cpu_platform = args.min_cpu_platform
-    ops.workload_metadata_from_node = args.workload_metadata_from_node
     ops.enable_pod_security_policy = args.enable_pod_security_policy
     ops.allow_route_overlap = args.allow_route_overlap
     ops.private_cluster = args.private_cluster
-    ops.enable_binauthz = args.enable_binauthz
     ops.istio_config = args.istio_config
     ops.enable_vertical_pod_autoscaling = args.enable_vertical_pod_autoscaling
     ops.security_group = args.security_group
     ops.identity_namespace = args.identity_namespace
-    ops.enable_shielded_nodes = args.enable_shielded_nodes
     flags.ValidateIstioConfigCreateArgs(args.istio_config, args.addons)
-    ops.database_encryption_key = args.database_encryption_key
-    ops.shielded_secure_boot = args.shielded_secure_boot
-    ops.shielded_integrity_monitoring = args.shielded_integrity_monitoring
-    ops.maintenance_window_start = args.maintenance_window_start
-    ops.maintenance_window_end = args.maintenance_window_end
-    ops.maintenance_window_recurrence = args.maintenance_window_recurrence
     ops.release_channel = args.release_channel
+    ops.max_surge_upgrade = args.max_surge_upgrade
+    ops.max_unavailable_upgrade = args.max_unavailable_upgrade
+    ops.autoscaling_profile = args.autoscaling_profile
+    ops.enable_tpu_service_networking = args.enable_tpu_service_networking
+    ops.enable_logging_monitoring_system_only = args.enable_logging_monitoring_system_only
+    ops.enable_master_global_access = args.enable_master_global_access
+    ops.enable_gvnic = args.enable_gvnic
     return ops
 
 
@@ -541,33 +575,36 @@ class CreateAlpha(Create):
     _AddAdditionalZonesFlag(group, deprecated=True)
     flags.AddNodeLocationsFlag(group)
     flags.AddAlphaAddonsFlags(parser)
+    flags.AddBootDiskKmsKeyFlag(parser)
     flags.AddClusterAutoscalingFlags(parser)
     flags.AddMaxPodsPerNodeFlag(parser)
     flags.AddEnableAutoRepairFlag(parser, for_create=True)
     flags.AddEnableBinAuthzFlag(parser)
     flags.AddEnableKubernetesAlphaFlag(parser)
+    flags.AddEnableCloudRunAlphaFlag(parser)
     flags.AddEnableLegacyAuthorizationFlag(parser)
     flags.AddIPAliasFlags(parser)
     flags.AddIstioConfigFlag(parser)
     flags.AddLabelsFlag(parser)
     flags.AddLocalSSDAndLocalSSDVolumeConfigsFlag(parser)
-    flags.AddMaintenanceWindowGroup(parser, emw_hidden=True, add_emw_flags=True)
+    flags.AddMaintenanceWindowGroup(parser)
     flags.AddMasterAuthorizedNetworksFlags(parser)
     flags.AddMinCpuPlatformFlag(parser)
-    flags.AddWorkloadMetadataFromNodeFlag(parser)
     flags.AddNetworkPolicyFlags(parser)
+    flags.AddILBSubsettingFlags(parser)
     flags.AddAutoprovisioningFlags(parser, hidden=False, for_create=True)
-    flags.AddAutoscalingProfilesFlag(parser, hidden=True)
+    flags.AddAutoscalingProfilesFlag(parser)
     flags.AddNodeTaintsFlag(parser)
     flags.AddPreemptibleFlag(parser)
     flags.AddPodSecurityPolicyFlag(parser)
     flags.AddAllowRouteOverlapFlag(parser)
-    flags.AddPrivateClusterFlags(parser, with_deprecated=True, with_alpha=True)
+    flags.AddPrivateClusterFlags(parser, with_deprecated=True)
     flags.AddClusterNodeIdentityFlags(parser)
     flags.AddTpuFlags(parser, hidden=False, enable_tpu_service_networking=True)
     flags.AddEnableStackdriverKubernetesFlag(parser)
-    flags.AddManagedPodIdentityFlags(parser)
-    flags.AddWorkloadIdentityFlags(parser)
+    flags.AddEnableLoggingMonitoringSystemOnlyFlag(parser)
+    flags.AddWorkloadIdentityFlags(parser, use_workload_pool=False)
+    flags.AddWorkloadMetadataFlag(parser, use_mode=False)
     flags.AddResourceUsageExportFlags(parser)
     flags.AddAuthenticatorSecurityGroupFlags(parser)
     flags.AddVerticalPodAutoscalingFlag(parser)
@@ -575,32 +612,26 @@ class CreateAlpha(Create):
     flags.AddInitialNodePoolNameArg(parser, hidden=False)
     flags.AddEnablePrivateIpv6AccessFlag(parser, hidden=True)
     flags.AddEnableIntraNodeVisibilityFlag(parser)
-    flags.AddEnableShieldedNodesFlags(parser)
     flags.AddDisableDefaultSnatFlag(parser, for_cluster_create=True)
     _AddReleaseChannelGroup(parser)
     flags.AddEnableAutoUpgradeFlag(parser, default=True)
-    flags.AddDatabaseEncryptionFlag(parser)
-    flags.AddSurgeUpgradeFlag(parser)
-    flags.AddMaxUnavailableUpgradeFlag(parser)
+    flags.AddSurgeUpgradeFlag(parser, default=1)
+    flags.AddMaxUnavailableUpgradeFlag(parser, is_create=True)
     flags.AddLinuxSysctlFlags(parser)
-    flags.AddShieldedInstanceFlags(parser)
     flags.AddNodeConfigFlag(parser)
+    flags.AddCostManagementConfigFlag(parser)
+    flags.AddReservationAffinityFlags(parser)
+    flags.AddDatapathProviderFlag(parser)
+    flags.AddMasterGlobalAccessFlag(parser)
+    flags.AddEnableGvnicFlag(parser)
 
   def ParseCreateOptions(self, args):
     ops = ParseCreateOptionsBase(args)
     flags.WarnForNodeVersionAutoUpgrade(args)
-    ops.enable_autoprovisioning = args.enable_autoprovisioning
-    ops.autoprovisioning_config_file = args.autoprovisioning_config_file
-    ops.min_cpu = args.min_cpu
-    ops.max_cpu = args.max_cpu
-    ops.min_memory = args.min_memory
-    ops.max_memory = args.max_memory
-    ops.min_accelerator = args.min_accelerator
-    ops.max_accelerator = args.max_accelerator
+    flags.ValidateSurgeUpgradeSettings(args)
+    ops.boot_disk_kms_key = args.boot_disk_kms_key
     ops.autoscaling_profile = args.autoscaling_profile
     ops.local_ssd_volume_configs = args.local_ssd_volumes
-    ops.enable_binauthz = args.enable_binauthz
-    ops.workload_metadata_from_node = args.workload_metadata_from_node
     ops.enable_pod_security_policy = args.enable_pod_security_policy
     ops.allow_route_overlap = args.allow_route_overlap
     ops.private_cluster = args.private_cluster
@@ -609,9 +640,7 @@ class CreateAlpha(Create):
     ops.master_ipv4_cidr = args.master_ipv4_cidr
     ops.enable_tpu_service_networking = args.enable_tpu_service_networking
     ops.istio_config = args.istio_config
-    ops.enable_managed_pod_identity = args.enable_managed_pod_identity
     ops.identity_namespace = args.identity_namespace
-    ops.federating_service_account = args.federating_service_account
     ops.security_group = args.security_group
     flags.ValidateIstioConfigCreateArgs(args.istio_config, args.addons)
     ops.enable_vertical_pod_autoscaling = args.enable_vertical_pod_autoscaling
@@ -621,26 +650,18 @@ class CreateAlpha(Create):
     ops.enable_network_egress_metering = args.enable_network_egress_metering
     ops.enable_resource_consumption_metering = args.enable_resource_consumption_metering
     ops.enable_private_ipv6_access = args.enable_private_ipv6_access
-    ops.enable_peering_route_sharing = args.enable_peering_route_sharing
-    ops.enable_shielded_nodes = args.enable_shielded_nodes
     ops.release_channel = args.release_channel
-    ops.autoprovisioning_service_account = args.autoprovisioning_service_account
-    ops.autoprovisioning_scopes = args.autoprovisioning_scopes
-    ops.autoprovisioning_locations = args.autoprovisioning_locations
-    ops.database_encryption_key = args.database_encryption_key
     ops.max_surge_upgrade = args.max_surge_upgrade
     ops.max_unavailable_upgrade = args.max_unavailable_upgrade
     ops.linux_sysctls = args.linux_sysctls
+    ops.enable_l4_ilb_subsetting = args.enable_l4_ilb_subsetting
     ops.disable_default_snat = args.disable_default_snat
-
-    ops.shielded_secure_boot = args.shielded_secure_boot
-    ops.shielded_integrity_monitoring = args.shielded_integrity_monitoring
     ops.node_config = args.node_config
-
-    ops.maintenance_window_start = args.maintenance_window_start
-    ops.maintenance_window_end = args.maintenance_window_end
-    ops.maintenance_window_recurrence = args.maintenance_window_recurrence
-
+    ops.enable_cost_management = args.enable_cost_management
+    ops.enable_logging_monitoring_system_only = args.enable_logging_monitoring_system_only
+    ops.datapath_provider = args.datapath_provider
+    ops.enable_master_global_access = args.enable_master_global_access
+    ops.enable_gvnic = args.enable_gvnic
     return ops
 
 
@@ -656,4 +677,3 @@ Custom Version Flags:
 """)
   flags.AddClusterVersionFlag(custom_version_group)
   flags.AddNodeVersionFlag(custom_version_group)
-
