@@ -45,14 +45,14 @@ DETAILED_HELP = {
         To create a new node pool "node-pool-1" with the default options in the
         cluster "sample-cluster", run:
 
-          $ {command} node-pool-1 --cluster=example-cluster
+          $ {command} node-pool-1 --cluster=sample-cluster
 
         The new node pool will show up in the cluster after all the nodes have
         been provisioned.
 
         To create a node pool with 5 nodes, run:
 
-          $ {command} node-pool-1 --cluster=example-cluster --num-nodes=5
+          $ {command} node-pool-1 --cluster=sample-cluster --num-nodes=5
         """,
 }
 
@@ -106,12 +106,14 @@ for examples.
   flags.AddAcceleratorArgs(parser)
   flags.AddDiskTypeFlag(parser)
   flags.AddMetadataFlags(parser)
+  flags.AddShieldedInstanceFlags(parser)
 
 
 def ParseCreateNodePoolOptionsBase(args):
   """Parses the flags provided with the node pool creation command."""
   enable_autorepair = cmd_util.GetAutoRepair(args)
   flags.WarnForNodeModification(args, enable_autorepair)
+  flags.ValidateSurgeUpgradeSettings(args)
   metadata = metadata_utils.ConstructMetadataDict(args.metadata,
                                                   args.metadata_from_file)
   return api_adapter.CreateNodePoolOptions(
@@ -139,7 +141,17 @@ def ParseCreateNodePoolOptionsBase(args):
       service_account=args.service_account,
       disk_type=args.disk_type,
       metadata=metadata,
-      max_pods_per_node=args.max_pods_per_node)
+      max_pods_per_node=args.max_pods_per_node,
+      enable_autoprovisioning=args.enable_autoprovisioning,
+      workload_metadata=args.workload_metadata,
+      workload_metadata_from_node=args.workload_metadata_from_node,
+      shielded_secure_boot=args.shielded_secure_boot,
+      shielded_integrity_monitoring=args.shielded_integrity_monitoring,
+      reservation_affinity=args.reservation_affinity,
+      reservation=args.reservation,
+      sandbox=args.sandbox,
+      max_surge_upgrade=args.max_surge_upgrade,
+      max_unavailable_upgrade=args.max_unavailable_upgrade)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
@@ -154,14 +166,23 @@ class Create(base.CreateCommand):
     flags.AddPreemptibleFlag(parser, for_node_pool=True)
     flags.AddEnableAutoRepairFlag(parser, for_node_pool=True, for_create=True)
     flags.AddMinCpuPlatformFlag(parser, for_node_pool=True)
+    flags.AddWorkloadMetadataFlag(parser)
     flags.AddNodeTaintsFlag(parser, for_node_pool=True)
     flags.AddNodePoolNodeIdentityFlags(parser)
+    flags.AddNodePoolAutoprovisioningFlag(parser, hidden=False)
     flags.AddMaxPodsPerNodeFlag(parser, for_node_pool=True)
-    flags.AddEnableAutoUpgradeFlag(parser, for_node_pool=True)
+    flags.AddEnableAutoUpgradeFlag(parser, for_node_pool=True, default=True)
+    flags.AddReservationAffinityFlags(parser, for_node_pool=True)
+    flags.AddSandboxFlag(parser)
+    flags.AddNodePoolLocationsFlag(parser, for_create=True)
+    flags.AddSurgeUpgradeFlag(parser, for_node_pool=True)
+    flags.AddMaxUnavailableUpgradeFlag(
+        parser, for_node_pool=True, is_create=True)
 
   def ParseCreateNodePoolOptions(self, args):
-    flags.WarnGAForFutureAutoUpgradeChange()
-    return ParseCreateNodePoolOptionsBase(args)
+    ops = ParseCreateNodePoolOptionsBase(args)
+    ops.node_locations = args.node_locations
+    return ops
 
   def Run(self, args):
     """This is what gets called when the user runs this command.
@@ -224,28 +245,30 @@ class CreateBeta(Create):
     _Args(parser)
     flags.AddClusterAutoscalingFlags(parser)
     flags.AddLocalSSDFlag(parser)
+    flags.AddBootDiskKmsKeyFlag(parser)
     flags.AddPreemptibleFlag(parser, for_node_pool=True)
     flags.AddEnableAutoRepairFlag(parser, for_node_pool=True, for_create=True)
     flags.AddMinCpuPlatformFlag(parser, for_node_pool=True)
-    flags.AddWorkloadMetadataFromNodeFlag(parser)
+    flags.AddWorkloadMetadataFlag(parser, use_mode=False)
     flags.AddNodeTaintsFlag(parser, for_node_pool=True)
     flags.AddNodePoolNodeIdentityFlags(parser)
-    flags.AddNodePoolAutoprovisioningFlag(parser, hidden=True)
+    flags.AddNodePoolAutoprovisioningFlag(parser, hidden=False)
     flags.AddMaxPodsPerNodeFlag(parser, for_node_pool=True)
     flags.AddEnableAutoUpgradeFlag(parser, for_node_pool=True, default=True)
     flags.AddSandboxFlag(parser)
     flags.AddNodePoolLocationsFlag(parser, for_create=True)
-    flags.AddShieldedInstanceFlags(parser)
+    flags.AddSurgeUpgradeFlag(parser, for_node_pool=True, default=1)
+    flags.AddMaxUnavailableUpgradeFlag(
+        parser, for_node_pool=True, is_create=True)
+    flags.AddReservationAffinityFlags(parser, for_node_pool=True)
 
   def ParseCreateNodePoolOptions(self, args):
     ops = ParseCreateNodePoolOptionsBase(args)
     flags.WarnForNodeVersionAutoUpgrade(args)
-    ops.workload_metadata_from_node = args.workload_metadata_from_node
-    ops.enable_autoprovisioning = args.enable_autoprovisioning
+    flags.ValidateSurgeUpgradeSettings(args)
+    ops.boot_disk_kms_key = args.boot_disk_kms_key
     ops.sandbox = args.sandbox
     ops.node_locations = args.node_locations
-    ops.shielded_secure_boot = args.shielded_secure_boot
-    ops.shielded_integrity_monitoring = args.shielded_integrity_monitoring
     return ops
 
 
@@ -256,17 +279,13 @@ class CreateAlpha(Create):
   def ParseCreateNodePoolOptions(self, args):
     ops = ParseCreateNodePoolOptionsBase(args)
     flags.WarnForNodeVersionAutoUpgrade(args)
-    ops.workload_metadata_from_node = args.workload_metadata_from_node
-    ops.enable_autoprovisioning = args.enable_autoprovisioning
+    flags.ValidateSurgeUpgradeSettings(args)
     ops.local_ssd_volume_configs = args.local_ssd_volumes
+    ops.boot_disk_kms_key = args.boot_disk_kms_key
     ops.sandbox = args.sandbox
     ops.node_group = args.node_group
     ops.linux_sysctls = args.linux_sysctls
-    ops.max_surge_upgrade = args.max_surge_upgrade
-    ops.max_unavailable_upgrade = args.max_unavailable_upgrade
     ops.node_locations = args.node_locations
-    ops.shielded_secure_boot = args.shielded_secure_boot
-    ops.shielded_integrity_monitoring = args.shielded_integrity_monitoring
     ops.node_config = args.node_config
     return ops
 
@@ -274,12 +293,13 @@ class CreateAlpha(Create):
   def Args(parser):
     _Args(parser)
     flags.AddClusterAutoscalingFlags(parser)
-    flags.AddNodePoolAutoprovisioningFlag(parser, hidden=True)
+    flags.AddNodePoolAutoprovisioningFlag(parser, hidden=False)
     flags.AddLocalSSDAndLocalSSDVolumeConfigsFlag(parser, for_node_pool=True)
+    flags.AddBootDiskKmsKeyFlag(parser)
     flags.AddPreemptibleFlag(parser, for_node_pool=True)
     flags.AddEnableAutoRepairFlag(parser, for_node_pool=True, for_create=True)
     flags.AddMinCpuPlatformFlag(parser, for_node_pool=True)
-    flags.AddWorkloadMetadataFromNodeFlag(parser)
+    flags.AddWorkloadMetadataFlag(parser, use_mode=False)
     flags.AddNodeTaintsFlag(parser, for_node_pool=True)
     flags.AddNodePoolNodeIdentityFlags(parser)
     flags.AddMaxPodsPerNodeFlag(parser, for_node_pool=True)
@@ -287,11 +307,12 @@ class CreateAlpha(Create):
     flags.AddNodeGroupFlag(parser)
     flags.AddEnableAutoUpgradeFlag(parser, for_node_pool=True, default=True)
     flags.AddLinuxSysctlFlags(parser, for_node_pool=True)
-    flags.AddSurgeUpgradeFlag(parser, for_node_pool=True)
-    flags.AddMaxUnavailableUpgradeFlag(parser, for_node_pool=True)
+    flags.AddSurgeUpgradeFlag(parser, for_node_pool=True, default=1)
+    flags.AddMaxUnavailableUpgradeFlag(
+        parser, for_node_pool=True, is_create=True)
     flags.AddNodePoolLocationsFlag(parser, for_create=True)
-    flags.AddShieldedInstanceFlags(parser)
     flags.AddNodeConfigFlag(parser)
+    flags.AddReservationAffinityFlags(parser, for_node_pool=True)
 
 
 Create.detailed_help = DETAILED_HELP
