@@ -23,7 +23,9 @@ from apitools.base.py import list_pager
 
 from googlecloudsdk.api_lib.dataproc import constants
 from googlecloudsdk.api_lib.dataproc import dataproc as dp
+from googlecloudsdk.api_lib.dataproc import util
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.dataproc import flags
 from googlecloudsdk.core import properties
 
 
@@ -48,35 +50,44 @@ class List(base.ListCommand):
 
   ## EXAMPLES
 
-  To see the list of all clusters, run:
+  To see the list of all clusters in Dataproc's 'us-central1' region, run:
 
-    $ {command}
+    $ {command} --region='us=central1'
 
-  To show a cluster whose name is `mycluster`, run:
+  To show a cluster in Dataproc's 'global' region with the name 'mycluster',
+  run:
 
-    $ {command} --filter='clusterName = mycluster'
+    $ {command} --region='global' --filter='clusterName = mycluster'
 
-  To see the list of all clusters with particular labels, run:
+  To see the list of all clusters in Dataproc's 'global' region with specified
+  labels, run:
 
-    $ {command} --filter='labels.env = staging AND labels.starred = *'
+    $ {command} --region='global' --filter='labels.env = staging AND
+      labels.starred = *'
 
-  To see a list of all active clusters with particular labels, run:
+  To see a list of all active clusters in Dataproc's 'europe-west1' region with
+  specified labels, run:
 
-    $ {command} --filter='status.state = ACTIVE labels.env = staging AND
-     labels.starred = *'
+    $ {command} --region='europe-west1' --filter='status.state = ACTIVE AND
+      labels.env = staging AND labels.starred = *'
   """
 
   @staticmethod
   def Args(parser):
+    flags.AddRegionFlag(parser)
     base.URI_FLAG.RemoveFromParser(parser)
     base.PAGE_SIZE_FLAG.SetDefault(parser, constants.DEFAULT_PAGE_SIZE)
     parser.display_info.AddFormat("""
           table(
             clusterName:label=NAME,
+            config.gkeClusterConfig.yesno(yes=GKE, no=GCE):label=PLATFORM,
             config.workerConfig.numInstances:label=WORKER_COUNT,
             config.secondaryWorkerConfig.numInstances:label=PREEMPTIBLE_WORKER_COUNT,
             status.state:label=STATUS,
-            config.gceClusterConfig.zoneUri.scope(zone):label=ZONE,
+            config.firstof(
+                gkeClusterConfig.namespacedGkeDeploymentTarget.targetGkeCluster,
+                gceClusterConfig.zoneUri
+              ).scope('locations').segment(0):label=ZONE,
             config.lifecycleConfig.yesno(yes=enabled, no=''):label=SCHEDULED_DELETE
           )
     """)
@@ -85,14 +96,15 @@ class List(base.ListCommand):
     dataproc = dp.Dataproc(self.ReleaseTrack())
 
     project = properties.VALUES.core.project.GetOrFail()
-    region = properties.VALUES.dataproc.region.GetOrFail()
+    region = util.ResolveRegion()
 
     request = self.GetRequest(dataproc.messages, project, region, args)
 
     return list_pager.YieldFromList(
         dataproc.client.projects_regions_clusters,
         request,
-        limit=args.limit, field='clusters',
+        limit=args.limit,
+        field='clusters',
         batch_size=args.page_size,
         batch_size_attribute='pageSize')
 
