@@ -30,6 +30,7 @@ from googlecloudsdk.core import properties
 
 
 def _AddArgsCommon(parser, messages):
+  """Adds the common arguments for all versions."""
   flags.GetDnsZoneArg(
       'The name of the managed-zone to be created.').AddToParser(parser)
   flags.GetManagedZonesDnsNameArg().AddToParser(parser)
@@ -39,6 +40,7 @@ def _AddArgsCommon(parser, messages):
   flags.GetManagedZoneNetworksArg().AddToParser(parser)
   flags.GetManagedZoneVisibilityArg().AddToParser(parser)
   flags.GetForwardingTargetsArg().AddToParser(parser)
+  flags.GetDnsPeeringArgs().AddToParser(parser)
 
 
 def _MakeDnssecConfig(args, messages):
@@ -47,8 +49,10 @@ def _MakeDnssecConfig(args, messages):
   if args.dnssec_state is not None:
     dnssec_config = command_util.ParseDnssecConfigArgs(args, messages)
   else:
-    bad_args = ['denial_of_existence', 'ksk_algorithm', 'zsk_algorithm',
-                'ksk_key_length', 'zsk_key_length']
+    bad_args = [
+        'denial_of_existence', 'ksk_algorithm', 'zsk_algorithm',
+        'ksk_key_length', 'zsk_key_length'
+    ]
     for bad_arg in bad_args:
       if getattr(args, bad_arg, None) is not None:
         raise exceptions.InvalidArgumentException(
@@ -86,14 +90,14 @@ class Create(base.CreateCommand):
           'If --visibility is set to public (default), setting networks is '
           'not allowed.')
     if args.visibility == 'private' and args.networks is None:
-      raise exceptions.RequiredArgumentException('--networks', ("""\
+      raise exceptions.RequiredArgumentException('--networks', ("""
            If --visibility is set to private, a list of networks must be
            provided.'
          NOTE: You can provide an empty value ("") for private zones that
           have NO network binding.
           """))
 
-    dns = apis.GetClientInstance('dns', 'v1')
+    dns = util.GetApiClient('v1')
     messages = apis.GetMessagesModule('dns', 'v1')
 
     registry = util.GetRegistry('v1')
@@ -137,6 +141,14 @@ class Create(base.CreateCommand):
 
     labels = labels_util.ParseCreateArgs(args, messages.ManagedZone.LabelsValue)
 
+    peering_config = None
+    if args.target_project and args.target_network:
+      peering_network = 'https://www.googleapis.com/compute/v1/projects/{}/global/networks/{}'.format(
+          args.target_project, args.target_network)
+      peering_config = messages.ManagedZonePeeringConfig()
+      peering_config.targetNetwork = messages.ManagedZonePeeringConfigTargetNetwork(
+          networkUrl=peering_network)
+
     zone = messages.ManagedZone(
         name=zone_ref.managedZone,
         dnsName=util.AppendTrailingDot(args.dns_name),
@@ -145,7 +157,8 @@ class Create(base.CreateCommand):
         labels=labels,
         visibility=visibility,
         forwardingConfig=forward_config,
-        privateVisibilityConfig=visibility_config)
+        privateVisibilityConfig=visibility_config,
+        peeringConfig=peering_config)
 
     result = dns.managedZones.Create(
         messages.DnsManagedZonesCreateRequest(managedZone=zone,
@@ -168,8 +181,8 @@ class CreateBeta(base.CreateCommand):
 
   To create a managed-zone with DNSSEC, run:
 
-    $ {command} my_zone_2 --description "Signed Zone" \
-        --dns-name myzone.example \
+    $ {command} my_zone_2 --description "Signed Zone"
+        --dns-name myzone.example
         --dnssec-state=on
   """
 
@@ -178,7 +191,9 @@ class CreateBeta(base.CreateCommand):
     messages = apis.GetMessagesModule('dns', 'v1beta2')
     _AddArgsCommon(parser, messages)
     parser.display_info.AddCacheUpdater(flags.ManagedZoneCompleter)
-    flags.GetDnsPeeringArgs().AddToParser(parser)
+    flags.GetPrivateForwardingTargetsArg().AddToParser(parser)
+    flags.GetReverseLookupArg().AddToParser(parser)
+    flags.GetServiceDirectoryArg().AddToParser(parser)
 
   def Run(self, args):
     # We explicitly want to allow --networks='' as a valid option and we need
@@ -190,8 +205,7 @@ class CreateBeta(base.CreateCommand):
             'If --visibility is set to public (default), setting networks is '
             'not allowed.')
     if args.visibility == 'private' and args.networks is None:
-      raise exceptions.RequiredArgumentException(
-          '--networks', ("""\
+      raise exceptions.RequiredArgumentException('--networks', ("""
            If --visibility is set to private, a list of networks must be
            provided.'
          NOTE: You can provide an empty value ("") for private zones that
@@ -199,7 +213,7 @@ class CreateBeta(base.CreateCommand):
           """))
 
     api_version = util.GetApiFromTrack(self.ReleaseTrack())
-    dns = apis.GetClientInstance('dns', api_version)
+    dns = util.GetApiClient(api_version)
     messages = apis.GetMessagesModule('dns', api_version)
     registry = util.GetRegistry(api_version)
 
@@ -230,11 +244,13 @@ class CreateBeta(base.CreateCommand):
       visibility_config = messages.ManagedZonePrivateVisibilityConfig(
           networks=network_configs)
 
-    if args.forwarding_targets:
-      forward_config = command_util.ParseManagedZoneForwardingConfig(
-          args.forwarding_targets, messages)
+    if args.forwarding_targets or args.private_forwarding_targets:
+      forwarding_config = command_util.ParseManagedZoneForwardingConfigWithForwardingPath(
+          messages=messages,
+          server_list=args.forwarding_targets,
+          private_server_list=args.private_forwarding_targets)
     else:
-      forward_config = None
+      forwarding_config = None
 
     dnssec_config = _MakeDnssecConfig(args, messages)
     labels = labels_util.ParseCreateArgs(args, messages.ManagedZone.LabelsValue)
@@ -247,6 +263,18 @@ class CreateBeta(base.CreateCommand):
       peering_config.targetNetwork = messages.ManagedZonePeeringConfigTargetNetwork(
           networkUrl=peering_network)
 
+    reverse_lookup_config = None
+    if args.IsSpecified(
+        'managed_reverse_lookup') and args.managed_reverse_lookup:
+      reverse_lookup_config = messages.ManagedZoneReverseLookupConfig()
+
+    service_directory_config = None
+    if args.IsSpecified(
+        'service_directory_namespace') and args.service_directory_namespace:
+      service_directory_config = messages.ManagedZoneServiceDirectoryConfig(
+          namespace=messages.ManagedZoneServiceDirectoryConfigNamespace(
+              namespaceUrl=args.service_directory_namespace))
+
     zone = messages.ManagedZone(
         name=zone_ref.managedZone,
         dnsName=util.AppendTrailingDot(args.dns_name),
@@ -254,9 +282,11 @@ class CreateBeta(base.CreateCommand):
         dnssecConfig=dnssec_config,
         labels=labels,
         visibility=visibility,
-        forwardingConfig=forward_config,
+        forwardingConfig=forwarding_config,
         privateVisibilityConfig=visibility_config,
-        peeringConfig=peering_config)
+        peeringConfig=peering_config,
+        reverseLookupConfig=reverse_lookup_config,
+        serviceDirectoryConfig=service_directory_config)
 
     result = dns.managedZones.Create(
         messages.DnsManagedZonesCreateRequest(managedZone=zone,
@@ -289,4 +319,6 @@ class CreateAlpha(CreateBeta):
     messages = apis.GetMessagesModule('dns', 'v1alpha2')
     _AddArgsCommon(parser, messages)
     parser.display_info.AddCacheUpdater(flags.ManagedZoneCompleter)
-    flags.GetDnsPeeringArgs().AddToParser(parser)
+    flags.GetPrivateForwardingTargetsArg().AddToParser(parser)
+    flags.GetReverseLookupArg().AddToParser(parser)
+    flags.GetServiceDirectoryArg().AddToParser(parser)
