@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,6 +31,7 @@ from googlecloudsdk.api_lib.firebase.test.android import arg_manager
 from googlecloudsdk.api_lib.firebase.test.android import matrix_creator
 from googlecloudsdk.calliope import base
 from googlecloudsdk.core import log
+import six
 
 
 @base.UnicodeIsSupported
@@ -38,7 +39,8 @@ class _BaseRun(object):
   """Invoke a test in Firebase Test Lab for Android and view test results."""
 
   detailed_help = {
-      'DESCRIPTION': """\
+      'DESCRIPTION':
+          """
           *{command}* invokes and monitors tests in Firebase Test Lab for
           Android.
 
@@ -48,7 +50,7 @@ class _BaseRun(object):
             and builds an activity map with associated screenshots and video.
           - *instrumentation*: runs automated unit or integration tests written
             using a testing framework. Firebase Test Lab for Android currently
-            supports the Espresso, Robotium and UI Automator 2.0 testing
+            supports the Espresso and UI Automator 2.0 testing
             frameworks.
           - *game-loop*: executes a special intent built into the game app (a
             "demo mode") that simulates the actions of a real player. This test
@@ -68,27 +70,32 @@ class _BaseRun(object):
           and/or within an argument file. Run *$ gcloud topic arg-files* for
           more information about argument files.
           """,
-
-      'EXAMPLES': """\
+      'EXAMPLES':
+          """
           To invoke a robo test lasting 100 seconds against the default device
           environment, run:
 
-            $ {command} --app APP_APK --timeout 100s
+            $ {command} --app=APP_APK --timeout=100s
 
           When specifying devices to test against, the preferred method is to
           use the --device flag. For example, to invoke a robo test against a
           virtual, generic MDPI Nexus device in landscape orientation, run:
 
-            $ {command} --app APP_APK --device model=NexusLowRes,orientation=landscape
+            $ {command} --app=APP_APK --device=model=NexusLowRes,orientation=landscape
 
           To invoke an instrumentation test against a physical Nexus 6 device
           (MODEL_ID: shamu) which is running Android API level 21 in French, run:
 
-            $ {command} --app APP_APK --test TEST_APK --device model=shamu,version=21,locale=fr
+            $ {command} --app=APP_APK --test=TEST_APK --device=model=shamu,version=21,locale=fr
 
           To test against multiple devices, specify --device more than once:
 
-            $ {command} --app APP_APK --test TEST_APK --device model=Nexus4,version=19 --device model=Nexus4,version=21 --device model=NexusLowRes,version=25
+            $ {command} --app=APP_APK --test=TEST_APK --device=model=Nexus4,version=19 --device=model=Nexus4,version=21 --device=model=NexusLowRes,version=25
+
+          To invoke a robo test on an Android App Bundle, pass the .aab file
+          using the --app flag.
+
+            $ {command} --app=bundle.aab
 
           You may also use the legacy dimension flags (deprecated) to specify
           which devices to use. Firebase Test Lab will run tests against every
@@ -101,7 +108,7 @@ class _BaseRun(object):
           comprehensive matrix of virtual and physical devices, OS versions,
           locales and orientations, run:
 
-            $ {command} --app APP_APK --timeout 5m --device-ids=shamu,NexusLowRes,Nexus5,g3,zeroflte --os-version-ids=19,21,22,23,24,25 --locales=en_GB,es,fr,ru,zh --orientations=portrait,landscape
+            $ {command} --app=APP_APK --timeout=5m --device-ids=shamu,NexusLowRes,Nexus5,g3,zeroflte --os-version-ids=19,21,22,23,24,25 --locales=en_GB,es,fr,ru,zh --orientations=portrait,landscape
 
           The above command will generate a test matrix with a total of 300 test
           executions, but only the subset of executions with valid dimension
@@ -130,13 +137,13 @@ class _BaseRun(object):
           For example, to run a robo test using a specific Google Cloud Storage
           location to hold the raw test results, run:
 
-            $ {command} --app APP_APK --results-bucket=gs://my-bucket --results-dir=my/test/results/<unique-value>
+            $ {command} --app=APP_APK --results-bucket=gs://my-bucket --results-dir=my/test/results/<unique-value>
 
           To run an instrumentation test and specify a custom name under which
           the history of your tests will be collected and displayed in the
           Firebase console, run:
 
-            $ {command} --app APP_APK --test TEST_APK --results-history-name='Excelsior App Test History'
+            $ {command} --app=APP_APK --test=TEST_APK --results-history-name='Excelsior App Test History'
 
           Argument Files
 
@@ -168,7 +175,7 @@ class _BaseRun(object):
         - a list of TestOutcome tuples (if ToolResults are available).
         - a URL string pointing to the user's results in ToolResults or GCS.
     """
-    if args.async and not args.IsSpecified('format'):
+    if args.async_ and not args.IsSpecified('format'):
       args.format = """
           value(format(
             'Final test results will be available at [{0}].', [])
@@ -197,8 +204,11 @@ class _BaseRun(object):
     additional_apks = getattr(args, 'additional_apks', None) or []
     for additional_apk in additional_apks:
       bucket_ops.UploadFileToGcs(additional_apk)
-    for other_files in getattr(args, 'other-files', None) or {}:
-      bucket_ops.UploadFileToGcs(other_files)
+    other_files = getattr(args, 'other_files', None) or {}
+    for device_path, file_to_upload in six.iteritems(other_files):
+      bucket_ops.UploadFileToGcs(
+          file_to_upload,
+          destination_object=util.GetRelativeDevicePath(device_path))
     bucket_ops.LogGcsResultsUrl()
 
     tr_history_picker = history_picker.ToolResultsHistoryPicker(
@@ -208,7 +218,7 @@ class _BaseRun(object):
 
     matrix = matrix_creator.CreateMatrix(args, self.context, history_id,
                                          bucket_ops.gcs_results_root,
-                                         str(self.ReleaseTrack()))
+                                         six.text_type(self.ReleaseTrack()))
     monitor = matrix_ops.MatrixMonitor(
         matrix.testMatrixId, args.type, self.context)
 
@@ -218,12 +228,12 @@ class _BaseRun(object):
 
       url = tool_results.CreateToolResultsUiUrl(project, tr_ids)
       log.status.Print('')
-      if args.async:
+      if args.async_:
         return url
       log.status.Print('Test results will be streamed to [{0}].'.format(url))
 
       # If we have exactly one testExecution, show detailed progress info.
-      if len(supported_executions) == 1:
+      if len(supported_executions) == 1 and args.num_flaky_test_attempts == 0:
         monitor.MonitorTestExecutionProgress(supported_executions[0].id)
       else:
         monitor.MonitorTestMatrixProgress()
@@ -236,7 +246,7 @@ class _BaseRun(object):
     self.exit_code = exit_code.ExitCodeFromRollupOutcome(
         summary_fetcher.FetchMatrixRollupOutcome(),
         tr_messages.Outcome.SummaryValueValuesEnum)
-    return summary_fetcher.CreateMatrixOutcomeSummary()
+    return summary_fetcher.CreateMatrixOutcomeSummaryUsingEnvironments()
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
@@ -262,6 +272,7 @@ class RunBeta(_BaseRun, base.ListCommand):
     arg_util.AddCommonTestRunArgs(parser)
     arg_util.AddMatrixArgs(parser)
     arg_util.AddAndroidTestArgs(parser)
+    arg_util.AddAndroidBetaArgs(parser)
     arg_util.AddBetaArgs(parser)
     base.URI_FLAG.RemoveFromParser(parser)
     parser.display_info.AddFormat(util.OUTCOMES_FORMAT)
