@@ -62,23 +62,21 @@ class Run(base.ListCommand):
           To invoke an XCTest lasting up to five minutes against the default
           device environment, run:
 
-            $ {command} --test XCTEST_ZIP --timeout 5m
+            $ {command} --test=XCTEST_ZIP --timeout=5m
 
           To invoke an XCTest against an iPad 5 running iOS 11.2, run:
 
-            $ {command} --test XCTEST_ZIP --device model=ipad5,version=11.2
+            $ {command} --test=XCTEST_ZIP --device=model=ipad5,version=11.2
 
           To run your tests against multiple iOS devices simultaneously, specify
           the *--device* flag more than once:
 
-            $ {command} --test XCTEST_ZIP \
-              --device model=iphone7 \
-              --device model=ipadmini4,version=11.2 \
-              --device model=iphonese
+            $ {command} --test=XCTEST_ZIP --device=model=iphone7
+            --device=model=ipadmini4,version=11.2 --device=model=iphonese
 
           To run your XCTest using a specific version of Xcode, say 9.4.1, run:
 
-            $ {command} --test XCTEST_ZIP --xcode-version=9.4.1
+            $ {command} --test=XCTEST_ZIP --xcode-version=9.4.1
 
           All test arguments for a given test may alternatively be stored in an
           argument group within a YAML-formatted argument file. The _ARG_FILE_
@@ -182,12 +180,7 @@ class Run(base.ListCommand):
     self.exit_code = exit_code.ExitCodeFromRollupOutcome(
         summary_fetcher.FetchMatrixRollupOutcome(),
         tr_messages.Outcome.SummaryValueValuesEnum)
-    if args.num_flaky_test_attempts > 0:
-      if not args.IsSpecified('format'):
-        args.format = util.FLAKY_ATTEMPTS_OUTCOMES_FORMAT
-      return summary_fetcher.CreateFlakyAttemptsMatrixOutcomeSummary()
-    else:
-      return summary_fetcher.CreateMatrixOutcomeSummary()
+    return summary_fetcher.CreateMatrixOutcomeSummaryUsingEnvironments()
 
 
 def PickHistoryName(args):
@@ -243,32 +236,30 @@ class RunBeta(Run):
           To invoke an XCTest lasting up to five minutes against the default
           device environment, run:
 
-            $ {command} --test XCTEST_ZIP --timeout 5m
+            $ {command} --test=XCTEST_ZIP --timeout=5m
 
           To invoke an XCTest against an iPad 5 running iOS 11.2, run:
 
-            $ {command} --test XCTEST_ZIP --device model=ipad5,version=11.2
+            $ {command} --test=XCTEST_ZIP --device=model=ipad5,version=11.2
 
           To run your tests against multiple iOS devices simultaneously, specify
           the *--device* flag more than once:
 
-            $ {command} --test XCTEST_ZIP \
-              --device model=iphone7 \
-              --device model=ipadmini4,version=11.2 \
-              --device model=iphonese
+            $ {command} --test=XCTEST_ZIP --device=model=iphone7
+            --device=model=ipadmini4,version=11.2 --device=model=iphonese
 
           To run your XCTest using a specific version of Xcode, say 9.4.1, run:
 
-            $ {command} --test XCTEST_ZIP --xcode-version=9.4.1
+            $ {command} --test=XCTEST_ZIP --xcode-version=9.4.1
 
           To run an iOS game loop, specify the *--type* and *--app* flags:
 
-            $ {command} --type game-loop --app app.ipa
+            $ {command} --type=game-loop --app=app.ipa
 
           To run an iOS game loop with specific scenario(s), use the
           *--scenario-numbers* flag:
 
-            $ {command} --type game-loop --app app.ipa --scenario-numbers 1,2,3
+            $ {command} --type=game-loop --app=app.ipa --scenario-numbers=1,2,3
 
           All test arguments for a given test may alternatively be stored in an
           argument group within a YAML-formatted argument file. The _ARG_FILE_
