@@ -12,13 +12,14 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """Creates or updates a Google Cloud Function."""
 
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+from apitools.base.py import encoding
+
 from googlecloudsdk.api_lib.compute import utils
 from googlecloudsdk.api_lib.functions import env_vars as env_vars_api_util
 from googlecloudsdk.api_lib.functions import util as api_util
@@ -36,6 +37,8 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.console import console_io
 
+from six.moves import urllib
+
 
 def _ApplyEnvVarsArgsToFunction(function, args):
   updated_fields = []
@@ -50,13 +53,20 @@ def _ApplyEnvVarsArgsToFunction(function, args):
 
 
 def _CreateBindPolicyCommand(function_name, region):
-  template = (
-      'gcloud alpha functions add-iam-policy-binding %s %s'
-      '--member=allUsers --role=roles/cloudfunctions.invoker')
+  template = ('gcloud alpha functions add-iam-policy-binding %s %s'
+              '--member=allUsers --role=roles/cloudfunctions.invoker')
   region_flag = '--region=%s ' % region if region else ''
   return template % (function_name, region_flag)
 
 
+def _CreateStackdriverURLforBuildLogs(build_id, project_id):
+  query_param = ('resource.type=build\nresource.labels.build_id=%s\n'
+                 'logName=projects/%s/logs/cloudbuild' % (build_id, project_id))
+  return ('https://console.cloud.google.com/logs/viewer?'
+          'project=%s&advancedFilter=%s' %
+          (project_id, urllib.parse.quote(query_param, safe='')))  # pylint: disable=redundant-keyword-arg
+
+
 def _GetProject(args):
   return args.project or properties.VALUES.core.project.Get(required=True)
 
@@ -64,21 +74,21 @@ def _GetProject(args):
 def _Run(args,
          track=None,
          enable_runtime=True,
-         enable_vpc_connector=False,
-         enable_traffic_control=False,
-         enable_allow_unauthenticated=False):
+         enable_build_worker_pool=False):
   """Run a function deployment with the given args."""
   # Check for labels that start with `deployment`, which is not allowed.
   labels_util.CheckNoDeploymentLabels('--remove-labels', args.remove_labels)
   labels_util.CheckNoDeploymentLabels('--update-labels', args.update_labels)
 
   # Check that exactly one trigger type is specified properly.
-  trigger_util.ValidateTriggerArgs(
-      args.trigger_event, args.trigger_resource,
-      args.IsSpecified('retry'), args.IsSpecified('trigger_http'))
-  trigger_params = trigger_util.GetTriggerEventParams(
-      args.trigger_http, args.trigger_bucket, args.trigger_topic,
-      args.trigger_event, args.trigger_resource)
+  trigger_util.ValidateTriggerArgs(args.trigger_event, args.trigger_resource,
+                                   args.IsSpecified('retry'),
+                                   args.IsSpecified('trigger_http'))
+  trigger_params = trigger_util.GetTriggerEventParams(args.trigger_http,
+                                                      args.trigger_bucket,
+                                                      args.trigger_topic,
+                                                      args.trigger_event,
+                                                      args.trigger_resource)
 
   function_ref = args.CONCEPTS.name.Parse()
   function_url = function_ref.RelativeName()
@@ -135,35 +145,38 @@ def _Run(args,
     elif is_new_function:
       raise exceptions.RequiredArgumentException(
           'runtime', 'Flag `--runtime` is required for new functions.')
-  if enable_vpc_connector:
-    if args.vpc_connector or args.clear_vpc_connector:
-      function.vpcConnector = ('' if args.clear_vpc_connector else
+  if args.vpc_connector or args.clear_vpc_connector:
+    function.vpcConnector = ('' if args.clear_vpc_connector else
+                             args.vpc_connector)
+    updated_fields.append('vpcConnector')
+  if args.IsSpecified('egress_settings'):
+    will_have_vpc_connector = ((had_vpc_connector and
+                                not args.clear_vpc_connector) or
                                args.vpc_connector)
-      updated_fields.append('vpcConnector')
-  if enable_traffic_control:
-    if args.IsSpecified('egress_settings'):
-      will_have_vpc_connector = ((had_vpc_connector and
-                                  not args.clear_vpc_connector) or
-                                 args.vpc_connector)
-      if not will_have_vpc_connector:
-        raise exceptions.RequiredArgumentException(
-            'vpc-connector', 'Flag `--vpc-connector` is '
-            'required for setting `egress-settings`.')
-      egress_settings_enum = arg_utils.ChoiceEnumMapper(
-          arg_name='egress_settings',
-          message_enum=function.VpcConnectorEgressSettingsValueValuesEnum,
-          custom_mappings=flags.EGRESS_SETTINGS_MAPPING).GetEnumForChoice(
-              args.egress_settings)
-      function.vpcConnectorEgressSettings = egress_settings_enum
-      updated_fields.append('vpcConnectorEgressSettings')
-    if args.IsSpecified('ingress_settings'):
-      ingress_settings_enum = arg_utils.ChoiceEnumMapper(
-          arg_name='ingress_settings',
-          message_enum=function.IngressSettingsValueValuesEnum,
-          custom_mappings=flags.INGRESS_SETTINGS_MAPPING).GetEnumForChoice(
-              args.ingress_settings)
-      function.ingressSettings = ingress_settings_enum
-      updated_fields.append('ingressSettings')
+    if not will_have_vpc_connector:
+      raise exceptions.RequiredArgumentException(
+          'vpc-connector', 'Flag `--vpc-connector` is '
+          'required for setting `egress-settings`.')
+    egress_settings_enum = arg_utils.ChoiceEnumMapper(
+        arg_name='egress_settings',
+        message_enum=function.VpcConnectorEgressSettingsValueValuesEnum,
+        custom_mappings=flags.EGRESS_SETTINGS_MAPPING).GetEnumForChoice(
+            args.egress_settings)
+    function.vpcConnectorEgressSettings = egress_settings_enum
+    updated_fields.append('vpcConnectorEgressSettings')
+  if args.IsSpecified('ingress_settings'):
+    ingress_settings_enum = arg_utils.ChoiceEnumMapper(
+        arg_name='ingress_settings',
+        message_enum=function.IngressSettingsValueValuesEnum,
+        custom_mappings=flags.INGRESS_SETTINGS_MAPPING).GetEnumForChoice(
+            args.ingress_settings)
+    function.ingressSettings = ingress_settings_enum
+    updated_fields.append('ingressSettings')
+  if enable_build_worker_pool:
+    if args.build_worker_pool or args.clear_build_worker_pool:
+      function.buildWorkerPool = ('' if args.clear_build_worker_pool else
+                                  args.build_worker_pool)
+      updated_fields.append('buildWorkerPool')
   # Populate trigger properties of function based on trigger args.
   if args.trigger_http:
     function.httpsTrigger = messages.HttpsTrigger()
@@ -189,9 +202,9 @@ def _Run(args,
   # used local source.
   if (args.source or args.stage_bucket or is_new_function or
       function.sourceUploadUrl):
-    updated_fields.extend(source_util.SetFunctionSourceProps(
-        function, function_ref, args.source, args.stage_bucket,
-        args.ignore_file))
+    updated_fields.extend(
+        source_util.SetFunctionSourceProps(function, function_ref, args.source,
+                                           args.stage_bucket, args.ignore_file))
 
   # Apply label args to function
   if labels_util.SetFunctionLabels(function, args.update_labels,
@@ -201,16 +214,12 @@ def _Run(args,
   # Apply environment variables args to function
   updated_fields.extend(_ApplyEnvVarsArgsToFunction(function, args))
 
-  ensure_all_users_invoke = (
-      enable_allow_unauthenticated and flags.ShouldEnsureAllUsersInvoke(args))
-  deny_all_users_invoke = (
-      enable_allow_unauthenticated and flags.ShouldDenyAllUsersInvoke(args))
+  ensure_all_users_invoke = flags.ShouldEnsureAllUsersInvoke(args)
+  deny_all_users_invoke = flags.ShouldDenyAllUsersInvoke(args)
 
   if is_new_function:
-    if (enable_allow_unauthenticated
-        and not ensure_all_users_invoke
-        and not deny_all_users_invoke
-        and api_util.CanAddFunctionIamPolicyBinding(_GetProject(args))):
+    if (not ensure_all_users_invoke and not deny_all_users_invoke and
+        api_util.CanAddFunctionIamPolicyBinding(_GetProject(args))):
       ensure_all_users_invoke = console_io.PromptContinue(
           prompt_string=(
               'Allow unauthenticated invocations of new function [{}]?'.format(
@@ -218,12 +227,9 @@ def _Run(args,
           default=False)
 
     op = api_util.CreateFunction(function, function_ref.Parent().RelativeName())
-    if (enable_allow_unauthenticated
-        and not ensure_all_users_invoke
-        and not deny_all_users_invoke):
-      template = (
-          'Function created with limited-access IAM policy. '
-          'To enable unauthorized access consider "%s"')
+    if (not ensure_all_users_invoke and not deny_all_users_invoke):
+      template = ('Function created with limited-access IAM policy. '
+                  'To enable unauthorized access consider "%s"')
       log.warning(template % _CreateBindPolicyCommand(args.NAME, args.region))
       deny_all_users_invoke = True
 
@@ -261,13 +267,34 @@ def _Run(args,
             api_util.RemoveFunctionIamPolicyBindingIfFound(function.name))
     except exceptions.HttpException:
       stop_trying_perm_set[0] = True
-      log.warning(
-          'Setting IAM policy failed, try "%s"' % _CreateBindPolicyCommand(
-              args.NAME, args.region))
+      log.warning('Setting IAM policy failed, try "%s"' %
+                  _CreateBindPolicyCommand(args.NAME, args.region))
+
+  log_stackdriver_url = [True]
+
+  def TryToLogStackdriverURL(op):
+    """Logs stackdriver URL.
+
+    This is for executing in the polling loop, and will stop trying as soon as
+    it succeeds at making a change.
+
+    Args:
+      op: the operation
+    """
+    if log_stackdriver_url[0] and op.metadata:
+      metadata = encoding.PyValueToMessage(
+          messages.OperationMetadataV1, encoding.MessageToPyValue(op.metadata))
+      if metadata.buildId:
+        sd_info_template = '\nFor Cloud Build Stackdriver Logs, visit: %s'
+        log.status.Print(sd_info_template %
+                         _CreateStackdriverURLforBuildLogs(metadata.buildId,
+                                                           _GetProject(args)))
+        log_stackdriver_url[0] = False
 
   if op:
     api_util.WaitForFunctionUpdateOperation(
-        op, do_every_poll=TryToSetInvokerPermission)
+        op, try_set_invoker=TryToSetInvokerPermission,
+        on_every_poll=[TryToLogStackdriverURL])
   return api_util.GetFunction(function.name)
 
 
@@ -290,6 +317,7 @@ class Deploy(base.Command):
         extra_remove_message=labels_util.NO_LABELS_STARTING_WITH_DEPLOY_MESSAGE)
 
     flags.AddServiceAccountFlag(parser)
+    flags.AddAllowUnauthenticatedFlag(parser)
 
     # Add args for specifying the function source code
     flags.AddSourceFlag(parser)
@@ -307,6 +335,10 @@ class Deploy(base.Command):
     # Add args for specifying ignore files to upload source
     flags.AddIgnoreFileFlag(parser)
 
+    flags.AddVPCConnectorMutexGroup(parser)
+    flags.AddEgressSettingsFlag(parser)
+    flags.AddIngressSettingsFlag(parser)
+
   def Run(self, args):
     return _Run(args, track=self.ReleaseTrack())
 
@@ -319,15 +351,9 @@ class DeployBeta(base.Command):
   def Args(parser):
     """Register flags for this command."""
     Deploy.Args(parser)
-    flags.AddVPCConnectorMutexGroup(parser)
-    flags.AddAllowUnauthenticatedFlag(parser)
 
   def Run(self, args):
-    return _Run(
-        args,
-        track=self.ReleaseTrack(),
-        enable_vpc_connector=True,
-        enable_allow_unauthenticated=True)
+    return _Run(args, track=self.ReleaseTrack())
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@@ -338,15 +364,10 @@ class DeployAlpha(base.Command):
   def Args(parser):
     """Register flags for this command."""
     Deploy.Args(parser)
-    flags.AddVPCConnectorMutexGroup(parser)
-    flags.AddEgressSettingsFlag(parser)
-    flags.AddIngressSettingsFlag(parser)
-    flags.AddAllowUnauthenticatedFlag(parser)
+    flags.AddBuildWorkerPoolMutexGroup(parser)
 
   def Run(self, args):
     return _Run(
         args,
         track=self.ReleaseTrack(),
-        enable_vpc_connector=True,
-        enable_traffic_control=True,
-        enable_allow_unauthenticated=True)
+        enable_build_worker_pool=True)
