@@ -1,4 +1,5 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,19 +12,23 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command for deleting service accounts."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 import textwrap
 
+from googlecloudsdk.api_lib.iam import util
 from googlecloudsdk.calliope import base
-from googlecloudsdk.command_lib.iam import base_classes
 from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.core import log
 from googlecloudsdk.core.console import console_io
 
 
-class Delete(base_classes.BaseIamCommand, base.DeleteCommand):
+class Delete(base.DeleteCommand):
   """Delete a service account from a project."""
 
   detailed_help = {
@@ -44,8 +49,9 @@ class Delete(base_classes.BaseIamCommand, base.DeleteCommand):
         message='You are about to delete service '
         'account [{0}].'.format(args.service_account),
         cancel_on_no=True)
-    self.iam_client.projects_serviceAccounts.Delete(
-        self.messages.IamProjectsServiceAccountsDeleteRequest(
+    client, messages = util.GetClientAndMessages()
+    client.projects_serviceAccounts.Delete(
+        messages.IamProjectsServiceAccountsDeleteRequest(
             name=iam_util.EmailToAccountResourceName(args.service_account)))
 
     log.status.Print('deleted service account [{0}]'.format(
