@@ -1,4 +1,5 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,15 +12,22 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command for deleting user-managed service account keys."""
 
-from googlecloudsdk.command_lib.iam import base_classes
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.api_lib.iam import util
+from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.core import log
+from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import console_io
 
 
-class Delete(base_classes.BaseIamCommand):
+class Delete(base.Command):
   """Delete a user-managed key from a service account."""
 
   @staticmethod
@@ -35,14 +43,23 @@ class Delete(base_classes.BaseIamCommand):
                         help='The key to delete.')
 
   def Run(self, args):
+    key_ref = resources.REGISTRY.Parse(
+        args.key,
+        collection='iam.projects.serviceAccounts.keys',
+        params={
+            'serviceAccountsId': args.iam_account,
+            'projectsId': '-'
+        })
+    key = key_ref.keysId
     console_io.PromptContinue(
         message='You are about to delete key [{0}] for service '
-        'account [{1}].'.format(args.key, args.account),
+        'account [{1}].'.format(args.key, args.iam_account),
         cancel_on_no=True)
-    self.iam_client.projects_serviceAccounts_keys.Delete(
-        self.messages.IamProjectsServiceAccountsKeysDeleteRequest(
-            name=iam_util.EmailAndKeyToResourceName(args.iam_account,
-                                                    args.key)))
+
+    client, messages = util.GetClientAndMessages()
+    client.projects_serviceAccounts_keys.Delete(
+        messages.IamProjectsServiceAccountsKeysDeleteRequest(
+            name=key_ref.RelativeName()))
 
     log.status.Print('deleted key [{1}] for service account [{0}]'.format(
-        args.iam_account, args.key))
+        args.iam_account, key))
