@@ -1,4 +1,5 @@
-# Copyright 2017 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,8 +14,13 @@
 # limitations under the License.
 """Describe a keyring."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
 from googlecloudsdk.calliope import base
+from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.kms import flags
 
 
@@ -28,17 +34,21 @@ class Describe(base.DescribeCommand):
   The following command returns the metadata for the keyring `towers`
   in the location `us-east1`:
 
-    $ {command} towers --location us-east1
+    $ {command} towers --location=us-east1
   """
 
   @staticmethod
   def Args(parser):
+    flags.AddLocationFlag(parser, 'keyring')
     flags.AddKeyRingArgument(parser, 'to describe')
 
   def Run(self, args):
     client = cloudkms_base.GetClientInstance()
     messages = cloudkms_base.GetMessagesModule()
     key_ring_ref = flags.ParseKeyRingName(args)
+    if not key_ring_ref.Name():
+      raise exceptions.InvalidArgumentException('keyring',
+                                                'keyring id must be non-empty.')
     return client.projects_locations_keyRings.Get(
         messages.CloudkmsProjectsLocationsKeyRingsGetRequest(
             name=key_ring_ref.RelativeName()))
