@@ -1,4 +1,5 @@
-# Copyright 2017 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,12 +14,14 @@
 # limitations under the License.
 """List the keys within a keyring."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import list_pager
 from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.kms import flags
-from googlecloudsdk.core import properties
-from googlecloudsdk.core import resources
 
 
 class List(base.ListCommand):
@@ -31,7 +34,7 @@ class List(base.ListCommand):
   The following command lists all keys within the
   keyring `fellowship` and location `global`:
 
-    $ {command} --keyring fellowship --location global
+    $ {command} --keyring=fellowship --location=global
   """
 
   @staticmethod
@@ -39,10 +42,15 @@ class List(base.ListCommand):
     # The format of a CryptoKeyVersion name is:
     # 'projects/*/locations/*/keyRings/*/cryptoKeys/*/cryptoKeyVersions/*'
     # The CryptoKeyVersionId is captured by segment(9).
+    flags.AddKeyRingFlag(parser, 'key')
+    flags.AddLocationFlag(parser, 'key')
     parser.display_info.AddFormat("""
         table(
           name,
           purpose,
+          version_template.algorithm,
+          version_template.protection_level,
+          labels.list(),
           primary.name.segment(9):label=PRIMARY_ID,
           primary.state:label=PRIMARY_STATE)
     """)
@@ -51,11 +59,7 @@ class List(base.ListCommand):
     client = cloudkms_base.GetClientInstance()
     messages = cloudkms_base.GetMessagesModule()
 
-    key_ring_ref = resources.REGISTRY.Create(
-        flags.KEY_RING_COLLECTION,
-        keyRingsId=args.MakeGetOrRaise('--keyring'),
-        locationsId=args.MakeGetOrRaise('--location'),
-        projectsId=properties.VALUES.core.project.GetOrFail)
+    key_ring_ref = flags.ParseKeyRingName(args)
 
     request = messages.CloudkmsProjectsLocationsKeyRingsCryptoKeysListRequest(
         parent=key_ring_ref.RelativeName())
