@@ -1,4 +1,5 @@
-# Copyright 2017 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Set the IAM policy for a key."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
 from googlecloudsdk.api_lib.cloudkms import iam
 from googlecloudsdk.calliope import base
@@ -20,10 +25,11 @@ from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.command_lib.kms import flags
 
 
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class SetIamPolicy(base.Command):
   """Set the IAM policy for a key.
 
-  Sets the IAM policy for the given key as defined in a JSON file.
+  Sets the IAM policy for the given key as defined in a JSON or YAML file.
 
   See https://cloud.google.com/iam/docs/managing-policies for details of
   the policy file format and contents.
@@ -33,7 +39,7 @@ class SetIamPolicy(base.Command):
   'policy.json' and set it for the key `frodo` with the keyring
   `fellowship` and location `global`:
 
-    $ {command} frodo policy.json --keyring fellowship --location global
+    $ {command} frodo policy.json --keyring=fellowship --location=global
   """
   # Text from above based on output of function call below
   # detailed_help = iam_util.GetDetailedHelpForSetIamPolicy(
@@ -41,14 +47,15 @@ class SetIamPolicy(base.Command):
 
   @staticmethod
   def Args(parser):
-    flags.AddCryptoKeyArgument(parser, 'whose IAM policy to update')
-    parser.add_argument('policy_file', help='JSON file with the IAM policy')
+    flags.AddKeyResourceArgument(parser, 'whose IAM policy to update')
+    parser.add_argument('policy_file', help=('JSON or YAML '
+                                             'file with the IAM policy'))
 
   def Run(self, args):
     messages = cloudkms_base.GetMessagesModule()
 
-    policy = iam_util.ParseJsonPolicyFile(args.policy_file, messages.Policy)
-    update_mask = iam_util.ConstructUpdateMaskFromPolicy(args.policy_file)
+    policy, update_mask = iam_util.ParseYamlOrJsonPolicyFile(args.policy_file,
+                                                             messages.Policy)
 
     crypto_key_ref = flags.ParseCryptoKeyName(args)
     result = iam.SetCryptoKeyIamPolicy(crypto_key_ref, policy, update_mask)
