@@ -20,6 +20,7 @@ from __future__ import division
 from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.logging import util
+from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions as calliope_exceptions
 from googlecloudsdk.core import log
@@ -80,6 +81,7 @@ class Update(base.UpdateCommand):
 
   def _Run(self, args, is_alpha=False):
     sink_ref = util.GetSinkReference(args.sink_name, args)
+    sink = self.GetSink(util.GetParentFromArgs(args), sink_ref)
 
     sink_data = {'name': sink_ref.sinksId}
     update_mask = []
@@ -111,8 +113,61 @@ class Update(base.UpdateCommand):
         sink_data['bigqueryOptions'] = bigquery_options
         update_mask.append('bigquery_options.use_partitioned_tables')
 
-      if args.IsSpecified('clear_exclusions'):
+      if args.IsSpecified('description'):
+        sink_data['description'] = args.description
+        update_mask.append('description')
+
+      if args.IsSpecified('disabled'):
+        sink_data['disabled'] = args.disabled
+        update_mask.append('disabled')
+
+      if (args.IsSpecified('clear_exclusions') or
+          args.IsSpecified('remove_exclusions') or
+          args.IsSpecified('add_exclusion') or
+          args.IsSpecified('update_exclusion')):
+        sink_data['exclusions'] = []
         update_mask.append('exclusions')
+        exclusions_to_remove = (args.remove_exclusions
+                                if args.IsSpecified('remove_exclusions')
+                                else [])
+        exclusions_to_update = (args.update_exclusion
+                                if args.IsSpecified('update_exclusion')
+                                else [])
+        for exclusion in sink.exclusions:
+          if exclusion.name in exclusions_to_remove:
+            exclusions_to_remove.remove(exclusion.name)
+          else:
+            for i in range(len(exclusions_to_update)):
+              if exclusion.name == exclusions_to_update[i]['name']:
+                for key, value in exclusions_to_update[i].items():
+                  if key == 'description':
+                    exclusion.description = value
+                  if key == 'filter':
+                    exclusion.filter = value
+                  if key == 'disabled':
+                    exclusion.disabled = value
+                exclusions_to_update.pop(i)
+                break
+            sink_data['exclusions'].append(exclusion)
+
+        if exclusions_to_remove:
+          raise calliope_exceptions.InvalidArgumentException(
+              '--remove-exclusions',
+              'Exclusions {0} do not exist'.format(
+                  ','.join(exclusions_to_remove)))
+
+        if exclusions_to_update:
+          raise calliope_exceptions.InvalidArgumentException(
+              '--update-exclusion',
+              'Exclusions {0} do not exist'.format(
+                  ','.join([exclusion['name']
+                            for exclusion in exclusions_to_update])))
+
+        if args.IsSpecified('clear_exclusions'):
+          sink_data['exclusions'] = []
+
+        if args.IsSpecified('add_exclusion'):
+          sink_data['exclusions'] += args.add_exclusion
 
     if not update_mask:
       raise calliope_exceptions.MinimumArgumentException(
@@ -120,8 +175,7 @@ class Update(base.UpdateCommand):
 
     # Check for legacy configuration, and let users decide if they still want
     # to update the sink with new settings.
-    sink = self.GetSink(util.GetParentFromArgs(args), sink_ref)
-    if 'cloud-logs@' in sink.writerIdentity:
+    if sink.writerIdentity and 'cloud-logs@' in sink.writerIdentity:
       console_io.PromptContinue(
           'This update will create a new writerIdentity (service account) for '
           'the sink. In order for the sink to continue working, grant that '
@@ -193,6 +247,66 @@ class UpdateAlpha(Update):
     parser.add_argument(
         '--clear-exclusions', action='store_true',
         help=('Remove all logging exclusions.'))
+    parser.add_argument(
+        '--remove-exclusions',
+        type=arg_parsers.ArgList(),
+        metavar='EXCLUSION ID',
+        help=('Specify the name of the Logging exclusion(s) to delete.'))
+    parser.add_argument(
+        '--add-exclusion', action='append',
+        type=arg_parsers.ArgDict(
+            spec={
+                'name': str,
+                'description': str,
+                'filter': str,
+                'disabled': bool
+            },
+            required_keys=['name', 'filter']
+        ),
+        help=('Add an exclusion filter for a log entry that is not to be '
+              'exported. This flag can be repeated.\n\n'
+              'The ``name\'\' and ``filter\'\' attributes are required. The '
+              'following keys are accepted:\n\n'
+              '*name*::: An identifier, such as ``load-balancer-exclusion\'\'. '
+              'Identifiers are limited to 100 characters and can include only '
+              'letters, digits, underscores, hyphens, and periods.\n\n'
+              '*description*::: A description of this exclusion.\n\n'
+              '*filter*::: An advanced log filter that matches the log entries '
+              'to be excluded.\n\n'
+              '*disabled*::: If this exclusion should be disabled and not '
+              'exclude the log entries.'))
+
+    parser.add_argument(
+        '--update-exclusion', action='append',
+        type=arg_parsers.ArgDict(
+            spec={
+                'name': str,
+                'description': str,
+                'filter': str,
+                'disabled': bool
+            },
+            required_keys=['name']
+        ),
+        help=('Update an exclusion filter for a log entry that is not to be '
+              'exported. This flag can be repeated.\n\n'
+              'The ``name\'\' and ``filter\'\' attributes are required. '
+              'following keys are accepted:\n\n'
+              'name*::: An identifier, such as ``load-balancer-exclusion\'\'. '
+              'Identifiers are limited to 100 characters and can include only '
+              'letters, digits, underscores, hyphens, and periods.\n\n'
+              '*description*::: A description of this exclusion.\n\n'
+              '*filter*::: An advanced log filter that matches the log entries '
+              'to be excluded.\n\n'
+              '*disabled*::: If this exclusion should be disabled and not '
+              'exclude the log entries.'))
+
+    parser.add_argument(
+        '--description',
+        help='Description of the sink.')
+
+    parser.add_argument(
+        '--disabled', action='store_true',
+        help=('Disable the sink. Disabled sinks do not export logs.'))
 
   def Run(self, args):
     return self._Run(args, is_alpha=True)
