@@ -1,4 +1,5 @@
-# Copyright 2017 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,11 +14,16 @@
 # limitations under the License.
 """Command to add values to an Organization Policy whitelist."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.resource_manager import exceptions
 from googlecloudsdk.api_lib.resource_manager import org_policies
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.resource_manager import org_policies_base
 from googlecloudsdk.command_lib.resource_manager import org_policies_flags as flags
+import six
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
@@ -39,26 +45,33 @@ class Allow(base.Command):
   @staticmethod
   def Args(parser):
     flags.AddIdArgToParser(parser)
-    flags.AddResourceFlagsToParser(parser)
+    flags.AddParentResourceFlagsToParser(parser)
     base.Argument(
-        'allowed_value', metavar='ALLOWED_VALUE', nargs='+').AddToParser(parser)
+        'allowed_value',
+        metavar='ALLOWED_VALUE',
+        nargs='+',
+        help='The values to add to the allowed_values list policy.',
+    ).AddToParser(parser)
 
+  # TODO(b/73831954):consider refactoring
   def Run(self, args):
-    flags.CheckResourceFlags(args)
     messages = org_policies.OrgPoliciesMessages()
     service = org_policies_base.OrgPoliciesService(args)
 
     policy = service.GetOrgPolicy(org_policies_base.GetOrgPolicyRequest(args))
 
-    if policy.booleanPolicy or (
-        policy.listPolicy and
-        (policy.listPolicy.deniedValues or policy.listPolicy.allValues)):
+    if policy.booleanPolicy or (policy.listPolicy and
+                                policy.listPolicy.deniedValues):
       raise exceptions.ResourceManagerError(
           'Cannot add values to a non-allowed_values list policy.')
 
+    if policy.listPolicy and policy.listPolicy.allValues:
+      raise exceptions.ResourceManagerError(
+          'Cannot add values if all_values is already specified.')
+
     if policy.listPolicy and policy.listPolicy.allowedValues:
       for value in args.allowed_value:
-        policy.listPolicy.allowedValues.append(unicode(value))
+        policy.listPolicy.allowedValues.append(six.text_type(value))
     else:
       policy.listPolicy = messages.ListPolicy(allowedValues=args.allowed_value)
 
