@@ -19,7 +19,13 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
+import uuid
+
+from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
+from googlecloudsdk.api_lib.run import traffic
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.builds import flags as build_flags
+from googlecloudsdk.command_lib.builds import submit_util
 from googlecloudsdk.command_lib.run import config_changes as config_changes_mod
 from googlecloudsdk.command_lib.run import connection_context
 from googlecloudsdk.command_lib.run import flags
@@ -29,6 +35,8 @@ from googlecloudsdk.command_lib.run import serverless_operations
 from googlecloudsdk.command_lib.run import stages
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
 from googlecloudsdk.command_lib.util.concepts import presentation_specs
+from googlecloudsdk.core import properties
+from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import progress_tracker
 
 
@@ -49,9 +57,9 @@ def GetAllowUnauth(args, operations, service_ref, service_exists):
      None means to retain the current value for the service.
   """
   allow_unauth = None
-  if flags.IsManaged(args):
-    allow_unauth = flags.GetAllowUnauthenticated(
-        args, operations, service_ref, not service_exists)
+  if flags.GetPlatform() == flags.PLATFORM_MANAGED:
+    allow_unauth = flags.GetAllowUnauthenticated(args, operations, service_ref,
+                                                 not service_exists)
     # Avoid failure removing a policy binding for a service that
     # doesn't exist.
     if not service_exists and not allow_unauth:
@@ -92,18 +100,13 @@ def GetSuccessMessageForSynchronousDeploy(operations, service_ref):
   """
   service = operations.GetService(service_ref)
   latest_ready = service.status.latestReadyRevisionName
-  latest_percent_traffic = sum(
-      target.percent for target in service.status.traffic
-      if target.latestRevision or (
-          latest_ready and target.revisionName == latest_ready))
-  msg = (
-      'Service [{{bold}}{serv}{{reset}}] '
-      'revision [{{bold}}{rev}{{reset}}] '
-      'has been deployed')
+  latest_percent_traffic = service.latest_percent_traffic
+  msg = ('Service [{{bold}}{serv}{{reset}}] '
+         'revision [{{bold}}{rev}{{reset}}] '
+         'has been deployed and is serving '
+         '{{bold}}{latest_percent_traffic}{{reset}} percent of traffic')
   if latest_percent_traffic:
-    msg += (
-        ' and is serving {{bold}}{latest_percent_traffic}{{reset}} '
-        'percent of traffic at {{bold}}{url}{{reset}}')
+    msg += (' at {{bold}}{url}{{reset}}')
   return msg.format(
       serv=service_ref.servicesId,
       rev=latest_ready,
@@ -111,7 +114,7 @@ def GetSuccessMessageForSynchronousDeploy(operations, service_ref):
       latest_percent_traffic=latest_percent_traffic)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class Deploy(base.Command):
   """Deploy a container to Cloud Run."""
 
@@ -122,16 +125,16 @@ class Deploy(base.Command):
       'EXAMPLES': """\
           To deploy a container to the service `my-backend` on Cloud Run:
 
-              $ {command} my-backend --image gcr.io/my/image
+              $ {command} my-backend --image=gcr.io/my/image
 
           You may also omit the service name. Then a prompt will be displayed
           with a suggested default value:
 
-              $ {command} --image gcr.io/my/image
+              $ {command} --image=gcr.io/my/image
 
           To deploy to Cloud Run on Kubernetes Engine, you need to specify a cluster:
 
-              $ {command} --image gcr.io/my/image --cluster my-cluster
+              $ {command} --image=gcr.io/my/image --cluster=my-cluster
           """,
   }
 
@@ -139,21 +142,17 @@ class Deploy(base.Command):
   def CommonArgs(parser):
     # Flags specific to managed CR
     managed_group = flags.GetManagedArgGroup(parser)
-    flags.AddRegionArg(managed_group)
     flags.AddAllowUnauthenticatedFlag(managed_group)
-    flags.AddServiceAccountFlag(managed_group)
     flags.AddCloudSQLFlags(managed_group)
-    # Flags specific to CRoGKE
-    gke_group = flags.GetGkeArgGroup(parser)
-    concept_parsers.ConceptParser([resource_args.CLUSTER_PRESENTATION
-                                  ]).AddToParser(gke_group)
-    # Flags specific to connecting to a Kubernetes cluster (kubeconfig)
-    kubernetes_group = flags.GetKubernetesArgGroup(parser)
-    flags.AddKubeconfigFlags(kubernetes_group)
+    flags.AddRevisionSuffixArg(managed_group)
+
     # Flags specific to connecting to a cluster
     cluster_group = flags.GetClusterArgGroup(parser)
     flags.AddEndpointVisibilityEnum(cluster_group)
-    flags.AddCpuFlag(cluster_group)
+    flags.AddSecretsFlags(cluster_group)
+    flags.AddConfigMapsFlags(cluster_group)
+    flags.AddHttp2Flag(cluster_group)
+
     # Flags not specific to any platform
     service_presentation = presentation_specs.ResourcePresentationSpec(
         'SERVICE',
@@ -161,8 +160,6 @@ class Deploy(base.Command):
         'Service to deploy to.',
         required=True,
         prefixes=False)
-    flags.AddImageArg(parser)
-    flags.AddPlatformArg(parser)
     flags.AddFunctionArg(parser)
     flags.AddMutexEnvVarsFlags(parser)
     flags.AddMemoryFlag(parser)
@@ -171,33 +168,80 @@ class Deploy(base.Command):
     flags.AddAsyncFlag(parser)
     flags.AddLabelsFlags(parser)
     flags.AddMaxInstancesFlag(parser)
+    flags.AddCommandFlag(parser)
+    flags.AddArgsFlag(parser)
+    flags.AddPortFlag(parser)
+    flags.AddCpuFlag(parser)
+    flags.AddNoTrafficFlag(parser)
     concept_parsers.ConceptParser([service_presentation]).AddToParser(parser)
 
   @staticmethod
   def Args(parser):
     Deploy.CommonArgs(parser)
+    flags.AddImageArg(parser)
+
+    # Flags specific to managed CR
+    managed_group = flags.GetManagedArgGroup(parser)
+    flags.AddServiceAccountFlag(managed_group)
+
+    # Flags only supported on GKE and Knative
+    cluster_group = flags.GetClusterArgGroup(parser)
+    flags.AddMinInstancesFlag(cluster_group)
 
   def Run(self, args):
     """Deploy a container to Cloud Run."""
-    image = args.image
-
-    conn_context = connection_context.GetConnectionContext(args)
-    config_changes = flags.GetConfigurationChanges(args)
-
     service_ref = flags.GetService(args)
+    build_op_ref = None
+    messages = None
+    build_log_url = None
+    image = args.image
+    include_build = flags.FlagIsExplicitlySet(args, 'source')
+    # Build an image from source if source specified.
+    if include_build:
+      # Create a tag for the image creation
+      if image is None and not args.IsSpecified('config'):
+        image = 'gcr.io/{projectID}/cloud-run-source-deploy/{service}:{tag}'.format(
+            projectID=properties.VALUES.core.project.Get(required=True),
+            service=service_ref.servicesId,
+            tag=uuid.uuid4().hex)
+      messages = cloudbuild_util.GetMessagesModule()
+      build_config = submit_util.CreateBuildConfig(
+          image, args.no_cache, messages, args.substitutions, args.config,
+          args.IsSpecified('source'), False, args.source,
+          args.gcs_source_staging_dir, args.ignore_file, args.gcs_log_dir,
+          args.machine_type, args.disk_size)
+
+      build, build_op = submit_util.Build(messages, True, build_config, True)
+      build_op_ref = resources.REGISTRY.ParseRelativeName(
+          build_op.name, 'cloudbuild.operations'
+      )
+      build_log_url = build.logUrl
+    # Deploy a container with an image
+    conn_context = connection_context.GetConnectionContext(
+        args, flags.Product.RUN, self.ReleaseTrack())
+    config_changes = flags.GetConfigurationChanges(args)
 
     with serverless_operations.Connect(conn_context) as operations:
       image_change = config_changes_mod.ImageChange(image)
       changes = [image_change]
       if config_changes:
         changes.extend(config_changes)
-      exists = operations.GetService(service_ref)
-      allow_unauth = GetAllowUnauth(args, operations, service_ref, exists)
+      service = operations.GetService(service_ref)
+      allow_unauth = GetAllowUnauth(args, operations, service_ref, service)
 
       pretty_print.Info(GetStartDeployMessage(conn_context, service_ref))
-
-      deployment_stages = stages.ServiceStages(allow_unauth is not None)
-      header = 'Deploying...' if exists else 'Deploying new service...'
+      has_latest = (service is None or
+                    traffic.LATEST_REVISION_KEY in service.spec_traffic)
+      deployment_stages = stages.ServiceStages(
+          include_iam_policy_set=allow_unauth is not None,
+          include_route=has_latest,
+          include_build=include_build)
+      header = 'Deploying'
+      if include_build:
+        header += ' and building'
+      if service is None:
+        header += ' new service'
+      header += '...'
       with progress_tracker.StagedProgressTracker(
           header,
           deployment_stages,
@@ -208,7 +252,10 @@ class Deploy(base.Command):
             changes,
             tracker,
             asyn=args.async_,
-            allow_unauthenticated=allow_unauth)
+            allow_unauthenticated=allow_unauth,
+            prefetch=service,
+            build_op_ref=build_op_ref,
+            build_log_url=build_log_url)
       if args.async_:
         pretty_print.Success(
             'Service [{{bold}}{serv}{{reset}}] is deploying '
@@ -225,16 +272,25 @@ class AlphaDeploy(Deploy):
   @staticmethod
   def Args(parser):
     Deploy.CommonArgs(parser)
-    # Flags specific to managed CR
-    managed_group = flags.GetManagedArgGroup(parser)
-    flags.AddRevisionSuffixArg(managed_group)
-    # Flags specific to connecting to a cluster
-    cluster_group = flags.GetClusterArgGroup(parser)
-    flags.AddSecretsFlags(cluster_group)
-    flags.AddConfigMapsFlags(cluster_group)
+
+    # Flags specific to VPCAccess
+    flags.AddVpcConnectorArg(parser)
+
     # Flags not specific to any platform
     flags.AddMinInstancesFlag(parser)
-    flags.AddCommandFlag(parser)
-    flags.AddArgsFlag(parser)
+    flags.AddServiceAccountFlagAlpha(parser)
+
+    # Flags inherited from gcloud builds submit
+    flags.AddConfigFlags(parser)
+    flags.AddSourceFlag(parser)
+    flags.AddBuildTimeoutFlag(parser)
+    build_flags.AddGcsSourceStagingDirFlag(parser, True)
+    build_flags.AddGcsLogDirFlag(parser, True)
+    build_flags.AddMachineTypeFlag(parser, True)
+    build_flags.AddDiskSizeFlag(parser, True)
+    build_flags.AddSubstitutionsFlag(parser, True)
+    build_flags.AddNoCacheFlag(parser, True)
+    build_flags.AddIgnoreFileFlag(parser, True)
+
 
 AlphaDeploy.__doc__ = Deploy.__doc__
