@@ -27,34 +27,27 @@ from googlecloudsdk.command_lib.util.concepts import concept_parsers
 from googlecloudsdk.command_lib.util.concepts import presentation_specs
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 class Describe(base.Command):
-  """Describe domain mappings."""
+  """Describe domain mappings for Cloud Run for Anthos."""
 
   detailed_help = {
       'DESCRIPTION':
-          '{description}',
+          """\
+          {description}
+
+          For domain mapping support with fully managed Cloud Run, use
+          `gcloud beta run domain-mappings describe`.""",
       'EXAMPLES':
           """\
           To describe a Cloud Run domain mapping, run:
 
-              $ {command} --domain www.example.com
+              $ {command} --domain=www.example.com
           """,
   }
 
   @staticmethod
   def CommonArgs(parser):
-    # Flags specific to managed CR
-    managed_group = flags.GetManagedArgGroup(parser)
-    flags.AddRegionArg(managed_group)
-    # Flags specific to CRoGKE
-    gke_group = flags.GetGkeArgGroup(parser)
-    concept_parsers.ConceptParser([resource_args.CLUSTER_PRESENTATION
-                                  ]).AddToParser(gke_group)
-    # Flags specific to connecting to a Kubernetes cluster (kubeconfig)
-    kubernetes_group = flags.GetKubernetesArgGroup(parser)
-    flags.AddKubeconfigFlags(kubernetes_group)
-    # Flags not specific to any platform
     domain_mapping_presentation = presentation_specs.ResourcePresentationSpec(
         '--domain',
         resource_args.GetDomainMappingResourceSpec(),
@@ -63,7 +56,7 @@ class Describe(base.Command):
         prefixes=False)
     concept_parsers.ConceptParser([
         domain_mapping_presentation]).AddToParser(parser)
-    flags.AddPlatformArg(parser)
+
     parser.display_info.AddFormat('yaml')
 
   @staticmethod
@@ -72,7 +65,14 @@ class Describe(base.Command):
 
   def Run(self, args):
     """Describe a domain mapping."""
-    conn_context = connection_context.GetConnectionContext(args)
+    # domains.cloudrun.com api group only supports v1alpha1 on clusters.
+    conn_context = connection_context.GetConnectionContext(
+        args,
+        flags.Product.RUN,
+        self.ReleaseTrack(),
+        version_override=('v1alpha1'
+                          if flags.GetPlatform() != flags.PLATFORM_MANAGED else
+                          None))
     domain_mapping_ref = args.CONCEPTS.domain.Parse()
     with serverless_operations.Connect(conn_context) as client:
       domain_mapping = client.GetDomainMapping(domain_mapping_ref)
@@ -83,12 +83,30 @@ class Describe(base.Command):
       return domain_mapping
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class AlphaDescribe(Describe):
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
+class BetaDescribe(Describe):
   """Describe domain mappings."""
 
+  detailed_help = {
+      'DESCRIPTION':
+          '{description}',
+      'EXAMPLES':
+          """\
+          To describe a Cloud Run domain mapping, run:
+
+              $ {command} --domain=www.example.com
+          """,
+  }
+
   @staticmethod
   def Args(parser):
     Describe.CommonArgs(parser)
 
-AlphaDescribe.__doc__ = Describe.__doc__
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class AlphaDescribe(BetaDescribe):
+  """Describe domain mappings."""
+
+  @staticmethod
+  def Args(parser):
+    Describe.CommonArgs(parser)
