@@ -21,22 +21,26 @@ from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.run import service
 from googlecloudsdk.api_lib.util import messages as messages_util
+from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.projects import util as projects_util
 from googlecloudsdk.command_lib.run import config_changes
 from googlecloudsdk.command_lib.run import connection_context
+from googlecloudsdk.command_lib.run import exceptions
 from googlecloudsdk.command_lib.run import flags
 from googlecloudsdk.command_lib.run import pretty_print
 from googlecloudsdk.command_lib.run import resource_args
 from googlecloudsdk.command_lib.run import serverless_operations
 from googlecloudsdk.command_lib.run import stages
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
+from googlecloudsdk.command_lib.util.concepts import presentation_specs
+from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
-from googlecloudsdk.core import yaml
 from googlecloudsdk.core.console import progress_tracker
 from surface.run import deploy
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
 class Replace(base.Command):
   """Creates or replaces a service from a YAML Service specification."""
 
@@ -56,39 +60,58 @@ class Replace(base.Command):
 
   @staticmethod
   def Args(parser):
-    # Flags specific to managed CR
-    managed_group = flags.GetManagedArgGroup(parser)
-    flags.AddRegionArg(managed_group)
 
-    # Flags specific to CRoGKE
-    gke_group = flags.GetGkeArgGroup(parser)
-    concept_parsers.ConceptParser([resource_args.CLUSTER_PRESENTATION
-                                  ]).AddToParser(gke_group)
+    # Flags specific to connecting to a cluster
+    cluster_group = flags.GetClusterArgGroup(parser)
+    namespace_presentation = presentation_specs.ResourcePresentationSpec(
+        '--namespace',
+        resource_args.GetNamespaceResourceSpec(),
+        'Namespace to replace service.',
+        required=True,
+        prefixes=False)
+    concept_parsers.ConceptParser(
+        [namespace_presentation]).AddToParser(cluster_group)
 
     # Flags not specific to any platform
     flags.AddAsyncFlag(parser)
-    flags.AddPlatformArg(parser)
     parser.add_argument(
         'FILE',
         action='store',
+        type=arg_parsers.YAMLFileContents(),
         help='The absolute path to the YAML file with a Knative '
         'service definition for the service to update or deploy.')
 
   def Run(self, args):
     """Create or Update service from YAML."""
-    conn_context = connection_context.GetConnectionContext(args)
-    if conn_context.supports_one_platform:
-      flags.VerifyOnePlatformFlags(args)
-    else:
-      flags.VerifyGKEFlags(args)
+    conn_context = connection_context.GetConnectionContext(
+        args, flags.Product.RUN, self.ReleaseTrack())
 
     with serverless_operations.Connect(conn_context) as client:
-      message_dict = yaml.load_path(args.FILE)
       new_service = service.Service(
           messages_util.DictToMessageWithErrorCheck(
-              message_dict, client.messages_module.Service),
+              args.FILE, client.messages_module.Service),
           client.messages_module)
 
+      # If managed, namespace must match project (or will default to project if
+      # not specified).
+      # If not managed, namespace simply must not conflict if specified in
+      # multiple places (or will default to "default" if not specified).
+      namespace = args.CONCEPTS.namespace.Parse().Name()  # From flag or default
+      if new_service.metadata.namespace is not None:
+        if (args.IsSpecified('namespace') and
+            namespace != new_service.metadata.namespace):
+          raise exceptions.ConfigurationError(
+              'Namespace specified in file does not match passed flag.')
+        namespace = new_service.metadata.namespace
+        if flags.GetPlatform() == flags.PLATFORM_MANAGED:
+          project = properties.VALUES.core.project.Get()
+          project_number = projects_util.GetProjectNumber(project)
+          if namespace != project and namespace != str(project_number):
+            raise exceptions.ConfigurationError(
+                'Namespace must be project ID [{}] or quoted number [{}] for '
+                'Cloud Run (fully managed).'.format(project, project_number))
+      new_service.metadata.namespace = namespace
+
       changes = [config_changes.ReplaceServiceChange(new_service)]
       service_ref = resources.REGISTRY.Parse(
           new_service.metadata.name,
