@@ -19,7 +19,7 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-from googlecloudsdk.api_lib.run import traffic
+from googlecloudsdk.api_lib.run import traffic_pair
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import display
 from googlecloudsdk.command_lib.run import connection_context
@@ -29,12 +29,15 @@ from googlecloudsdk.command_lib.run import pretty_print
 from googlecloudsdk.command_lib.run import resource_args
 from googlecloudsdk.command_lib.run import serverless_operations
 from googlecloudsdk.command_lib.run import stages
+from googlecloudsdk.command_lib.run import traffic_printer
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
 from googlecloudsdk.command_lib.util.concepts import presentation_specs
 from googlecloudsdk.core.console import progress_tracker
+from googlecloudsdk.core.resource import resource_printer
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+@base.ReleaseTracks(base.ReleaseTrack.GA,
+                    base.ReleaseTrack.BETA)
 class AdjustTraffic(base.Command):
   """Adjust the trafic assignments for a Cloud Run service."""
 
@@ -45,22 +48,36 @@ class AdjustTraffic(base.Command):
           """,
       'EXAMPLES':
           """\
-          To adjust the traffic assignments for myservice
+          To assign 10% of traffic to revision myservice-s5sxn and
+          90% of traffic to revision myservice-cp9kw run:
 
-              $ {command} myservice --to-revision=revis-name-1=10,revis-name-2=34 --to-latest=23`
+              $ {command} myservice --to-revisions=myservice-s5sxn=10,myservice-cp9kw=90
+
+          To increase the traffic to revision myservice-s5sxn to 20% and
+          by reducing the traffic to revision myservice-cp9kw to 80% run:
+
+              $ {command} myservice --to-revisions=myservice-s5sxn=20
+
+          To rollback to revision myservice-cp9kw run:
+
+              $ {command} myservice --to-revisions=myservice-cp9kw=100
+
+          To assign 100% of traffic to the current or future LATEST revision
+          run:
+
+              $ {command} myservice --to-latest
+
+          You can also refer to the current or future LATEST revision in
+          --to-revisions by the string "LATEST". For example, to set 10% of
+          traffic to always float to the latest revision:
+
+              $ {command} myservice --to-revisions=LATEST=10
 
-         Sets 10% on revis-name1, 34% on revis-name-2, and 23% on the latest revision
          """,
   }
 
   @staticmethod
-  def Args(parser):
-    # Flags specific to CRoGKE
-    gke_group = flags.GetGkeArgGroup(parser)
-    concept_parsers.ConceptParser([resource_args.CLUSTER_PRESENTATION
-                                  ]).AddToParser(gke_group)
-
-    # Flags not specific to any platform
+  def CommonArgs(parser):
     service_presentation = presentation_specs.ResourcePresentationSpec(
         'SERVICE',
         resource_args.GetServiceResourceSpec(prompt=True),
@@ -70,20 +87,10 @@ class AdjustTraffic(base.Command):
     flags.AddAsyncFlag(parser)
     flags.AddUpdateTrafficFlags(parser)
     concept_parsers.ConceptParser([service_presentation]).AddToParser(parser)
-    flags.AddPlatformArg(parser)
 
-  def _SetFormat(self, args):
-    """Set display format for output.
-
-    Args:
-      args: Namespace, the args namespace
-    """
-    columns = [
-        'displayPercent:label=TRAFFIC',
-        'displayRevisionId:label=REVISION',
-    ]
-    args.GetDisplayInfo().AddFormat(
-        'table({})'.format(','.join(columns)))
+  @staticmethod
+  def Args(parser):
+    AdjustTraffic.CommonArgs(parser)
 
   def Run(self, args):
     """Update the traffic split for the service.
@@ -94,21 +101,23 @@ class AdjustTraffic(base.Command):
     Returns:
       List of traffic.TrafficTargetStatus instances reflecting the change.
     """
-    conn_context = connection_context.GetConnectionContext(args)
-    service_ref = flags.GetService(args)
+    # TODO(b/143898356) Begin code that should be in Args
+    resource_printer.RegisterFormatter(
+        traffic_printer.TRAFFIC_PRINTER_FORMAT,
+        traffic_printer.TrafficPrinter)
+    args.GetDisplayInfo().AddFormat('traffic')
+    # End code that should be in Args
 
-    if conn_context.supports_one_platform:
-      flags.VerifyOnePlatformFlags(args)
-    else:
-      flags.VerifyGKEFlags(args)
+    conn_context = connection_context.GetConnectionContext(
+        args, flags.Product.RUN, self.ReleaseTrack())
+    service_ref = flags.GetService(args)
 
     changes = flags.GetConfigurationChanges(args)
     if not changes:
       raise exceptions.NoConfigurationChangeError(
           'No traffic configuration change requested.')
 
-    self._SetFormat(args)
-
+    is_managed = flags.GetPlatform() == flags.PLATFORM_MANAGED
     with serverless_operations.Connect(conn_context) as client:
       deployment_stages = stages.UpdateTrafficStages()
       try:
@@ -117,26 +126,40 @@ class AdjustTraffic(base.Command):
             deployment_stages,
             failure_message='Updating traffic failed',
             suppress_output=args.async_) as tracker:
-          client.UpdateTraffic(
-              service_ref, changes, tracker, args.async_, flags.IsManaged(args))
+          client.UpdateTraffic(service_ref, changes, tracker, args.async_)
       except:
         serv = client.GetService(service_ref)
-        resources = traffic.GetTrafficTargetPairs(
-            serv.spec.traffic,
-            serv.status.traffic,
-            flags.IsManaged(args),
-            serv.status.latestReadyRevisionName)
-        display.Displayer(
-            self, args, resources, display_info=args.GetDisplayInfo()).Display()
+        if serv:
+          resources = traffic_pair.GetTrafficTargetPairs(
+              serv.spec_traffic,
+              serv.status_traffic,
+              is_managed,
+              serv.status.latestReadyRevisionName,
+              serv.status.url)
+          display.Displayer(
+              self, args, resources,
+              display_info=args.GetDisplayInfo()).Display()
         raise
 
     if args.async_:
       pretty_print.Success('Updating traffic asynchronously.')
     else:
       serv = client.GetService(service_ref)
-      resources = traffic.GetTrafficTargetPairs(
-          serv.spec.traffic,
-          serv.status.traffic,
-          flags.IsManaged(args),
-          serv.status.latestReadyRevisionName)
+      resources = traffic_pair.GetTrafficTargetPairs(
+          serv.spec_traffic,
+          serv.status_traffic,
+          is_managed,
+          serv.status.latestReadyRevisionName,
+          serv.status.url)
       return resources
+
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class AlphaAdjustTraffic(AdjustTraffic):
+  """Adjust the traffic assignments for a Cloud Run service."""
+
+  @staticmethod
+  def Args(parser):
+    AdjustTraffic.CommonArgs(parser)
+    flags.AddTrafficTagsFlags(parser)
+
