@@ -85,6 +85,7 @@ def _WhitelistClientIP(instance_ref,
 
   acl_name = 'sql connect at time {0}'.format(time_of_connection)
   user_acl = sql_messages.AclEntry(
+      kind='sql#aclEntry',
       name=acl_name,
       expirationTime=iso_duration.Duration(
           minutes=minutes).GetRelativeDateTime(time_of_connection)
@@ -92,7 +93,7 @@ def _WhitelistClientIP(instance_ref,
       # Setting the microseconds component to 10 milliseconds. This complies
       # with backend formatting restrictions, since backend requires a microsecs
       # component and anything less than 1 milli will get truncated.
-      .replace(microsecond=10000),
+      .replace(microsecond=10000).isoformat(),
       value='CLIENT_IP')
 
   try:
@@ -219,7 +220,7 @@ def RunConnectCommand(args, supports_database=False):
                                  'not reply with the whitelisted IP.')
 
   # Check for the mysql or psql executable based on the db version.
-  db_type = instance_info.databaseVersion.split('_')[0]
+  db_type = instance_info.databaseVersion.name.split('_')[0]
   exe_name = constants.DB_EXE.get(db_type, 'mysql')
   exe = files.FindExecutableOnPath(exe_name)
   if not exe:
@@ -253,7 +254,8 @@ def RunConnectCommand(args, supports_database=False):
   flags = constants.EXE_FLAGS[exe_name]
   sql_args = [exe_name, flags['hostname'], ip_address]
   sql_args.extend([flags['user'], sql_user])
-  sql_args.append(flags['password'])
+  if 'password' in flags:
+    sql_args.append(flags['password'])
 
   if supports_database:
     sql_args.extend(instances_command_util.GetDatabaseArgs(args, flags))
@@ -262,16 +264,13 @@ def RunConnectCommand(args, supports_database=False):
 
 
 def RunProxyConnectCommand(args,
-                           supports_database=False,
-                           supports_mssqlcli=False):
+                           supports_database=False):
   """Connects to a Cloud SQL instance through the Cloud SQL Proxy.
 
   Args:
     args: argparse.Namespace, The arguments that this command was invoked with.
     supports_database: Whether or not the `--database` flag needs to be
       accounted for.
-    supports_mssqlcli: Whether or not mssql-cli's hostname handling needs to be
-      accounted for.
 
   Returns:
     If no exception is raised this method does not return. A new process is
@@ -292,15 +291,15 @@ def RunProxyConnectCommand(args,
       sql_messages.SqlInstancesGetRequest(
           project=instance_ref.project, instance=instance_ref.instance))
 
-  if not instances_api_util.IsInstanceV2(instance_info):
+  if not instances_api_util.IsInstanceV2(sql_messages, instance_info):
     # The Cloud SQL Proxy does not support V1 instances.
     return RunConnectCommand(args, supports_database)
 
   # If the instance is V2, keep going with the proxy.
   util.EnsureComponentIsInstalled('cloud_sql_proxy', '`sql connect` command')
 
-  # Check for the mysql or psql executable based on the db version.
-  db_type = instance_info.databaseVersion.split('_')[0]
+  # Check for the executable based on the db version.
+  db_type = instance_info.databaseVersion.name.split('_')[0]
   exe_name = constants.DB_EXE.get(db_type, 'mysql')
   exe = files.FindExecutableOnPath(exe_name)
   if not exe:
@@ -322,14 +321,15 @@ def RunProxyConnectCommand(args,
   # We have everything we need, time to party!
   flags = constants.EXE_FLAGS[exe_name]
   sql_args = [exe_name]
-  if supports_mssqlcli and exe_name == 'mssql-cli':
+  if exe_name == 'mssql-cli':
     # mssql-cli merges hostname and port into a single argument
     hostname = 'tcp:127.0.0.1,{0}'.format(port)
     sql_args.extend([flags['hostname'], hostname])
   else:
     sql_args.extend([flags['hostname'], '127.0.0.1', flags['port'], port])
   sql_args.extend([flags['user'], sql_user])
-  sql_args.append(flags['password'])
+  if 'password' in flags:
+    sql_args.append(flags['password'])
 
   if supports_database:
     sql_args.extend(instances_command_util.GetDatabaseArgs(args, flags))
@@ -348,13 +348,15 @@ class Connect(base.Command):
   def Args(parser):
     """Args is called by calliope to gather arguments for this command."""
     AddBaseArgs(parser)
+    sql_flags.AddDatabase(
+        parser, 'The SQL Server database to connect to.')
 
   def Run(self, args):
     """Connects to a Cloud SQL instance."""
     return RunConnectCommand(args)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
 class ConnectBeta(base.Command):
   """Connects to a Cloud SQL instance.
 
@@ -364,28 +366,6 @@ class ConnectBeta(base.Command):
 
   detailed_help = DETAILED_HELP
 
-  @staticmethod
-  def Args(parser):
-    """Args is called by calliope to gather arguments for this command."""
-    AddBaseArgs(parser)
-    AddBetaArgs(parser)
-    sql_flags.AddDatabase(parser, 'The PostgreSQL database to connect to.')
-
-  def Run(self, args):
-    """Connects to a Cloud SQL instance."""
-    return RunProxyConnectCommand(args, supports_database=True)
-
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class ConnectAlpha(base.Command):
-  """Connects to a Cloud SQL instance.
-
-  Connects to Cloud SQL V2 instances through the Cloud SQL Proxy. Connects to
-  Cloud SQL V1 instances directly.
-  """
-
-  detailed_help = DETAILED_HELP
-
   @staticmethod
   def Args(parser):
     """Args is called by calliope to gather arguments for this command."""
@@ -396,5 +376,4 @@ class ConnectAlpha(base.Command):
 
   def Run(self, args):
     """Connects to a Cloud SQL instance."""
-    return RunProxyConnectCommand(
-        args, supports_database=True, supports_mssqlcli=True)
+    return RunProxyConnectCommand(args, supports_database=True)
