@@ -21,6 +21,7 @@ from __future__ import unicode_literals
 from apitools.base.py import exceptions as apitools_exceptions
 
 from googlecloudsdk.api_lib.sql import api_util as common_api_util
+from googlecloudsdk.api_lib.sql import exceptions as sql_exceptions
 from googlecloudsdk.api_lib.sql import instances as api_util
 from googlecloudsdk.api_lib.sql import operations
 from googlecloudsdk.api_lib.sql import validate
@@ -33,11 +34,35 @@ from googlecloudsdk.command_lib.sql import validate as command_validate
 from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
-from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.resource import resource_lex
 from googlecloudsdk.core.resource import resource_property
 import six
 
+DETAILED_HELP = {
+    'EXAMPLES':
+        """\
+        To create a MySQL 5.7 instance with ID ``prod-instance'' and machine
+        type ``db-n1-standard-1'', in the region ``us-central1'' (a zone will be
+        auto-assigned), where the 'root' user has its password set to
+        ``password123'', run:
+
+          $ {command} prod-instance --database-version=MYSQL_5_7 --tier=db-n1-standard-1 --region=us-central1 --root-password=password123
+
+        To create a Postgres 9.6 instance with ID ``prod-instance'' that has 2
+        CPUs, 8 GiB of RAM, and is in the zone ``us-central1-a'', where the
+        'postgres' user has its password set to ``password123'', run:
+
+          $ {command} prod-instance --database-version=POSTGRES_9_6 --cpu=2 --memory=8GiB --zone=us-central1-a --root-password=password123
+
+        To create a SQL Server 2017 Express instance with ID ``prod-instance''
+        that has 2 CPUs, 3840MiB of RAM, and is in the zone ``us-central1-a'',
+        where the 'sqlserver' user has its password set to ``password123'',
+        run:
+
+          $ {command} prod-instance --database-version=SQLSERVER_2017_EXPRESS --cpu=2 --memory=3840MiB --zone=us-central1-a --root-password=password123
+        """,
+}
+
 
 def AddBaseArgs(parser):
   """Declare flag and positional arguments for this command parser."""
@@ -46,7 +71,6 @@ def AddBaseArgs(parser):
   parser.display_info.AddFormat(flags.GetInstanceListFormat())
   flags.AddActivationPolicy(parser)
   flags.AddAssignIp(parser)
-  flags.AddAuthorizedGAEApps(parser)
   flags.AddAuthorizedNetworks(parser)
   flags.AddAvailabilityType(parser)
   parser.add_argument(
@@ -56,6 +80,7 @@ def AddBaseArgs(parser):
       default=True,
       help='Enables daily backup.')
   flags.AddBackupStartTime(parser)
+  flags.AddBackupLocation(parser, allow_empty=False)
   flags.AddCPU(parser)
   flags.AddDatabaseFlags(parser)
   flags.AddEnableBinLog(parser, show_negated_in_help=False)
@@ -63,12 +88,6 @@ def AddBaseArgs(parser):
       '--failover-replica-name',
       required=False,
       help='Also create a failover replica with the specified name.')
-  parser.add_argument(
-      '--follow-gae-app',
-      required=False,
-      help=('First Generation instances only. The App Engine app this '
-            'instance should follow. It must be in the same region as '
-            'the instance.'))
   parser.add_argument(
       'instance',
       type=command_validate.InstanceNameRegexpValidator(),
@@ -83,14 +102,6 @@ def AddBaseArgs(parser):
             'replication setup. The newly created instance will be a read '
             'replica of the specified master instance.'))
   flags.AddMemory(parser)
-  parser.add_argument(
-      '--pricing-plan',
-      '-p',
-      required=False,
-      choices=['PER_USE', 'PACKAGE'],
-      default='PER_USE',
-      help=('First Generation instances only. The pricing plan for this '
-            'instance.'))
   # TODO(b/31989340): add remote completion
   # TODO(b/73362371): Make specifying a location required.
   location_group = parser.add_mutually_exclusive_group()
@@ -126,6 +137,16 @@ def AddBaseArgs(parser):
       default=None,
       help='The storage type for the instance. The default is SSD.')
   flags.AddTier(parser)
+  kms_flag_overrides = {
+      'kms-key': '--disk-encryption-key',
+      'kms-keyring': '--disk-encryption-key-keyring',
+      'kms-location': '--disk-encryption-key-location',
+      'kms-project': '--disk-encryption-key-project'
+  }
+  kms_resource_args.AddKmsKeyResourceArg(
+      parser,
+      'instance',
+      flag_overrides=kms_flag_overrides)
 
 
 def AddBetaArgs(parser):
@@ -136,6 +157,11 @@ def AddBetaArgs(parser):
   labels_util.AddCreateLabelsFlags(parser)
 
 
+def AddAlphaArgs(parser):
+  """Declare alpha flags for this command parser."""
+  flags.AddEnablePointInTimeRecovery(parser)
+
+
 def RunBaseCreateCommand(args, release_track):
   """Creates a new Cloud SQL instance.
 
@@ -149,6 +175,11 @@ def RunBaseCreateCommand(args, release_track):
   Raises:
     HttpException: A http error response was received while executing api
         request.
+    ArgumentError: An argument supplied by the user was incorrect, such as
+      specifying an invalid CMEK configuration or attempting to create a V1
+      instance.
+    RequiredArgumentException: A required argument was not supplied by the user,
+      such as omitting --root-password on a SQL Server instance.
   """
   client = common_api_util.SqlClient(common_api_util.API_VERSION_DEFAULT)
   sql_client = client.sql_client
@@ -187,12 +218,31 @@ def RunBaseCreateCommand(args, release_track):
     if not args.IsSpecified('region'):
       args.region = master_instance_resource.region
     if not args.IsSpecified('database_version'):
-      args.database_version = master_instance_resource.databaseVersion
+      args.database_version = master_instance_resource.databaseVersion.name
     if not args.IsSpecified('tier') and master_instance_resource.settings:
       args.tier = master_instance_resource.settings.tier
-    # Check for CMEK usage; warn the user about replica inheriting the setting.
+
+    # Validate master/replica CMEK configurations.
     if master_instance_resource.diskEncryptionConfiguration:
-      command_util.ShowCmekWarning('replica', 'the master instance')
+      if args.region == master_instance_resource.region:
+        # Warn user that same-region replicas inherit their master's CMEK
+        # configuration.
+        command_util.ShowCmekWarning('replica', 'the master instance')
+      elif not args.IsSpecified('disk_encryption_key'):
+        # Raise error that cross-region replicas require their own CMEK key if
+        # the master is CMEK.
+        raise exceptions.RequiredArgumentException(
+            '--disk-encryption-key',
+            '`--disk-encryption-key` is required when creating a cross-region '
+            'replica of an instance with customer-managed encryption.')
+      else:
+        command_util.ShowCmekWarning('replica')
+    elif args.IsSpecified('disk_encryption_key'):
+      # Raise error that cross-region replicas cannot be CMEK encrypted if their
+      # master is not.
+      raise sql_exceptions.ArgumentError(
+          '`--disk-encryption-key` cannot be specified when creating a replica '
+          'of an instance without customer-managed encryption.')
 
   # --root-password is required when creating SQL Server instances
   if args.IsSpecified('database_version') and args.database_version.startswith(
@@ -209,20 +259,13 @@ def RunBaseCreateCommand(args, release_track):
           release_track=release_track))
 
   # TODO(b/122660263): Remove when V1 instances are no longer supported.
-  # V1 instances are deprecated. Prompt to continue if one is being created.
-  if api_util.IsInstanceV1(instance_resource):
-    log.warning(
-        'First Generation instances will be automatically upgraded '
-        'to Second Generation starting March 4th, 2020, and First Generation '
-        'will be fully decommissioned on March 25, 2020. We recommend you '
-        'create a Second Generation instance.')
-    console_io.PromptContinue(cancel_on_no=True)
-
-  if args.pricing_plan == 'PACKAGE':
-    console_io.PromptContinue(
-        'Charges will begin accruing immediately. Really create Cloud '
-        'SQL instance?',
-        cancel_on_no=True)
+  # V1 instances are deprecated.
+  # Note that the exception type is intentionally vague because the user may not
+  # have directly supplied the offending argument.  For example, creating a read
+  # replica defaults its tier to that of its master.
+  if api_util.IsInstanceV1(sql_messages, instance_resource):
+    raise sql_exceptions.ArgumentError(
+        'First Generation instances can no longer be created.')
 
   operation_ref = None
   try:
@@ -269,6 +312,8 @@ def RunBaseCreateCommand(args, release_track):
 class Create(base.Command):
   """Creates a new Cloud SQL instance."""
 
+  detailed_help = DETAILED_HELP
+
   def Run(self, args):
     return RunBaseCreateCommand(args, self.ReleaseTrack())
 
@@ -283,6 +328,8 @@ class Create(base.Command):
 class CreateBeta(base.Command):
   """Creates a new Cloud SQL instance."""
 
+  detailed_help = DETAILED_HELP
+
   def Run(self, args):
     return RunBaseCreateCommand(args, self.ReleaseTrack())
 
@@ -298,6 +345,8 @@ class CreateBeta(base.Command):
 class CreateAlpha(base.Command):
   """Creates a new Cloud SQL instance."""
 
+  detailed_help = DETAILED_HELP
+
   def Run(self, args):
     return RunBaseCreateCommand(args, self.ReleaseTrack())
 
@@ -306,19 +355,5 @@ class CreateAlpha(base.Command):
     """Args is called by calliope to gather arguments for this command."""
     AddBaseArgs(parser)
     AddBetaArgs(parser)
+    AddAlphaArgs(parser)
     flags.AddDatabaseVersion(parser, restrict_choices=False)
-    kms_flag_overrides = {
-        'kms-key': '--disk-encryption-key',
-        'kms-keyring': '--disk-encryption-key-keyring',
-        'kms-location': '--disk-encryption-key-location',
-        'kms-project': '--disk-encryption-key-project'
-    }
-    permission_info = (
-        'Please ensure that you have the '
-        '`resourcemanager.projects.setIamPolicy` permission for the project '
-        'associated with the key')
-    kms_resource_args.AddKmsKeyResourceArg(
-        parser,
-        'instance',
-        flag_overrides=kms_flag_overrides,
-        permission_info=permission_info)
