@@ -1,4 +1,5 @@
-# Copyright 2013 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2013 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,6 +18,10 @@ Lists instances in a given project in the alphabetical order of the
 instance name.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.sql import api_util
 from googlecloudsdk.api_lib.sql import instances
 from googlecloudsdk.calliope import base
@@ -31,7 +36,8 @@ def _GetUriFromResource(resource):
       instance=resource.name).SelfLink()
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
+                    base.ReleaseTrack.ALPHA)
 class List(base.ListCommand):
   """Lists Cloud SQL instances in a given project.
 
@@ -41,7 +47,7 @@ class List(base.ListCommand):
 
   @staticmethod
   def Args(parser):
-    parser.display_info.AddFormat(flags.INSTANCES_FORMAT_BETA)
+    parser.display_info.AddFormat(flags.GetInstanceListFormat())
     parser.display_info.AddUriFunc(_GetUriFromResource)
 
   def Run(self, args):
@@ -53,10 +59,6 @@ class List(base.ListCommand):
 
     Returns:
       SQL instance resource iterator.
-    Raises:
-      HttpException: An http error response was received while executing api
-          request.
-      ToolException: An error other than an http error occured while executing
-          the command.
     """
-    return instances.InstancesV1Beta4.GetDatabaseInstances()
+    return instances.InstancesV1Beta4.GetDatabaseInstances(
+        limit=args.limit, batch_size=args.page_size)
