@@ -18,6 +18,9 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import copy
+
+from apitools.base.protorpclite import messages
 from apitools.base.py import encoding
 
 from googlecloudsdk.api_lib.sql import api_util as common_api_util
@@ -47,10 +50,9 @@ def _PrintAndConfirmWarningMessage(args, database_version):
   """Print and confirm warning indicating the effect of applying the patch."""
   continue_msg = None
   if any([args.tier, args.enable_database_replication is not None]):
-    continue_msg = (
-        'WARNING: This patch modifies a value that requires '
-        'your instance to be restarted. Submitting this patch '
-        'will immediately restart your instance if it\'s running.')
+    continue_msg = ('WARNING: This patch modifies a value that requires '
+                    'your instance to be restarted. Submitting this patch '
+                    'will immediately restart your instance if it\'s running.')
   elif any([args.database_flags, args.clear_database_flags]):
     database_type_fragment = 'mysql'
     if api_util.InstancesV1Beta4.IsPostgresDatabaseVersion(database_version):
@@ -75,6 +77,22 @@ def _PrintAndConfirmWarningMessage(args, database_version):
     raise exceptions.CancelledError('canceled by the user.')
 
 
+def WithoutKind(message, inline=False):
+  result = message if inline else copy.deepcopy(message)
+  for field in result.all_fields():
+    if field.name == 'kind':
+      result.kind = None
+    elif isinstance(field, messages.MessageField):
+      value = getattr(result, field.name)
+      if value is not None:
+        if isinstance(value, list):
+          setattr(result, field.name,
+                  [WithoutKind(item, True) for item in value])
+        else:
+          setattr(result, field.name, WithoutKind(value, True))
+  return result
+
+
 def _GetConfirmedClearedFields(args, patch_instance, original_instance):
   """Clear fields according to args and confirm with user."""
   cleared_fields = []
@@ -89,8 +107,8 @@ def _GetConfirmedClearedFields(args, patch_instance, original_instance):
   log.status.write(
       'The following message will be used for the patch API method.\n')
   log.status.write(
-      encoding.MessageToJson(patch_instance, include_fields=cleared_fields) +
-      '\n')
+      encoding.MessageToJson(
+          WithoutKind(patch_instance), include_fields=cleared_fields) + '\n')
 
   _PrintAndConfirmWarningMessage(args, original_instance.databaseVersion)
 
@@ -120,13 +138,19 @@ def AddBaseArgs(parser):
       help=('Clear the list of external networks that are allowed to connect '
             'to the instance.'))
   flags.AddAvailabilityType(parser)
+
   backups_group = parser.add_mutually_exclusive_group()
-  flags.AddBackupStartTime(backups_group)
+
+  backups_enabled_group = backups_group.add_group()
+  flags.AddBackupStartTime(backups_enabled_group)
+  flags.AddBackupLocation(backups_enabled_group, allow_empty=True)
+
   backups_group.add_argument(
       '--no-backup',
       required=False,
       action='store_true',
       help='Specified if daily backup should be disabled.')
+
   database_flags_group = parser.add_mutually_exclusive_group()
   flags.AddDatabaseFlags(database_flags_group)
   database_flags_group.add_argument(
@@ -152,9 +176,10 @@ def AddBaseArgs(parser):
       help=('First Generation instances only. The App Engine app '
             'this instance should follow. It must be in the same region as '
             'the instance. WARNING: Instance may be restarted.'))
-  flags.AddZone(parser, help_text=(
-      'Preferred Compute Engine zone (e.g. us-central1-a, '
-      'us-central1-b, etc.). WARNING: Instance may be restarted.'))
+  flags.AddZone(
+      parser,
+      help_text=('Preferred Compute Engine zone (e.g. us-central1-a, '
+                 'us-central1-b, etc.). WARNING: Instance may be restarted.'))
   parser.add_argument(
       'instance',
       completer=flags.InstanceCompleter,
@@ -192,12 +217,16 @@ def AddBetaArgs(parser):
   labels_util.AddUpdateLabelsFlags(parser, enable_clear=True)
 
 
+def AddAlphaArgs(parser):
+  """Adds alpha args and flags to the parser."""
+  flags.AddEnablePointInTimeRecovery(parser)
+
+
 def RunBasePatchCommand(args, release_track):
   """Updates settings of a Cloud SQL instance using the patch api method.
 
   Args:
-    args: argparse.Namespace, The arguments that this command was invoked
-        with.
+    args: argparse.Namespace, The arguments that this command was invoked with.
     release_track: base.ReleaseTrack, the release track that this was run under.
 
   Returns:
@@ -228,18 +257,17 @@ def RunBasePatchCommand(args, release_track):
       sql_messages.SqlInstancesGetRequest(
           project=instance_ref.project, instance=instance_ref.instance))
 
-  patch_instance = (
-      command_util.InstancesV1Beta4.ConstructPatchInstanceFromArgs(
-          sql_messages,
-          args,
-          original=original_instance_resource,
-          release_track=release_track))
+  patch_instance = command_util.InstancesV1Beta4.ConstructPatchInstanceFromArgs(
+      sql_messages,
+      args,
+      original=original_instance_resource,
+      release_track=release_track)
   patch_instance.project = instance_ref.project
   patch_instance.name = instance_ref.instance
 
   # TODO(b/122660263): Remove when V1 instances are no longer supported.
   # V1 deprecation notice.
-  if api_util.IsInstanceV1(original_instance_resource):
+  if api_util.IsInstanceV1(sql_messages, original_instance_resource):
     command_util.ShowV1DeprecationWarning()
 
   cleared_fields = _GetConfirmedClearedFields(args, patch_instance,
@@ -289,7 +317,7 @@ class Patch(base.UpdateCommand):
     AddBaseArgs(parser)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
 class PatchBeta(base.UpdateCommand):
   """Updates the settings of a Cloud SQL instance."""
 
@@ -301,3 +329,18 @@ class PatchBeta(base.UpdateCommand):
     """Args is called by calliope to gather arguments for this command."""
     AddBaseArgs(parser)
     AddBetaArgs(parser)
+
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class PatchAlpha(base.UpdateCommand):
+  """Updates the settings of a Cloud SQL instance."""
+
+  def Run(self, args):
+    return RunBasePatchCommand(args, self.ReleaseTrack())
+
+  @staticmethod
+  def Args(parser):
+    """Args is called by calliope to gather arguments for this command."""
+    AddBaseArgs(parser)
+    AddBetaArgs(parser)
+    AddAlphaArgs(parser)
