@@ -1,4 +1,5 @@
-# Copyright 2015 Google Inc. All Rights Reserved.
+# -*- coding: utf-8 -*- #
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +15,10 @@
 
 """Gcloud named configuration files supplementary help."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 from googlecloudsdk.core import properties
 
@@ -83,6 +88,13 @@ class TestingArgFiles(base.TopicCommand):
 
     $ gcloud config configurations activate my-config
 
+  To display the path of the active configuration, run:
+
+    $ gcloud info --format='get(config.paths.active_config_path)'
+
+  Note that changes to your OS login, Google Cloud Platform account or project
+  could change the path.
+
   You can view and change the properties of your active configuration
   using the following commands:
 
