@@ -526,9 +526,9 @@ software_update_check_period = 0
 # that is done we will re-enable parallel composite uploads by default in
 # gsutil.
 #
-# Note: Parallel composite uploads should not be used with NEARLINE or COLDLINE
-# storage class buckets, as doing this incurs an early deletion charge for
-# each component object.
+# Note: Parallel composite uploads should not be used with NEARLINE, COLDLINE,
+# or ARCHIVE storage class buckets, as doing this incurs an early deletion
+# charge for each component object.
 #
 # Note: Parallel composite uploads are not enabled with Cloud KMS encrypted
 # objects as a source or destination, as composition with KMS objects is not yet
@@ -585,6 +585,11 @@ software_update_check_period = 0
 #test_hmac_alt_service_account =
 #test_hmac_list_service_account =
 
+# Service account emails for testing impersonation credentials. If this field is
+# not populated with a service account the tests for service account
+# impersonation will not run.  Primarily useful for tool developers.
+#test_impersonate_service_account =
+
 # 'content_language' specifies the ISO 639-1 language code of the content, to be
 # passed in the Content-Language header. By default no Content-Language is sent.
 # See the ISO 639-1 column of
@@ -1118,6 +1123,10 @@ class ConfigCommand(Command):
             '#%s_json_port = <alternate JSON API storage host port>\n'
             '#%s_json_host_header = <alternate JSON API storage host header>\n\n'
             % (host_key, host_key, host_key))
+        config_file.write(
+            '# To impersonate a service account for "%s://" URIs over\n'
+            '# JSON API, edit and uncomment the following line:\n'
+            '#%s_impersonate_service_account = <service account email>\n\n')
       config_file.write('\n')
 
     # Write the config file Boto section.
@@ -1136,7 +1145,8 @@ class ConfigCommand(Command):
 # version to use. If not set below gsutil defaults to API version 1.
 """)
     api_version = 2
-    if cred_type == CredTypes.HMAC: api_version = 1
+    if cred_type == CredTypes.HMAC:
+      api_version = 1
 
     config_file.write('default_api_version = %d\n' % api_version)
 
