@@ -15,9 +15,14 @@
 """Implementation of Unix-like du command for cloud storage providers."""
 
 from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import division
+from __future__ import unicode_literals
 
+import locale
 import sys
 
+import six
 from gslib.bucket_listing_ref import BucketListingObject
 from gslib.command import Command
 from gslib.command_argument import CommandArgument
@@ -29,7 +34,9 @@ from gslib.utils import ls_helper
 from gslib.utils.constants import NO_MAX
 from gslib.utils.constants import S3_DELETE_MARKER_GUID
 from gslib.utils.constants import UTF8
+from gslib.utils.text_util import print_to_fd
 from gslib.utils.unit_util import MakeHumanReadable
+from gslib.utils import text_util
 
 _SYNOPSIS = """
   gsutil du url...
@@ -43,7 +50,8 @@ _DETAILED_HELP_TEXT = ("""
 <B>DESCRIPTION</B>
   The du command displays the amount of space (in bytes) being used by the
   objects in the file or object hierarchy under a given URL. The syntax emulates
-  the Linux du command (which stands for disk usage). For example, the command:
+  the Linux du -b command (which stands for disk usage). For example, the
+  command:
 
   gsutil du -s gs://your-bucket/dir
 
@@ -125,8 +133,8 @@ class DuCommand(Command):
       gs_api_support=[ApiSelector.XML, ApiSelector.JSON],
       gs_default_api=ApiSelector.JSON,
       argparse_arguments=[
-          CommandArgument.MakeZeroOrMoreCloudURLsArgument()
-      ]
+          CommandArgument.MakeZeroOrMoreCloudURLsArgument(),
+      ],
   )
   # Help specification. See help_provider.py for documentation.
   help_spec = Command.HelpSpec(
@@ -140,9 +148,10 @@ class DuCommand(Command):
 
   def _PrintSummaryLine(self, num_bytes, name):
     size_string = (MakeHumanReadable(num_bytes)
-                   if self.human_readable else str(num_bytes))
-    sys.stdout.write('%(size)-10s  %(name)s%(ending)s' % {
-        'size': size_string, 'name': name, 'ending': self.line_ending})
+                   if self.human_readable else six.text_type(num_bytes))
+    text_util.print_to_fd('{size:<11}  {name}'.format(
+        size=size_string, name=six.ensure_text(name)),
+                          end=self.line_ending)
 
   def _PrintInfoAboutBucketListingRef(self, bucket_listing_ref):
     """Print listing info for given bucket_listing_ref.
@@ -158,8 +167,8 @@ class DuCommand(Command):
     """
     obj = bucket_listing_ref.root_object
     url_str = bucket_listing_ref.url_string
-    if (obj.metadata and S3_DELETE_MARKER_GUID in
-        obj.metadata.additionalProperties):
+    if (obj.metadata and
+        S3_DELETE_MARKER_GUID in obj.metadata.additionalProperties):
       size_string = '0'
       num_bytes = 0
       num_objs = 0
@@ -171,10 +180,11 @@ class DuCommand(Command):
       num_objs = 1
 
     if not self.summary_only:
-      sys.stdout.write('%(size)-10s  %(url)s%(ending)s' % {
-          'size': size_string,
-          'url': url_str.encode(UTF8),
-          'ending': self.line_ending})
+      url_detail = '{size:<11}  {url}{ending}'.format(
+          size=size_string,
+          url=six.ensure_text(url_str),
+          ending=six.ensure_text(self.line_ending))
+      print_to_fd(url_detail, file=sys.stdout, end='')
 
     return (num_objs, num_bytes)
 
@@ -203,14 +213,12 @@ class DuCommand(Command):
         elif o == '-X':
           if a == '-':
             f = sys.stdin
+            f_close = False
           else:
-            f = open(a, 'r')
-          try:
-            for line in f:
-              line = line.strip().decode(UTF8)
-              if line:
-                self.exclude_patterns.append(line)
-          finally:
+            f = open(a, 'r') if six.PY2 else open(a, 'r', encoding=UTF8)
+            f_close = True
+          self.exclude_patterns = [six.ensure_text(line.strip()) for line in f]
+          if f_close:
             f.close()
 
     if not self.args:
@@ -233,17 +241,21 @@ class DuCommand(Command):
     for url_arg in self.args:
       top_level_storage_url = StorageUrlFromString(url_arg)
       if top_level_storage_url.IsFileUrl():
-        raise CommandException('Only cloud URLs are supported for %s'
-                               % self.command_name)
+        raise CommandException('Only cloud URLs are supported for %s' %
+                               self.command_name)
       bucket_listing_fields = ['size']
 
       listing_helper = ls_helper.LsHelper(
-          self.WildcardIterator, self.logger,
-          print_object_func=_PrintObjectLong, print_dir_func=_PrintNothing,
+          self.WildcardIterator,
+          self.logger,
+          print_object_func=_PrintObjectLong,
+          print_dir_func=_PrintNothing,
           print_dir_header_func=_PrintNothing,
           print_dir_summary_func=_PrintDirectory,
-          print_newline_func=_PrintNothing, all_versions=self.all_versions,
-          should_recurse=True, exclude_patterns=self.exclude_patterns,
+          print_newline_func=_PrintNothing,
+          all_versions=self.all_versions,
+          should_recurse=True,
+          exclude_patterns=self.exclude_patterns,
           fields=bucket_listing_fields)
 
       # LsHelper expands to objects and prefixes, so perform a top-level
@@ -251,8 +263,8 @@ class DuCommand(Command):
       if top_level_storage_url.IsProvider():
         # Provider URL: use bucket wildcard to iterate over all buckets.
         top_level_iter = self.WildcardIterator(
-            '%s://*' % top_level_storage_url.scheme).IterBuckets(
-                bucket_fields=['id'])
+            '%s://*' %
+            top_level_storage_url.scheme).IterBuckets(bucket_fields=['id'])
       elif top_level_storage_url.IsBucket():
         top_level_iter = self.WildcardIterator(
             '%s://%s' % (top_level_storage_url.scheme,
