@@ -16,6 +16,8 @@
 
 from __future__ import absolute_import
 from __future__ import print_function
+from __future__ import division
+from __future__ import unicode_literals
 
 import getopt
 import re
@@ -70,7 +72,7 @@ _SYNOPSIS = (
     _DELETE_SYNOPSIS.lstrip('\n') +
     _LIST_SYNOPSIS.lstrip('\n') +
     _WATCHBUCKET_SYNOPSIS +
-    _STOPCHANNEL_SYNOPSIS.lstrip('\n') + '\n')
+    _STOPCHANNEL_SYNOPSIS.lstrip('\n') + '\n')  # yapf: disable
 
 _LIST_DESCRIPTION = """
 <B>LIST</B>
@@ -87,7 +89,7 @@ _LIST_DESCRIPTION = """
     gsutil notification list gs://example-bucket
 
   The same as above, but for Object Change Notifications instead of Cloud
-  Pub/Sub notification subscription configs.
+  Pub/Sub notification subscription configs:
 
     gsutil notification list -o gs://example-bucket
 
@@ -140,10 +142,11 @@ _CREATE_DESCRIPTION = """
   create command will use the Cloud Pub/Sub topic
   "projects/default-project/topics/example-bucket".
 
-  In order to enable notifications, a special Cloud Storage service account
-  unique to each project must have the IAM permission "projects.topics.publish".
-  This command will check to see if that permission exists and, if not, will
-  attempt to grant it.
+  In order to enable notifications, a `special Cloud Storage service account
+  <https://cloud.google.com/storage/docs/projects#service-accounts>`_ unique to
+  each project must have the IAM permission "pubsub.topics.publish". This
+  command will check to see if that permission exists and, if not, will attempt
+  to grant it.
 
   You can create multiple notification configurations for a bucket, but their
   triggers cannot overlap such that a single event could send multiple
@@ -203,7 +206,8 @@ _CREATE_DESCRIPTION = """
               OBJECT_FINALIZE - An object has been created.
               OBJECT_METADATA_UPDATE - The metadata of an object has changed.
               OBJECT_DELETE - An object has been permanently deleted.
-              OBJECT_ARCHIVE - A live Cloud Storage object has been archived.
+              OBJECT_ARCHIVE - A live version of an object has become a
+                noncurrent version.
 
   -f        Specifies the payload format of notification messages. Must be
             either "json" for a payload matches the object metadata for the
@@ -328,7 +332,6 @@ _DESCRIPTION = """
 
 """.format(composite_namespace=copy_helper.PARALLEL_UPLOAD_TEMP_NAMESPACE)
 
-
 NOTIFICATION_AUTHORIZATION_FAILED_MESSAGE = """
 Watch bucket attempt failed:
   {watch_error}
@@ -348,6 +351,7 @@ verified using Google Webmaster Tools. For instructions, please see
 
 _DETAILED_HELP_TEXT = CreateHelpText(_SYNOPSIS, _DESCRIPTION)
 
+# yapf: disable
 _create_help_text = (
     CreateHelpText(_CREATE_SYNOPSIS, _CREATE_DESCRIPTION))
 _list_help_text = (
@@ -358,10 +362,11 @@ _watchbucket_help_text = (
     CreateHelpText(_WATCHBUCKET_SYNOPSIS, _WATCHBUCKET_DESCRIPTION))
 _stopchannel_help_text = (
     CreateHelpText(_STOPCHANNEL_SYNOPSIS, _STOPCHANNEL_DESCRIPTION))
+# yapf: enable
 
 PAYLOAD_FORMAT_MAP = {
     'none': 'NONE',
-    'json': 'JSON_API_V1'
+    'json': 'JSON_API_V1',
 }
 
 
@@ -385,7 +390,11 @@ class NotificationCommand(Command):
   command_spec = Command.CreateCommandSpec(
       'notification',
       command_name_aliases=[
-          'notify', 'notifyconfig', 'notifications', 'notif'],
+          'notify',
+          'notifyconfig',
+          'notifications',
+          'notif',
+      ],
       usage_synopsis=_SYNOPSIS,
       min_args=2,
       max_args=NO_MAX,
@@ -398,28 +407,30 @@ class NotificationCommand(Command):
       argparse_arguments={
           'watchbucket': [
               CommandArgument.MakeFreeTextArgument(),
-              CommandArgument.MakeZeroOrMoreCloudBucketURLsArgument()
+              CommandArgument.MakeZeroOrMoreCloudBucketURLsArgument(),
           ],
           'stopchannel': [],
-          'list': [
-              CommandArgument.MakeZeroOrMoreCloudBucketURLsArgument()
-          ],
+          'list': [CommandArgument.MakeZeroOrMoreCloudBucketURLsArgument(),],
           'delete': [
               # Takes a list of one of the following:
               #   notification: projects/_/buckets/bla/notificationConfigs/5,
               #   bucket: gs://foobar
-              CommandArgument.MakeZeroOrMoreCloudURLsArgument()
+              CommandArgument.MakeZeroOrMoreCloudURLsArgument(),
           ],
           'create': [
               CommandArgument.MakeFreeTextArgument(),  # Cloud Pub/Sub topic
-              CommandArgument.MakeNCloudBucketURLsArgument(1)
+              CommandArgument.MakeNCloudBucketURLsArgument(1),
           ]
-      }
+      },
   )
   # Help specification. See help_provider.py for documentation.
   help_spec = Command.HelpSpec(
       help_name='notification',
-      help_name_aliases=['watchbucket', 'stopchannel', 'notifyconfig'],
+      help_name_aliases=[
+          'watchbucket',
+          'stopchannel',
+          'notifyconfig',
+      ],
       help_type='command_help',
       help_one_line_summary='Configure object change notification',
       help_text=_DETAILED_HELP_TEXT,
@@ -428,7 +439,8 @@ class NotificationCommand(Command):
           'list': _list_help_text,
           'delete': _delete_help_text,
           'watchbucket': _watchbucket_help_text,
-          'stopchannel': _stopchannel_help_text},
+          'stopchannel': _stopchannel_help_text,
+      },
   )
 
   def _WatchBucket(self):
@@ -463,12 +475,15 @@ class NotificationCommand(Command):
                      bucket_url, watch_url)
 
     try:
-      channel = self.gsutil_api.WatchBucket(
-          bucket_url.bucket_name, watch_url, identifier, token=client_token,
-          provider=bucket_url.scheme)
-    except AccessDeniedException, e:
-      self.logger.warn(NOTIFICATION_AUTHORIZATION_FAILED_MESSAGE.format(
-          watch_error=str(e), watch_url=watch_url))
+      channel = self.gsutil_api.WatchBucket(bucket_url.bucket_name,
+                                            watch_url,
+                                            identifier,
+                                            token=client_token,
+                                            provider=bucket_url.scheme)
+    except AccessDeniedException as e:
+      self.logger.warn(
+          NOTIFICATION_AUTHORIZATION_FAILED_MESSAGE.format(watch_error=str(e),
+                                                           watch_url=watch_url))
       raise
 
     channel_id = channel.id
@@ -502,16 +517,20 @@ class NotificationCommand(Command):
     if not bucket_url.IsBucket():
       raise CommandException('URL must name a bucket for the %s command.' %
                              self.command_name)
-    channels = self.gsutil_api.ListChannels(
-          bucket_url.bucket_name, provider='gs').items
-    self.logger.info('Bucket %s has the following active Object Change Notifications:', bucket_url.bucket_name)
+    channels = self.gsutil_api.ListChannels(bucket_url.bucket_name,
+                                            provider='gs').items
+    self.logger.info(
+        'Bucket %s has the following active Object Change Notifications:',
+        bucket_url.bucket_name)
     for idx, channel in enumerate(channels):
-      self.logger.info('\tNotification channel %d:', idx+1)
+      self.logger.info('\tNotification channel %d:', idx + 1)
       self.logger.info('\t\tChannel identifier: %s', channel.channel_id)
       self.logger.info('\t\tResource identifier: %s', channel.resource_id)
       self.logger.info('\t\tApplication URL: %s', channel.push_url)
       self.logger.info('\t\tCreated by: %s', channel.subscriber_email)
-      self.logger.info('\t\tCreation time: %s', str(datetime.fromtimestamp(channel.creation_time_ms/1000)))
+      self.logger.info(
+          '\t\tCreation time: %s',
+          str(datetime.fromtimestamp(channel.creation_time_ms / 1000)))
 
     return 0
 
@@ -566,22 +585,21 @@ class NotificationCommand(Command):
     self.logger.debug('Creating notification for bucket %s', bucket_url)
 
     # Find the project this bucket belongs to
-    bucket_metadata = self.gsutil_api.GetBucket(
-        bucket_name,
-        fields=['projectNumber'],
-        provider=bucket_url.scheme)
+    bucket_metadata = self.gsutil_api.GetBucket(bucket_name,
+                                                fields=['projectNumber'],
+                                                provider=bucket_url.scheme)
     bucket_project_number = bucket_metadata.projectNumber
 
     # If not specified, choose a sensible default for the Cloud Pub/Sub topic
     # name.
     if not pubsub_topic:
-      pubsub_topic = 'projects/%s/topics/%s' % (
-          PopulateProjectId(None), bucket_name)
+      pubsub_topic = 'projects/%s/topics/%s' % (PopulateProjectId(None),
+                                                bucket_name)
     if not pubsub_topic.startswith('projects/'):
       # If a user picks a topic ID (mytopic) but doesn't pass the whole name (
       # projects/my-project/topics/mytopic ), pick a default project.
-      pubsub_topic = 'projects/%s/topics/%s' % (
-          PopulateProjectId(None), pubsub_topic)
+      pubsub_topic = 'projects/%s/topics/%s' % (PopulateProjectId(None),
+                                                pubsub_topic)
     self.logger.debug('Using Cloud Pub/Sub topic %s', pubsub_topic)
 
     just_modified_topic_permissions = False
@@ -669,8 +687,7 @@ class NotificationCommand(Command):
                         pubsub_topic)
       return False
 
-  def _EnumerateNotificationsFromArgs(
-      self, accept_notification_configs=True):
+  def _EnumerateNotificationsFromArgs(self, accept_notification_configs=True):
     """Yields bucket/notification tuples from command-line args.
 
     Given a list of strings that are bucket names (gs://foo) or notification
@@ -688,8 +705,8 @@ class NotificationCommand(Command):
       if match:
         if not accept_notification_configs:
           raise CommandException(
-              '%s %s accepts only bucket names, but you provided %s' % (
-                  self.command_name, self.subcommand_name, list_entry))
+              '%s %s accepts only bucket names, but you provided %s' %
+              (self.command_name, self.subcommand_name, list_entry))
         bucket_name = match.group('bucket')
         notification_id = match.group('notification')
         found = False
@@ -729,7 +746,7 @@ class NotificationCommand(Command):
     if self.sub_opts:
       if '-o' in dict(self.sub_opts):
         for bucket_name in self.args:
-          self._ListChannels(bucket_name) 
+          self._ListChannels(bucket_name)
     else:
       for bucket_name, notification in self._EnumerateNotificationsFromArgs(
           accept_notification_configs=False):
@@ -748,11 +765,11 @@ class NotificationCommand(Command):
         print('\t\t%s: %s' % (attr.key, attr.value))
     filters = []
     if notification.event_types:
-      filters.append(
-          '\t\tEvent Types: %s' % ', '.join(notification.event_types))
+      filters.append('\t\tEvent Types: %s' %
+                     ', '.join(notification.event_types))
     if notification.object_name_prefix:
-      filters.append(
-          "\t\tObject name prefix: '%s'" % notification.object_name_prefix)
+      filters.append("\t\tObject name prefix: '%s'" %
+                     notification.object_name_prefix)
     if filters:
       print('\tFilters:')
       for line in filters:
@@ -765,14 +782,16 @@ class NotificationCommand(Command):
     return 0
 
   def _DeleteNotification(self, bucket_name, notification_id):
-    self.gsutil_api.DeleteNotificationConfig(
-        bucket_name, notification=notification_id, provider='gs')
+    self.gsutil_api.DeleteNotificationConfig(bucket_name,
+                                             notification=notification_id,
+                                             provider='gs')
     return 0
 
   def _RunSubCommand(self, func):
     try:
-      (self.sub_opts, self.args) = getopt.getopt(
-          self.args, self.command_spec.supported_sub_args)
+      (self.sub_opts,
+       self.args) = getopt.getopt(self.args,
+                                  self.command_spec.supported_sub_args)
       # Commands with both suboptions and subcommands need to reparse for
       # suboptions, so we log again.
       metrics.LogCommandParams(sub_opts=self.sub_opts)
@@ -785,15 +804,16 @@ class NotificationCommand(Command):
       'list': _List,
       'delete': _Delete,
       'watchbucket': _WatchBucket,
-      'stopchannel': _StopChannel}
+      'stopchannel': _StopChannel
+  }
 
   def RunCommand(self):
     """Command entry point for the notification command."""
     self.subcommand_name = self.args.pop(0)
     if self.subcommand_name in NotificationCommand.SUBCOMMANDS:
       metrics.LogCommandParams(subcommands=[self.subcommand_name])
-      return self._RunSubCommand(NotificationCommand.SUBCOMMANDS[
-          self.subcommand_name])
+      return self._RunSubCommand(
+          NotificationCommand.SUBCOMMANDS[self.subcommand_name])
     else:
       raise CommandException('Invalid subcommand "%s" for the %s command.' %
                              (self.subcommand_name, self.command_name))
