@@ -36,6 +36,7 @@ from gslib.command_runner import CommandRunner
 from gslib.cs_api_map import ApiMapConstants
 from gslib.cs_api_map import ApiSelector
 from gslib.discard_messages_queue import DiscardMessagesQueue
+from gslib.gcs_json_api import GcsJsonApi
 from gslib.tests.mock_logging_handler import MockLoggingHandler
 from gslib.tests.testcase import base
 import gslib.tests.util as util
@@ -71,8 +72,12 @@ class GsutilApiUnitTestClassMapFactory(object):
   def GetClassMap(cls):
     """Returns a class map for use in unit tests."""
     gs_class_map = {
-        ApiSelector.XML: BotoTranslation,
-        ApiSelector.JSON: BotoTranslation
+        ApiSelector.XML:
+            BotoTranslation,
+        # TODO: This should be replaced with 'ApiSelector.JSON: GcsJsonApi'.
+        # Refer Issue https://github.com/GoogleCloudPlatform/gsutil/issues/970
+        ApiSelector.JSON:
+            BotoTranslation
     }
     s3_class_map = {ApiSelector.XML: BotoTranslation}
     class_map = {'gs': gs_class_map, 's3': s3_class_map}
