@@ -29,8 +29,11 @@ from gslib.third_party.storage_apitools import storage_v1_messages as apitools_m
 
 TYPES = set([
     'user',
+    'deleted:user',
     'serviceAccount',
+    'deleted:serviceAccount',
     'group',
+    'deleted:group',
     'domain',
 ])
 
@@ -203,6 +206,9 @@ def BindingStringToTuple(is_grant, input_str):
                e.g. user:foo@bar.com:objectAdmin
                     user:foo@bar.com:objectAdmin,objectViewer
                     user:foo@bar.com
+                    allUsers
+                    deleted:user:foo@bar.com?uid=123:objectAdmin,objectViewer
+                    deleted:serviceAccount:foo@bar.com?uid=123
 
   Raises:
     CommandException in the case of invalid input.
@@ -215,6 +221,9 @@ def BindingStringToTuple(is_grant, input_str):
     input_str += ':'
   if input_str.count(':') == 1:
     tokens = input_str.split(':')
+    if '%s:%s' % (tokens[0], tokens[1]) in TYPES:
+      raise CommandException('Incorrect public member type for binding %s' %
+                             input_str)
     if tokens[0] in PUBLIC_MEMBERS:
       (member, roles) = tokens
     elif tokens[0] in TYPES:
@@ -224,11 +233,20 @@ def BindingStringToTuple(is_grant, input_str):
       raise CommandException('Incorrect public member type for binding %s' %
                              input_str)
   elif input_str.count(':') == 2:
-    (member_type, member_id, roles) = input_str.split(':')
-    if member_type in DISCOURAGED_TYPES:
-      raise CommandException(DISCOURAGED_TYPES_MSG)
-    elif member_type not in TYPES:
-      raise CommandException('Incorrect member type for binding %s' % input_str)
+    tokens = input_str.split(':')
+    if '%s:%s' % (tokens[0], tokens[1]) in TYPES:
+      # case "deleted:user:foo@bar.com?uid=1234"
+      member = ':'.join(tokens)
+      roles = DROP_ALL
+    else:
+      (member_type, member_id, roles) = tokens
+      _check_member_type(member_type, input_str)
+      member = '%s:%s' % (member_type, member_id)
+  elif input_str.count(':') == 3:
+    # case "deleted:user:foo@bar.com?uid=1234:objectAdmin,objectViewer"
+    (member_type_p1, member_type_p2, member_id, roles) = input_str.split(':')
+    member_type = '%s:%s' % (member_type_p1, member_type_p2)
+    _check_member_type(member_type, input_str)
     member = '%s:%s' % (member_type, member_id)
   else:
     raise CommandException('Invalid ch format %s' % input_str)
@@ -245,6 +263,13 @@ def BindingStringToTuple(is_grant, input_str):
   return BindingsTuple(is_grant=is_grant, bindings=bindings)
 
 
+def _check_member_type(member_type, input_str):
+  if member_type in DISCOURAGED_TYPES:
+    raise CommandException(DISCOURAGED_TYPES_MSG)
+  elif member_type not in TYPES:
+    raise CommandException('Incorrect member type for binding %s' % input_str)
+
+
 def ResolveRole(role):
   if not role:
     return DROP_ALL
